@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 cp .github/bot-pr-format-base.sh /tmp
 source /tmp/bot-pr-format-base.sh
@@ -11,5 +11,5 @@ if [[ "$LIST_FILES" != "" ]]; then
   git commit -a -m "Format files
 
 Co-authored-by: $USER_COMBINED"
-  git push fork $LOCAL_BRANCH:$HEAD_BRANCH 2>&1 || bot_error "Cannot push formatted branch, are edits for maintainers allowed?"
+  git push fork "$LOCAL_BRANCH:$HEAD_BRANCH" 2>&1 || bot_error "Cannot push formatted branch, are edits for maintainers allowed?"
 fi
