@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,14 +35,15 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <array>
+#include <cinttypes>
 #include <type_traits>
 #include <utility>
 
 
 #include "accessor_helper.hpp"
-#include "accessor_references.hpp"
 #include "index_span.hpp"
 #include "range.hpp"
+#include "scaled_reduced_row_major_reference.hpp"
 #include "utils.hpp"
 
 
@@ -53,8 +54,6 @@ namespace gko {
  * @ingroup accessor
  */
 namespace acc {
-
-
 namespace detail {
 
 
@@ -85,11 +84,11 @@ struct enable_write_scalar<Dimensionality, Accessor, ScalarType, false> {
      */
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES scalar_type
-    write_scalar_masked(scalar_type value, Indices &&... indices) const
+    write_scalar_masked(scalar_type value, Indices&&... indices) const
     {
         static_assert(sizeof...(Indices) == Dimensionality,
                       "Number of indices must match dimensionality!");
-        scalar_type *GKO_ACC_RESTRICT rest_scalar = self()->scalar_;
+        scalar_type* GKO_ACC_RESTRICT rest_scalar = self()->scalar_;
         return rest_scalar[self()->compute_mask_scalar_index(
                    std::forward<Indices>(indices)...)] = value;
     }
@@ -107,18 +106,18 @@ struct enable_write_scalar<Dimensionality, Accessor, ScalarType, false> {
      */
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES scalar_type
-    write_scalar_direct(scalar_type value, Indices &&... indices) const
+    write_scalar_direct(scalar_type value, Indices&&... indices) const
     {
-        scalar_type *GKO_ACC_RESTRICT rest_scalar = self()->scalar_;
+        scalar_type* GKO_ACC_RESTRICT rest_scalar = self()->scalar_;
         return rest_scalar[self()->compute_direct_scalar_index(
                    std::forward<Indices>(indices)...)] = value;
     }
 
 
 private:
-    constexpr GKO_ACC_ATTRIBUTES const Accessor *self() const
+    constexpr GKO_ACC_ATTRIBUTES const Accessor* self() const
     {
-        return static_cast<const Accessor *>(this);
+        return static_cast<const Accessor*>(this);
     }
 };
 
@@ -156,8 +155,8 @@ private:
  *
  * @note  This class only manages the accesses and not the memory itself.
  */
-template <int Dimensionality, typename ArithmeticType, typename StorageType,
-          size_type ScalarMask>
+template <std::size_t Dimensionality, typename ArithmeticType,
+          typename StorageType, std::uint64_t ScalarMask>
 class scaled_reduced_row_major
     : public detail::enable_write_scalar<
           Dimensionality,
@@ -167,8 +166,8 @@ class scaled_reduced_row_major
 public:
     using arithmetic_type = std::remove_cv_t<ArithmeticType>;
     using storage_type = StorageType;
-    static constexpr size_type dimensionality{Dimensionality};
-    static constexpr size_type scalar_mask{ScalarMask};
+    static constexpr auto dimensionality = Dimensionality;
+    static constexpr auto scalar_mask = ScalarMask;
     static constexpr bool is_const{std::is_const<storage_type>::value};
     using scalar_type =
         std::conditional_t<is_const, const arithmetic_type, arithmetic_type>;
@@ -191,9 +190,9 @@ public:
     friend class range<scaled_reduced_row_major>;
 
 protected:
-    static constexpr size_type scalar_dim{
+    static constexpr std::size_t scalar_dim{
         helper::count_mask_dimensionality<scalar_mask, dimensionality>()};
-    static constexpr size_type scalar_stride_dim{
+    static constexpr std::size_t scalar_stride_dim{
         scalar_dim == 0 ? 0 : (scalar_dim - 1)};
 
     using dim_type = std::array<size_type, dimensionality>;
@@ -202,6 +201,10 @@ protected:
     using reference_type =
         reference_class::scaled_reduced_storage<arithmetic_type, StorageType>;
 
+private:
+    using index_type = std::int64_t;
+
+protected:
     /**
      * Creates the accessor for an already allocated storage space with a
      * stride. The first stride is used for computing the index for the first
@@ -215,8 +218,8 @@ protected:
      * @param scalar_stride  stride array used for memory accesses to scalar
      */
     constexpr GKO_ACC_ATTRIBUTES scaled_reduced_row_major(
-        dim_type size, storage_type *storage,
-        storage_stride_type storage_stride, scalar_type *scalar,
+        dim_type size, storage_type* storage,
+        storage_stride_type storage_stride, scalar_type* scalar,
         scalar_stride_type scalar_stride)
         : size_(size),
           storage_{storage},
@@ -237,13 +240,12 @@ protected:
      *                values.
      */
     constexpr GKO_ACC_ATTRIBUTES scaled_reduced_row_major(
-        dim_type size, storage_type *storage, storage_stride_type stride,
-        scalar_type *scalar)
+        dim_type size, storage_type* storage, storage_stride_type stride,
+        scalar_type* scalar)
         : scaled_reduced_row_major{
               size, storage, stride, scalar,
               helper::compute_default_masked_row_major_stride_array<
-                  typename scalar_stride_type::value_type, scalar_mask,
-                  scalar_stride_dim, dimensionality>(size)}
+                  scalar_mask, scalar_stride_dim, dimensionality>(size)}
     {}
 
     /**
@@ -256,13 +258,11 @@ protected:
      *                values.
      */
     constexpr GKO_ACC_ATTRIBUTES scaled_reduced_row_major(dim_type size,
-                                                          storage_type *storage,
-                                                          scalar_type *scalar)
+                                                          storage_type* storage,
+                                                          scalar_type* scalar)
         : scaled_reduced_row_major{
               size, storage,
-              helper::compute_default_row_major_stride_array<
-                  typename storage_stride_type::value_type>(size),
-              scalar}
+              helper::compute_default_row_major_stride_array(size), scalar}
     {}
 
     /**
@@ -295,9 +295,9 @@ public:
      */
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES scalar_type
-    read_scalar_masked(Indices &&... indices) const
+    read_scalar_masked(Indices&&... indices) const
     {
-        const arithmetic_type *GKO_ACC_RESTRICT rest_scalar = scalar_;
+        const arithmetic_type* GKO_ACC_RESTRICT rest_scalar = scalar_;
         return rest_scalar[compute_mask_scalar_index(
             std::forward<Indices>(indices)...)];
     }
@@ -313,9 +313,9 @@ public:
      */
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES scalar_type
-    read_scalar_direct(Indices &&... indices) const
+    read_scalar_direct(Indices&&... indices) const
     {
-        const arithmetic_type *GKO_ACC_RESTRICT rest_scalar = scalar_;
+        const arithmetic_type* GKO_ACC_RESTRICT rest_scalar = scalar_;
         return rest_scalar[compute_direct_scalar_index(
             std::forward<Indices>(indices)...)];
     }
@@ -389,7 +389,7 @@ public:
      * @returns a const reference to the storage stride array of size
      *          dimensionality - 1
      */
-    constexpr GKO_ACC_ATTRIBUTES const storage_stride_type &get_storage_stride()
+    constexpr GKO_ACC_ATTRIBUTES const storage_stride_type& get_storage_stride()
         const
     {
         return storage_stride_;
@@ -400,7 +400,7 @@ public:
      *
      * @returns a const reference to the scalar stride array
      */
-    constexpr GKO_ACC_ATTRIBUTES const scalar_stride_type &get_scalar_stride()
+    constexpr GKO_ACC_ATTRIBUTES const scalar_stride_type& get_scalar_stride()
         const
     {
         return scalar_stride_;
@@ -411,7 +411,7 @@ public:
      *
      * @returns the pointer to the storage data
      */
-    constexpr GKO_ACC_ATTRIBUTES storage_type *get_stored_data() const
+    constexpr GKO_ACC_ATTRIBUTES storage_type* get_stored_data() const
     {
         return storage_;
     }
@@ -421,7 +421,7 @@ public:
      *
      * @returns a const pointer to the storage data
      */
-    constexpr GKO_ACC_ATTRIBUTES const storage_type *get_const_storage() const
+    constexpr GKO_ACC_ATTRIBUTES const storage_type* get_const_storage() const
     {
         return storage_;
     }
@@ -431,7 +431,7 @@ public:
      *
      * @returns the pointer to the scalar data
      */
-    constexpr GKO_ACC_ATTRIBUTES scalar_type *get_scalar() const
+    constexpr GKO_ACC_ATTRIBUTES scalar_type* get_scalar() const
     {
         return scalar_;
     }
@@ -441,7 +441,7 @@ public:
      *
      * @returns a const pointer to the scalar data
      */
-    constexpr GKO_ACC_ATTRIBUTES const scalar_type *get_const_scalar() const
+    constexpr GKO_ACC_ATTRIBUTES const scalar_type* get_const_scalar() const
     {
         return scalar_;
     }
@@ -449,33 +449,33 @@ public:
 protected:
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES size_type
-    compute_index(Indices &&... indices) const
+    compute_index(Indices&&... indices) const
     {
         static_assert(sizeof...(Indices) == dimensionality,
                       "Number of indices must match dimensionality!");
-        return helper::compute_row_major_index<size_type, dimensionality>(
+        return helper::compute_row_major_index<index_type, dimensionality>(
             size_, storage_stride_, std::forward<Indices>(indices)...);
     }
 
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES size_type
-    compute_mask_scalar_index(Indices &&... indices) const
+    compute_mask_scalar_index(Indices&&... indices) const
     {
         static_assert(sizeof...(Indices) == dimensionality,
                       "Number of indices must match dimensionality!");
-        return helper::compute_masked_index<size_type, scalar_mask,
+        return helper::compute_masked_index<index_type, scalar_mask,
                                             scalar_stride_dim>(
             size_, scalar_stride_, std::forward<Indices>(indices)...);
     }
 
     template <typename... Indices>
     constexpr GKO_ACC_ATTRIBUTES size_type
-    compute_direct_scalar_index(Indices &&... indices) const
+    compute_direct_scalar_index(Indices&&... indices) const
     {
         static_assert(
             sizeof...(Indices) == scalar_dim,
             "Number of indices must match number of set bits in scalar mask!");
-        return helper::compute_masked_index_direct<size_type, scalar_mask,
+        return helper::compute_masked_index_direct<index_type, scalar_mask,
                                                    scalar_stride_dim>(
             size_, scalar_stride_, std::forward<Indices>(indices)...);
     }
@@ -483,9 +483,9 @@ protected:
 
 private:
     const dim_type size_;
-    storage_type *const storage_;
+    storage_type* const storage_;
     const storage_stride_type storage_stride_;
-    scalar_type *const scalar_;
+    scalar_type* const scalar_;
     const scalar_stride_type scalar_stride_;
 };
 
