@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,8 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <algorithm>
 #include <cctype>
+#include <cstring>
+#include <limits>
 #include <map>
 #include <regex>
 #include <string>
@@ -77,7 +79,7 @@ public:
      *
      * @return an instance of the matrix.
      */
-    static const mtx_io &get()
+    static const mtx_io& get()
     {
         static mtx_io instance;
         return instance;
@@ -90,7 +92,7 @@ public:
      *
      * @return the matrix data.
      */
-    matrix_data<ValueType, IndexType> read(std::istream &is) const
+    matrix_data<ValueType, IndexType> read(std::istream& is) const
     {
         auto parsed_header = this->read_header(is);
         std::istringstream dimensions_stream(parsed_header.dimensions_line);
@@ -107,8 +109,8 @@ public:
      * @param data  the matrix data to be written.
      * @param header  the header to be printed at the start of the file.
      */
-    void write(std::ostream &os, const matrix_data<ValueType, IndexType> &data,
-               const std::string &header) const
+    void write(std::ostream& os, const matrix_data<ValueType, IndexType>& data,
+               const std::string& header) const
     {
         std::istringstream header_stream(header);
         auto parsed_header = this->read_description_line(header_stream);
@@ -124,15 +126,15 @@ private:
      * entry of the matrix, depending on its storage scheme:
      */
     struct entry_format {
-        virtual ValueType read_entry(std::istream &is) const = 0;
-        virtual void write_entry(std::ostream &os,
-                                 const ValueType &value) const = 0;
+        virtual ValueType read_entry(std::istream& is) const = 0;
+        virtual void write_entry(std::ostream& os,
+                                 const ValueType& value) const = 0;
     };
 
     /**
      * maps entry format specification strings to algorithms
      */
-    std::map<std::string, const entry_format *> format_map;
+    std::map<std::string, const entry_format*> format_map;
 
     /**
      * the value is encoded as a decimal number
@@ -145,7 +147,7 @@ private:
          *
          * @return the matrix entry.
          */
-        ValueType read_entry(std::istream &is) const override
+        ValueType read_entry(std::istream& is) const override
         {
             double result{};
             GKO_CHECK_STREAM(is >> result, "error while reading matrix entry");
@@ -158,8 +160,8 @@ private:
          * @param  os the output stream
          * @param  value the matrix entry to be written
          */
-        void write_entry(std::ostream &os,
-                         const ValueType &value) const override
+        void write_entry(std::ostream& os,
+                         const ValueType& value) const override
         {
             write_entry_impl<ValueType>(os, value);
         }
@@ -167,7 +169,7 @@ private:
     private:
         template <typename T>
         static std::enable_if_t<is_complex_s<T>::value> write_entry_impl(
-            std::ostream &, const T &)
+            std::ostream&, const T&)
         {
             throw GKO_STREAM_ERROR(
                 "trying to write a complex matrix into a real entry format");
@@ -175,7 +177,7 @@ private:
 
         template <typename T>
         static std::enable_if_t<!is_complex_s<T>::value> write_entry_impl(
-            std::ostream &os, const T &value)
+            std::ostream& os, const T& value)
         {
             GKO_CHECK_STREAM(os << static_cast<double>(value),
                              "error while writing matrix entry");
@@ -194,7 +196,7 @@ private:
          *
          * @return the matrix entry.
          */
-        ValueType read_entry(std::istream &is) const override
+        ValueType read_entry(std::istream& is) const override
         {
             return read_entry_impl<ValueType>(is);
         }
@@ -205,8 +207,8 @@ private:
          * @param  os the output stream
          * @param  value the matrix entry to be written
          */
-        void write_entry(std::ostream &os,
-                         const ValueType &value) const override
+        void write_entry(std::ostream& os,
+                         const ValueType& value) const override
         {
             GKO_CHECK_STREAM(os << static_cast<double>(real(value)) << ' '
                                 << static_cast<double>(imag(value)),
@@ -216,7 +218,7 @@ private:
     private:
         template <typename T>
         static std::enable_if_t<is_complex_s<T>::value, T> read_entry_impl(
-            std::istream &is)
+            std::istream& is)
         {
             using real_type = remove_complex<T>;
             double real{};
@@ -228,7 +230,7 @@ private:
 
         template <typename T>
         static std::enable_if_t<!is_complex_s<T>::value, T> read_entry_impl(
-            std::istream &)
+            std::istream&)
         {
             throw GKO_STREAM_ERROR(
                 "trying to read a complex matrix into a real storage type");
@@ -247,7 +249,7 @@ private:
          *
          * @return the matrix entry(one).
          */
-        ValueType read_entry(std::istream &) const override
+        ValueType read_entry(std::istream&) const override
         {
             return one<ValueType>();
         }
@@ -258,7 +260,7 @@ private:
          * @param  dummy output stream
          * @param  dummy matrix entry to be written
          */
-        void write_entry(std::ostream &, const ValueType &) const override {}
+        void write_entry(std::ostream&, const ValueType&) const override {}
 
     } pattern_format{};
 
@@ -274,8 +276,8 @@ private:
             size_type num_nonzeros) const = 0;
 
         virtual void insert_entry(
-            const IndexType &row, const IndexType &col, const ValueType &entry,
-            matrix_data<ValueType, IndexType> &data) const = 0;
+            const IndexType& row, const IndexType& col, const ValueType& entry,
+            matrix_data<ValueType, IndexType>& data) const = 0;
 
         virtual size_type get_row_start(size_type col) const = 0;
     };
@@ -283,7 +285,7 @@ private:
     /**
      * maps storage modifier specification strings to algorithms
      */
-    std::map<std::string, const storage_modifier *> modifier_map;
+    std::map<std::string, const storage_modifier*> modifier_map;
 
     /**
      * all (nonzero) elements of the matrix are stored
@@ -313,8 +315,8 @@ private:
          * @param data  the data holding the matrix.
          */
         void insert_entry(
-            const IndexType &row, const IndexType &col, const ValueType &entry,
-            matrix_data<ValueType, IndexType> &data) const override
+            const IndexType& row, const IndexType& col, const ValueType& entry,
+            matrix_data<ValueType, IndexType>& data) const override
         {
             data.nonzeros.emplace_back(row, col, entry);
         }
@@ -341,7 +343,8 @@ private:
         size_type get_reservation_size(size_type num_rows, size_type num_cols,
                                        size_type num_nonzeros) const override
         {
-            return 2 * num_nonzeros - max(num_rows, num_cols);
+            return 2 * num_nonzeros -
+                   min(2 * num_nonzeros, max(num_rows, num_cols));
         }
 
         /**
@@ -353,8 +356,8 @@ private:
          * @param data  the data holding the matrix.
          */
         void insert_entry(
-            const IndexType &row, const IndexType &col, const ValueType &entry,
-            matrix_data<ValueType, IndexType> &data) const override
+            const IndexType& row, const IndexType& col, const ValueType& entry,
+            matrix_data<ValueType, IndexType>& data) const override
         {
             data.nonzeros.emplace_back(row, col, entry);
             if (row != col) {
@@ -398,11 +401,13 @@ private:
          * @param data  the data holding the matrix.
          */
         void insert_entry(
-            const IndexType &row, const IndexType &col, const ValueType &entry,
-            matrix_data<ValueType, IndexType> &data) const override
+            const IndexType& row, const IndexType& col, const ValueType& entry,
+            matrix_data<ValueType, IndexType>& data) const override
         {
             data.nonzeros.emplace_back(row, col, entry);
-            data.nonzeros.emplace_back(col, row, -entry);
+            if (row != col) {
+                data.nonzeros.emplace_back(col, row, -entry);
+            }
         }
 
         /**
@@ -431,7 +436,8 @@ private:
         size_type get_reservation_size(size_type num_rows, size_type num_cols,
                                        size_type num_nonzeros) const override
         {
-            return 2 * num_nonzeros - max(num_rows, num_cols);
+            return 2 * num_nonzeros -
+                   min(2 * num_nonzeros, max(num_rows, num_cols));
         }
 
         /**
@@ -443,8 +449,8 @@ private:
          * @param data  the data holding the matrix.
          */
         void insert_entry(
-            const IndexType &row, const IndexType &col, const ValueType &entry,
-            matrix_data<ValueType, IndexType> &data) const override
+            const IndexType& row, const IndexType& col, const ValueType& entry,
+            matrix_data<ValueType, IndexType>& data) const override
         {
             data.nonzeros.emplace_back(row, col, entry);
             if (row != col) {
@@ -476,9 +482,9 @@ private:
          * @return the matrix data
          */
         virtual matrix_data<ValueType, IndexType> read_data(
-            std::istream &header, std::istream &content,
-            const entry_format *entry_reader,
-            const storage_modifier *modifier) const = 0;
+            std::istream& header, std::istream& content,
+            const entry_format* entry_reader,
+            const storage_modifier* modifier) const = 0;
         /**
          * Write the matrix data
          *
@@ -487,16 +493,16 @@ private:
          * @param entry_writer  The entry format to write in.
          * @param modifier  The strorage modifer
          */
-        virtual void write_data(std::ostream &os,
-                                const matrix_data<ValueType, IndexType> &data,
-                                const entry_format *entry_writer,
-                                const storage_modifier *modifier) const = 0;
+        virtual void write_data(std::ostream& os,
+                                const matrix_data<ValueType, IndexType>& data,
+                                const entry_format* entry_writer,
+                                const storage_modifier* modifier) const = 0;
     };
 
     /**
      * maps storage layout specification strings to algorithms
      */
-    std::map<std::string, const storage_layout *> layout_map;
+    std::map<std::string, const storage_layout*> layout_map;
 
     /**
      * the matrix is sparse, and every nonzero is stored together with its
@@ -514,9 +520,9 @@ private:
          * @return the matrix data
          */
         matrix_data<ValueType, IndexType> read_data(
-            std::istream &header, std::istream &content,
-            const entry_format *entry_reader,
-            const storage_modifier *modifier) const override
+            std::istream& header, std::istream& content,
+            const entry_format* entry_reader,
+            const storage_modifier* modifier) const override
         {
             size_type num_rows{};
             size_type num_cols{};
@@ -550,16 +556,16 @@ private:
          * @param entry_writer  The entry format to write in.
          * @param modifier  The strorage modifer
          */
-        void write_data(std::ostream &os,
-                        const matrix_data<ValueType, IndexType> &data,
-                        const entry_format *entry_writer,
-                        const storage_modifier *) const override
+        void write_data(std::ostream& os,
+                        const matrix_data<ValueType, IndexType>& data,
+                        const entry_format* entry_writer,
+                        const storage_modifier*) const override
         {
             // TODO: use the storage modifier
             GKO_CHECK_STREAM(os << data.size[0] << ' ' << data.size[1] << ' '
                                 << data.nonzeros.size() << '\n',
                              "error when writing size information");
-            for (const auto &nonzero : data.nonzeros) {
+            for (const auto& nonzero : data.nonzeros) {
                 GKO_CHECK_STREAM(
                     os << nonzero.row + 1 << ' ' << nonzero.column + 1 << ' ',
                     "error when writing matrix index");
@@ -585,9 +591,9 @@ private:
          * @return the matrix data
          */
         matrix_data<ValueType, IndexType> read_data(
-            std::istream &header, std::istream &content,
-            const entry_format *entry_reader,
-            const storage_modifier *modifier) const override
+            std::istream& header, std::istream& content,
+            const entry_format* entry_reader,
+            const storage_modifier* modifier) const override
         {
             size_type num_rows{};
             size_type num_cols{};
@@ -619,10 +625,10 @@ private:
          * @param entry_writer  The entry format to write in.
          * @param modifier  The strorage modifer
          */
-        void write_data(std::ostream &os,
-                        const matrix_data<ValueType, IndexType> &data,
-                        const entry_format *entry_writer,
-                        const storage_modifier *) const override
+        void write_data(std::ostream& os,
+                        const matrix_data<ValueType, IndexType>& data,
+                        const entry_format* entry_writer,
+                        const storage_modifier*) const override
         {
             using nt = typename matrix_data<ValueType, IndexType>::nonzero_type;
             auto nonzeros = data.nonzeros;
@@ -673,9 +679,9 @@ private:
      * read/write the rest of the file
      */
     struct header_data {
-        const entry_format *entry{};
-        const storage_modifier *modifier{};
-        const storage_layout *layout{};
+        const entry_format* entry{};
+        const storage_modifier* modifier{};
+        const storage_layout* layout{};
         std::string dimensions_line{};
     };
 
@@ -686,7 +692,7 @@ private:
      *
      * @return the data containing the description
      */
-    header_data read_description_line(std::istream &is) const
+    header_data read_description_line(std::istream& is) const
     {
         header_data data{};
 
@@ -731,7 +737,7 @@ private:
      *
      * @return the header data
      */
-    header_data read_header(std::istream &is) const
+    header_data read_header(std::istream& is) const
     {
         auto data = read_description_line(is);
         do {
@@ -754,12 +760,204 @@ private:
  * @return matrix_data  the matrix data.
  */
 template <typename ValueType, typename IndexType>
-matrix_data<ValueType, IndexType> read_raw(std::istream &is)
+matrix_data<ValueType, IndexType> read_raw(std::istream& is)
 {
     return mtx_io<ValueType, IndexType>::get().read(is);
 }
 
 
+/**
+ * Returns the magic number at the beginning of the binary format header for the
+ * given type parameters.
+ *
+ * @tparam ValueType  the value type to be used for the binary storage
+ * @tparam IndexType  the index type to be used for the binary storage
+ */
+template <typename ValueType, typename IndexType>
+static constexpr uint64 binary_format_magic()
+{
+    constexpr auto is_int = std::is_same<IndexType, int32>::value;
+    constexpr auto is_long = std::is_same<IndexType, int64>::value;
+    constexpr auto is_double = std::is_same<ValueType, double>::value;
+    constexpr auto is_float = std::is_same<ValueType, float>::value;
+    constexpr auto is_complex_double =
+        std::is_same<ValueType, std::complex<double>>::value;
+    constexpr auto is_complex_float =
+        std::is_same<ValueType, std::complex<float>>::value;
+    static_assert(is_int || is_long, "invalid storage index type");
+    static_assert(
+        is_double || is_float || is_complex_double || is_complex_float,
+        "invalid storage value type");
+    constexpr auto index_bit = is_int ? 'I' : 'L';
+    constexpr auto value_bit =
+        is_double ? 'D' : (is_float ? 'S' : (is_complex_double ? 'Z' : 'C'));
+    constexpr uint64 shift = 256;
+    constexpr uint64 type_bits = index_bit * shift + value_bit;
+    return 'G' +
+           shift *
+               ('I' +
+                shift *
+                    ('N' +
+                     shift *
+                         ('K' +
+                          shift * ('G' + shift * ('O' + shift * type_bits)))));
+}
+
+
+namespace {
+
+
+template <bool first>
+struct select_helper {};
+
+template <>
+struct select_helper<true> {
+    template <typename T1, typename T2>
+    static T1 get(T1 val, T2)
+    {
+        return val;
+    }
+};
+
+template <>
+struct select_helper<false> {
+    template <typename T1, typename T2>
+    static T2 get(T1, T2 val)
+    {
+        return val;
+    }
+};
+
+
+template <typename FileValueType, typename FileIndexType, typename ValueType,
+          typename IndexType>
+matrix_data<ValueType, IndexType> read_binary_convert(std::istream& is,
+                                                      uint64 num_rows,
+                                                      uint64 num_cols,
+                                                      uint64 num_entries)
+{
+    if (num_rows > std::numeric_limits<IndexType>::max() ||
+        num_cols > std::numeric_limits<IndexType>::max()) {
+        throw GKO_STREAM_ERROR(
+            "cannot read into this format, its index type would overflow");
+    }
+    if (is_complex<FileValueType>() && !is_complex<ValueType>()) {
+        throw GKO_STREAM_ERROR(
+            "cannot read into this format, would assign complex to real");
+    }
+    matrix_data<ValueType, IndexType> result(gko::dim<2>{
+        static_cast<size_type>(num_rows), static_cast<size_type>(num_cols)});
+    result.nonzeros.resize(num_entries);
+    constexpr auto entry_binary_size =
+        sizeof(FileValueType) + 2 * sizeof(FileIndexType);
+    for (size_type i = 0; i < num_entries; i++) {
+        std::array<char, entry_binary_size> block;
+        GKO_CHECK_STREAM(is.read(block.data(), entry_binary_size),
+                         "failed reading entry " + std::to_string(i));
+        FileValueType value{};
+        FileIndexType row{};
+        FileIndexType column{};
+        std::memcpy(&row, &block[0], sizeof(FileIndexType));
+        std::memcpy(&column, &block[sizeof(FileIndexType)],
+                    sizeof(FileIndexType));
+        std::memcpy(&value, &block[2 * sizeof(FileIndexType)],
+                    sizeof(FileValueType));
+        result.nonzeros[i].value = static_cast<ValueType>(
+            select_helper<is_complex<ValueType>()>::get(value, real(value)));
+        result.nonzeros[i].row = row;
+        result.nonzeros[i].column = column;
+    }
+    // sort the entries
+    result.ensure_row_major_order();
+    return result;
+}
+
+
+}  // namespace
+
+
+template <typename ValueType, typename IndexType>
+matrix_data<ValueType, IndexType> read_binary_raw(std::istream& is)
+{
+    static_assert(sizeof(uint64) == 8, "c++ is broken");  // just to be sure
+    std::array<char, 32> header{};
+    GKO_CHECK_STREAM(is.read(header.data(), 32), "failed reading header");
+    uint64 magic{};
+    uint64 num_rows{};
+    uint64 num_cols{};
+    uint64 num_entries{};
+    std::memcpy(&magic, &header[0], 8);
+    std::memcpy(&num_rows, &header[8], 8);
+    std::memcpy(&num_cols, &header[16], 8);
+    std::memcpy(&num_entries, &header[24], 8);
+#define DECLARE_OVERLOAD(_vtype, _itype)                                  \
+    else if (magic == binary_format_magic<_vtype, _itype>())              \
+    {                                                                     \
+        return read_binary_convert<_vtype, _itype, ValueType, IndexType>( \
+            is, num_rows, num_cols, num_entries);                         \
+    }
+    if (false) {
+    }
+    DECLARE_OVERLOAD(double, int32)
+    DECLARE_OVERLOAD(float, int32)
+    DECLARE_OVERLOAD(std::complex<double>, int32)
+    DECLARE_OVERLOAD(std::complex<float>, int32)
+    DECLARE_OVERLOAD(double, int64)
+    DECLARE_OVERLOAD(float, int64)
+    DECLARE_OVERLOAD(std::complex<double>, int64)
+    DECLARE_OVERLOAD(std::complex<float>, int64)
+#undef DECLARE_OVERLOAD
+    else
+    {
+        throw GKO_STREAM_ERROR("invalid header magic number '" +
+                               std::string(header.data(), 8) + "'");
+    }
+}
+
+
+template <typename ValueType, typename IndexType>
+matrix_data<ValueType, IndexType> read_generic_raw(std::istream& is)
+{
+    auto first_char = is.peek();
+    GKO_CHECK_STREAM(is, "failed reading from stream");
+    if (first_char == '%') {
+        return read_raw<ValueType, IndexType>(is);
+    } else {
+        return read_binary_raw<ValueType, IndexType>(is);
+    }
+}
+
+
+template <typename ValueType, typename IndexType>
+void write_binary_raw(std::ostream& os,
+                      const matrix_data<ValueType, IndexType>& mtx)
+{
+    uint64 magic = binary_format_magic<ValueType, IndexType>();
+    uint64 num_rows = mtx.size[0];
+    uint64 num_cols = mtx.size[1];
+    uint64 num_entries = mtx.nonzeros.size();
+    std::array<char, 32> header{};
+    std::memcpy(&header[0], &magic, 8);
+    std::memcpy(&header[8], &num_rows, 8);
+    std::memcpy(&header[16], &num_cols, 8);
+    std::memcpy(&header[24], &num_entries, 8);
+    GKO_CHECK_STREAM(os.write(header.data(), 32), "failed writing header");
+    constexpr auto entry_binary_size =
+        sizeof(ValueType) + 2 * sizeof(IndexType);
+    for (size_type i = 0; i < num_entries; i++) {
+        std::array<char, entry_binary_size> block;
+        std::memcpy(&block[0], &mtx.nonzeros[i].row, sizeof(IndexType));
+        std::memcpy(&block[sizeof(IndexType)], &mtx.nonzeros[i].column,
+                    sizeof(IndexType));
+        std::memcpy(&block[2 * sizeof(IndexType)], &mtx.nonzeros[i].value,
+                    sizeof(ValueType));
+        GKO_CHECK_STREAM(os.write(block.data(), entry_binary_size),
+                         "failed writing entry " + std::to_string(i));
+    }
+    os.flush();
+}
+
+
 /**
  * Writes raw data to the stream.
  *
@@ -768,7 +966,7 @@ matrix_data<ValueType, IndexType> read_raw(std::istream &is)
  * @param layout  the layout type which the data should be written in.
  */
 template <typename ValueType, typename IndexType>
-void write_raw(std::ostream &os, const matrix_data<ValueType, IndexType> &data,
+void write_raw(std::ostream& os, const matrix_data<ValueType, IndexType>& data,
                layout_type layout)
 {
     // TODO: add support for all layout combinations
@@ -781,13 +979,23 @@ void write_raw(std::ostream &os, const matrix_data<ValueType, IndexType> &data,
 
 
 #define GKO_DECLARE_READ_RAW(ValueType, IndexType) \
-    matrix_data<ValueType, IndexType> read_raw(std::istream &is)
+    matrix_data<ValueType, IndexType> read_raw(std::istream& is)
 #define GKO_DECLARE_WRITE_RAW(ValueType, IndexType)               \
-    void write_raw(std::ostream &os,                              \
-                   const matrix_data<ValueType, IndexType> &data, \
+    void write_raw(std::ostream& os,                              \
+                   const matrix_data<ValueType, IndexType>& data, \
                    layout_type layout)
+#define GKO_DECLARE_READ_BINARY_RAW(ValueType, IndexType) \
+    matrix_data<ValueType, IndexType> read_binary_raw(std::istream& is)
+#define GKO_DECLARE_WRITE_BINARY_RAW(ValueType, IndexType) \
+    void write_binary_raw(std::ostream& os,                \
+                          const matrix_data<ValueType, IndexType>& data)
+#define GKO_DECLARE_READ_GENERIC_RAW(ValueType, IndexType) \
+    matrix_data<ValueType, IndexType> read_generic_raw(std::istream& is)
 GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_READ_RAW);
 GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_WRITE_RAW);
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_READ_BINARY_RAW);
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_WRITE_BINARY_RAW);
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(GKO_DECLARE_READ_GENERIC_RAW);
 
 
 }  // namespace gko
