@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -54,17 +54,17 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 namespace gko {
 namespace solver {
-
-
 namespace cb_gmres {
+namespace {
 
 
-GKO_REGISTER_OPERATION(initialize_1, cb_gmres::initialize_1);
-GKO_REGISTER_OPERATION(initialize_2, cb_gmres::initialize_2);
-GKO_REGISTER_OPERATION(step_1, cb_gmres::step_1);
-GKO_REGISTER_OPERATION(step_2, cb_gmres::step_2);
+GKO_REGISTER_OPERATION(initialize, cb_gmres::initialize);
+GKO_REGISTER_OPERATION(restart, cb_gmres::restart);
+GKO_REGISTER_OPERATION(arnoldi, cb_gmres::arnoldi);
+GKO_REGISTER_OPERATION(solve_krylov, cb_gmres::solve_krylov);
 
 
+}  // anonymous namespace
 }  // namespace cb_gmres
 
 
@@ -187,8 +187,11 @@ struct helper<std::complex<T>> {
 
 
 template <typename ValueType>
-void CbGmres<ValueType>::apply_impl(const LinOp *b, LinOp *x) const
+void CbGmres<ValueType>::apply_impl(const LinOp* b, LinOp* x) const
 {
+    if (!this->get_system_matrix()) {
+        return;
+    }
     precision_dispatch_real_complex<ValueType>(
         [this](auto dense_b, auto dense_x) {
             this->apply_dense_impl(dense_b, dense_x);
@@ -199,14 +202,13 @@ void CbGmres<ValueType>::apply_impl(const LinOp *b, LinOp *x) const
 
 template <typename ValueType>
 void CbGmres<ValueType>::apply_dense_impl(
-    const matrix::Dense<ValueType> *dense_b,
-    matrix::Dense<ValueType> *dense_x) const
+    const matrix::Dense<ValueType>* dense_b,
+    matrix::Dense<ValueType>* dense_x) const
 {
     // Current workaround to get a lambda with a template argument (only
     // the type of `value` matters, the content does not)
     auto apply_templated = [&](auto value) {
         using storage_type = decltype(value);
-        GKO_ASSERT_IS_SQUARE_MATRIX(system_matrix_);
 
         using Vector = matrix::Dense<ValueType>;
         using VectorNorms = matrix::Dense<remove_complex<ValueType>>;
@@ -221,6 +223,9 @@ void CbGmres<ValueType>::apply_dense_impl(
         auto one_op = initialize<Vector>({one<ValueType>()}, exec);
         auto neg_one_op = initialize<Vector>({-one<ValueType>()}, exec);
 
+        const auto num_rows = this->get_size()[0];
+        const auto num_rhs = dense_b->get_size()[1];
+        const auto krylov_dim = this->get_krylov_dim();
         auto residual = Vector::create_with_config_of(dense_b);
         /* The dimensions {x, y, z} explained for the krylov_bases:
          * - x: selects the krylov vector (which has krylov_dim + 1 vectors)
@@ -228,28 +233,21 @@ void CbGmres<ValueType>::apply_dense_impl(
          * - z: selects which column-element of said krylov vector should be
          *      used
          */
-        const dim<3> krylov_bases_dim{krylov_dim_ + 1,
-                                      system_matrix_->get_size()[1],
-                                      dense_b->get_size()[1]};
+        const dim<3> krylov_bases_dim{krylov_dim + 1, num_rows, num_rhs};
         Range3dHelper helper(exec, krylov_bases_dim);
         auto krylov_bases_range = helper.get_range();
 
         auto next_krylov_basis = Vector::create_with_config_of(dense_b);
         std::shared_ptr<matrix::Dense<ValueType>> preconditioned_vector =
             Vector::create_with_config_of(dense_b);
-        auto hessenberg = Vector::create(
-            exec,
-            dim<2>{krylov_dim_ + 1, krylov_dim_ * dense_b->get_size()[1]});
-        auto buffer = Vector::create(
-            exec, dim<2>{krylov_dim_ + 1, dense_b->get_size()[1]});
-        auto givens_sin =
-            Vector::create(exec, dim<2>{krylov_dim_, dense_b->get_size()[1]});
-        auto givens_cos =
-            Vector::create(exec, dim<2>{krylov_dim_, dense_b->get_size()[1]});
-        auto residual_norm_collection = Vector::create(
-            exec, dim<2>{krylov_dim_ + 1, dense_b->get_size()[1]});
-        auto residual_norm =
-            VectorNorms::create(exec, dim<2>{1, dense_b->get_size()[1]});
+        auto hessenberg =
+            Vector::create(exec, dim<2>{krylov_dim + 1, krylov_dim * num_rhs});
+        auto buffer = Vector::create(exec, dim<2>{krylov_dim + 1, num_rhs});
+        auto givens_sin = Vector::create(exec, dim<2>{krylov_dim, num_rhs});
+        auto givens_cos = Vector::create(exec, dim<2>{krylov_dim, num_rhs});
+        auto residual_norm_collection =
+            Vector::create(exec, dim<2>{krylov_dim + 1, num_rhs});
+        auto residual_norm = VectorNorms::create(exec, dim<2>{1, num_rhs});
         // 1st row of arnoldi_norm: == eta * norm2(old_next_krylov_basis)
         //                          with eta == 1 / sqrt(2)
         //                          (computed right before updating
@@ -258,49 +256,44 @@ void CbGmres<ValueType>::apply_dense_impl(
         //                          == norm2(next_krylov_basis)
         // 3rd row of arnoldi_norm: the infinity norm of next_krylov_basis
         //                          (ONLY when using a scalar accessor)
-        auto arnoldi_norm =
-            VectorNorms::create(exec, dim<2>{3, dense_b->get_size()[1]});
-        Array<size_type> final_iter_nums(this->get_executor(),
-                                         dense_b->get_size()[1]);
-        auto y =
-            Vector::create(exec, dim<2>{krylov_dim_, dense_b->get_size()[1]});
+        auto arnoldi_norm = VectorNorms::create(exec, dim<2>{3, num_rhs});
+        array<size_type> final_iter_nums(this->get_executor(), num_rhs);
+        auto y = Vector::create(exec, dim<2>{krylov_dim, num_rhs});
 
         bool one_changed{};
-        Array<stopping_status> stop_status(this->get_executor(),
-                                           dense_b->get_size()[1]);
+        array<char> reduction_tmp{this->get_executor()};
+        array<stopping_status> stop_status(this->get_executor(), num_rhs);
         // reorth_status and num_reorth are both helper variables for GPU
         // implementations at the moment.
         // num_reorth := Number of vectors which require a re-orthogonalization
         // reorth_status := stopping status for the re-orthogonalization,
         //                  marking which RHS requires one, and which does not
-        Array<stopping_status> reorth_status(this->get_executor(),
-                                             dense_b->get_size()[1]);
-        Array<size_type> num_reorth(this->get_executor(), 1);
+        array<stopping_status> reorth_status(this->get_executor(), num_rhs);
+        array<size_type> num_reorth(this->get_executor(), 1);
 
         // Initialization
-        exec->run(cb_gmres::make_initialize_1(
-            dense_b, residual.get(), givens_sin.get(), givens_cos.get(),
-            &stop_status, krylov_dim_));
+        exec->run(cb_gmres::make_initialize(dense_b, residual.get(),
+                                            givens_sin.get(), givens_cos.get(),
+                                            &stop_status, krylov_dim));
         // residual = dense_b
         // givens_sin = givens_cos = 0
-        system_matrix_->apply(neg_one_op.get(), dense_x, one_op.get(),
-                              residual.get());
+        this->get_system_matrix()->apply(neg_one_op, dense_x, one_op, residual);
         // residual = residual - Ax
 
-        exec->run(cb_gmres::make_initialize_2(
+        exec->run(cb_gmres::make_restart(
             residual.get(), residual_norm.get(), residual_norm_collection.get(),
             arnoldi_norm.get(), krylov_bases_range, next_krylov_basis.get(),
-            &final_iter_nums, krylov_dim_));
+            &final_iter_nums, reduction_tmp, krylov_dim));
         // residual_norm = norm(residual)
         // residual_norm_collection = {residual_norm, 0, ..., 0}
         // krylov_bases(:, 1) = residual / residual_norm
         // next_krylov_basis = residual / residual_norm
         // final_iter_nums = {0, ..., 0}
 
-        auto stop_criterion = stop_criterion_factory_->generate(
-            system_matrix_,
-            std::shared_ptr<const LinOp>(dense_b, [](const LinOp *) {}),
-            dense_x, residual.get());
+        auto stop_criterion = this->get_stop_criterion_factory()->generate(
+            this->get_system_matrix(),
+            std::shared_ptr<const LinOp>(dense_b, [](const LinOp*) {}), dense_x,
+            residual.get());
 
         int total_iter = -1;
         size_type restart_iter = 0;
@@ -310,11 +303,9 @@ void CbGmres<ValueType>::apply_dense_impl(
         auto after_preconditioner =
             matrix::Dense<ValueType>::create_with_config_of(dense_x);
 
-        Array<bool> stop_encountered_rhs(exec->get_master(),
-                                         dense_b->get_size()[1]);
-        Array<bool> fully_converged_rhs(exec->get_master(),
-                                        dense_b->get_size()[1]);
-        Array<stopping_status> host_stop_status(
+        array<bool> stop_encountered_rhs(exec->get_master(), num_rhs);
+        array<bool> fully_converged_rhs(exec->get_master(), num_rhs);
+        array<stopping_status> host_stop_status(
             this->get_executor()->get_master(), stop_status);
         for (size_type i = 0; i < stop_encountered_rhs.get_num_elems(); ++i) {
             stop_encountered_rhs.get_data()[i] = false;
@@ -322,33 +313,36 @@ void CbGmres<ValueType>::apply_dense_impl(
         }
         // Start only after this value with performing forced iterations after
         // convergence detection
-        constexpr decltype(total_iter) start_force_reset{10};
+        constexpr int start_force_reset{10};
         bool perform_reset{false};
-        // Fraction of the krylov_dim_ (or total_iter if it is lower),
+        // Fraction of the krylov_dim (or total_iter if it is lower),
         // determining the number of forced iteration to perform
-        constexpr decltype(krylov_dim_) forced_iteration_fraction{10};
-        const decltype(krylov_dim_) forced_limit{krylov_dim_ /
-                                                 forced_iteration_fraction};
+        constexpr size_type forced_iteration_fraction{10};
+        const size_type forced_limit{krylov_dim / forced_iteration_fraction};
         // Counter for the forced iterations. Start at max in order to properly
         // test convergence at the beginning
-        decltype(krylov_dim_) forced_iterations{forced_limit};
+        size_type forced_iterations{forced_limit};
 
         while (true) {
             ++total_iter;
-            this->template log<log::Logger::iteration_complete>(
-                this, total_iter, residual.get(), dense_x, residual_norm.get());
             // In the beginning, only force a fraction of the total iterations
             if (forced_iterations < forced_limit &&
                 forced_iterations < total_iter / forced_iteration_fraction) {
+                this->template log<log::Logger::iteration_complete>(
+                    this, dense_b, dense_x, total_iter, residual.get(),
+                    residual_norm.get(), nullptr, &stop_status, false);
                 ++forced_iterations;
             } else {
                 bool all_changed = stop_criterion->update()
                                        .num_iterations(total_iter)
-                                       .residual(residual.get())
-                                       .residual_norm(residual_norm.get())
+                                       .residual(residual)
+                                       .residual_norm(residual_norm)
                                        .solution(dense_x)
                                        .check(RelativeStoppingId, true,
                                               &stop_status, &one_changed);
+                this->template log<log::Logger::iteration_complete>(
+                    this, dense_b, dense_x, total_iter, residual.get(),
+                    residual_norm.get(), nullptr, &stop_status, all_changed);
                 if (one_changed || all_changed) {
                     host_stop_status = stop_status;
                     bool host_array_changed{false};
@@ -391,15 +385,14 @@ void CbGmres<ValueType>::apply_dense_impl(
                 }
             }
 
-            if (perform_reset || restart_iter == krylov_dim_) {
+            if (perform_reset || restart_iter == krylov_dim) {
                 perform_reset = false;
                 // Restart
                 // use a view in case this is called earlier
                 auto hessenberg_view = hessenberg->create_submatrix(
-                    span{0, restart_iter},
-                    span{0, dense_b->get_size()[1] * (restart_iter)});
+                    span{0, restart_iter}, span{0, num_rhs * (restart_iter)});
 
-                exec->run(cb_gmres::make_step_2(
+                exec->run(cb_gmres::make_solve_krylov(
                     residual_norm_collection.get(),
                     krylov_bases_range.get_accessor().to_const(),
                     hessenberg_view.get(), y.get(), before_preconditioner.get(),
@@ -407,21 +400,21 @@ void CbGmres<ValueType>::apply_dense_impl(
                 // Solve upper triangular.
                 // y = hessenberg \ residual_norm_collection
 
-                this->get_preconditioner()->apply(before_preconditioner.get(),
-                                                  after_preconditioner.get());
-                dense_x->add_scaled(one_op.get(), after_preconditioner.get());
+                this->get_preconditioner()->apply(before_preconditioner,
+                                                  after_preconditioner);
+                dense_x->add_scaled(one_op, after_preconditioner);
                 // Solve x
                 // x = x + get_preconditioner() * krylov_bases * y
                 residual->copy_from(dense_b);
                 // residual = dense_b
-                system_matrix_->apply(neg_one_op.get(), dense_x, one_op.get(),
-                                      residual.get());
+                this->get_system_matrix()->apply(neg_one_op, dense_x, one_op,
+                                                 residual);
                 // residual = residual - Ax
-                exec->run(cb_gmres::make_initialize_2(
+                exec->run(cb_gmres::make_restart(
                     residual.get(), residual_norm.get(),
                     residual_norm_collection.get(), arnoldi_norm.get(),
                     krylov_bases_range, next_krylov_basis.get(),
-                    &final_iter_nums, krylov_dim_));
+                    &final_iter_nums, reduction_tmp, krylov_dim));
                 // residual_norm = norm(residual)
                 // residual_norm_collection = {residual_norm, 0, ..., 0}
                 // krylov_bases(:, 1) = residual / residual_norm
@@ -430,24 +423,23 @@ void CbGmres<ValueType>::apply_dense_impl(
                 restart_iter = 0;
             }
 
-            this->get_preconditioner()->apply(next_krylov_basis.get(),
-                                              preconditioned_vector.get());
+            this->get_preconditioner()->apply(next_krylov_basis,
+                                              preconditioned_vector);
             // preconditioned_vector = get_preconditioner() *
             // next_krylov_basis
 
             // Do Arnoldi and givens rotation
             auto hessenberg_iter = hessenberg->create_submatrix(
                 span{0, restart_iter + 2},
-                span{dense_b->get_size()[1] * restart_iter,
-                     dense_b->get_size()[1] * (restart_iter + 1)});
+                span{num_rhs * restart_iter, num_rhs * (restart_iter + 1)});
             auto buffer_iter = buffer->create_submatrix(
-                span{0, restart_iter + 2}, span{0, dense_b->get_size()[1]});
+                span{0, restart_iter + 2}, span{0, num_rhs});
 
             // Start of arnoldi
-            system_matrix_->apply(preconditioned_vector.get(),
-                                  next_krylov_basis.get());
+            this->get_system_matrix()->apply(preconditioned_vector,
+                                             next_krylov_basis);
             // next_krylov_basis = A * preconditioned_vector
-            exec->run(cb_gmres::make_step_1(
+            exec->run(cb_gmres::make_arnoldi(
                 next_krylov_basis.get(), givens_sin.get(), givens_cos.get(),
                 residual_norm.get(), residual_norm_collection.get(),
                 krylov_bases_range, hessenberg_iter.get(), buffer_iter.get(),
@@ -481,44 +473,49 @@ void CbGmres<ValueType>::apply_dense_impl(
         // Solve x
 
         auto hessenberg_small = hessenberg->create_submatrix(
-            span{0, restart_iter},
-            span{0, dense_b->get_size()[1] * (restart_iter)});
+            span{0, restart_iter}, span{0, num_rhs * restart_iter});
 
-        exec->run(cb_gmres::make_step_2(
+        exec->run(cb_gmres::make_solve_krylov(
             residual_norm_collection.get(),
             krylov_bases_range.get_accessor().to_const(),
             hessenberg_small.get(), y.get(), before_preconditioner.get(),
             &final_iter_nums));
         // Solve upper triangular.
         // y = hessenberg \ residual_norm_collection
-        this->get_preconditioner()->apply(before_preconditioner.get(),
-                                          after_preconditioner.get());
-        dense_x->add_scaled(one_op.get(), after_preconditioner.get());
+        this->get_preconditioner()->apply(before_preconditioner,
+                                          after_preconditioner);
+        dense_x->add_scaled(one_op, after_preconditioner);
         // Solve x
         // x = x + get_preconditioner() * krylov_bases * y
     };  // End of apply_lambda
 
     // Look which precision to use as the storage type
-    helper<ValueType>::call(apply_templated, storage_precision_);
+    helper<ValueType>::call(apply_templated, this->get_storage_precision());
 }
 
 
 template <typename ValueType>
-void CbGmres<ValueType>::apply_impl(const LinOp *alpha, const LinOp *b,
-                                    const LinOp *beta, LinOp *x) const
+void CbGmres<ValueType>::apply_impl(const LinOp* alpha, const LinOp* b,
+                                    const LinOp* beta, LinOp* x) const
 {
+    if (!this->get_system_matrix()) {
+        return;
+    }
     precision_dispatch_real_complex<ValueType>(
         [this](auto dense_alpha, auto dense_b, auto dense_beta, auto dense_x) {
             auto x_clone = dense_x->clone();
             this->apply_dense_impl(dense_b, x_clone.get());
             dense_x->scale(dense_beta);
-            dense_x->add_scaled(dense_alpha, x_clone.get());
+            dense_x->add_scaled(dense_alpha, x_clone);
         },
         alpha, b, beta, x);
 }
 
 #define GKO_DECLARE_CB_GMRES(_type1) class CbGmres<_type1>
+#define GKO_DECLARE_CB_GMRES_TRAITS(_type1) \
+    struct workspace_traits<CbGmres<_type1>>
 GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_CB_GMRES);
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_CB_GMRES_TRAITS);
 
 
 }  // namespace solver
