@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -34,6 +34,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <algorithm>
+#include <type_traits>
 
 
 #include <gtest/gtest.h>
@@ -57,7 +58,7 @@ protected:
         x.get_data()[1] = 2;
     }
 
-    static void assert_equal_to_original_x(gko::Array<T> &a,
+    static void assert_equal_to_original_x(gko::array<T>& a,
                                            bool check_zero = true)
     {
         ASSERT_EQ(a.get_num_elems(), 2);
@@ -68,15 +69,15 @@ protected:
     }
 
     std::shared_ptr<const gko::Executor> exec;
-    gko::Array<T> x;
+    gko::array<T> x;
 };
 
-TYPED_TEST_SUITE(Array, gko::test::ValueAndIndexTypes);
+TYPED_TEST_SUITE(Array, gko::test::ValueAndIndexTypes, TypenameNameGenerator);
 
 
 TYPED_TEST(Array, CanBeCreatedWithoutAnExecutor)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     ASSERT_EQ(a.get_executor(), nullptr);
     ASSERT_EQ(a.get_num_elems(), 0);
@@ -85,7 +86,7 @@ TYPED_TEST(Array, CanBeCreatedWithoutAnExecutor)
 
 TYPED_TEST(Array, CanBeEmpty)
 {
-    gko::Array<TypeParam> a(this->exec);
+    gko::array<TypeParam> a(this->exec);
 
     ASSERT_EQ(a.get_num_elems(), 0);
 }
@@ -93,7 +94,7 @@ TYPED_TEST(Array, CanBeEmpty)
 
 TYPED_TEST(Array, ReturnsNullWhenEmpty)
 {
-    gko::Array<TypeParam> a(this->exec);
+    gko::array<TypeParam> a(this->exec);
 
     EXPECT_EQ(a.get_const_data(), nullptr);
     ASSERT_EQ(a.get_data(), nullptr);
@@ -102,7 +103,7 @@ TYPED_TEST(Array, ReturnsNullWhenEmpty)
 
 TYPED_TEST(Array, CanBeCreatedFromExistingData)
 {
-    gko::Array<TypeParam> a{this->exec, 3, new TypeParam[3],
+    gko::array<TypeParam> a{this->exec, 3, new TypeParam[3],
                             std::default_delete<TypeParam[]>{}};
 
     EXPECT_EQ(a.get_num_elems(), 3);
@@ -111,7 +112,7 @@ TYPED_TEST(Array, CanBeCreatedFromExistingData)
 
 TYPED_TEST(Array, CanBeCreatedFromDataOnExecutor)
 {
-    gko::Array<TypeParam> a{this->exec, 3,
+    gko::array<TypeParam> a{this->exec, 3,
                             this->exec->template alloc<TypeParam>(3)};
 
     EXPECT_EQ(a.get_num_elems(), 3);
@@ -123,7 +124,7 @@ TYPED_TEST(Array, CanBeCreatedFromRange)
     using std::begin;
     auto data = {1, 2, 3};
 
-    gko::Array<TypeParam> a{this->exec, begin(data), end(data)};
+    gko::array<TypeParam> a{this->exec, begin(data), end(data)};
 
     EXPECT_EQ(a.get_const_data()[0], TypeParam{1});
     EXPECT_EQ(a.get_const_data()[1], TypeParam{2});
@@ -133,7 +134,7 @@ TYPED_TEST(Array, CanBeCreatedFromRange)
 
 TYPED_TEST(Array, CanBeCreatedFromInitializerList)
 {
-    gko::Array<TypeParam> a{this->exec, {1, 2, 3}};
+    gko::array<TypeParam> a{this->exec, {1, 2, 3}};
 
     EXPECT_EQ(a.get_const_data()[0], TypeParam{1});
     EXPECT_EQ(a.get_const_data()[1], TypeParam{2});
@@ -166,7 +167,7 @@ TYPED_TEST(Array, KnowsItsExecutor)
 
 TYPED_TEST(Array, CanBeCopyConstructed)
 {
-    gko::Array<TypeParam> a(this->x);
+    gko::array<TypeParam> a(this->x);
     this->x.get_data()[0] = 7;
 
     this->assert_equal_to_original_x(a);
@@ -175,7 +176,7 @@ TYPED_TEST(Array, CanBeCopyConstructed)
 
 TYPED_TEST(Array, CanBeMoveConstructed)
 {
-    gko::Array<TypeParam> a(std::move(this->x));
+    gko::array<TypeParam> a(std::move(this->x));
 
     this->assert_equal_to_original_x(a);
 }
@@ -183,7 +184,7 @@ TYPED_TEST(Array, CanBeMoveConstructed)
 
 TYPED_TEST(Array, CanBeCopyConstructedToADifferentExecutor)
 {
-    gko::Array<TypeParam> a{this->exec, this->x};
+    gko::array<TypeParam> a{this->exec, this->x};
 
     this->assert_equal_to_original_x(a);
 }
@@ -191,16 +192,46 @@ TYPED_TEST(Array, CanBeCopyConstructedToADifferentExecutor)
 
 TYPED_TEST(Array, CanBeMoveConstructedToADifferentExecutor)
 {
-    gko::Array<TypeParam> a{this->exec, std::move(this->x)};
+    gko::array<TypeParam> a{this->exec, std::move(this->x)};
 
     this->assert_equal_to_original_x(a);
 }
 
 
+TYPED_TEST(Array, MoveConstructedFromArrayExecutorlessIsEmpty)
+{
+    gko::array<TypeParam> a{std::move(this->x)};
+
+    a = std::move(this->x);
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
+TYPED_TEST(Array, MoveConstructedFromArraySameExecutorIsEmpty)
+{
+    gko::array<TypeParam> a{this->exec, std::move(this->x)};
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
+TYPED_TEST(Array, MoveConstructedFromArrayDifferentExecutorIsEmpty)
+{
+    gko::array<TypeParam> a{gko::ReferenceExecutor::create(),
+                            std::move(this->x)};
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
 TYPED_TEST(Array, CanBeCopied)
 {
     auto omp = gko::OmpExecutor::create();
-    gko::Array<TypeParam> a(omp, 3);
+    gko::array<TypeParam> a(omp, 3);
 
     a = this->x;
     this->x.get_data()[0] = 7;
@@ -211,7 +242,7 @@ TYPED_TEST(Array, CanBeCopied)
 
 TYPED_TEST(Array, CanBeCopiedToExecutorlessArray)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     a = this->x;
 
@@ -222,11 +253,11 @@ TYPED_TEST(Array, CanBeCopiedToExecutorlessArray)
 
 TYPED_TEST(Array, CanBeCopiedFromExecutorlessArray)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     this->x = a;
 
-    ASSERT_NE(this->x.get_executor(), nullptr);
+    ASSERT_EQ(this->x.get_executor(), this->exec);
     ASSERT_EQ(this->x.get_num_elems(), 0);
 }
 
@@ -234,7 +265,7 @@ TYPED_TEST(Array, CanBeCopiedFromExecutorlessArray)
 TYPED_TEST(Array, CanBeMoved)
 {
     auto omp = gko::OmpExecutor::create();
-    gko::Array<TypeParam> a(omp, 3);
+    gko::array<TypeParam> a(omp, 3);
 
     a = std::move(this->x);
 
@@ -244,22 +275,55 @@ TYPED_TEST(Array, CanBeMoved)
 
 TYPED_TEST(Array, CanBeMovedToExecutorlessArray)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     a = std::move(this->x);
 
-    ASSERT_NE(a.get_executor(), nullptr);
+    ASSERT_EQ(a.get_executor(), this->exec);
     this->assert_equal_to_original_x(a);
 }
 
 
 TYPED_TEST(Array, CanBeMovedFromExecutorlessArray)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     this->x = std::move(a);
 
-    ASSERT_NE(this->x.get_executor(), nullptr);
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
+TYPED_TEST(Array, MovedFromArrayExecutorlessIsEmpty)
+{
+    gko::array<TypeParam> a;
+
+    a = std::move(this->x);
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
+TYPED_TEST(Array, MovedFromArraySameExecutorIsEmpty)
+{
+    gko::array<TypeParam> a{this->exec};
+
+    a = std::move(this->x);
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
+    ASSERT_EQ(this->x.get_num_elems(), 0);
+}
+
+
+TYPED_TEST(Array, MovedFromArrayDifferentExecutorIsEmpty)
+{
+    gko::array<TypeParam> a{gko::ReferenceExecutor::create()};
+
+    a = std::move(this->x);
+
+    ASSERT_EQ(this->x.get_executor(), this->exec);
     ASSERT_EQ(this->x.get_num_elems(), 0);
 }
 
@@ -355,7 +419,7 @@ TYPED_TEST(Array, CanBeResized)
 TYPED_TEST(Array, ViewCannotBeResized)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::Array<TypeParam>::view(this->exec, 3, data);
+    auto view = gko::make_array_view(this->exec, 3, data);
 
     EXPECT_THROW(view.resize_and_reset(1), gko::NotSupported);
     EXPECT_EQ(view.get_num_elems(), 3);
@@ -366,7 +430,7 @@ TYPED_TEST(Array, ViewCannotBeResized)
 template <typename T>
 class my_null_deleter {
 public:
-    using pointer = T *;
+    using pointer = T*;
 
     void operator()(pointer) const noexcept {}
 };
@@ -383,7 +447,7 @@ public:
 TYPED_TEST(Array, CustomDeleterCannotBeResized)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view_custom_deleter = gko::Array<TypeParam>(
+    auto view_custom_deleter = gko::array<TypeParam>(
         this->exec, 3, data, my_null_deleter<TypeParam[]>{});
 
     EXPECT_THROW(view_custom_deleter.resize_and_reset(1), gko::NotSupported);
@@ -394,7 +458,7 @@ TYPED_TEST(Array, CustomDeleterCannotBeResized)
 
 TYPED_TEST(Array, CanBeAssignedAnExecutor)
 {
-    gko::Array<TypeParam> a;
+    gko::array<TypeParam> a;
 
     a.set_executor(this->exec);
 
@@ -415,7 +479,7 @@ TYPED_TEST(Array, ChangesExecutors)
 TYPED_TEST(Array, ViewModifiesOriginalData)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::Array<TypeParam>::view(this->exec, 3, data);
+    auto view = gko::make_array_view(this->exec, 3, data);
 
     TypeParam new_data[] = {5, 4, 2};
     std::copy(new_data, new_data + 3, view.get_data());
@@ -429,8 +493,8 @@ TYPED_TEST(Array, ViewModifiesOriginalData)
 
 TYPED_TEST(Array, CopyArrayToArray)
 {
-    gko::Array<TypeParam> array(this->exec, {1, 2, 3});
-    gko::Array<TypeParam> array2(this->exec, {5, 4, 2, 1});
+    gko::array<TypeParam> array(this->exec, {1, 2, 3});
+    gko::array<TypeParam> array2(this->exec, {5, 4, 2, 1});
 
     array = array2;
 
@@ -447,11 +511,11 @@ TYPED_TEST(Array, CopyArrayToArray)
 TYPED_TEST(Array, CopyViewToView)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::Array<TypeParam>::view(this->exec, 3, data);
+    auto view = gko::make_array_view(this->exec, 3, data);
     TypeParam data2[] = {5, 4, 2};
-    auto view2 = gko::Array<TypeParam>::view(this->exec, 3, data2);
+    auto view2 = gko::make_array_view(this->exec, 3, data2);
     TypeParam data_size4[] = {5, 4, 2, 1};
-    auto view_size4 = gko::Array<TypeParam>::view(this->exec, 4, data_size4);
+    auto view_size4 = gko::make_array_view(this->exec, 4, data_size4);
 
     view = view2;
     view2.get_data()[0] = 2;
@@ -469,8 +533,8 @@ TYPED_TEST(Array, CopyViewToView)
 TYPED_TEST(Array, CopyViewToArray)
 {
     TypeParam data[] = {1, 2, 3, 4};
-    auto view = gko::Array<TypeParam>::view(this->exec, 4, data);
-    gko::Array<TypeParam> array(this->exec, {5, 4, 2});
+    auto view = gko::make_array_view(this->exec, 4, data);
+    gko::array<TypeParam> array(this->exec, {5, 4, 2});
 
     array = view;
     view.get_data()[0] = 2;
@@ -487,9 +551,9 @@ TYPED_TEST(Array, CopyViewToArray)
 TYPED_TEST(Array, CopyArrayToView)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::Array<TypeParam>::view(this->exec, 3, data);
-    gko::Array<TypeParam> array_size2(this->exec, {5, 4});
-    gko::Array<TypeParam> array_size4(this->exec, {5, 4, 2, 1});
+    auto view = gko::make_array_view(this->exec, 3, data);
+    gko::array<TypeParam> array_size2(this->exec, {5, 4});
+    gko::array<TypeParam> array_size4(this->exec, {5, 4, 2, 1});
 
     view = array_size2;
 
@@ -504,8 +568,8 @@ TYPED_TEST(Array, CopyArrayToView)
 
 TYPED_TEST(Array, MoveArrayToArray)
 {
-    gko::Array<TypeParam> array(this->exec, {1, 2, 3});
-    gko::Array<TypeParam> array2(this->exec, {5, 4, 2, 1});
+    gko::array<TypeParam> array(this->exec, {1, 2, 3});
+    gko::array<TypeParam> array2(this->exec, {5, 4, 2, 1});
     auto data2 = array2.get_data();
 
     array = std::move(array2);
@@ -524,9 +588,9 @@ TYPED_TEST(Array, MoveArrayToArray)
 TYPED_TEST(Array, MoveViewToView)
 {
     TypeParam data[] = {1, 2, 3, 4};
-    auto view = gko::Array<TypeParam>::view(this->exec, 4, data);
+    auto view = gko::make_array_view(this->exec, 4, data);
     TypeParam data2[] = {5, 4, 2};
-    auto view2 = gko::Array<TypeParam>::view(this->exec, 3, data2);
+    auto view2 = gko::make_array_view(this->exec, 3, data2);
 
     view = std::move(view2);
 
@@ -548,8 +612,8 @@ TYPED_TEST(Array, MoveViewToView)
 TYPED_TEST(Array, MoveViewToArray)
 {
     TypeParam data[] = {1, 2, 3, 4};
-    gko::Array<TypeParam> array(this->exec, {5, 4, 2});
-    auto view = gko::Array<TypeParam>::view(this->exec, 4, data);
+    gko::array<TypeParam> array(this->exec, {5, 4, 2});
+    auto view = gko::make_array_view(this->exec, 4, data);
 
     array = std::move(view);
 
@@ -571,9 +635,9 @@ TYPED_TEST(Array, MoveViewToArray)
 TYPED_TEST(Array, MoveArrayToView)
 {
     TypeParam data[] = {1, 2, 3};
-    auto view = gko::Array<TypeParam>::view(this->exec, 3, data);
-    gko::Array<TypeParam> array_size2(this->exec, {5, 4});
-    gko::Array<TypeParam> array_size4(this->exec, {5, 4, 2, 1});
+    auto view = gko::make_array_view(this->exec, 3, data);
+    gko::array<TypeParam> array_size2(this->exec, {5, 4});
+    gko::array<TypeParam> array_size4(this->exec, {5, 4, 2, 1});
     auto size2_ptr = array_size2.get_data();
     auto size4_ptr = array_size4.get_data();
 
@@ -591,4 +655,64 @@ TYPED_TEST(Array, MoveArrayToView)
 }
 
 
+TYPED_TEST(Array, AsView)
+{
+    auto ptr = this->x.get_data();
+    auto size = this->x.get_num_elems();
+    auto exec = this->x.get_executor();
+    auto view = this->x.as_view();
+
+    ASSERT_EQ(ptr, this->x.get_data());
+    ASSERT_EQ(ptr, view.get_data());
+    ASSERT_EQ(size, this->x.get_num_elems());
+    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(exec, this->x.get_executor());
+    ASSERT_EQ(exec, view.get_executor());
+    ASSERT_TRUE(this->x.is_owning());
+    ASSERT_FALSE(view.is_owning());
+}
+
+
+TYPED_TEST(Array, AsConstView)
+{
+    auto ptr = this->x.get_data();
+    auto size = this->x.get_num_elems();
+    auto exec = this->x.get_executor();
+    auto view = this->x.as_const_view();
+
+    ASSERT_EQ(ptr, this->x.get_data());
+    ASSERT_EQ(ptr, view.get_const_data());
+    ASSERT_EQ(size, this->x.get_num_elems());
+    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(exec, this->x.get_executor());
+    ASSERT_EQ(exec, view.get_executor());
+    ASSERT_TRUE(this->x.is_owning());
+    ASSERT_FALSE(view.is_owning());
+}
+
+
+TYPED_TEST(Array, ArrayConstCastWorksOnView)
+{
+    auto ptr = this->x.get_data();
+    auto size = this->x.get_num_elems();
+    auto exec = this->x.get_executor();
+    auto const_view = this->x.as_const_view();
+    auto view = gko::detail::array_const_cast(std::move(const_view));
+    static_assert(std::is_same<decltype(view), decltype(this->x)>::value,
+                  "wrong return type");
+
+    ASSERT_EQ(nullptr, const_view.get_const_data());
+    ASSERT_EQ(0, const_view.get_num_elems());
+    ASSERT_EQ(exec, const_view.get_executor());
+    ASSERT_EQ(ptr, this->x.get_data());
+    ASSERT_EQ(ptr, view.get_const_data());
+    ASSERT_EQ(size, this->x.get_num_elems());
+    ASSERT_EQ(size, view.get_num_elems());
+    ASSERT_EQ(exec, this->x.get_executor());
+    ASSERT_EQ(exec, view.get_executor());
+    ASSERT_TRUE(this->x.is_owning());
+    ASSERT_FALSE(view.is_owning());
+}
+
+
 }  // namespace
