@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -55,8 +55,8 @@ const std::string apply_str = "Dummy::apply";
 TEST(Record, CanGetData)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::iteration_complete_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::iteration_complete_mask);
 
     ASSERT_EQ(logger->get().allocation_started.size(), 0);
 }
@@ -65,12 +65,12 @@ TEST(Record, CanGetData)
 TEST(Record, CatchesAllocationStarted)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::allocation_started_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::allocation_started_mask);
 
     logger->on<gko::log::Logger::allocation_started>(exec.get(), 42);
 
-    auto &data = logger->get().allocation_started.back();
+    auto& data = logger->get().allocation_started.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->num_bytes, 42);
     ASSERT_EQ(data->location, 0);
@@ -80,14 +80,14 @@ TEST(Record, CatchesAllocationStarted)
 TEST(Record, CatchesAllocationCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::allocation_completed_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::allocation_completed_mask);
     int dummy = 1;
     auto ptr = reinterpret_cast<gko::uintptr>(&dummy);
 
     logger->on<gko::log::Logger::allocation_completed>(exec.get(), 42, ptr);
 
-    auto &data = logger->get().allocation_completed.back();
+    auto& data = logger->get().allocation_completed.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->num_bytes, 42);
     ASSERT_EQ(data->location, ptr);
@@ -97,14 +97,13 @@ TEST(Record, CatchesAllocationCompleted)
 TEST(Record, CatchesFreeStarted)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger =
-        gko::log::Record::create(exec, gko::log::Logger::free_started_mask);
+    auto logger = gko::log::Record::create(gko::log::Logger::free_started_mask);
     int dummy = 1;
     auto ptr = reinterpret_cast<gko::uintptr>(&dummy);
 
     logger->on<gko::log::Logger::free_started>(exec.get(), ptr);
 
-    auto &data = logger->get().free_started.back();
+    auto& data = logger->get().free_started.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->num_bytes, 0);
     ASSERT_EQ(data->location, ptr);
@@ -115,13 +114,13 @@ TEST(Record, CatchesFreeCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger =
-        gko::log::Record::create(exec, gko::log::Logger::free_completed_mask);
+        gko::log::Record::create(gko::log::Logger::free_completed_mask);
     int dummy = 1;
     auto ptr = reinterpret_cast<gko::uintptr>(&dummy);
 
     logger->on<gko::log::Logger::free_completed>(exec.get(), ptr);
 
-    auto &data = logger->get().free_completed.back();
+    auto& data = logger->get().free_completed.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->num_bytes, 0);
     ASSERT_EQ(data->location, ptr);
@@ -131,8 +130,7 @@ TEST(Record, CatchesFreeCompleted)
 TEST(Record, CatchesCopyStarted)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger =
-        gko::log::Record::create(exec, gko::log::Logger::copy_started_mask);
+    auto logger = gko::log::Record::create(gko::log::Logger::copy_started_mask);
     int dummy_from = 1;
     int dummy_to = 1;
     auto ptr_from = reinterpret_cast<gko::uintptr>(&dummy_from);
@@ -141,7 +139,7 @@ TEST(Record, CatchesCopyStarted)
     logger->on<gko::log::Logger::copy_started>(exec.get(), exec.get(), ptr_from,
                                                ptr_to, 42);
 
-    auto &data = logger->get().copy_started.back();
+    auto& data = logger->get().copy_started.back();
     auto data_from = std::get<0>(*data);
     auto data_to = std::get<1>(*data);
     ASSERT_EQ(data_from.exec, exec.get());
@@ -157,7 +155,7 @@ TEST(Record, CatchesCopyCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger =
-        gko::log::Record::create(exec, gko::log::Logger::copy_completed_mask);
+        gko::log::Record::create(gko::log::Logger::copy_completed_mask);
     int dummy_from = 1;
     int dummy_to = 1;
     auto ptr_from = reinterpret_cast<gko::uintptr>(&dummy_from);
@@ -166,7 +164,7 @@ TEST(Record, CatchesCopyCompleted)
     logger->on<gko::log::Logger::copy_completed>(exec.get(), exec.get(),
                                                  ptr_from, ptr_to, 42);
 
-    auto &data = logger->get().copy_completed.back();
+    auto& data = logger->get().copy_completed.back();
     auto data_from = std::get<0>(*data);
     auto data_to = std::get<1>(*data);
     ASSERT_EQ(data_from.exec, exec.get());
@@ -181,13 +179,13 @@ TEST(Record, CatchesCopyCompleted)
 TEST(Record, CatchesOperationLaunched)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::operation_launched_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::operation_launched_mask);
     gko::Operation op;
 
     logger->on<gko::log::Logger::operation_launched>(exec.get(), &op);
 
-    auto &data = logger->get().operation_launched.back();
+    auto& data = logger->get().operation_launched.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->operation, &op);
 }
@@ -196,13 +194,13 @@ TEST(Record, CatchesOperationLaunched)
 TEST(Record, CatchesOperationCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::operation_completed_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::operation_completed_mask);
     gko::Operation op;
 
     logger->on<gko::log::Logger::operation_completed>(exec.get(), &op);
 
-    auto &data = logger->get().operation_completed.back();
+    auto& data = logger->get().operation_completed.back();
     ASSERT_EQ(data->exec, exec.get());
     ASSERT_EQ(data->operation, &op);
 }
@@ -213,16 +211,16 @@ TEST(Record, CatchesPolymorphicObjectCreateStarted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::polymorphic_object_create_started_mask);
+        gko::log::Logger::polymorphic_object_create_started_mask);
     auto po = gko::matrix::Dense<>::create(exec);
 
     logger->on<gko::log::Logger::polymorphic_object_create_started>(exec.get(),
                                                                     po.get());
 
 
-    auto &data = logger->get().polymorphic_object_create_started.back();
+    auto& data = logger->get().polymorphic_object_create_started.back();
     ASSERT_EQ(data->exec, exec.get());
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po.get(), 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po, 0);
     ASSERT_EQ(data->output.get(), nullptr);
 }
 
@@ -232,17 +230,17 @@ TEST(Record, CatchesPolymorphicObjectCreateCompleted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::polymorphic_object_create_completed_mask);
+        gko::log::Logger::polymorphic_object_create_completed_mask);
     auto po = gko::matrix::Dense<>::create(exec);
     auto output = gko::matrix::Dense<>::create(exec);
 
     logger->on<gko::log::Logger::polymorphic_object_create_completed>(
         exec.get(), po.get(), output.get());
 
-    auto &data = logger->get().polymorphic_object_create_completed.back();
+    auto& data = logger->get().polymorphic_object_create_completed.back();
     ASSERT_EQ(data->exec, exec.get());
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po.get(), 0);
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), output.get(), 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), output, 0);
 }
 
 
@@ -251,17 +249,17 @@ TEST(Record, CatchesPolymorphicObjectCopyStarted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::polymorphic_object_copy_started_mask);
+        gko::log::Logger::polymorphic_object_copy_started_mask);
     auto from = gko::matrix::Dense<>::create(exec);
     auto to = gko::matrix::Dense<>::create(exec);
 
     logger->on<gko::log::Logger::polymorphic_object_copy_started>(
         exec.get(), from.get(), to.get());
 
-    auto &data = logger->get().polymorphic_object_copy_started.back();
+    auto& data = logger->get().polymorphic_object_copy_started.back();
     ASSERT_EQ(data->exec, exec.get());
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from.get(), 0);
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to.get(), 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to, 0);
 }
 
 
@@ -270,7 +268,7 @@ TEST(Record, CatchesPolymorphicObjectCopyCompleted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::polymorphic_object_copy_completed_mask);
+        gko::log::Logger::polymorphic_object_copy_completed_mask);
     auto from = gko::matrix::Dense<>::create(exec);
     auto to = gko::matrix::Dense<>::create(exec);
 
@@ -278,10 +276,49 @@ TEST(Record, CatchesPolymorphicObjectCopyCompleted)
         exec.get(), from.get(), to.get());
 
 
-    auto &data = logger->get().polymorphic_object_copy_completed.back();
+    auto& data = logger->get().polymorphic_object_copy_completed.back();
     ASSERT_EQ(data->exec, exec.get());
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from.get(), 0);
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to.get(), 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to, 0);
+}
+
+
+TEST(Record, CatchesPolymorphicObjectMoveStarted)
+{
+    using Dense = gko::matrix::Dense<>;
+    auto exec = gko::ReferenceExecutor::create();
+    auto logger = gko::log::Record::create(
+        gko::log::Logger::polymorphic_object_move_started_mask);
+    auto from = gko::matrix::Dense<>::create(exec);
+    auto to = gko::matrix::Dense<>::create(exec);
+
+    logger->on<gko::log::Logger::polymorphic_object_move_started>(
+        exec.get(), from.get(), to.get());
+
+    auto& data = logger->get().polymorphic_object_move_started.back();
+    ASSERT_EQ(data->exec, exec.get());
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to, 0);
+}
+
+
+TEST(Record, CatchesPolymorphicObjectMoveCompleted)
+{
+    using Dense = gko::matrix::Dense<>;
+    auto exec = gko::ReferenceExecutor::create();
+    auto logger = gko::log::Record::create(
+        gko::log::Logger::polymorphic_object_move_completed_mask);
+    auto from = gko::matrix::Dense<>::create(exec);
+    auto to = gko::matrix::Dense<>::create(exec);
+
+    logger->on<gko::log::Logger::polymorphic_object_move_completed>(
+        exec.get(), from.get(), to.get());
+
+
+    auto& data = logger->get().polymorphic_object_move_completed.back();
+    ASSERT_EQ(data->exec, exec.get());
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), from, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->output.get()), to, 0);
 }
 
 
@@ -290,16 +327,16 @@ TEST(Record, CatchesPolymorphicObjectDeleted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::polymorphic_object_deleted_mask);
+        gko::log::Logger::polymorphic_object_deleted_mask);
     auto po = gko::matrix::Dense<>::create(exec);
 
     logger->on<gko::log::Logger::polymorphic_object_deleted>(exec.get(),
                                                              po.get());
 
 
-    auto &data = logger->get().polymorphic_object_deleted.back();
+    auto& data = logger->get().polymorphic_object_deleted.back();
     ASSERT_EQ(data->exec, exec.get());
-    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po.get(), 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->input.get()), po, 0);
     ASSERT_EQ(data->output, nullptr);
 }
 
@@ -308,8 +345,8 @@ TEST(Record, CatchesLinOpApplyStarted)
 {
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_apply_started_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::linop_apply_started_mask);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
     auto x = gko::initialize<Dense>({3.3}, exec);
@@ -317,7 +354,7 @@ TEST(Record, CatchesLinOpApplyStarted)
     logger->on<gko::log::Logger::linop_apply_started>(A.get(), b.get(),
                                                       x.get());
 
-    auto &data = logger->get().linop_apply_started.back();
+    auto& data = logger->get().linop_apply_started.back();
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->A.get()), A, 0);
     ASSERT_EQ(data->alpha, nullptr);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->b.get()), b, 0);
@@ -330,8 +367,8 @@ TEST(Record, CatchesLinOpApplyCompleted)
 {
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_apply_completed_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::linop_apply_completed_mask);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
     auto x = gko::initialize<Dense>({3.3}, exec);
@@ -339,7 +376,7 @@ TEST(Record, CatchesLinOpApplyCompleted)
     logger->on<gko::log::Logger::linop_apply_completed>(A.get(), b.get(),
                                                         x.get());
 
-    auto &data = logger->get().linop_apply_completed.back();
+    auto& data = logger->get().linop_apply_completed.back();
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->A.get()), A, 0);
     ASSERT_EQ(data->alpha, nullptr);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->b.get()), b, 0);
@@ -353,7 +390,7 @@ TEST(Record, CatchesLinOpAdvancedApplyStarted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_advanced_apply_started_mask);
+        gko::log::Logger::linop_advanced_apply_started_mask);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto alpha = gko::initialize<Dense>({-4.4}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
@@ -363,7 +400,7 @@ TEST(Record, CatchesLinOpAdvancedApplyStarted)
     logger->on<gko::log::Logger::linop_advanced_apply_started>(
         A.get(), alpha.get(), b.get(), beta.get(), x.get());
 
-    auto &data = logger->get().linop_advanced_apply_started.back();
+    auto& data = logger->get().linop_advanced_apply_started.back();
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->A.get()), A, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->alpha.get()), alpha, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->b.get()), b, 0);
@@ -377,7 +414,7 @@ TEST(Record, CatchesLinOpAdvancedApplyCompleted)
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_advanced_apply_completed_mask);
+        gko::log::Logger::linop_advanced_apply_completed_mask);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto alpha = gko::initialize<Dense>({-4.4}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
@@ -387,7 +424,7 @@ TEST(Record, CatchesLinOpAdvancedApplyCompleted)
     logger->on<gko::log::Logger::linop_advanced_apply_completed>(
         A.get(), alpha.get(), b.get(), beta.get(), x.get());
 
-    auto &data = logger->get().linop_advanced_apply_completed.back();
+    auto& data = logger->get().linop_advanced_apply_completed.back();
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->A.get()), A, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->alpha.get()), alpha, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->b.get()), b, 0);
@@ -400,7 +437,7 @@ TEST(Record, CatchesLinopFactoryGenerateStarted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_factory_generate_started_mask);
+        gko::log::Logger::linop_factory_generate_started_mask);
     auto factory =
         gko::solver::Bicgstab<>::build()
             .with_criteria(
@@ -411,7 +448,7 @@ TEST(Record, CatchesLinopFactoryGenerateStarted)
     logger->on<gko::log::Logger::linop_factory_generate_started>(factory.get(),
                                                                  input.get());
 
-    auto &data = logger->get().linop_factory_generate_started.back();
+    auto& data = logger->get().linop_factory_generate_started.back();
     ASSERT_EQ(data->factory, factory.get());
     ASSERT_NE(data->input.get(), nullptr);
     ASSERT_EQ(data->output.get(), nullptr);
@@ -422,7 +459,7 @@ TEST(Record, CatchesLinopFactoryGenerateCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::linop_factory_generate_completed_mask);
+        gko::log::Logger::linop_factory_generate_completed_mask);
     auto factory =
         gko::solver::Bicgstab<>::build()
             .with_criteria(
@@ -434,7 +471,7 @@ TEST(Record, CatchesLinopFactoryGenerateCompleted)
     logger->on<gko::log::Logger::linop_factory_generate_completed>(
         factory.get(), input.get(), output.get());
 
-    auto &data = logger->get().linop_factory_generate_completed.back();
+    auto& data = logger->get().linop_factory_generate_completed.back();
     ASSERT_EQ(data->factory, factory.get());
     ASSERT_NE(data->input.get(), nullptr);
     ASSERT_NE(data->output.get(), nullptr);
@@ -445,7 +482,7 @@ TEST(Record, CatchesCriterionCheckStarted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::criterion_check_started_mask);
+        gko::log::Logger::criterion_check_started_mask);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
@@ -455,7 +492,7 @@ TEST(Record, CatchesCriterionCheckStarted)
         criterion.get(), 1, nullptr, nullptr, nullptr, RelativeStoppingId,
         true);
 
-    auto &data = logger->get().criterion_check_started.back();
+    auto& data = logger->get().criterion_check_started.back();
     ASSERT_NE(data->criterion, nullptr);
     ASSERT_EQ(data->stopping_id, RelativeStoppingId);
     ASSERT_EQ(data->set_finalized, true);
@@ -468,12 +505,12 @@ TEST(Record, CatchesCriterionCheckCompletedOld)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::criterion_check_completed_mask);
+        gko::log::Logger::criterion_check_completed_mask);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
     constexpr gko::uint8 RelativeStoppingId{42};
-    gko::Array<gko::stopping_status> stop_status(exec, 1);
+    gko::array<gko::stopping_status> stop_status(exec, 1);
 
     logger->on<gko::log::Logger::criterion_check_completed>(
         criterion.get(), 1, nullptr, nullptr, nullptr, RelativeStoppingId, true,
@@ -481,7 +518,7 @@ TEST(Record, CatchesCriterionCheckCompletedOld)
 
     stop_status.get_data()->reset();
     stop_status.get_data()->stop(RelativeStoppingId);
-    auto &data = logger->get().criterion_check_completed.back();
+    auto& data = logger->get().criterion_check_completed.back();
     ASSERT_NE(data->criterion, nullptr);
     ASSERT_EQ(data->stopping_id, RelativeStoppingId);
     ASSERT_EQ(data->set_finalized, true);
@@ -498,12 +535,12 @@ TEST(Record, CatchesCriterionCheckCompleted)
 {
     auto exec = gko::ReferenceExecutor::create();
     auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::criterion_check_completed_mask);
+        gko::log::Logger::criterion_check_completed_mask);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
     constexpr gko::uint8 RelativeStoppingId{42};
-    gko::Array<gko::stopping_status> stop_status(exec, 1);
+    gko::array<gko::stopping_status> stop_status(exec, 1);
 
     logger->on<gko::log::Logger::criterion_check_completed>(
         criterion.get(), 1, nullptr, nullptr, nullptr, nullptr,
@@ -511,7 +548,7 @@ TEST(Record, CatchesCriterionCheckCompleted)
 
     stop_status.get_data()->reset();
     stop_status.get_data()->stop(RelativeStoppingId);
-    auto &data = logger->get().criterion_check_completed.back();
+    auto& data = logger->get().criterion_check_completed.back();
     ASSERT_NE(data->criterion, nullptr);
     ASSERT_EQ(data->stopping_id, RelativeStoppingId);
     ASSERT_EQ(data->set_finalized, true);
@@ -528,33 +565,47 @@ TEST(Record, CatchesIterations)
 {
     using Dense = gko::matrix::Dense<>;
     auto exec = gko::ReferenceExecutor::create();
-    auto logger = gko::log::Record::create(
-        exec, gko::log::Logger::iteration_complete_mask);
+    auto logger =
+        gko::log::Record::create(gko::log::Logger::iteration_complete_mask);
     auto factory =
         gko::solver::Bicgstab<>::build()
             .with_criteria(
                 gko::stop::Iteration::build().with_max_iters(3u).on(exec))
             .on(exec);
     auto solver = factory->generate(gko::initialize<Dense>({1.1}, exec));
+    auto right_hand_side = gko::initialize<Dense>({-5.5}, exec);
     auto residual = gko::initialize<Dense>({-4.4}, exec);
     auto solution = gko::initialize<Dense>({-2.2}, exec);
     auto residual_norm = gko::initialize<Dense>({-3.3}, exec);
     auto implicit_sq_residual_norm = gko::initialize<Dense>({-3.5}, exec);
-
+    constexpr gko::uint8 RelativeStoppingId{42};
+    gko::array<gko::stopping_status> stop_status(exec, 1);
+    stop_status.get_data()->reset();
+    stop_status.get_data()->converge(RelativeStoppingId);
 
     logger->on<gko::log::Logger::iteration_complete>(
-        solver.get(), num_iters, residual.get(), solution.get(),
-        residual_norm.get(), implicit_sq_residual_norm.get());
+        solver.get(), right_hand_side.get(), solution.get(), num_iters,
+        residual.get(), residual_norm.get(), implicit_sq_residual_norm.get(),
+        &stop_status, true);
 
-    auto &data = logger->get().iteration_completed.back();
+    stop_status.get_data()->reset();
+    stop_status.get_data()->stop(RelativeStoppingId);
+    auto& data = logger->get().iteration_completed.back();
     ASSERT_NE(data->solver.get(), nullptr);
     ASSERT_EQ(data->num_iterations, num_iters);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->residual.get()), residual, 0);
+    GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->right_hand_side.get()),
+                        right_hand_side, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->solution.get()), solution, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->residual_norm.get()),
                         residual_norm, 0);
     GKO_ASSERT_MTX_NEAR(gko::as<Dense>(data->implicit_sq_residual_norm.get()),
                         implicit_sq_residual_norm, 0);
+    ASSERT_EQ(data->status.get_const_data()->has_stopped(), true);
+    ASSERT_EQ(data->status.get_const_data()->get_id(),
+              stop_status.get_const_data()->get_id());
+    ASSERT_EQ(data->status.get_const_data()->is_finalized(), true);
+    ASSERT_TRUE(data->all_stopped);
 }
 
 
