@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,7 @@ constexpr int num_iters = 10;
 template <typename T>
 class Stream : public ::testing::Test {};
 
-TYPED_TEST_SUITE(Stream, gko::test::ValueTypes);
+TYPED_TEST_SUITE(Stream, gko::test::ValueTypes, TypenameNameGenerator);
 
 
 TYPED_TEST(Stream, CatchesAllocationStarted)
@@ -67,7 +67,7 @@ TYPED_TEST(Stream, CatchesAllocationStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::allocation_started_mask, out);
+        gko::log::Logger::allocation_started_mask, out);
 
     logger->template on<gko::log::Logger::allocation_started>(exec.get(), 42);
 
@@ -82,7 +82,7 @@ TYPED_TEST(Stream, CatchesAllocationCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::allocation_completed_mask, out);
+        gko::log::Logger::allocation_completed_mask, out);
     int dummy = 1;
     std::stringstream ptrstream;
     ptrstream << std::hex << "0x" << reinterpret_cast<gko::uintptr>(&dummy);
@@ -102,7 +102,7 @@ TYPED_TEST(Stream, CatchesFreeStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::free_started_mask, out);
+        gko::log::Logger::free_started_mask, out);
     int dummy = 1;
     std::stringstream ptrstream;
     ptrstream << std::hex << "0x" << reinterpret_cast<gko::uintptr>(&dummy);
@@ -121,7 +121,7 @@ TYPED_TEST(Stream, CatchesFreeCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::free_completed_mask, out);
+        gko::log::Logger::free_completed_mask, out);
     int dummy = 1;
     std::stringstream ptrstream;
     ptrstream << std::hex << "0x" << reinterpret_cast<gko::uintptr>(&dummy);
@@ -140,7 +140,7 @@ TYPED_TEST(Stream, CatchesCopyStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::copy_started_mask, out);
+        gko::log::Logger::copy_started_mask, out);
     int dummy_in = 1;
     int dummy_out = 1;
     std::stringstream ptrstream_in;
@@ -167,7 +167,7 @@ TYPED_TEST(Stream, CatchesCopyCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::copy_completed_mask, out);
+        gko::log::Logger::copy_completed_mask, out);
     int dummy_in = 1;
     int dummy_out = 1;
     std::stringstream ptrstream_in;
@@ -194,7 +194,7 @@ TYPED_TEST(Stream, CatchesOperationLaunched)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::operation_launched_mask, out);
+        gko::log::Logger::operation_launched_mask, out);
     gko::Operation op;
     std::stringstream ptrstream;
     ptrstream << &op;
@@ -212,7 +212,7 @@ TYPED_TEST(Stream, CatchesOperationCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::operation_completed_mask, out);
+        gko::log::Logger::operation_completed_mask, out);
     gko::Operation op;
     std::stringstream ptrstream;
     ptrstream << &op;
@@ -230,7 +230,7 @@ TYPED_TEST(Stream, CatchesPolymorphicObjectCreateStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::polymorphic_object_create_started_mask, out);
+        gko::log::Logger::polymorphic_object_create_started_mask, out);
     auto po = gko::matrix::Dense<TypeParam>::create(exec);
     std::stringstream ptrstream;
     ptrstream << po.get();
@@ -249,7 +249,7 @@ TYPED_TEST(Stream, CatchesPolymorphicObjectCreateCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::polymorphic_object_create_completed_mask, out);
+        gko::log::Logger::polymorphic_object_create_completed_mask, out);
     auto po = gko::matrix::Dense<TypeParam>::create(exec);
     auto output = gko::matrix::Dense<TypeParam>::create(exec);
     std::stringstream ptrstream_in;
@@ -272,7 +272,7 @@ TYPED_TEST(Stream, CatchesPolymorphicObjectCopyStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::polymorphic_object_copy_started_mask, out);
+        gko::log::Logger::polymorphic_object_copy_started_mask, out);
     auto from = gko::matrix::Dense<TypeParam>::create(exec);
     auto to = gko::matrix::Dense<TypeParam>::create(exec);
     std::stringstream ptrstream_from;
@@ -295,7 +295,7 @@ TYPED_TEST(Stream, CatchesPolymorphicObjectCopyCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::polymorphic_object_copy_completed_mask, out);
+        gko::log::Logger::polymorphic_object_copy_completed_mask, out);
     auto from = gko::matrix::Dense<TypeParam>::create(exec);
     auto to = gko::matrix::Dense<TypeParam>::create(exec);
     std::stringstream ptrstream_from;
@@ -313,12 +313,58 @@ TYPED_TEST(Stream, CatchesPolymorphicObjectCopyCompleted)
 }
 
 
+TYPED_TEST(Stream, CatchesPolymorphicObjectMoveStarted)
+{
+    auto exec = gko::ReferenceExecutor::create();
+    std::stringstream out;
+    auto logger = gko::log::Stream<TypeParam>::create(
+        gko::log::Logger::polymorphic_object_move_started_mask, out);
+    auto from = gko::matrix::Dense<TypeParam>::create(exec);
+    auto to = gko::matrix::Dense<TypeParam>::create(exec);
+    std::stringstream ptrstream_from;
+    ptrstream_from << from.get();
+    std::stringstream ptrstream_to;
+    ptrstream_to << to.get();
+
+    logger->template on<gko::log::Logger::polymorphic_object_move_started>(
+        exec.get(), from.get(), to.get());
+
+    auto os = out.str();
+    GKO_ASSERT_STR_CONTAINS(os, ptrstream_from.str());
+    GKO_ASSERT_STR_CONTAINS(os, "move started to");
+    GKO_ASSERT_STR_CONTAINS(os, ptrstream_to.str());
+}
+
+
+TYPED_TEST(Stream, CatchesPolymorphicObjectMoveCompleted)
+{
+    auto exec = gko::ReferenceExecutor::create();
+    std::stringstream out;
+    auto logger = gko::log::Stream<TypeParam>::create(
+        gko::log::Logger::polymorphic_object_move_completed_mask, out);
+    auto from = gko::matrix::Dense<TypeParam>::create(exec);
+    auto to = gko::matrix::Dense<TypeParam>::create(exec);
+    std::stringstream ptrstream_from;
+    ptrstream_from << from.get();
+    std::stringstream ptrstream_to;
+    ptrstream_to << to.get();
+
+    logger->template on<gko::log::Logger::polymorphic_object_move_completed>(
+        exec.get(), from.get(), to.get());
+
+    auto os = out.str();
+    GKO_ASSERT_STR_CONTAINS(os, ptrstream_from.str());
+    GKO_ASSERT_STR_CONTAINS(os, "move completed to");
+    GKO_ASSERT_STR_CONTAINS(os, ptrstream_to.str());
+}
+
+
 TYPED_TEST(Stream, CatchesPolymorphicObjectDeleted)
 {
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::polymorphic_object_deleted_mask, out);
+        gko::log::Logger::polymorphic_object_deleted_mask, out);
     auto po = gko::matrix::Dense<TypeParam>::create(exec);
     std::stringstream ptrstream;
     ptrstream << po.get();
@@ -338,7 +384,7 @@ TYPED_TEST(Stream, CatchesLinOpApplyStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_apply_started_mask, out);
+        gko::log::Logger::linop_apply_started_mask, out);
     auto A = Dense::create(exec);
     auto b = Dense::create(exec);
     auto x = Dense::create(exec);
@@ -366,7 +412,7 @@ TYPED_TEST(Stream, CatchesLinOpApplyStartedWithVerbose)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_apply_started_mask, out, true);
+        gko::log::Logger::linop_apply_started_mask, out, true);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
     auto x = gko::initialize<Dense>({3.3}, exec);
@@ -387,7 +433,7 @@ TYPED_TEST(Stream, CatchesLinOpApplyCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_apply_completed_mask, out);
+        gko::log::Logger::linop_apply_completed_mask, out);
     auto A = Dense::create(exec);
     auto b = Dense::create(exec);
     auto x = Dense::create(exec);
@@ -415,7 +461,7 @@ TYPED_TEST(Stream, CatchesLinOpApplyCompletedWithVerbose)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_apply_completed_mask, out, true);
+        gko::log::Logger::linop_apply_completed_mask, out, true);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
     auto x = gko::initialize<Dense>({3.3}, exec);
@@ -436,7 +482,7 @@ TYPED_TEST(Stream, CatchesLinOpAdvancedApplyStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_advanced_apply_started_mask, out);
+        gko::log::Logger::linop_advanced_apply_started_mask, out);
     auto A = Dense::create(exec);
     auto alpha = Dense::create(exec);
     auto b = Dense::create(exec);
@@ -472,7 +518,7 @@ TYPED_TEST(Stream, CatchesLinOpAdvancedApplyStartedWithVerbose)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_advanced_apply_started_mask, out, true);
+        gko::log::Logger::linop_advanced_apply_started_mask, out, true);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto alpha = gko::initialize<Dense>({-4.4}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
@@ -497,7 +543,7 @@ TYPED_TEST(Stream, CatchesLinOpAdvancedApplyCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_advanced_apply_completed_mask, out);
+        gko::log::Logger::linop_advanced_apply_completed_mask, out);
     auto A = Dense::create(exec);
     auto alpha = Dense::create(exec);
     auto b = Dense::create(exec);
@@ -533,7 +579,7 @@ TYPED_TEST(Stream, CatchesLinOpAdvancedApplyCompletedWithVerbose)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_advanced_apply_completed_mask, out, true);
+        gko::log::Logger::linop_advanced_apply_completed_mask, out, true);
     auto A = gko::initialize<Dense>({1.1}, exec);
     auto alpha = gko::initialize<Dense>({-4.4}, exec);
     auto b = gko::initialize<Dense>({-2.2}, exec);
@@ -557,7 +603,7 @@ TYPED_TEST(Stream, CatchesLinopFactoryGenerateStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_factory_generate_started_mask, out);
+        gko::log::Logger::linop_factory_generate_started_mask, out);
     auto factory =
         gko::solver::Bicgstab<TypeParam>::build()
             .with_criteria(
@@ -584,7 +630,7 @@ TYPED_TEST(Stream, CatchesLinopFactoryGenerateCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::linop_factory_generate_completed_mask, out);
+        gko::log::Logger::linop_factory_generate_completed_mask, out);
     auto factory =
         gko::solver::Bicgstab<TypeParam>::build()
             .with_criteria(
@@ -616,7 +662,7 @@ TYPED_TEST(Stream, CatchesCriterionCheckStarted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::criterion_check_started_mask, out);
+        gko::log::Logger::criterion_check_started_mask, out);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
@@ -643,12 +689,12 @@ TYPED_TEST(Stream, CatchesCriterionCheckCompleted)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::criterion_check_completed_mask, out);
+        gko::log::Logger::criterion_check_completed_mask, out);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
     constexpr gko::uint8 RelativeStoppingId{42};
-    gko::Array<gko::stopping_status> stop_status(exec, 1);
+    gko::array<gko::stopping_status> stop_status(exec, 1);
     std::stringstream ptrstream;
     ptrstream << criterion.get();
     std::stringstream true_in_stream;
@@ -674,12 +720,12 @@ TYPED_TEST(Stream, CatchesCriterionCheckCompletedWithVerbose)
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::criterion_check_completed_mask, out, true);
+        gko::log::Logger::criterion_check_completed_mask, out, true);
     auto criterion =
         gko::stop::Iteration::build().with_max_iters(3u).on(exec)->generate(
             nullptr, nullptr, nullptr);
     constexpr gko::uint8 RelativeStoppingId{42};
-    gko::Array<gko::stopping_status> stop_status(exec, 1);
+    gko::array<gko::stopping_status> stop_status(exec, 1);
     std::stringstream true_in_stream;
     true_in_stream << true;
 
@@ -697,14 +743,15 @@ TYPED_TEST(Stream, CatchesCriterionCheckCompletedWithVerbose)
 }
 
 
-TYPED_TEST(Stream, CatchesIterations)
+TYPED_TEST(Stream, CatchesIterationsWithoutStoppingStatus)
 {
     using Dense = gko::matrix::Dense<TypeParam>;
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::iteration_complete_mask, out);
+        gko::log::Logger::iteration_complete_mask, out);
     auto solver = Dense::create(exec);
+    auto right_hand_side = Dense::create(exec);
     auto residual = Dense::create(exec);
     auto solution = Dense::create(exec);
     auto residual_norm = Dense::create(exec);
@@ -715,8 +762,9 @@ TYPED_TEST(Stream, CatchesIterations)
     ptrstream_residual << residual.get();
 
     logger->template on<gko::log::Logger::iteration_complete>(
-        solver.get(), num_iters, residual.get(), solution.get(),
-        residual_norm.get(), implicit_sq_residual_norm.get());
+        solver.get(), right_hand_side.get(), solution.get(), num_iters,
+        residual.get(), residual_norm.get(), implicit_sq_residual_norm.get(),
+        nullptr, false);
 
     GKO_ASSERT_STR_CONTAINS(out.str(),
                             "iteration " + std::to_string(num_iters));
@@ -725,13 +773,45 @@ TYPED_TEST(Stream, CatchesIterations)
 }
 
 
+TYPED_TEST(Stream, CatchesIterationsWithStoppingStatus)
+{
+    using Dense = gko::matrix::Dense<TypeParam>;
+    auto exec = gko::ReferenceExecutor::create();
+    std::stringstream out;
+    auto logger = gko::log::Stream<TypeParam>::create(
+        gko::log::Logger::iteration_complete_mask, out);
+    auto solver = Dense::create(exec);
+    auto right_hand_side = Dense::create(exec);
+    auto residual = Dense::create(exec);
+    auto solution = Dense::create(exec);
+    auto residual_norm = Dense::create(exec);
+    auto implicit_sq_residual_norm = Dense::create(exec);
+    std::stringstream ptrstream_solver;
+    ptrstream_solver << solver.get();
+    std::stringstream ptrstream_residual;
+    ptrstream_residual << residual.get();
+    gko::array<gko::stopping_status> stop_status(exec, 1);
+
+    logger->template on<gko::log::Logger::iteration_complete>(
+        solver.get(), right_hand_side.get(), solution.get(), num_iters,
+        residual.get(), residual_norm.get(), implicit_sq_residual_norm.get(),
+        &stop_status, true);
+
+    GKO_ASSERT_STR_CONTAINS(out.str(),
+                            "iteration " + std::to_string(num_iters));
+    GKO_ASSERT_STR_CONTAINS(out.str(), ptrstream_solver.str());
+    GKO_ASSERT_STR_CONTAINS(out.str(), ptrstream_residual.str());
+    GKO_ASSERT_STR_CONTAINS(out.str(), "Stopped the iteration process true");
+}
+
+
 TYPED_TEST(Stream, CatchesIterationsWithVerbose)
 {
     using Dense = gko::matrix::Dense<TypeParam>;
     auto exec = gko::ReferenceExecutor::create();
     std::stringstream out;
     auto logger = gko::log::Stream<TypeParam>::create(
-        exec, gko::log::Logger::iteration_complete_mask, out, true);
+        gko::log::Logger::iteration_complete_mask, out, true);
 
     auto factory =
         gko::solver::Bicgstab<TypeParam>::build()
@@ -739,18 +819,22 @@ TYPED_TEST(Stream, CatchesIterationsWithVerbose)
                 gko::stop::Iteration::build().with_max_iters(3u).on(exec))
             .on(exec);
     auto solver = factory->generate(gko::initialize<Dense>({1.1}, exec));
+    auto right_hand_side = gko::initialize<Dense>({-5.5}, exec);
     auto residual = gko::initialize<Dense>({-4.4}, exec);
     auto solution = gko::initialize<Dense>({-2.2}, exec);
     auto residual_norm = gko::initialize<Dense>({-3.3}, exec);
+    gko::array<gko::stopping_status> stop_status(exec, 1);
 
     logger->template on<gko::log::Logger::iteration_complete>(
-        solver.get(), num_iters, residual.get(), solution.get(),
-        residual_norm.get());
+        solver.get(), right_hand_side.get(), solution.get(), num_iters,
+        residual.get(), residual_norm.get(), nullptr, &stop_status, true);
 
     auto os = out.str();
+    GKO_ASSERT_STR_CONTAINS(os, "-5.5");
     GKO_ASSERT_STR_CONTAINS(os, "-4.4");
     GKO_ASSERT_STR_CONTAINS(os, "-2.2");
     GKO_ASSERT_STR_CONTAINS(os, "-3.3");
+    GKO_ASSERT_STR_CONTAINS(os, "Finalized:")
 }
 
 
