@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -52,87 +52,6 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 namespace {
 
 
-template <typename ValueType, typename IndexType>
-void assert_matrices_are_same(
-    const gko::matrix::Fbcsr<ValueType, IndexType> *const bm,
-    const gko::matrix::Csr<ValueType, IndexType> *const cm,
-    const gko::matrix::Diagonal<ValueType> *const diam = nullptr,
-    const gko::matrix_data<ValueType, IndexType> *const md = nullptr)
-{
-    if (cm) {
-        ASSERT_EQ(bm->get_size(), cm->get_size());
-        ASSERT_EQ(bm->get_num_stored_elements(), cm->get_num_stored_elements());
-    }
-    if (md) {
-        ASSERT_EQ(bm->get_size(), md->size);
-        ASSERT_EQ(bm->get_num_stored_elements(), md->nonzeros.size());
-    }
-    if (diam) {
-        const gko::size_type minsize =
-            std::min(bm->get_size()[0], bm->get_size()[1]);
-        ASSERT_EQ(minsize, diam->get_size()[0]);
-        ASSERT_EQ(minsize, diam->get_size()[1]);
-    }
-
-    const IndexType nbrows = bm->get_num_block_rows();
-    const int bs = bm->get_block_size();
-    const auto nbnz = bm->get_num_stored_blocks();
-    gko::acc::range<gko::acc::block_col_major<const ValueType, 3>> fbvals(
-        std::array<gko::size_type, 3>{nbnz, static_cast<gko::size_type>(bs),
-                                      static_cast<gko::size_type>(bs)},
-        bm->get_const_values());
-
-    for (IndexType ibrow = 0; ibrow < nbrows; ibrow++) {
-        const IndexType *const browptr = bm->get_const_row_ptrs();
-        const IndexType numblocksbrow = browptr[ibrow + 1] - browptr[ibrow];
-        for (IndexType irow = ibrow * bs; irow < ibrow * bs + bs; irow++) {
-            const IndexType rowstart = browptr[ibrow] * bs * bs +
-                                       (irow - ibrow * bs) * numblocksbrow * bs;
-            if (cm) {
-                ASSERT_EQ(cm->get_const_row_ptrs()[irow], rowstart);
-            }
-        }
-
-        const IndexType iz_browstart = browptr[ibrow] * bs * bs;
-        const IndexType *const bcolinds = bm->get_const_col_idxs();
-
-        for (IndexType ibnz = browptr[ibrow]; ibnz < browptr[ibrow + 1];
-             ibnz++) {
-            const IndexType bcol = bcolinds[ibnz];
-            const IndexType blkoffset_frombrowstart = ibnz - browptr[ibrow];
-
-            for (int ib = 0; ib < bs; ib++) {
-                const IndexType row = ibrow * bs + ib;
-                const IndexType inz_rowstart =
-                    iz_browstart + ib * numblocksbrow * bs;
-                const IndexType inz_blockstart_row =
-                    inz_rowstart + blkoffset_frombrowstart * bs;
-
-                for (int jb = 0; jb < bs; jb++) {
-                    const IndexType col = bcol * bs + jb;
-                    const IndexType inz = inz_blockstart_row + jb;
-                    if (cm) {
-                        ASSERT_EQ(col, cm->get_const_col_idxs()[inz]);
-                        ASSERT_EQ(fbvals(ibnz, ib, jb),
-                                  cm->get_const_values()[inz]);
-                    }
-                    if (md) {
-                        ASSERT_EQ(row, md->nonzeros[inz].row);
-                        ASSERT_EQ(col, md->nonzeros[inz].column);
-                        ASSERT_EQ(fbvals(ibnz, ib, jb),
-                                  md->nonzeros[inz].value);
-                    }
-                    if (row == col && diam) {
-                        ASSERT_EQ(fbvals(ibnz, ib, jb),
-                                  diam->get_const_values()[row]);
-                    }
-                }
-            }
-        }
-    }
-}
-
-
 template <typename SampleGenerator>
 void check_sample_generator_common(const SampleGenerator sg)
 {
@@ -156,11 +75,97 @@ protected:
 
     FbcsrSample() : ref(gko::ReferenceExecutor::create()) {}
 
+    void assert_matrices_are_same(
+        gko::ptr_param<const gko::matrix::Fbcsr<value_type, index_type>> bm,
+        gko::ptr_param<const gko::matrix::Csr<value_type, index_type>> cm,
+        const gko::matrix::Diagonal<value_type>* const diam = nullptr,
+        const gko::matrix_data<value_type, index_type>* const md = nullptr)
+    {
+        if (cm) {
+            ASSERT_EQ(bm->get_size(), cm->get_size());
+            ASSERT_EQ(bm->get_num_stored_elements(),
+                      cm->get_num_stored_elements());
+        }
+        if (md) {
+            ASSERT_EQ(bm->get_size(), md->size);
+            ASSERT_EQ(bm->get_num_stored_elements(), md->nonzeros.size());
+        }
+        if (diam) {
+            const gko::size_type minsize =
+                std::min(bm->get_size()[0], bm->get_size()[1]);
+            ASSERT_EQ(minsize, diam->get_size()[0]);
+            ASSERT_EQ(minsize, diam->get_size()[1]);
+        }
+
+        const index_type nbrows = bm->get_num_block_rows();
+        const int bs = bm->get_block_size();
+        const auto nbnz = bm->get_num_stored_blocks();
+        gko::acc::range<gko::acc::block_col_major<const value_type, 3>> fbvals(
+            std::array<gko::acc::size_type, 3>{
+                static_cast<gko::acc::size_type>(nbnz),
+                static_cast<gko::acc::size_type>(bs),
+                static_cast<gko::acc::size_type>(bs)},
+            bm->get_const_values());
+
+        for (index_type ibrow = 0; ibrow < nbrows; ibrow++) {
+            const index_type* const browptr = bm->get_const_row_ptrs();
+            const index_type numblocksbrow =
+                browptr[ibrow + 1] - browptr[ibrow];
+            for (index_type irow = ibrow * bs; irow < ibrow * bs + bs; irow++) {
+                const index_type rowstart =
+                    browptr[ibrow] * bs * bs +
+                    (irow - ibrow * bs) * numblocksbrow * bs;
+                if (cm) {
+                    ASSERT_EQ(cm->get_const_row_ptrs()[irow], rowstart);
+                }
+            }
+
+            const index_type iz_browstart = browptr[ibrow] * bs * bs;
+            const index_type* const bcolinds = bm->get_const_col_idxs();
+
+            for (index_type ibnz = browptr[ibrow]; ibnz < browptr[ibrow + 1];
+                 ibnz++) {
+                const index_type bcol = bcolinds[ibnz];
+                const index_type blkoffset_frombrowstart =
+                    ibnz - browptr[ibrow];
+
+                for (int ib = 0; ib < bs; ib++) {
+                    const index_type row = ibrow * bs + ib;
+                    const index_type inz_rowstart =
+                        iz_browstart + ib * numblocksbrow * bs;
+                    const index_type inz_blockstart_row =
+                        inz_rowstart + blkoffset_frombrowstart * bs;
+
+                    for (int jb = 0; jb < bs; jb++) {
+                        const index_type col = bcol * bs + jb;
+                        const index_type inz = inz_blockstart_row + jb;
+                        if (cm) {
+                            ASSERT_EQ(col, cm->get_const_col_idxs()[inz]);
+                            ASSERT_EQ(fbvals(ibnz, ib, jb),
+                                      cm->get_const_values()[inz]);
+                        }
+                        if (md) {
+                            ASSERT_EQ(row, md->nonzeros[inz].row);
+                            ASSERT_EQ(col, md->nonzeros[inz].column);
+                            ASSERT_EQ(fbvals(ibnz, ib, jb),
+                                      md->nonzeros[inz].value);
+                        }
+                        if (row == col && diam) {
+                            ASSERT_EQ(fbvals(ibnz, ib, jb),
+                                      diam->get_const_values()[row]);
+                        }
+                    }
+                }
+            }
+        }
+    }
+
     std::shared_ptr<const gko::ReferenceExecutor> ref;
 };
 
 
-TYPED_TEST_SUITE(FbcsrSample, gko::test::ValueIndexTypes);
+TYPED_TEST_SUITE(FbcsrSample, gko::test::ValueIndexTypes,
+                 PairTypenameNameGenerator);
 
 
 TYPED_TEST(FbcsrSample, SampleGeneratorsAreCorrect)
@@ -181,15 +186,15 @@ TYPED_TEST(FbcsrSample, SampleGeneratorsAreCorrect)
     std::unique_ptr<const Mtx> fbmtx2 = fbsample2.generate_fbcsr();
     std::unique_ptr<const Csr> csmtx2 = fbsample2.generate_csr();
     std::unique_ptr<const Diag> diag2 = fbsample2.extract_diagonal();
-    const gko::Array<index_type> nnzperrow = fbsample2.getNonzerosPerRow();
+    const gko::array<index_type> nnzperrow = fbsample2.getNonzerosPerRow();
 
     check_sample_generator_common(fbsample);
-    assert_matrices_are_same(fbmtx.get(), csmtx.get(),
-                             static_cast<const Diag *>(nullptr), &mdata);
+    this->assert_matrices_are_same(fbmtx, csmtx,
+                                   static_cast<const Diag*>(nullptr), &mdata);
     check_sample_generator_common(fbsample2);
-    assert_matrices_are_same(fbmtx2.get(), csmtx2.get(), diag2.get());
+    this->assert_matrices_are_same(fbmtx2, csmtx2, diag2.get());
     for (index_type irow = 0; irow < fbsample2.nrows; irow++) {
-        const index_type *const row_ptrs = csmtx2->get_const_row_ptrs();
+        const index_type* const row_ptrs = csmtx2->get_const_row_ptrs();
         const index_type num_nnz_row = row_ptrs[irow + 1] - row_ptrs[irow];
         ASSERT_EQ(nnzperrow.get_const_data()[irow], num_nnz_row);
         for (index_type iz = row_ptrs[irow]; iz < row_ptrs[irow + 1]; iz++) {
@@ -208,20 +213,11 @@ TYPED_TEST(FbcsrSample, SampleGeneratorsAreCorrect)
 
 
 template <typename ValueIndexType>
-class FbcsrSampleComplex : public ::testing::Test {
-protected:
-    using value_type =
-        typename std::tuple_element<0, decltype(ValueIndexType())>::type;
-    using index_type =
-        typename std::tuple_element<1, decltype(ValueIndexType())>::type;
-
-    FbcsrSampleComplex() : ref(gko::ReferenceExecutor::create()) {}
-
-    std::shared_ptr<const gko::ReferenceExecutor> ref;
-};
+class FbcsrSampleComplex : public FbcsrSample<ValueIndexType> {};
 
 
-TYPED_TEST_SUITE(FbcsrSampleComplex, gko::test::ComplexValueIndexTypes);
+TYPED_TEST_SUITE(FbcsrSampleComplex, gko::test::ComplexValueIndexTypes,
+                 PairTypenameNameGenerator);
 
 
 TYPED_TEST(FbcsrSampleComplex, ComplexSampleGeneratorIsCorrect)
@@ -237,7 +233,7 @@ TYPED_TEST(FbcsrSampleComplex, ComplexSampleGeneratorIsCorrect)
     std::unique_ptr<const Csr> csmtx3 = fbsample3.generate_csr();
 
     check_sample_generator_common(fbsample3);
-    assert_matrices_are_same(fbmtx3.get(), csmtx3.get());
+    this->assert_matrices_are_same(fbmtx3, csmtx3);
 }
 
 
@@ -258,9 +254,9 @@ protected:
           mtx(fbsample.generate_fbcsr())
     {
         // backup for move tests
-        const value_type *const v = mtx->get_values();
-        const index_type *const c = mtx->get_col_idxs();
-        const index_type *const r = mtx->get_row_ptrs();
+        const value_type* const v = mtx->get_values();
+        const index_type* const c = mtx->get_col_idxs();
+        const index_type* const r = mtx->get_row_ptrs();
         orig_size = mtx->get_size();
         orig_rowptrs.resize(fbsample.nbrows + 1);
         orig_colinds.resize(fbsample.nbnz);
@@ -279,7 +275,7 @@ protected:
     std::vector<index_type> orig_rowptrs;
     std::vector<index_type> orig_colinds;
 
-    void assert_equal_to_original_mtx(const Mtx *m)
+    void assert_equal_to_original_mtx(gko::ptr_param<const Mtx> m)
     {
         auto v = m->get_const_values();
         auto c = m->get_const_col_idxs();
@@ -294,7 +290,7 @@ protected:
         ASSERT_EQ(m->get_num_block_cols(), m->get_size()[1] / bs);
 
         for (index_type irow = 0; irow < orig_size[0] / bs; irow++) {
-            const index_type *const rowptr = &orig_rowptrs[0];
+            const index_type* const rowptr = &orig_rowptrs[0];
             ASSERT_EQ(r[irow], rowptr[irow]);
 
             for (index_type inz = rowptr[irow]; inz < rowptr[irow + 1]; inz++) {
@@ -308,7 +304,7 @@ protected:
         }
     }
 
-    void assert_empty(const Mtx *m)
+    void assert_empty(const Mtx* m)
     {
         ASSERT_EQ(m->get_size(), gko::dim<2>(0, 0));
         ASSERT_EQ(m->get_num_stored_elements(), 0);
@@ -319,7 +315,7 @@ protected:
     }
 };
 
-TYPED_TEST_SUITE(Fbcsr, gko::test::ValueIndexTypes);
+TYPED_TEST_SUITE(Fbcsr, gko::test::ValueIndexTypes, PairTypenameNameGenerator);
 
 
 TYPED_TEST(Fbcsr, GetNumBlocksCorrectlyThrows)
@@ -355,15 +351,15 @@ TYPED_TEST(Fbcsr, KnowsItsSize)
 
 TYPED_TEST(Fbcsr, ContainsCorrectData)
 {
-    this->assert_equal_to_original_mtx(this->mtx.get());
+    this->assert_equal_to_original_mtx(this->mtx);
 }
 
 
 TYPED_TEST(Fbcsr, BlockSizeIsSetCorrectly)
 {
     using Mtx = typename TestFixture::Mtx;
-    auto m = Mtx::create(this->exec);
-    m->set_block_size(6);
+    auto m = Mtx::create(this->exec, 6);
+
     ASSERT_EQ(m->get_block_size(), 6);
 }
 
@@ -388,15 +384,41 @@ TYPED_TEST(Fbcsr, CanBeCreatedFromExistingData)
     const size_type nbcols = this->fbsample.nbcols;
     const size_type bnnz = this->fbsample.nbnz;
     std::unique_ptr<Mtx> refmat = this->fbsample.generate_fbcsr();
-    value_type *const values = refmat->get_values();
-    index_type *const col_idxs = refmat->get_col_idxs();
-    index_type *const row_ptrs = refmat->get_row_ptrs();
+    value_type* const values = refmat->get_values();
+    index_type* const col_idxs = refmat->get_col_idxs();
+    index_type* const row_ptrs = refmat->get_row_ptrs();
 
     auto mtx = gko::matrix::Fbcsr<value_type, index_type>::create(
         this->exec, gko::dim<2>{nbrows * bs, nbcols * bs}, bs,
-        gko::Array<value_type>::view(this->exec, bnnz * bs * bs, values),
-        gko::Array<index_type>::view(this->exec, bnnz, col_idxs),
-        gko::Array<index_type>::view(this->exec, nbrows + 1, row_ptrs));
+        gko::make_array_view(this->exec, bnnz * bs * bs, values),
+        gko::make_array_view(this->exec, bnnz, col_idxs),
+        gko::make_array_view(this->exec, nbrows + 1, row_ptrs));
+
+    ASSERT_EQ(mtx->get_const_values(), values);
+    ASSERT_EQ(mtx->get_const_col_idxs(), col_idxs);
+    ASSERT_EQ(mtx->get_const_row_ptrs(), row_ptrs);
+}
+
+
+TYPED_TEST(Fbcsr, CanBeCreatedFromExistingConstData)
+{
+    using value_type = typename TestFixture::value_type;
+    using index_type = typename TestFixture::index_type;
+    using size_type = gko::size_type;
+    const int bs = this->fbsample.bs;
+    const size_type nbrows = this->fbsample.nbrows;
+    const size_type nbcols = this->fbsample.nbcols;
+    const size_type bnnz = this->fbsample.nbnz;
+    auto refmat = this->fbsample.generate_fbcsr();
+    auto values = refmat->get_const_values();
+    auto col_idxs = refmat->get_const_col_idxs();
+    auto row_ptrs = refmat->get_const_row_ptrs();
+
+    auto mtx = gko::matrix::Fbcsr<value_type, index_type>::create_const(
+        this->exec, gko::dim<2>{nbrows * bs, nbcols * bs}, bs,
+        gko::array<value_type>::const_view(this->exec, bnnz * bs * bs, values),
+        gko::array<index_type>::const_view(this->exec, bnnz, col_idxs),
+        gko::array<index_type>::const_view(this->exec, nbrows + 1, row_ptrs));
 
     ASSERT_EQ(mtx->get_const_values(), values);
     ASSERT_EQ(mtx->get_const_col_idxs(), col_idxs);
@@ -409,11 +431,11 @@ TYPED_TEST(Fbcsr, CanBeCopied)
     using Mtx = typename TestFixture::Mtx;
     auto copy = Mtx::create(this->exec);
 
-    copy->copy_from(this->mtx.get());
+    copy->copy_from(this->mtx);
 
-    this->assert_equal_to_original_mtx(this->mtx.get());
+    this->assert_equal_to_original_mtx(this->mtx);
     this->mtx->get_values()[1] = 3.0;
-    this->assert_equal_to_original_mtx(copy.get());
+    this->assert_equal_to_original_mtx(copy);
 }
 
 
@@ -422,9 +444,9 @@ TYPED_TEST(Fbcsr, CanBeMoved)
     using Mtx = typename TestFixture::Mtx;
     auto copy = Mtx::create(this->exec);
 
-    copy->copy_from(std::move(this->mtx));
+    copy->move_from(this->mtx);
 
-    this->assert_equal_to_original_mtx(copy.get());
+    this->assert_equal_to_original_mtx(copy);
 }
 
 
@@ -434,9 +456,9 @@ TYPED_TEST(Fbcsr, CanBeCloned)
 
     auto clone = this->mtx->clone();
 
-    this->assert_equal_to_original_mtx(this->mtx.get());
+    this->assert_equal_to_original_mtx(this->mtx);
     this->mtx->get_values()[1] = 5.0;
-    this->assert_equal_to_original_mtx(dynamic_cast<Mtx *>(clone.get()));
+    this->assert_equal_to_original_mtx(dynamic_cast<Mtx*>(clone.get()));
 }
 
 
@@ -451,12 +473,11 @@ TYPED_TEST(Fbcsr, CanBeCleared)
 TYPED_TEST(Fbcsr, CanBeReadFromMatrixData)
 {
     using Mtx = typename TestFixture::Mtx;
-    auto m = Mtx::create(this->exec);
-    m->set_block_size(this->fbsample.bs);
+    auto m = Mtx::create(this->exec, this->fbsample.bs);
 
     m->read(this->fbsample.generate_matrix_data());
 
-    this->assert_equal_to_original_mtx(m.get());
+    this->assert_equal_to_original_mtx(m);
 }
 
 
@@ -464,8 +485,7 @@ TYPED_TEST(Fbcsr, CanBeReadFromEmptyMatrixData)
 {
     using Mtx = typename TestFixture::Mtx;
     using MtxData = typename TestFixture::MtxData;
-    auto m = Mtx::create(this->exec);
-    m->set_block_size(this->fbsample.bs);
+    auto m = Mtx::create(this->exec, this->fbsample.bs);
     MtxData mdata;
     mdata.size = gko::dim<2>{0, 0};
 
