@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -96,9 +96,9 @@ public:
                                       static_cast<size_type>(ncols)},
                           nnz, bs);
 
-        value_type *const v = mtx->get_values();
-        index_type *const c = mtx->get_col_idxs();
-        index_type *const r = mtx->get_row_ptrs();
+        value_type* const v = mtx->get_values();
+        index_type* const c = mtx->get_col_idxs();
+        index_type* const r = mtx->get_row_ptrs();
         r[0] = 0;
         r[1] = 2;
         r[2] = 4;
@@ -108,26 +108,31 @@ public:
         c[3] = 2;
 
         gko::acc::range<gko::acc::block_col_major<value_type, 3>> vals(
-            std::array<size_type, 3>{nbnz, static_cast<size_type>(bs),
-                                     static_cast<size_type>(bs)},
+            std::array<gko::acc::size_type, 3>{
+                static_cast<gko::acc::size_type>(nbnz),
+                static_cast<gko::acc::size_type>(bs),
+                static_cast<gko::acc::size_type>(bs)},
             v);
 
-        if (mtx->get_size()[0] % bs != 0)
+        if (mtx->get_size()[0] % bs != 0) {
             throw gko::BadDimension(__FILE__, __LINE__, __func__, "test fbcsr",
                                     mtx->get_size()[0], mtx->get_size()[1],
                                     "block size does not divide the size!");
+        }
 
         for (index_type ibrow = 0; ibrow < mtx->get_num_block_rows(); ibrow++) {
-            const index_type *const browptr = mtx->get_row_ptrs();
+            const index_type* const browptr = mtx->get_row_ptrs();
             for (index_type inz = browptr[ibrow]; inz < browptr[ibrow + 1];
                  inz++) {
                 const index_type bcolind = mtx->get_col_idxs()[inz];
                 const value_type base = (ibrow + 1) * (bcolind + 1);
-                for (int ival = 0; ival < bs; ival++)
-                    for (int jval = 0; jval < bs; jval++)
+                for (int ival = 0; ival < bs; ival++) {
+                    for (int jval = 0; jval < bs; jval++) {
                         vals(inz, ival, jval) =
                             base + static_cast<gko::remove_complex<value_type>>(
                                        ival * bs + jval);
+                    }
+                }
             }
         }
 
@@ -149,12 +154,12 @@ public:
      */
     std::unique_ptr<Csr> generate_csr() const
     {
-        gko::Array<index_type> csrrow(exec, {0, 6, 12, 18, 24, 30, 36});
-        gko::Array<index_type> csrcols(
+        gko::array<index_type> csrrow(exec, {0, 6, 12, 18, 24, 30, 36});
+        gko::array<index_type> csrcols(
             exec, {3, 4, 5, 9, 10, 11, 3, 4, 5, 9, 10, 11, 3, 4, 5, 9, 10, 11,
                    0, 1, 2, 6, 7,  8,  0, 1, 2, 6, 7,  8,  0, 1, 2, 6, 7,  8});
         // clang-format off
-        gko::Array<value_type> csrvals(exec, I<value_type>
+        gko::array<value_type> csrvals(exec, I<value_type>
             {2, 3, 4, 4, 5, 6, 5, 6, 7, 7, 8, 9, 0, 9, 0,
 	         10, 11, 12, 2, 3, 4, 0, 7, 8, 5, 6, 7,
 	         9, 10, 11, 8, 9, 10, 12,
@@ -170,19 +175,21 @@ public:
      */
     std::unique_ptr<SparCsr> generate_sparsity_csr() const
     {
-        gko::Array<IndexType> colids(exec, nbnz);
-        gko::Array<IndexType> rowptrs(exec, nbrows + 1);
+        gko::array<IndexType> colids(exec, nbnz);
+        gko::array<IndexType> rowptrs(exec, nbrows + 1);
         const std::unique_ptr<const Fbcsr> fbmat = generate_fbcsr();
-        for (index_type i = 0; i < nbrows + 1; i++)
+        for (index_type i = 0; i < nbrows + 1; i++) {
             rowptrs.get_data()[i] = fbmat->get_const_row_ptrs()[i];
-        for (index_type i = 0; i < nbnz; i++)
+        }
+        for (index_type i = 0; i < nbnz; i++) {
             colids.get_data()[i] = fbmat->get_const_col_idxs()[i];
+        }
         return SparCsr::create(exec, gko::dim<2>{nbrows, nbcols}, colids,
                                rowptrs);
     }
 
     /**
-     * @return Array of COO triplets that represent the matrix
+     * @return array of COO triplets that represent the matrix
      *
      * @note The order of the triplets assumes the blocks are stored row-major
      */
@@ -228,7 +235,7 @@ public:
     }
 
     /**
-     * @return Array of COO triplets that represent the matrix; includes
+     * @return array of COO triplets that represent the matrix; includes
      *         explicit zeros
      *
      * @note The order of the triplets assumes the blocks are stored row-major
@@ -296,11 +303,13 @@ public:
 
     std::unique_ptr<Fbcsr> generate_fbcsr() const
     {
-        gko::Array<index_type> r(exec, {0, 1, 3, 4});
-        gko::Array<index_type> c(exec, {0, 0, 3, 2});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
-        for (IndexType i = 0; i < nnz; i++) v[i] = 0.15 + fbcsr_test_offset;
+        gko::array<index_type> r(exec, {0, 1, 3, 4});
+        gko::array<index_type> c(exec, {0, 0, 3, 2});
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
+        for (IndexType i = 0; i < nnz; i++) {
+            v[i] = 0.15 + fbcsr_test_offset;
+        }
 
         v[0] = 1;
         v[1] = 3;
@@ -321,12 +330,14 @@ public:
 
     std::unique_ptr<Csr> generate_csr() const
     {
-        gko::Array<index_type> r(exec, {0, 2, 4, 8, 12, 14, 16});
-        gko::Array<index_type> c(
+        gko::array<index_type> r(exec, {0, 2, 4, 8, 12, 14, 16});
+        gko::array<index_type> c(
             exec, {0, 1, 0, 1, 0, 1, 6, 7, 0, 1, 6, 7, 4, 5, 4, 5});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
-        for (IndexType i = 0; i < nnz; i++) v[i] = 0.15 + fbcsr_test_offset;
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
+        for (IndexType i = 0; i < nnz; i++) {
+            v[i] = 0.15 + fbcsr_test_offset;
+        }
         v[0] = 1;
         v[1] = 2;
         v[2] = 3;
@@ -347,13 +358,13 @@ public:
 
     std::unique_ptr<Diagonal> extract_diagonal() const
     {
-        gko::Array<ValueType> dvals(exec, {1, 0, 0, 0, -12, -11});
+        gko::array<ValueType> dvals(exec, {1, 0, 0, 0, -12, -11});
         return Diagonal::create(exec, nrows, dvals);
     }
 
-    gko::Array<index_type> getNonzerosPerRow() const
+    gko::array<index_type> getNonzerosPerRow() const
     {
-        return gko::Array<index_type>(exec, {2, 2, 4, 4, 2, 2});
+        return gko::array<index_type>(exec, {2, 2, 4, 4, 2, 2});
     }
 
 
@@ -395,11 +406,13 @@ public:
 
     std::unique_ptr<Fbcsr> generate_fbcsr() const
     {
-        gko::Array<index_type> c(exec, {1, 1});
-        gko::Array<index_type> r(exec, {0, 1, 2});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
-        for (IndexType i = 0; i < nnz; i++) v[i] = i;
+        gko::array<index_type> c(exec, {1, 1});
+        gko::array<index_type> r(exec, {0, 1, 2});
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
+        for (IndexType i = 0; i < nnz; i++) {
+            v[i] = i;
+        }
 
         return Fbcsr::create(exec,
                              gko::dim<2>{static_cast<size_type>(nrows),
@@ -441,23 +454,24 @@ public:
 
     std::unique_ptr<Fbcsr> generate_fbcsr() const
     {
-        gko::Array<index_type> r(exec, {0, 1, 3, 4});
-        gko::Array<index_type> c(exec, {0, 0, 3, 2});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
-        for (IndexType i = 0; i < nnz; i++) v[i] = 0.15 + fbcsr_test_offset;
+        gko::array<index_type> r(exec, {0, 1, 3, 4});
+        gko::array<index_type> c(exec, {0, 0, 3, 2});
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
+        for (IndexType i = 0; i < nnz; i++) {
+            v[i] = 0.15 + fbcsr_test_offset;
+        }
 
-        using namespace std::complex_literals;
-        v[0] = 1.0 + 1.15i;
-        v[2] = 2.0 + 2.15i;
-        v[1] = 3.0 - 3.15i;
-        v[3] = 0.0 - 0.15i;
+        v[0] = value_type{1.0, 1.15};
+        v[2] = value_type{2.0, 2.15};
+        v[1] = value_type{3.0, -3.15};
+        v[3] = value_type{0.0, -0.15};
         v[9] = 0.0;
         v[11] = 0.0;
-        v[12] = -12.0 + 12.15i;
-        v[14] = -1.0 + 1.15i;
-        v[13] = -2.0 - 2.15i;
-        v[15] = -11.0 - 11.15i;
+        v[12] = -value_type{12.0, 12.15};
+        v[14] = -value_type{1.0, 1.15};
+        v[13] = -value_type{2.0, -2.15};
+        v[15] = -value_type{11.0, -11.15};
 
         return Fbcsr::create(exec,
                              gko::dim<2>{static_cast<size_type>(nrows),
@@ -467,24 +481,25 @@ public:
 
     std::unique_ptr<Csr> generate_csr() const
     {
-        gko::Array<index_type> r(exec, {0, 2, 4, 8, 12, 14, 16});
-        gko::Array<index_type> c(
+        gko::array<index_type> r(exec, {0, 2, 4, 8, 12, 14, 16});
+        gko::array<index_type> c(
             exec, {0, 1, 0, 1, 0, 1, 6, 7, 0, 1, 6, 7, 4, 5, 4, 5});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
-        for (IndexType i = 0; i < nnz; i++) v[i] = 0.15 + fbcsr_test_offset;
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
+        for (IndexType i = 0; i < nnz; i++) {
+            v[i] = 0.15 + fbcsr_test_offset;
+        }
 
-        using namespace std::complex_literals;
-        v[0] = 1.0 + 1.15i;
-        v[1] = 2.0 + 2.15i;
-        v[2] = 3.0 - 3.15i;
-        v[3] = 0.0 - 0.15i;
+        v[0] = value_type{1.0, 1.15};
+        v[1] = value_type{2.0, 2.15};
+        v[2] = value_type{3.0, -3.15};
+        v[3] = value_type{0.0, -0.15};
         v[10] = 0.0;
         v[11] = 0.0;
-        v[12] = -12.0 + 12.15i;
-        v[13] = -1.0 + 1.15i;
-        v[14] = -2.0 - 2.15i;
-        v[15] = -11.0 - 11.15i;
+        v[12] = -value_type{12.0, 12.15};
+        v[13] = -value_type{1.0, 1.15};
+        v[14] = -value_type{2.0, -2.15};
+        v[15] = -value_type{11.0, -11.15};
 
         return Csr::create(exec,
                            gko::dim<2>{static_cast<size_type>(nrows),
@@ -522,12 +537,12 @@ public:
 
     std::unique_ptr<Fbcsr> generate_fbcsr() const
     {
-        gko::Array<index_type> r(exec, {0, 8, 19, 30});
-        gko::Array<index_type> c(
+        gko::array<index_type> r(exec, {0, 8, 19, 30});
+        gko::array<index_type> c(
             exec, {0,  1,  20, 15, 12, 18, 5, 28, 3,  10, 29, 5,  9,  2,  16,
                    12, 21, 2,  0,  1,  5,  9, 12, 15, 17, 20, 22, 24, 27, 28});
-        gko::Array<value_type> vals(exec, nnz);
-        value_type *const v = vals.get_data();
+        gko::array<value_type> vals(exec, nnz);
+        value_type* const v = vals.get_data();
         for (IndexType i = 0; i < nnz; i++) {
             v[i] = static_cast<value_type>(i + 0.15 + fbcsr_test_offset);
         }
