@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -74,34 +74,33 @@ std::unique_ptr<vec<ValueType>> create_vector(
 
 // utilities for computing norms and residuals
 template <typename ValueType>
-ValueType get_first_element(const vec<ValueType> *norm)
+ValueType get_first_element(const vec<ValueType>* norm)
 {
     return norm->get_executor()->copy_val_to_host(norm->get_const_values());
 }
 
 
 template <typename ValueType>
-gko::remove_complex<ValueType> compute_norm(const vec<ValueType> *b)
+gko::remove_complex<ValueType> compute_norm(const vec<ValueType>* b)
 {
     auto exec = b->get_executor();
     auto b_norm = gko::initialize<real_vec<ValueType>>({0.0}, exec);
-    b->compute_norm2(gko::lend(b_norm));
-    return get_first_element(gko::lend(b_norm));
+    b->compute_norm2(b_norm);
+    return get_first_element(b_norm.get());
 }
 
 
 template <typename ValueType>
 gko::remove_complex<ValueType> compute_residual_norm(
-    const gko::LinOp *system_matrix, const vec<ValueType> *b,
-    const vec<ValueType> *x)
+    const gko::LinOp* system_matrix, const vec<ValueType>* b,
+    const vec<ValueType>* x)
 {
     auto exec = system_matrix->get_executor();
     auto one = gko::initialize<vec<ValueType>>({1.0}, exec);
     auto neg_one = gko::initialize<vec<ValueType>>({-1.0}, exec);
     auto res = gko::clone(b);
-    system_matrix->apply(gko::lend(one), gko::lend(x), gko::lend(neg_one),
-                         gko::lend(res));
-    return compute_norm(gko::lend(res));
+    system_matrix->apply(one, x, neg_one, res);
+    return compute_norm(res.get());
 }
 
 
@@ -116,62 +115,62 @@ namespace loggers {
 // taken before and after. This can create significant overhead since to ensure
 // proper timings, calls to `synchronize` are required.
 struct OperationLogger : gko::log::Logger {
-    void on_allocation_started(const gko::Executor *exec,
-                               const gko::size_type &) const override
+    void on_allocation_started(const gko::Executor* exec,
+                               const gko::size_type&) const override
     {
         this->start_operation(exec, "allocate");
     }
 
-    void on_allocation_completed(const gko::Executor *exec,
-                                 const gko::size_type &,
-                                 const gko::uintptr &) const override
+    void on_allocation_completed(const gko::Executor* exec,
+                                 const gko::size_type&,
+                                 const gko::uintptr&) const override
     {
         this->end_operation(exec, "allocate");
     }
 
-    void on_free_started(const gko::Executor *exec,
-                         const gko::uintptr &) const override
+    void on_free_started(const gko::Executor* exec,
+                         const gko::uintptr&) const override
     {
         this->start_operation(exec, "free");
     }
 
-    void on_free_completed(const gko::Executor *exec,
-                           const gko::uintptr &) const override
+    void on_free_completed(const gko::Executor* exec,
+                           const gko::uintptr&) const override
     {
         this->end_operation(exec, "free");
     }
 
-    void on_copy_started(const gko::Executor *from, const gko::Executor *to,
-                         const gko::uintptr &, const gko::uintptr &,
-                         const gko::size_type &) const override
+    void on_copy_started(const gko::Executor* from, const gko::Executor* to,
+                         const gko::uintptr&, const gko::uintptr&,
+                         const gko::size_type&) const override
     {
         from->synchronize();
         this->start_operation(to, "copy");
     }
 
-    void on_copy_completed(const gko::Executor *from, const gko::Executor *to,
-                           const gko::uintptr &, const gko::uintptr &,
-                           const gko::size_type &) const override
+    void on_copy_completed(const gko::Executor* from, const gko::Executor* to,
+                           const gko::uintptr&, const gko::uintptr&,
+                           const gko::size_type&) const override
     {
         from->synchronize();
         this->end_operation(to, "copy");
     }
 
-    void on_operation_launched(const gko::Executor *exec,
-                               const gko::Operation *op) const override
+    void on_operation_launched(const gko::Executor* exec,
+                               const gko::Operation* op) const override
     {
         this->start_operation(exec, op->get_name());
     }
 
-    void on_operation_completed(const gko::Executor *exec,
-                                const gko::Operation *op) const override
+    void on_operation_completed(const gko::Executor* exec,
+                                const gko::Operation* op) const override
     {
         this->end_operation(exec, op->get_name());
     }
 
-    void write_data(std::ostream &ostream)
+    void write_data(std::ostream& ostream)
     {
-        for (const auto &entry : total) {
+        for (const auto& entry : total) {
             ostream << "\t" << entry.first.c_str() << ": "
                     << std::chrono::duration_cast<std::chrono::nanoseconds>(
                            entry.second)
@@ -180,14 +179,10 @@ struct OperationLogger : gko::log::Logger {
         }
     }
 
-    OperationLogger(std::shared_ptr<const gko::Executor> exec)
-        : gko::log::Logger(exec)
-    {}
-
 private:
     // Helper which synchronizes and starts the time before every operation.
-    void start_operation(const gko::Executor *exec,
-                         const std::string &name) const
+    void start_operation(const gko::Executor* exec,
+                         const std::string& name) const
     {
         nested.emplace_back(0);
         exec->synchronize();
@@ -196,7 +191,7 @@ private:
 
     // Helper to compute the end time and store the operation's time at its
     // end. Also time nested operations.
-    void end_operation(const gko::Executor *exec, const std::string &name) const
+    void end_operation(const gko::Executor* exec, const std::string& name) const
     {
         exec->synchronize();
         const auto end = std::chrono::steady_clock::now();
@@ -220,34 +215,30 @@ private:
 // This logger tracks the persistently allocated data
 struct StorageLogger : gko::log::Logger {
     // Store amount of bytes allocated on every allocation
-    void on_allocation_completed(const gko::Executor *,
-                                 const gko::size_type &num_bytes,
-                                 const gko::uintptr &location) const override
+    void on_allocation_completed(const gko::Executor*,
+                                 const gko::size_type& num_bytes,
+                                 const gko::uintptr& location) const override
     {
         storage[location] = num_bytes;
     }
 
     // Reset the amount of bytes on every free
-    void on_free_completed(const gko::Executor *,
-                           const gko::uintptr &location) const override
+    void on_free_completed(const gko::Executor*,
+                           const gko::uintptr& location) const override
     {
         storage[location] = 0;
     }
 
     // Write the data after summing the total from all allocations
-    void write_data(std::ostream &ostream)
+    void write_data(std::ostream& ostream)
     {
         gko::size_type total{};
-        for (const auto &e : storage) {
+        for (const auto& e : storage) {
             total += e.second;
         }
         ostream << "Storage: " << total << std::endl;
     }
 
-    StorageLogger(std::shared_ptr<const gko::Executor> exec)
-        : gko::log::Logger(exec)
-    {}
-
 private:
     mutable std::unordered_map<gko::uintptr, gko::size_type> storage;
 };
@@ -259,10 +250,10 @@ struct ResidualLogger : gko::log::Logger {
     // Depending on the available information, store the norm or compute it from
     // the residual. If the true residual norm could not be computed, store the
     // value `-1.0`.
-    void on_iteration_complete(const gko::LinOp *, const gko::size_type &,
-                               const gko::LinOp *residual,
-                               const gko::LinOp *solution,
-                               const gko::LinOp *residual_norm) const override
+    void on_iteration_complete(const gko::LinOp*, const gko::size_type&,
+                               const gko::LinOp* residual,
+                               const gko::LinOp* solution,
+                               const gko::LinOp* residual_norm) const override
     {
         if (residual_norm) {
             rec_res_norms.push_back(utils::get_first_element(
@@ -279,33 +270,32 @@ struct ResidualLogger : gko::log::Logger {
         }
     }
 
-    ResidualLogger(std::shared_ptr<const gko::Executor> exec,
-                   const gko::LinOp *matrix, const vec<ValueType> *b)
-        : gko::log::Logger(exec, gko::log::Logger::iteration_complete_mask),
+    ResidualLogger(const gko::LinOp* matrix, const vec<ValueType>* b)
+        : gko::log::Logger(gko::log::Logger::iteration_complete_mask),
           matrix{matrix},
           b{b}
     {}
 
-    void write_data(std::ostream &ostream)
+    void write_data(std::ostream& ostream)
     {
         ostream << "Recurrent Residual Norms: " << std::endl;
         ostream << "[" << std::endl;
-        for (const auto &entry : rec_res_norms) {
+        for (const auto& entry : rec_res_norms) {
             ostream << "\t" << entry << std::endl;
         }
         ostream << "];" << std::endl;
 
         ostream << "True Residual Norms: " << std::endl;
         ostream << "[" << std::endl;
-        for (const auto &entry : true_res_norms) {
+        for (const auto& entry : true_res_norms) {
             ostream << "\t" << entry << std::endl;
         }
         ostream << "];" << std::endl;
     }
 
 private:
-    const gko::LinOp *matrix;
-    const vec<ValueType> *b;
+    const gko::LinOp* matrix;
+    const vec<ValueType>* b;
     mutable std::vector<gko::remove_complex<ValueType>> rec_res_norms;
     mutable std::vector<gko::remove_complex<ValueType>> true_res_norms;
 };
@@ -318,7 +308,7 @@ namespace {
 
 
 // Print usage help
-void print_usage(const char *filename)
+void print_usage(const char* filename)
 {
     std::cerr << "Usage: " << filename << " [executor] [matrix file]"
               << std::endl;
@@ -330,7 +320,7 @@ void print_usage(const char *filename)
 
 
 template <typename ValueType>
-void print_vector(const gko::matrix::Dense<ValueType> *vec)
+void print_vector(const gko::matrix::Dense<ValueType>* vec)
 {
     auto elements_to_print = std::min(gko::size_type(10), vec->get_size()[0]);
     std::cout << "[" << std::endl;
@@ -344,7 +334,7 @@ void print_vector(const gko::matrix::Dense<ValueType> *vec)
 }  // namespace
 
 
-int main(int argc, char *argv[])
+int main(int argc, char* argv[])
 {
     // Parametrize the benchmark here
     // Pick a value type
@@ -407,13 +397,13 @@ int main(int argc, char *argv[])
 
     // Read data: A is read from disk
     // Create a StorageLogger to track the size of A
-    auto storage_logger = std::make_shared<loggers::StorageLogger>(exec);
+    auto storage_logger = std::make_shared<loggers::StorageLogger>();
     // Add the logger to the executor
     exec->add_logger(storage_logger);
     // Read the matrix A from file
     auto A = gko::share(gko::read<mtx>(std::ifstream(input_mtx), exec));
     // Remove the storage logger
-    exec->remove_logger(gko::lend(storage_logger));
+    exec->remove_logger(storage_logger);
 
     // Pick a maximum iteration count
     const auto max_iters = A->get_size()[0];
@@ -443,7 +433,7 @@ int main(int argc, char *argv[])
         auto x_clone = gko::clone(x);
 
         // Generate and call apply on a solver
-        solver_factory->generate(A)->apply(gko::lend(b), gko::lend(x_clone));
+        solver_factory->generate(A)->apply(b, x_clone);
         exec->synchronize();
     }
 
@@ -471,7 +461,7 @@ int main(int argc, char *argv[])
         // Similarly time the apply
         exec->synchronize();
         auto a_tic = std::chrono::steady_clock::now();
-        generated_solver->apply(gko::lend(b), gko::lend(x_clone));
+        generated_solver->apply(b, x_clone);
         exec->synchronize();
         auto a_tac = std::chrono::steady_clock::now();
         auto apply_time =
@@ -479,35 +469,35 @@ int main(int argc, char *argv[])
         output_file << "Apply time (ns): " << apply_time.count() << std::endl;
 
         // Compute the residual norm
-        auto residual = utils::compute_residual_norm(gko::lend(A), gko::lend(b),
-                                                     gko::lend(x_clone));
+        auto residual =
+            utils::compute_residual_norm(A.get(), b.get(), x_clone.get());
         output_file << "Residual_norm: " << residual << std::endl;
     }
 
     // Log the internal operations using the OperationLogger without timing
     {
         // Create an OperationLogger to analyze the generate step
-        auto gen_logger = std::make_shared<loggers::OperationLogger>(exec);
+        auto gen_logger = std::make_shared<loggers::OperationLogger>();
         // Add the generate logger to the executor
         exec->add_logger(gen_logger);
         // Generate a solver
         auto generated_solver = solver_factory->generate(A);
         // Remove the generate logger from the executor
-        exec->remove_logger(gko::lend(gen_logger));
+        exec->remove_logger(gen_logger);
         // Write the data to the output file
         output_file << "Generate operations times (ns):" << std::endl;
         gen_logger->write_data(output_file);
 
         // Create an OperationLogger to analyze the apply step
-        auto apply_logger = std::make_shared<loggers::OperationLogger>(exec);
+        auto apply_logger = std::make_shared<loggers::OperationLogger>();
         exec->add_logger(apply_logger);
         // Create a ResidualLogger to log the recurent residual
         auto res_logger = std::make_shared<loggers::ResidualLogger<ValueType>>(
-            exec, gko::lend(A), gko::lend(b));
+            A.get(), b.get());
         generated_solver->add_logger(res_logger);
         // Solve the system
-        generated_solver->apply(gko::lend(b), gko::lend(x));
-        exec->remove_logger(gko::lend(apply_logger));
+        generated_solver->apply(b, x);
+        exec->remove_logger(apply_logger);
         // Write the data to the output file
         output_file << "Apply operations times (ns):" << std::endl;
         apply_logger->write_data(output_file);
@@ -516,7 +506,7 @@ int main(int argc, char *argv[])
 
     // Print solution
     std::cout << "Solution, first ten entries: \n";
-    print_vector(gko::lend(x));
+    print_vector(x.get());
 
     // Print output file location
     std::cout << "The performance and residual data can be found in " << of_name
