@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -51,6 +51,10 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 namespace gko {
 
 
+template <typename ValueType>
+class array;
+
+
 namespace detail {
 
 
@@ -62,18 +66,117 @@ namespace detail {
  */
 template <typename SourceType, typename TargetType>
 void convert_data(std::shared_ptr<const Executor> exec, size_type size,
-                  const SourceType *src, TargetType *dst);
+                  const SourceType* src, TargetType* dst);
+
+
+/**
+ * @internal
+ *
+ * array-like non-owning wrapper for const data, to be used in conjunction with
+ * `array_const_cast` and `create_const` to create matrix type wrappers from
+ * constant data.
+ *
+ * @tparam ValueType  the type of elements stored in the array view.
+ */
+template <typename ValueType>
+class const_array_view {
+public:
+    /**
+     * The type of elements stored in the array view.
+     */
+    using value_type = ValueType;
+
+    /**
+     * Constructs an array view from existing data.
+     *
+     * @param exec  the executor in whose memory space the data resides.
+     * @param num_elems  the number of elements in this array view.
+     * @param data  a pointer to the first element of this array view.
+     */
+    const_array_view(std::shared_ptr<const Executor> exec, size_type num_elems,
+                     const ValueType* data)
+        : exec_{std::move(exec)}, num_elems_{num_elems}, data_{data}
+    {}
+
+    /*
+     * To avoid any collisions with the value semantics of normal arrays,
+     * disable assignment and copy-construction altogether.
+     */
+    const_array_view& operator=(const const_array_view&) = delete;
+    const_array_view& operator=(const_array_view&&) = delete;
+    const_array_view(const const_array_view&) = delete;
+    /*
+     * TODO C++17: delete this overload as well, it is no longer necessary due
+     * to guaranteed RVO.
+     */
+    const_array_view(const_array_view&& other)
+        : const_array_view{other.exec_, other.num_elems_, other.data_}
+    {
+        other.num_elems_ = 0;
+        other.data_ = nullptr;
+    }
+
+    /**
+     * Returns the number of elements in the array view.
+     *
+     * @return the number of elements in the array view
+     */
+    size_type get_num_elems() const noexcept { return num_elems_; }
+
+    /**
+     * Returns a constant pointer to the first element of this array view.
+     *
+     * @return a constant pointer to the first element of this array view.
+     */
+    const value_type* get_const_data() const noexcept { return data_; }
+
+    /**
+     * Returns the Executor associated with the array view.
+     *
+     * @return the Executor associated with the array view
+     */
+    std::shared_ptr<const Executor> get_executor() const noexcept
+    {
+        return exec_;
+    }
+
+    /**
+     * Returns false, to be consistent with the array interface.
+     */
+    bool is_owning() const noexcept { return false; }
+
+    /**
+     * Creates a array copied from const_array_view.
+     *
+     * @return an array constructed from const_array_view data
+     */
+    array<ValueType> copy_to_array() const;
+
+private:
+    std::shared_ptr<const Executor> exec_;
+    size_type num_elems_;
+    const ValueType* data_;
+};
+
+
+template <typename ValueType>
+using ConstArrayView [[deprecated("please use const_array_view")]] =
+    const_array_view<ValueType>;
+
+
+template <typename ValueType>
+array<ValueType> array_const_cast(const_array_view<ValueType> view);
 
 
 }  // namespace detail
 
 
 /**
- * An Array is a container which encapsulates fixed-sized arrays, stored on the
- * Executor tied to the Array.
+ * An array is a container which encapsulates fixed-sized arrays, stored on the
+ * Executor tied to the array.
  * The array stores and transfers its data as __raw__ memory, which means that
  * the constructors of its elements are not called when constructing, copying or
- * moving the Array. Thus, the Array class is most suitable for storing POD
+ * moving the array. Thus, the array class is most suitable for storing POD
  * types.
  *
  * @tparam ValueType  the type of elements stored in the array
@@ -81,7 +184,7 @@ void convert_data(std::shared_ptr<const Executor> exec, size_type size,
  * @ingroup array
  */
 template <typename ValueType>
-class Array {
+class array {
 public:
     /**
      * The type of elements stored in the array.
@@ -89,7 +192,7 @@ public:
     using value_type = ValueType;
 
     /**
-     * The default deleter type used by Array.
+     * The default deleter type used by array.
      */
     using default_deleter = executor_deleter<value_type[]>;
 
@@ -99,7 +202,7 @@ public:
     using view_deleter = null_deleter<value_type[]>;
 
     /**
-     * Creates an empty Array not tied to any executor.
+     * Creates an empty array not tied to any executor.
      *
      * An array without an assigned executor can only be empty. Attempts to
      * change its size (e.g. via the resize_and_reset method) will result in an
@@ -108,34 +211,34 @@ public:
      * will be cleared, but its executor will not be modified.
      *
      * The executor can later be set by using the set_executor method. If an
-     * Array with no assigned executor is assigned or moved to, it will inherit
-     * the executor of the source Array.
+     * array with no assigned executor is assigned or moved to, it will inherit
+     * the executor of the source array.
      */
-    Array() noexcept
+    array() noexcept
         : num_elems_(0),
           data_(nullptr, default_deleter{nullptr}),
           exec_(nullptr)
     {}
 
     /**
-     * Creates an empty Array tied to the specified Executor.
+     * Creates an empty array tied to the specified Executor.
      *
      * @param exec  the Executor where the array data is allocated
      */
-    explicit Array(std::shared_ptr<const Executor> exec) noexcept
+    explicit array(std::shared_ptr<const Executor> exec) noexcept
         : num_elems_(0),
           data_(nullptr, default_deleter{exec}),
           exec_(std::move(exec))
     {}
 
     /**
-     * Creates an Array on the specified Executor.
+     * Creates an array on the specified Executor.
      *
      * @param exec  the Executor where the array data will be allocated
      * @param num_elems  the amount of memory (expressed as the number of
      *                   `value_type` elements) allocated on the Executor
      */
-    Array(std::shared_ptr<const Executor> exec, size_type num_elems)
+    array(std::shared_ptr<const Executor> exec, size_type num_elems)
         : num_elems_(num_elems),
           data_(nullptr, default_deleter{exec}),
           exec_(std::move(exec))
@@ -146,7 +249,7 @@ public:
     }
 
     /**
-     * Creates an Array from existing memory.
+     * Creates an array from existing memory.
      *
      * The memory will be managed by the array, and deallocated using the
      * specified deleter (e.g. use std::default_delete for data allocated with
@@ -159,18 +262,18 @@ public:
      * @param data  chunk of memory used to create the array
      * @param deleter  the deleter used to free the memory
      *
-     * @see Array::view() to create an array that does not deallocate memory
-     * @see Array(std::shared_ptr<cont Executor>, size_type, value_type*) to
+     * @see array::view() to create an array that does not deallocate memory
+     * @see array(std::shared_ptr<cont Executor>, size_type, value_type*) to
      *      deallocate the memory using Executor::free() method
      */
     template <typename DeleterType>
-    Array(std::shared_ptr<const Executor> exec, size_type num_elems,
-          value_type *data, DeleterType deleter)
+    array(std::shared_ptr<const Executor> exec, size_type num_elems,
+          value_type* data, DeleterType deleter)
         : num_elems_{num_elems}, data_(data, deleter), exec_{exec}
     {}
 
     /**
-     * Creates an Array from existing memory.
+     * Creates an array from existing memory.
      *
      * The memory will be managed by the array, and deallocated using the
      * Executor::free method.
@@ -179,9 +282,9 @@ public:
      * @param num_elems  number of elements in `data`
      * @param data  chunk of memory used to create the array
      */
-    Array(std::shared_ptr<const Executor> exec, size_type num_elems,
-          value_type *data)
-        : Array(exec, num_elems, data, default_deleter{exec})
+    array(std::shared_ptr<const Executor> exec, size_type num_elems,
+          value_type* data)
+        : array(exec, num_elems, data, default_deleter{exec})
     {}
 
     /**
@@ -195,11 +298,11 @@ public:
      * @param end  end of range of values
      */
     template <typename RandomAccessIterator>
-    Array(std::shared_ptr<const Executor> exec, RandomAccessIterator begin,
+    array(std::shared_ptr<const Executor> exec, RandomAccessIterator begin,
           RandomAccessIterator end)
-        : Array(exec)
+        : array(exec)
     {
-        Array tmp(exec->get_master(), std::distance(begin, end));
+        array tmp(exec->get_master(), std::distance(begin, end));
         std::copy(begin, end, tmp.data_.get());
         *this = std::move(tmp);
     }
@@ -212,12 +315,12 @@ public:
      *            implicitly convertible to value_type)
      *
      * @param exec  the Executor where the array data will be allocated
-     * @param init_list  list of values used to initialize the Array
+     * @param init_list  list of values used to initialize the array
      */
     template <typename T>
-    Array(std::shared_ptr<const Executor> exec,
+    array(std::shared_ptr<const Executor> exec,
           std::initializer_list<T> init_list)
-        : Array(exec, begin(init_list), end(init_list))
+        : array(exec, begin(init_list), end(init_list))
     {}
 
     /**
@@ -227,10 +330,10 @@ public:
      * copied as POD types.
      *
      * @param exec  the executor where the new array will be created
-     * @param other  the Array to copy from
+     * @param other  the array to copy from
      */
-    Array(std::shared_ptr<const Executor> exec, const Array &other)
-        : Array(exec)
+    array(std::shared_ptr<const Executor> exec, const array& other)
+        : array(exec)
     {
         *this = other;
     }
@@ -241,9 +344,9 @@ public:
      * This does not invoke the constructors of the elements, instead they are
      * copied as POD types.
      *
-     * @param other  the Array to copy from
+     * @param other  the array to copy from
      */
-    Array(const Array &other) : Array(other.get_executor(), other) {}
+    array(const array& other) : array(other.get_executor(), other) {}
 
     /**
      * Moves another array to a different executor.
@@ -252,9 +355,9 @@ public:
      * copied as POD types.
      *
      * @param exec  the executor where the new array will be moved
-     * @param other  the Array to move
+     * @param other  the array to move
      */
-    Array(std::shared_ptr<const Executor> exec, Array &&other) : Array(exec)
+    array(std::shared_ptr<const Executor> exec, array&& other) : array(exec)
     {
         *this = std::move(other);
     }
@@ -265,14 +368,14 @@ public:
      * This does not invoke the constructors of the elements, instead they are
      * copied as POD types.
      *
-     * @param other  the Array to move
+     * @param other  the array to move
      */
-    Array(Array &&other) : Array(other.get_executor(), std::move(other)) {}
+    array(array&& other) : array(other.get_executor(), std::move(other)) {}
 
     /**
-     * Creates an Array from existing memory.
+     * Creates an array from existing memory.
      *
-     * The Array does not take ownership of the memory, and will not deallocate
+     * The array does not take ownership of the memory, and will not deallocate
      * it once it goes out of scope. This array type cannot use the function
      * `resize_and_reset` since it does not own the data it should resize.
      *
@@ -280,12 +383,52 @@ public:
      * @param num_elems  number of elements in `data`
      * @param data  chunk of memory used to create the array
      *
-     * @return an Array constructed from `data`
+     * @return an array constructed from `data`
      */
-    static Array view(std::shared_ptr<const Executor> exec, size_type num_elems,
-                      value_type *data)
+    static array view(std::shared_ptr<const Executor> exec, size_type num_elems,
+                      value_type* data)
     {
-        return Array{exec, num_elems, data, view_deleter{}};
+        return array{exec, num_elems, data, view_deleter{}};
+    }
+
+    /**
+     * Creates a constant (immutable) array from existing memory.
+     *
+     * The array does not take ownership of the memory, and will not deallocate
+     * it once it goes out of scope. This array type cannot use the function
+     * `resize_and_reset` since it does not own the data it should resize.
+     *
+     * @param exec  executor where `data` is located
+     * @param num_elems  number of elements in `data`
+     * @param data  chunk of memory used to create the array
+     *
+     * @return an array constructed from `data`
+     */
+    static detail::const_array_view<ValueType> const_view(
+        std::shared_ptr<const Executor> exec, size_type num_elems,
+        const value_type* data)
+    {
+        return {exec, num_elems, data};
+    }
+
+    /**
+     * Returns a non-owning view of the memory owned by this array.
+     * It can only be used until this array gets deleted, cleared or resized.
+     */
+    array<ValueType> as_view()
+    {
+        return view(this->get_executor(), this->get_num_elems(),
+                    this->get_data());
+    }
+
+    /**
+     * Returns a non-owning constant view of the memory owned by this array.
+     * It can only be used until this array gets deleted, cleared or resized.
+     */
+    detail::const_array_view<ValueType> as_const_view() const
+    {
+        return const_view(this->get_executor(), this->get_num_elems(),
+                          this->get_const_data());
     }
 
     /**
@@ -300,11 +443,11 @@ public:
      * The executor of this is preserved. In case this does not have an assigned
      * executor, it will inherit the executor of other.
      *
-     * @param other  the Array to copy from
+     * @param other  the array to copy from
      *
      * @return this
      */
-    Array &operator=(const Array &other)
+    array& operator=(const array& other)
     {
         if (&other == this) {
             return *this;
@@ -324,7 +467,7 @@ public:
             GKO_ENSURE_COMPATIBLE_BOUNDS(other.get_num_elems(),
                                          this->num_elems_);
         }
-        exec_->copy_from(other.get_executor().get(), other.get_num_elems(),
+        exec_->copy_from(other.get_executor(), other.get_num_elems(),
                          other.get_const_data(), this->get_data());
         return *this;
     }
@@ -334,8 +477,8 @@ public:
      * change, a copy only happens when targeting another executor's data. This
      * means that in the following situation:
      * ```cpp
-     *   gko::Array<int> a; // an existing array or view
-     *   gko::Array<int> b; // an existing array or view
+     *   gko::array<int> a; // an existing array or view
+     *   gko::array<int> b; // an existing array or view
      *   b = std::move(a);
      * ```
      * Depending on whether `a` and `b` are array or view, this happens:
@@ -346,7 +489,7 @@ public:
      * + `a` is an array and `b` is a view, `b` becomes the only valid array
      *    of `a`.
      *
-     * In all the previous cases, `a` becomes invalid (e.g., a `nullptr`).
+     * In all the previous cases, `a` becomes empty (e.g., a `nullptr`).
      *
      * This does not invoke the constructors of the elements, instead they are
      * copied as POD types.
@@ -354,18 +497,18 @@ public:
      * The executor of this is preserved. In case this does not have an assigned
      * executor, it will inherit the executor of other.
      *
-     * @param other  the Array to move data from
+     * @param other  the array to move data from
      *
      * @return this
      */
-    Array &operator=(Array &&other)
+    array& operator=(array&& other)
     {
         if (&other == this) {
             return *this;
         }
         if (exec_ == nullptr) {
             exec_ = other.get_executor();
-            data_ = data_manager{nullptr, other.data_.get_deleter()};
+            data_ = data_manager{nullptr, default_deleter{exec_}};
         }
         if (other.get_executor() == nullptr) {
             this->clear();
@@ -373,13 +516,13 @@ public:
         }
         if (exec_ == other.get_executor()) {
             // same device, only move the pointer
-            using std::swap;
-            swap(data_, other.data_);
-            swap(num_elems_, other.num_elems_);
-            other.clear();
+            data_ = std::exchange(
+                other.data_, data_manager{nullptr, default_deleter{exec_}});
+            num_elems_ = std::exchange(other.num_elems_, 0);
         } else {
             // different device, copy the data
             *this = other;
+            other.clear();
         }
         return *this;
     }
@@ -396,14 +539,14 @@ public:
      * The executor of this is preserved. In case this does not have an assigned
      * executor, it will inherit the executor of other.
      *
-     * @param other  the Array to copy from
+     * @param other  the array to copy from
      * @tparam OtherValueType  the value type of `other`
      *
      * @return this
      */
     template <typename OtherValueType>
-    std::enable_if_t<!std::is_same<ValueType, OtherValueType>::value, Array>
-        &operator=(const Array<OtherValueType> &other)
+    std::enable_if_t<!std::is_same<ValueType, OtherValueType>::value, array>&
+    operator=(const array<OtherValueType>& other)
     {
         if (this->exec_ == nullptr) {
             this->exec_ = other.get_executor();
@@ -420,8 +563,8 @@ public:
             GKO_ENSURE_COMPATIBLE_BOUNDS(other.get_num_elems(),
                                          this->num_elems_);
         }
-        Array<OtherValueType> tmp{this->exec_};
-        const OtherValueType *source = other.get_const_data();
+        array<OtherValueType> tmp{this->exec_};
+        const OtherValueType* source = other.get_const_data();
         // if we are on different executors: copy, then convert
         if (this->exec_ != other.get_executor()) {
             tmp = other;
@@ -433,10 +576,10 @@ public:
     }
 
     /**
-     * Deallocates all data used by the Array.
+     * Deallocates all data used by the array.
      *
      * The array is left in a valid, but empty state, so the same array can be
-     * used to allocate new memory. Calls to Array::get_data() will return
+     * used to allocate new memory. Calls to array::get_data() will return
      * a `nullptr`.
      */
     void clear() noexcept
@@ -447,12 +590,12 @@ public:
 
     /**
      * Resizes the array so it is able to hold the specified number of elements.
-     * For a view and other non-owning Array types, this throws an exception
+     * For a view and other non-owning array types, this throws an exception
      * since these types cannot be resized.
      *
      * All data stored in the array will be lost.
      *
-     * If the Array is not assigned an executor, an exception will be thrown.
+     * If the array is not assigned an executor, an exception will be thrown.
      *
      * @param num_elems  the amount of memory (expressed as the number of
      *                   `value_type` elements) allocated on the Executor
@@ -468,7 +611,7 @@ public:
         }
         if (!this->is_owning()) {
             throw gko::NotSupported(__FILE__, __LINE__, __func__,
-                                    "Non owning gko::Array cannot be resized.");
+                                    "Non owning gko::array cannot be resized.");
         }
 
         if (num_elems > 0 && this->is_owning()) {
@@ -484,32 +627,32 @@ public:
      *
      * @param value the value to be filled
      */
-    void fill(const ValueType value);
+    void fill(const value_type value);
 
     /**
-     * Returns the number of elements in the Array.
+     * Returns the number of elements in the array.
      *
-     * @return the number of elements in the Array
+     * @return the number of elements in the array
      */
     size_type get_num_elems() const noexcept { return num_elems_; }
 
     /**
      * Returns a pointer to the block of memory used to store the elements of
-     * the Array.
+     * the array.
      *
      * @return a pointer to the block of memory used to store the elements of
-     * the Array
+     * the array
      */
-    value_type *get_data() noexcept { return data_.get(); }
+    value_type* get_data() noexcept { return data_.get(); }
 
     /**
      * Returns a constant pointer to the block of memory used to store the
-     * elements of the Array.
+     * elements of the array.
      *
      * @return a constant pointer to the block of memory used to store the
-     * elements of the Array
+     * elements of the array
      */
-    const value_type *get_const_data() const noexcept { return data_.get(); }
+    const value_type* get_const_data() const noexcept { return data_.get(); }
 
     /**
      * Returns the Executor associated with the array.
@@ -522,7 +665,7 @@ public:
     }
 
     /**
-     * Changes the Executor of the Array, moving the allocated data to the new
+     * Changes the Executor of the array, moving the allocated data to the new
      * Executor.
      *
      * @param exec  the Executor where the data will be moved to
@@ -533,22 +676,22 @@ public:
             // moving to the same executor, no-op
             return;
         }
-        Array tmp(std::move(exec));
+        array tmp(std::move(exec));
         tmp = *this;
         exec_ = std::move(tmp.exec_);
         data_ = std::move(tmp.data_);
     }
 
     /**
-     * Tells whether this Array owns its data or not.
+     * Tells whether this array owns its data or not.
      *
      * Views do not own their data and this has multiple implications. They
-     * cannot be resized since the data is not owned by the Array which stores a
+     * cannot be resized since the data is not owned by the array which stores a
      * view. It is also unclear whether custom deleter types are owning types as
-     * they could be a user-created view-type, therefore only proper Array which
+     * they could be a user-created view-type, therefore only proper array which
      * use the `default_deleter` are considered owning types.
      *
-     * @return whether this Array can be resized or not.
+     * @return whether this array can be resized or not.
      */
     bool is_owning()
     {
@@ -557,9 +700,9 @@ public:
 
 
 private:
-    // Allow other Array types to access private members
+    // Allow other array types to access private members
     template <typename OtherValueType>
-    friend class Array;
+    friend class array;
 
     using data_manager =
         std::unique_ptr<value_type[], std::function<void(value_type[])>>;
@@ -570,38 +713,108 @@ private:
 };
 
 
+template <typename ValueType>
+using Array [[deprecated("please use array")]] = array<ValueType>;
+
+
+/**
+ * Reduce (sum) the values in the array
+ *
+ * @tparam The type of the input data
+ *
+ * @param [in] input_arr the input array to be reduced
+ * @param [in] init_val the initial value
+ *
+ * @return the reduced value
+ */
+template <typename ValueType>
+ValueType reduce_add(const array<ValueType>& input_arr,
+                     const ValueType init_val = 0);
+
+/**
+ * Reduce (sum) the values in the array
+ *
+ * @tparam The type of the input data
+ *
+ * @param [in] input_arr the input array to be reduced
+ * @param [out,in] result the reduced value. The result is written into the
+ *                 first entry and the value in the first entry is used as the
+ *                 initial value for the reduce.
+ */
+template <typename ValueType>
+void reduce_add(const array<ValueType>& input_arr, array<ValueType>& result);
+
+
+/**
+ * Helper function to create an array view deducing the value type.
+ *
+ * @param exec  the executor on which the array resides
+ * @param size  the number of elements for the array
+ * @param data  the pointer to the array we create a view on.
+ *
+ * @tparam ValueType  the type of the array elements
+ *
+ * @return `array<ValueType>::view(exec, size, data)`
+ */
+template <typename ValueType>
+array<ValueType> make_array_view(std::shared_ptr<const Executor> exec,
+                                 size_type size, ValueType* data)
+{
+    return array<ValueType>::view(exec, size, data);
+}
+
+
+/**
+ * Helper function to create a const array view deducing the value type.
+ *
+ * @param exec  the executor on which the array resides
+ * @param size  the number of elements for the array
+ * @param data  the pointer to the array we create a view on.
+ *
+ * @tparam ValueType  the type of the array elements
+ *
+ * @return `array<ValueType>::const_view(exec, size, data)`
+ */
+template <typename ValueType>
+detail::const_array_view<ValueType> make_const_array_view(
+    std::shared_ptr<const Executor> exec, size_type size, const ValueType* data)
+{
+    return array<ValueType>::const_view(exec, size, data);
+}
+
+
 namespace detail {
 
 
 template <typename T>
-struct temporary_clone_helper<Array<T>> {
-    static std::unique_ptr<Array<T>> create(
-        std::shared_ptr<const Executor> exec, Array<T> *ptr, bool copy_data)
+struct temporary_clone_helper<array<T>> {
+    static std::unique_ptr<array<T>> create(
+        std::shared_ptr<const Executor> exec, array<T>* ptr, bool copy_data)
     {
         if (copy_data) {
-            return std::make_unique<Array<T>>(std::move(exec), *ptr);
+            return std::make_unique<array<T>>(std::move(exec), *ptr);
         } else {
-            return std::make_unique<Array<T>>(std::move(exec),
+            return std::make_unique<array<T>>(std::move(exec),
                                               ptr->get_num_elems());
         }
     }
 };
 
 template <typename T>
-struct temporary_clone_helper<const Array<T>> {
-    static std::unique_ptr<const Array<T>> create(
-        std::shared_ptr<const Executor> exec, const Array<T> *ptr, bool)
+struct temporary_clone_helper<const array<T>> {
+    static std::unique_ptr<const array<T>> create(
+        std::shared_ptr<const Executor> exec, const array<T>* ptr, bool)
     {
-        return std::make_unique<const Array<T>>(std::move(exec), *ptr);
+        return std::make_unique<const array<T>>(std::move(exec), *ptr);
     }
 };
 
 
 // specialization for non-constant arrays, copying back via assignment
 template <typename T>
-class copy_back_deleter<Array<T>> {
+class copy_back_deleter<array<T>> {
 public:
-    using pointer = Array<T> *;
+    using pointer = array<T>*;
 
     /**
      * Creates a new deleter object.
@@ -627,6 +840,38 @@ private:
 };
 
 
+/**
+ * @internal
+ *
+ * Casts away const-ness from a array view to get a array representing the same
+ * data. This needs to be used carefully, as the class this array gets passed to
+ * must not modify its data. That is usually achieved by creating a `const`
+ * instance of the class.
+ *
+ * @param view  the array view to be cast.
+ * @returns a non-const non-owning array wrapping the same pointer on the same
+ *          executor as `view`.
+ */
+template <typename ValueType>
+array<ValueType> array_const_cast(const_array_view<ValueType> view)
+{
+    return array<ValueType>::view(
+        view.get_executor(), view.get_num_elems(),
+        const_cast<ValueType*>(view.get_const_data()));
+}
+
+
+template <typename ValueType>
+array<ValueType> const_array_view<ValueType>::copy_to_array() const
+{
+    array<ValueType> result(this->get_executor(), this->get_num_elems());
+    result.get_executor()->copy_from(this->get_executor(),
+                                     this->get_num_elems(),
+                                     this->get_const_data(), result.get_data());
+    return result;
+}
+
+
 }  // namespace detail
 }  // namespace gko
 
