@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -40,6 +40,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <ginkgo/core/base/abstract_factory.hpp>
+#include <ginkgo/core/base/device_matrix_data.hpp>
 #include <ginkgo/core/base/dim.hpp>
 #include <ginkgo/core/base/exception_helpers.hpp>
 #include <ginkgo/core/base/math.hpp>
@@ -59,7 +60,7 @@ template <typename ValueType>
 class Diagonal;
 
 
-}
+}  // namespace matrix
 
 
 /**
@@ -123,9 +124,9 @@ class Diagonal;
  *     auto tmp = gko::clone(x0);
  *     auto one = Dense<>::create(L->get_executor(), {1.0,});
  *     for (int i = 0; i < iters; ++i) {
- *         L->apply(gko::lend(tmp), gko::lend(x));
- *         x->add_scaled(gko::lend(one), gko::lend(b));
- *         tmp->copy_from(gko::lend(x));
+ *         L->apply(tmp, x);
+ *         x->add_scaled(one, b);
+ *         tmp->copy_from(x);
  *     }
  *     return x;
  * }
@@ -133,7 +134,7 @@ class Diagonal;
  *
  * Here, if $L$ is a matrix, LinOp::apply() refers to the matrix vector
  * product, and `L->apply(a, b)` computes $b = L \cdot a$.
- * `x->add_scaled(one.get(), b.get())` is the `axpy` vector update $x:=x+b$.
+ * `x->add_scaled(one, b)` is the `axpy` vector update $x:=x+b$.
  *
  * The interesting part of this example is the apply() routine at line 4 of the
  * function body. Since this routine is part of the LinOp base class, the
@@ -154,28 +155,32 @@ public:
      *
      * @return this
      */
-    LinOp *apply(const LinOp *b, LinOp *x)
+    LinOp* apply(ptr_param<const LinOp> b, ptr_param<LinOp> x)
     {
-        this->template log<log::Logger::linop_apply_started>(this, b, x);
-        this->validate_application_parameters(b, x);
+        this->template log<log::Logger::linop_apply_started>(this, b.get(),
+                                                             x.get());
+        this->validate_application_parameters(b.get(), x.get());
         auto exec = this->get_executor();
         this->apply_impl(make_temporary_clone(exec, b).get(),
                          make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_apply_completed>(this, b, x);
+        this->template log<log::Logger::linop_apply_completed>(this, b.get(),
+                                                               x.get());
         return this;
     }
 
     /**
      * @copydoc apply(const LinOp *, LinOp *)
      */
-    const LinOp *apply(const LinOp *b, LinOp *x) const
+    const LinOp* apply(ptr_param<const LinOp> b, ptr_param<LinOp> x) const
     {
-        this->template log<log::Logger::linop_apply_started>(this, b, x);
-        this->validate_application_parameters(b, x);
+        this->template log<log::Logger::linop_apply_started>(this, b.get(),
+                                                             x.get());
+        this->validate_application_parameters(b.get(), x.get());
         auto exec = this->get_executor();
         this->apply_impl(make_temporary_clone(exec, b).get(),
                          make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_apply_completed>(this, b, x);
+        this->template log<log::Logger::linop_apply_completed>(this, b.get(),
+                                                               x.get());
         return this;
     }
 
@@ -189,38 +194,40 @@ public:
      *
      * @return this
      */
-    LinOp *apply(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                 LinOp *x)
+    LinOp* apply(ptr_param<const LinOp> alpha, ptr_param<const LinOp> b,
+                 ptr_param<const LinOp> beta, ptr_param<LinOp> x)
     {
         this->template log<log::Logger::linop_advanced_apply_started>(
-            this, alpha, b, beta, x);
-        this->validate_application_parameters(alpha, b, beta, x);
+            this, alpha.get(), b.get(), beta.get(), x.get());
+        this->validate_application_parameters(alpha.get(), b.get(), beta.get(),
+                                              x.get());
         auto exec = this->get_executor();
         this->apply_impl(make_temporary_clone(exec, alpha).get(),
                          make_temporary_clone(exec, b).get(),
                          make_temporary_clone(exec, beta).get(),
                          make_temporary_clone(exec, x).get());
         this->template log<log::Logger::linop_advanced_apply_completed>(
-            this, alpha, b, beta, x);
+            this, alpha.get(), b.get(), beta.get(), x.get());
         return this;
     }
 
     /**
      * @copydoc apply(const LinOp *, const LinOp *, const LinOp *, LinOp *)
      */
-    const LinOp *apply(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                       LinOp *x) const
+    const LinOp* apply(ptr_param<const LinOp> alpha, ptr_param<const LinOp> b,
+                       ptr_param<const LinOp> beta, ptr_param<LinOp> x) const
     {
         this->template log<log::Logger::linop_advanced_apply_started>(
-            this, alpha, b, beta, x);
-        this->validate_application_parameters(alpha, b, beta, x);
+            this, alpha.get(), b.get(), beta.get(), x.get());
+        this->validate_application_parameters(alpha.get(), b.get(), beta.get(),
+                                              x.get());
         auto exec = this->get_executor();
         this->apply_impl(make_temporary_clone(exec, alpha).get(),
                          make_temporary_clone(exec, b).get(),
                          make_temporary_clone(exec, beta).get(),
                          make_temporary_clone(exec, x).get());
         this->template log<log::Logger::linop_advanced_apply_completed>(
-            this, alpha, b, beta, x);
+            this, alpha.get(), b.get(), beta.get(), x.get());
         return this;
     }
 
@@ -229,7 +236,7 @@ public:
      *
      * @return size of the operator
      */
-    const dim<2> &get_size() const noexcept { return size_; }
+    const dim<2>& get_size() const noexcept { return size_; }
 
     /**
      * Returns true if the linear operator uses the data given in x as
@@ -240,6 +247,36 @@ public:
      */
     virtual bool apply_uses_initial_guess() const { return false; }
 
+    /** Copy-assigns a LinOp. Preserves the executor and copies the size. */
+    LinOp& operator=(const LinOp&) = default;
+
+    /**
+     * Move-assigns a LinOp. Preserves the executor and moves the size.
+     * The moved-from object has size 0x0 afterwards, but its executor is
+     * unchanged.
+     */
+    LinOp& operator=(LinOp&& other)
+    {
+        if (this != &other) {
+            EnableAbstractPolymorphicObject<LinOp>::operator=(std::move(other));
+            this->set_size(other.get_size());
+            other.set_size({});
+        }
+        return *this;
+    }
+
+    /** Copy-constructs a LinOp. Inherits executor and size from the input. */
+    LinOp(const LinOp&) = default;
+
+    /**
+     * Move-constructs a LinOp. Inherits executor and size from the input,
+     * which will have size 0x0 and unchanged executor afterwards.
+     */
+    LinOp(LinOp&& other)
+        : EnableAbstractPolymorphicObject<LinOp>(std::move(other)),
+          size_{std::exchange(other.size_, dim<2>{})}
+    {}
+
 protected:
     /**
      * Creates a linear operator.
@@ -248,7 +285,7 @@ protected:
      * @param size  the size of the operator
      */
     explicit LinOp(std::shared_ptr<const Executor> exec,
-                   const dim<2> &size = dim<2>{})
+                   const dim<2>& size = dim<2>{})
         : EnableAbstractPolymorphicObject<LinOp>(exec), size_{size}
     {}
 
@@ -257,7 +294,7 @@ protected:
      *
      * @param value  the new size of the operator
      */
-    void set_size(const dim<2> &value) noexcept { size_ = value; }
+    void set_size(const dim<2>& value) noexcept { size_ = value; }
 
     /**
      * Implementers of LinOp should override this function instead
@@ -268,7 +305,7 @@ protected:
      * @param b  the input vector(s) on which the operator is applied
      * @param x  the output vector(s) where the result is stored
      */
-    virtual void apply_impl(const LinOp *b, LinOp *x) const = 0;
+    virtual void apply_impl(const LinOp* b, LinOp* x) const = 0;
 
     /**
      * Implementers of LinOp should override this function instead
@@ -279,8 +316,8 @@ protected:
      * @param beta  scaling of the input x
      * @param x  output vector(s)
      */
-    virtual void apply_impl(const LinOp *alpha, const LinOp *b,
-                            const LinOp *beta, LinOp *x) const = 0;
+    virtual void apply_impl(const LinOp* alpha, const LinOp* b,
+                            const LinOp* beta, LinOp* x) const = 0;
 
     /**
      * Throws a DimensionMismatch exception if the parameters to `apply` are of
@@ -289,7 +326,7 @@ protected:
      * @param b  vector(s) on which the operator is applied
      * @param x  output vector(s)
      */
-    void validate_application_parameters(const LinOp *b, const LinOp *x) const
+    void validate_application_parameters(const LinOp* b, const LinOp* x) const
     {
         GKO_ASSERT_CONFORMANT(this, b);
         GKO_ASSERT_EQUAL_ROWS(this, x);
@@ -305,9 +342,9 @@ protected:
      * @param beta  scaling of the input x
      * @param x  output vector(s)
      */
-    void validate_application_parameters(const LinOp *alpha, const LinOp *b,
-                                         const LinOp *beta,
-                                         const LinOp *x) const
+    void validate_application_parameters(const LinOp* alpha, const LinOp* b,
+                                         const LinOp* beta,
+                                         const LinOp* x) const
     {
         this->validate_application_parameters(b, x);
         GKO_ASSERT_EQUAL_DIMENSIONS(alpha, dim<2>(1, 1));
@@ -368,7 +405,7 @@ private:
  * // create a linear operator which represents the solver
  * auto cg = cg_factory->generate(A);
  * // solve the system
- * cg->apply(gko::lend(b), gko::lend(x));
+ * cg->apply(b, x);
  * ```
  *
  * @ingroup LinOp
@@ -382,7 +419,14 @@ public:
     {
         this->template log<log::Logger::linop_factory_generate_started>(
             this, input.get());
-        auto generated = AbstractFactory::generate(input);
+        const auto exec = this->get_executor();
+        std::unique_ptr<LinOp> generated;
+        if (input->get_executor() == exec) {
+            generated = this->AbstractFactory::generate(input);
+        } else {
+            generated =
+                this->AbstractFactory::generate(gko::clone(exec, input));
+        }
         this->template log<log::Logger::linop_factory_generate_completed>(
             this, input.get(), generated.get());
         return generated;
@@ -482,7 +526,7 @@ public:
      * @return a pointer to the new permuted object
      */
     virtual std::unique_ptr<LinOp> permute(
-        const Array<IndexType> *permutation_indices) const
+        const array<IndexType>* permutation_indices) const
     {
         return as<Permutable>(this->row_permute(permutation_indices))
             ->column_permute(permutation_indices);
@@ -500,7 +544,7 @@ public:
      * @return a pointer to the new permuted object
      */
     virtual std::unique_ptr<LinOp> inverse_permute(
-        const Array<IndexType> *permutation_indices) const
+        const array<IndexType>* permutation_indices) const
     {
         return as<Permutable>(this->inverse_row_permute(permutation_indices))
             ->inverse_column_permute(permutation_indices);
@@ -517,7 +561,7 @@ public:
      * @return a pointer to the new permuted object
      */
     virtual std::unique_ptr<LinOp> row_permute(
-        const Array<IndexType> *permutation_indices) const = 0;
+        const array<IndexType>* permutation_indices) const = 0;
 
     /**
      * Returns a LinOp representing the column permutation of the Permutable
@@ -531,7 +575,7 @@ public:
      * @return a pointer to the new column permuted object
      */
     virtual std::unique_ptr<LinOp> column_permute(
-        const Array<IndexType> *permutation_indices) const = 0;
+        const array<IndexType>* permutation_indices) const = 0;
 
     /**
      * Returns a LinOp representing the row permutation of the inverse permuted
@@ -544,7 +588,7 @@ public:
      * @return a pointer to the new inverse permuted object
      */
     virtual std::unique_ptr<LinOp> inverse_row_permute(
-        const Array<IndexType> *permutation_indices) const = 0;
+        const array<IndexType>* permutation_indices) const = 0;
 
     /**
      * Returns a LinOp representing the row permutation of the inverse permuted
@@ -558,7 +602,7 @@ public:
      * @return a pointer to the new inverse permuted object
      */
     virtual std::unique_ptr<LinOp> inverse_column_permute(
-        const Array<IndexType> *permutation_indices) const = 0;
+        const array<IndexType>* permutation_indices) const = 0;
 };
 
 
@@ -581,17 +625,39 @@ public:
      *
      * @param data  the matrix_data structure
      */
-    virtual void read(const matrix_data<ValueType, IndexType> &data) = 0;
+    virtual void read(const matrix_data<ValueType, IndexType>& data) = 0;
 
     /**
      * Reads a matrix from a matrix_assembly_data structure.
      *
      * @param data  the matrix_assembly_data structure
      */
-    void read(const matrix_assembly_data<ValueType, IndexType> &data)
+    void read(const matrix_assembly_data<ValueType, IndexType>& data)
     {
         this->read(data.get_ordered_data());
     }
+
+    /**
+     * Reads a matrix from a device_matrix_data structure.
+     *
+     * @param data  the device_matrix_data structure.
+     */
+    virtual void read(const device_matrix_data<ValueType, IndexType>& data)
+    {
+        this->read(data.copy_to_host());
+    }
+
+    /**
+     * Reads a matrix from a device_matrix_data structure.
+     * The structure may be emptied by this function.
+     *
+     * @param data  the device_matrix_data structure.
+     */
+    virtual void read(device_matrix_data<ValueType, IndexType>&& data)
+    {
+        this->read(data.copy_to_host());
+        data.empty_out();
+    }
 };
 
 
@@ -614,7 +680,7 @@ public:
      *
      * @param data  the matrix_data structure
      */
-    virtual void write(matrix_data<ValueType, IndexType> &data) const = 0;
+    virtual void write(matrix_data<ValueType, IndexType>& data) const = 0;
 };
 
 
@@ -756,6 +822,34 @@ public:
 };
 
 
+/**
+ * Adds the operation M <- a I + b M for matrix M, identity
+ * operator I and scalars a and b, where M is the calling object.
+ */
+class ScaledIdentityAddable {
+public:
+    /**
+     * Scales this and adds another scalar times the identity to it.
+     *
+     * @param a  Scalar to multiply the identity operator before adding.
+     * @param b  Scalar to multiply this before adding the scaled identity to
+     *           it.
+     */
+    void add_scaled_identity(ptr_param<const LinOp> const a,
+                             ptr_param<const LinOp> const b)
+    {
+        GKO_ASSERT_IS_SCALAR(a);
+        GKO_ASSERT_IS_SCALAR(b);
+        auto ae = make_temporary_clone(as<LinOp>(this)->get_executor(), a);
+        auto be = make_temporary_clone(as<LinOp>(this)->get_executor(), b);
+        add_scaled_identity_impl(ae.get(), be.get());
+    }
+
+private:
+    virtual void add_scaled_identity_impl(const LinOp* a, const LinOp* b) = 0;
+};
+
+
 /**
  * The EnableLinOp mixin can be used to provide sensible default implementations
  * of the majority of the LinOp and PolymorphicObject interface.
@@ -779,6 +873,10 @@ public:
  * 2.  Application of the LinOp: Implementers have to override the two
  *     overloads of the LinOp::apply_impl() virtual methods.
  *
+ * @note  This mixin can't be used with concrete types that derive from
+ *        experimental::distributed::DistributedBase. In that case use
+ *        experimental::EnableDistributedLinOp instead.
+ *
  * @tparam ConcreteLinOp  the concrete LinOp which is being implemented
  *                        [CRTP parameter]
  * @tparam PolymorphicBase  parent of ConcreteLinOp in the polymorphic
@@ -794,57 +892,32 @@ public:
     using EnablePolymorphicObject<ConcreteLinOp,
                                   PolymorphicBase>::EnablePolymorphicObject;
 
-    const ConcreteLinOp *apply(const LinOp *b, LinOp *x) const
+    const ConcreteLinOp* apply(ptr_param<const LinOp> b,
+                               ptr_param<LinOp> x) const
     {
-        this->template log<log::Logger::linop_apply_started>(this, b, x);
-        this->validate_application_parameters(b, x);
-        auto exec = this->get_executor();
-        this->apply_impl(make_temporary_clone(exec, b).get(),
-                         make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_apply_completed>(this, b, x);
+        PolymorphicBase::apply(b, x);
         return self();
     }
 
-    ConcreteLinOp *apply(const LinOp *b, LinOp *x)
+    ConcreteLinOp* apply(ptr_param<const LinOp> b, ptr_param<LinOp> x)
     {
-        this->template log<log::Logger::linop_apply_started>(this, b, x);
-        this->validate_application_parameters(b, x);
-        auto exec = this->get_executor();
-        this->apply_impl(make_temporary_clone(exec, b).get(),
-                         make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_apply_completed>(this, b, x);
+        PolymorphicBase::apply(b, x);
         return self();
     }
 
-    const ConcreteLinOp *apply(const LinOp *alpha, const LinOp *b,
-                               const LinOp *beta, LinOp *x) const
+    const ConcreteLinOp* apply(ptr_param<const LinOp> alpha,
+                               ptr_param<const LinOp> b,
+                               ptr_param<const LinOp> beta,
+                               ptr_param<LinOp> x) const
     {
-        this->template log<log::Logger::linop_advanced_apply_started>(
-            this, alpha, b, beta, x);
-        this->validate_application_parameters(alpha, b, beta, x);
-        auto exec = this->get_executor();
-        this->apply_impl(make_temporary_clone(exec, alpha).get(),
-                         make_temporary_clone(exec, b).get(),
-                         make_temporary_clone(exec, beta).get(),
-                         make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_advanced_apply_completed>(
-            this, alpha, b, beta, x);
+        PolymorphicBase::apply(alpha, b, beta, x);
         return self();
     }
 
-    ConcreteLinOp *apply(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                         LinOp *x)
+    ConcreteLinOp* apply(ptr_param<const LinOp> alpha, ptr_param<const LinOp> b,
+                         ptr_param<const LinOp> beta, ptr_param<LinOp> x)
     {
-        this->template log<log::Logger::linop_advanced_apply_started>(
-            this, alpha, b, beta, x);
-        this->validate_application_parameters(alpha, b, beta, x);
-        auto exec = this->get_executor();
-        this->apply_impl(make_temporary_clone(exec, alpha).get(),
-                         make_temporary_clone(exec, b).get(),
-                         make_temporary_clone(exec, beta).get(),
-                         make_temporary_clone(exec, x).get());
-        this->template log<log::Logger::linop_advanced_apply_completed>(
-            this, alpha, b, beta, x);
+        PolymorphicBase::apply(alpha, b, beta, x);
         return self();
     }
 
@@ -974,7 +1047,7 @@ public:                                                                 \
  */
 #define GKO_ENABLE_LIN_OP_FACTORY(_lin_op, _parameters_name, _factory_name)  \
 public:                                                                      \
-    const _parameters_name##_type &get_##_parameters_name() const            \
+    const _parameters_name##_type& get_##_parameters_name() const            \
     {                                                                        \
         return _parameters_name##_;                                          \
     }                                                                        \
@@ -992,7 +1065,7 @@ public:                                                                      \
                   std::move(exec))                                           \
         {}                                                                   \
         explicit _factory_name(std::shared_ptr<const ::gko::Executor> exec,  \
-                               const _parameters_name##_type &parameters)    \
+                               const _parameters_name##_type& parameters)    \
             : ::gko::EnableDefaultLinOpFactory<_factory_name, _lin_op,       \
                                                _parameters_name##_type>(     \
                   std::move(exec), parameters)                               \
@@ -1046,8 +1119,8 @@ public:                                                                      \
     mutable _name{__VA_ARGS__};                                              \
                                                                              \
     template <typename... Args>                                              \
-    auto with_##_name(Args &&... _value)                                     \
-        const->const std::decay_t<decltype(*this)> &                         \
+    auto with_##_name(Args&&... _value)                                      \
+        const->const std::decay_t<decltype(*this)>&                          \
     {                                                                        \
         using type = decltype(this->_name);                                  \
         this->_name = type{std::forward<Args>(_value)...};                   \
@@ -1097,8 +1170,8 @@ public:                                                                      \
     mutable _name{__VA_ARGS__};                                              \
                                                                              \
     template <typename... Args>                                              \
-    auto with_##_name(Args &&... _value)                                     \
-        const->const std::decay_t<decltype(*this)> &                         \
+    auto with_##_name(Args&&... _value)                                      \
+        const->const std::decay_t<decltype(*this)>&                          \
     {                                                                        \
         GKO_NOT_IMPLEMENTED;                                                 \
         return *this;                                                        \
@@ -1111,8 +1184,8 @@ public:                                                                      \
     mutable _name{_default};                                                 \
                                                                              \
     template <typename Arg>                                                  \
-    auto with_##_name(Arg &&_value)                                          \
-        const->const std::decay_t<decltype(*this)> &                         \
+    auto with_##_name(Arg&& _value)                                          \
+        const->const std::decay_t<decltype(*this)>&                          \
     {                                                                        \
         using type = decltype(this->_name);                                  \
         this->_name = type{std::forward<Arg>(_value)};                       \
@@ -1126,8 +1199,8 @@ public:                                                                      \
     mutable _name{__VA_ARGS__};                                              \
                                                                              \
     template <typename... Args>                                              \
-    auto with_##_name(Args &&... _value)                                     \
-        const->const std::decay_t<decltype(*this)> &                         \
+    auto with_##_name(Args&&... _value)                                      \
+        const->const std::decay_t<decltype(*this)>&                          \
     {                                                                        \
         using type = decltype(this->_name);                                  \
         this->_name = type{std::forward<Args>(_value)...};                   \
