@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -152,7 +152,7 @@ using uint64 = std::uint64_t;
 
 
 /**
- *
+ * Unsigned integer type capable of holding a pointer to void
  */
 using uintptr = std::uintptr_t;
 
@@ -461,6 +461,38 @@ GKO_ATTRIBUTES constexpr bool operator!=(precision_reduction x,
 #endif
 
 
+/**
+ * Instantiates a template for each value and scalar type compiled by Ginkgo.
+ * This means all value and scalar type combinations for which
+ * `value = scalar * value` is well-defined.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, which are replaced by the
+ *                value and scalar type, respectively.
+ */
+#if GINKGO_DPCPP_SINGLE_MODE
+#define GKO_INSTANTIATE_FOR_EACH_VALUE_AND_SCALAR_TYPE(_macro)              \
+    template _macro(float, float);                                          \
+    template <>                                                             \
+    _macro(double, double) GKO_NOT_IMPLEMENTED;                             \
+    template _macro(std::complex<float>, std::complex<float>);              \
+    template <>                                                             \
+    _macro(std::complex<double>, std::complex<double>) GKO_NOT_IMPLEMENTED; \
+    template _macro(std::complex<float>, float);                            \
+    template <>                                                             \
+    _macro(std::complex<double>, double) GKO_NOT_IMPLEMENTED;
+#else
+#define GKO_INSTANTIATE_FOR_EACH_VALUE_AND_SCALAR_TYPE(_macro)   \
+    template _macro(float, float);                               \
+    template _macro(double, double);                             \
+    template _macro(std::complex<float>, std::complex<float>);   \
+    template _macro(std::complex<double>, std::complex<double>); \
+    template _macro(std::complex<float>, float);                 \
+    template _macro(std::complex<double>, double)
+#endif
+
+
 /**
  * Instantiates a template for each index type compiled by Ginkgo.
  *
@@ -527,6 +559,73 @@ GKO_ATTRIBUTES constexpr bool operator!=(precision_reduction x,
 #endif
 
 
+/**
+ * Instantiates a template for each non-complex value, local and global index
+ * type compiled by Ginkgo.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take three arguments, which are replaced by the
+ *                value, the local and the global index types.
+ */
+#if GINKGO_DPCPP_SINGLE_MODE
+#define GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE( \
+    _macro)                                                                     \
+    template _macro(float, int32, int32);                                       \
+    template _macro(float, int32, int64);                                       \
+    template _macro(float, int64, int64);                                       \
+    template <>                                                                 \
+    _macro(double, int32, int32) GKO_NOT_IMPLEMENTED;                           \
+    template <>                                                                 \
+    _macro(double, int32, int64) GKO_NOT_IMPLEMENTED;                           \
+    template <>                                                                 \
+    _macro(double, int64, int64) GKO_NOT_IMPLEMENTED
+#else
+#define GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE( \
+    _macro)                                                                     \
+    template _macro(float, int32, int32);                                       \
+    template _macro(float, int32, int64);                                       \
+    template _macro(float, int64, int64);                                       \
+    template _macro(double, int32, int32);                                      \
+    template _macro(double, int32, int64);                                      \
+    template _macro(double, int64, int64)
+#endif
+
+
+/**
+ * Instantiates a template for each value and index type compiled by Ginkgo.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, which are replaced by the
+ *                value and index types.
+ */
+#if GINKGO_DPCPP_SINGLE_MODE
+#define GKO_INSTANTIATE_FOR_EACH_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE(_macro)  \
+    GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE( \
+        _macro);                                                            \
+    template _macro(std::complex<float>, int32, int32);                     \
+    template _macro(std::complex<float>, int32, int64);                     \
+    template _macro(std::complex<float>, int64, int64);                     \
+    template <>                                                             \
+    _macro(std::complex<double>, int32, int32) GKO_NOT_IMPLEMENTED;         \
+    template <>                                                             \
+    _macro(std::complex<double>, int32, int64) GKO_NOT_IMPLEMENTED;         \
+    template <>                                                             \
+    _macro(std::complex<double>, int64, int64) GKO_NOT_IMPLEMENTED
+#else
+#define GKO_INSTANTIATE_FOR_EACH_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE(_macro)  \
+    GKO_INSTANTIATE_FOR_EACH_NON_COMPLEX_VALUE_AND_LOCAL_GLOBAL_INDEX_TYPE( \
+        _macro);                                                            \
+    template _macro(std::complex<float>, int32, int32);                     \
+    template _macro(std::complex<float>, int32, int64);                     \
+    template _macro(std::complex<float>, int64, int64);                     \
+    template _macro(std::complex<double>, int32, int32);                    \
+    template _macro(std::complex<double>, int32, int64);                    \
+    template _macro(std::complex<double>, int64, int64)
+#endif
+
+
 #if GINKGO_DPCPP_SINGLE_MODE
 #define GKO_INSTANTIATE_FOR_EACH_VALUE_CONVERSION(_macro)                  \
     template <>                                                            \
@@ -548,8 +647,6 @@ GKO_ATTRIBUTES constexpr bool operator!=(precision_reduction x,
     template <>                                                   \
     _macro(std::complex<double>, std::complex<double>) GKO_NOT_IMPLEMENTED
 #else
-
-
 /**
  * Instantiates a template for each value type conversion pair compiled by
  * Ginkgo.
@@ -584,6 +681,63 @@ GKO_ATTRIBUTES constexpr bool operator!=(precision_reduction x,
 #endif
 
 
+/**
+ * Instantiates a template for each value type pair compiled by Ginkgo.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, which are replaced by the
+ *                value and index types.
+ */
+#define GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE_PAIR(_macro)       \
+    template _macro(float, float);                             \
+    template _macro(double, double);                           \
+    template _macro(std::complex<float>, float);               \
+    template _macro(std::complex<double>, double);             \
+    template _macro(std::complex<float>, std::complex<float>); \
+    template _macro(std::complex<double>, std::complex<double>)
+
+
+/**
+ * Instantiates a template for each combined value and index type compiled by
+ * Ginkgo.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, which are replaced by the
+ *                value and index types.
+ */
+#define GKO_INSTANTIATE_FOR_EACH_COMBINED_VALUE_AND_INDEX_TYPE(_macro) \
+    template _macro(char, char);                                       \
+    template _macro(int32, int32);                                     \
+    template _macro(int64, int64);                                     \
+    template _macro(unsigned int, unsigned int);                       \
+    template _macro(unsigned long, unsigned long);                     \
+    template _macro(float, float);                                     \
+    template _macro(double, double);                                   \
+    template _macro(long double, long double);                         \
+    template _macro(std::complex<float>, std::complex<float>);         \
+    template _macro(std::complex<double>, std::complex<double>)
+
+/**
+ * Instantiates a template for each value and index type compiled by Ginkgo.
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, which are replaced by the
+ *                value and index types.
+ */
+#define GKO_INSTANTIATE_FOR_EACH_POD_TYPE(_macro) \
+    template _macro(float);                       \
+    template _macro(double);                      \
+    template _macro(std::complex<float>);         \
+    template _macro(std::complex<double>);        \
+    template _macro(size_type);                   \
+    template _macro(bool);                        \
+    template _macro(int32);                       \
+    template _macro(int64)
+
+
 /**
  * Instantiates a template for each normal type
  *
@@ -595,9 +749,50 @@ GKO_ATTRIBUTES constexpr bool operator!=(precision_reduction x,
 #define GKO_INSTANTIATE_FOR_EACH_TEMPLATE_TYPE(_macro) \
     GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(_macro);       \
     GKO_INSTANTIATE_FOR_EACH_INDEX_TYPE(_macro);       \
-    template _macro(size_type)
+    template _macro(gko::size_type)
 
 
+/**
+ * Value for an invalid signed index type.
+ */
+template <typename IndexType>
+inline constexpr GKO_ATTRIBUTES IndexType invalid_index()
+{
+    static_assert(std::is_signed<IndexType>::value,
+                  "IndexType needs to be signed");
+    return static_cast<IndexType>(-1);
+}
+
+
+namespace experimental {
+namespace distributed {
+
+
+/**
+ * Index type for enumerating processes in a distributed application
+ *
+ * Conforms to the MPI C interface of e.g. MPI rank or size
+ */
+using comm_index_type = int;
+
+
+/**
+ * Instantiates a template for each valid combination of local and global index
+ * type
+ *
+ * @param _macro  A macro which expands the template instantiation
+ *                (not including the leading `template` specifier).
+ *                Should take two arguments, where the first is replaced by the
+ *                local index type and the second by the global index type.
+ */
+#define GKO_INSTANTIATE_FOR_EACH_LOCAL_GLOBAL_INDEX_TYPE(_macro) \
+    template _macro(int32, int32);                               \
+    template _macro(int32, int64);                               \
+    template _macro(int64, int64)
+
+
+}  // namespace distributed
+}  // namespace experimental
 }  // namespace gko
 
 
