@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ namespace gko {
 
 /* Eliminate circular dependencies the hard way */
 template <typename ValueType>
-class Array;
+class array;
 class Executor;
 class LinOp;
 class LinOpFactory;
@@ -130,7 +130,7 @@ protected:                                                           \
 public:                                                              \
     template <size_type Event, typename... Params>                   \
     std::enable_if_t<Event == _id && (_id < event_count_max)> on(    \
-        Params &&... params) const                                   \
+        Params&&... params) const                                    \
     {                                                                \
         if (enabled_events_ & (mask_type{1} << _id)) {               \
             this->on_##_event_name(std::forward<Params>(params)...); \
@@ -145,8 +145,8 @@ public:                                                              \
      * @param exec  the executor used
      * @param num_bytes  the number of bytes to allocate
      */
-    GKO_LOGGER_REGISTER_EVENT(0, allocation_started, const Executor *exec,
-                              const size_type &num_bytes)
+    GKO_LOGGER_REGISTER_EVENT(0, allocation_started, const Executor* exec,
+                              const size_type& num_bytes)
 
     /**
      * Executor's allocation completed event.
@@ -155,9 +155,9 @@ public:                                                              \
      * @param num_bytes  the number of bytes allocated
      * @param location  the address at which the data was allocated
      */
-    GKO_LOGGER_REGISTER_EVENT(1, allocation_completed, const Executor *exec,
-                              const size_type &num_bytes,
-                              const uintptr &location)
+    GKO_LOGGER_REGISTER_EVENT(1, allocation_completed, const Executor* exec,
+                              const size_type& num_bytes,
+                              const uintptr& location)
 
     /**
      * Executor's free started event.
@@ -165,8 +165,8 @@ public:                                                              \
      * @param exec  the executor used
      * @param location  the address at which the data will be freed
      */
-    GKO_LOGGER_REGISTER_EVENT(2, free_started, const Executor *exec,
-                              const uintptr &location)
+    GKO_LOGGER_REGISTER_EVENT(2, free_started, const Executor* exec,
+                              const uintptr& location)
 
     /**
      * Executor's free completed event.
@@ -174,8 +174,8 @@ public:                                                              \
      * @param exec  the executor used
      * @param location  the address at which the data was freed
      */
-    GKO_LOGGER_REGISTER_EVENT(3, free_completed, const Executor *exec,
-                              const uintptr &location)
+    GKO_LOGGER_REGISTER_EVENT(3, free_completed, const Executor* exec,
+                              const uintptr& location)
 
     /**
      * Executor's copy started event.
@@ -186,9 +186,9 @@ public:                                                              \
      * @param loc_to  the address at which the data will be copied to
      * @param num_bytes  the number of bytes to be copied
      */
-    GKO_LOGGER_REGISTER_EVENT(4, copy_started, const Executor *exec_from,
-                              const Executor *exec_to, const uintptr &loc_from,
-                              const uintptr &loc_to, const size_type &num_bytes)
+    GKO_LOGGER_REGISTER_EVENT(4, copy_started, const Executor* exec_from,
+                              const Executor* exec_to, const uintptr& loc_from,
+                              const uintptr& loc_to, const size_type& num_bytes)
 
     /**
      * Executor's copy completed event.
@@ -199,9 +199,9 @@ public:                                                              \
      * @param loc_to  the address at which the data was copied to
      * @param num_bytes  the number of bytes copied
      */
-    GKO_LOGGER_REGISTER_EVENT(5, copy_completed, const Executor *exec_from,
-                              const Executor *exec_to, const uintptr &loc_from,
-                              const uintptr &loc_to, const size_type &num_bytes)
+    GKO_LOGGER_REGISTER_EVENT(5, copy_completed, const Executor* exec_from,
+                              const Executor* exec_to, const uintptr& loc_from,
+                              const uintptr& loc_to, const size_type& num_bytes)
 
     /**
      * Executor's operation launched event (method run).
@@ -209,8 +209,8 @@ public:                                                              \
      * @param exec  the executor used
      * @param op  the operation launched
      */
-    GKO_LOGGER_REGISTER_EVENT(6, operation_launched, const Executor *exec,
-                              const Operation *op)
+    GKO_LOGGER_REGISTER_EVENT(6, operation_launched, const Executor* exec,
+                              const Operation* op)
 
     /**
      * Executor's operation completed event (method run).
@@ -223,8 +223,8 @@ public:                                                              \
      * the loggers will do lightweight logging, and therefore this operation for
      * the GPU just notes that the Operation has been sent to the GPU.
      */
-    GKO_LOGGER_REGISTER_EVENT(7, operation_completed, const Executor *exec,
-                              const Operation *op)
+    GKO_LOGGER_REGISTER_EVENT(7, operation_completed, const Executor* exec,
+                              const Operation* op)
 
     /**
      * PolymorphicObject's create started event.
@@ -233,7 +233,7 @@ public:                                                              \
      * @param po  the PolymorphicObject to be created
      */
     GKO_LOGGER_REGISTER_EVENT(8, polymorphic_object_create_started,
-                              const Executor *exec, const PolymorphicObject *po)
+                              const Executor* exec, const PolymorphicObject* po)
 
     /**
      * PolymorphicObject's create completed event.
@@ -243,9 +243,9 @@ public:                                                              \
      * @param output  the PolymorphicObject which was created
      */
     GKO_LOGGER_REGISTER_EVENT(9, polymorphic_object_create_completed,
-                              const Executor *exec,
-                              const PolymorphicObject *input,
-                              const PolymorphicObject *output)
+                              const Executor* exec,
+                              const PolymorphicObject* input,
+                              const PolymorphicObject* output)
 
     /**
      * PolymorphicObject's copy started event.
@@ -255,9 +255,9 @@ public:                                                              \
      * @param output  the PolymorphicObject to be copied to
      */
     GKO_LOGGER_REGISTER_EVENT(10, polymorphic_object_copy_started,
-                              const Executor *exec,
-                              const PolymorphicObject *input,
-                              const PolymorphicObject *output)
+                              const Executor* exec,
+                              const PolymorphicObject* input,
+                              const PolymorphicObject* output)
 
     /**
      * PolymorphicObject's copy completed event.
@@ -267,9 +267,9 @@ public:                                                              \
      * @param output  the PolymorphicObject to be copied to
      */
     GKO_LOGGER_REGISTER_EVENT(11, polymorphic_object_copy_completed,
-                              const Executor *exec,
-                              const PolymorphicObject *input,
-                              const PolymorphicObject *output)
+                              const Executor* exec,
+                              const PolymorphicObject* input,
+                              const PolymorphicObject* output)
 
     /**
      * PolymorphicObject's deleted event.
@@ -278,7 +278,7 @@ public:                                                              \
      * @param po  the PolymorphicObject to be deleted
      */
     GKO_LOGGER_REGISTER_EVENT(12, polymorphic_object_deleted,
-                              const Executor *exec, const PolymorphicObject *po)
+                              const Executor* exec, const PolymorphicObject* po)
 
     /**
      * LinOp's apply started event.
@@ -287,8 +287,8 @@ public:                                                              \
      * @param b  the input vector(s)
      * @param x  the output vector(s)
      */
-    GKO_LOGGER_REGISTER_EVENT(13, linop_apply_started, const LinOp *A,
-                              const LinOp *b, const LinOp *x)
+    GKO_LOGGER_REGISTER_EVENT(13, linop_apply_started, const LinOp* A,
+                              const LinOp* b, const LinOp* x)
 
     /**
      * LinOp's apply completed event.
@@ -297,8 +297,8 @@ public:                                                              \
      * @param b  the input vector(s)
      * @param x  the output vector(s)
      */
-    GKO_LOGGER_REGISTER_EVENT(14, linop_apply_completed, const LinOp *A,
-                              const LinOp *b, const LinOp *x)
+    GKO_LOGGER_REGISTER_EVENT(14, linop_apply_completed, const LinOp* A,
+                              const LinOp* b, const LinOp* x)
 
     /**
      * LinOp's advanced apply started event.
@@ -309,9 +309,9 @@ public:                                                              \
      * @param beta  scaling of the input x
      * @param x  the output vector(s)
      */
-    GKO_LOGGER_REGISTER_EVENT(15, linop_advanced_apply_started, const LinOp *A,
-                              const LinOp *alpha, const LinOp *b,
-                              const LinOp *beta, const LinOp *x)
+    GKO_LOGGER_REGISTER_EVENT(15, linop_advanced_apply_started, const LinOp* A,
+                              const LinOp* alpha, const LinOp* b,
+                              const LinOp* beta, const LinOp* x)
 
     /**
      * LinOp's advanced apply completed event.
@@ -323,8 +323,8 @@ public:                                                              \
      * @param x  the output vector(s)
      */
     GKO_LOGGER_REGISTER_EVENT(16, linop_advanced_apply_completed,
-                              const LinOp *A, const LinOp *alpha,
-                              const LinOp *b, const LinOp *beta, const LinOp *x)
+                              const LinOp* A, const LinOp* alpha,
+                              const LinOp* b, const LinOp* beta, const LinOp* x)
 
     /**
      * LinOp Factory's generate started event.
@@ -334,7 +334,7 @@ public:                                                              \
      *               a system matrix)
      */
     GKO_LOGGER_REGISTER_EVENT(17, linop_factory_generate_started,
-                              const LinOpFactory *factory, const LinOp *input)
+                              const LinOpFactory* factory, const LinOp* input)
 
     /**
      * LinOp Factory's generate completed event.
@@ -345,8 +345,8 @@ public:                                                              \
      * @param output  the generated LinOp object
      */
     GKO_LOGGER_REGISTER_EVENT(18, linop_factory_generate_completed,
-                              const LinOpFactory *factory, const LinOp *input,
-                              const LinOp *output)
+                              const LinOpFactory* factory, const LinOp* input,
+                              const LinOp* output)
 
     /**
      * stop::Criterion's check started event.
@@ -360,11 +360,11 @@ public:                                                              \
      * @param set_finalized  whether this finalizes the iteration
      */
     GKO_LOGGER_REGISTER_EVENT(19, criterion_check_started,
-                              const stop::Criterion *criterion,
-                              const size_type &it, const LinOp *r,
-                              const LinOp *tau, const LinOp *x,
-                              const uint8 &stopping_id,
-                              const bool &set_finalized)
+                              const stop::Criterion* criterion,
+                              const size_type& it, const LinOp* r,
+                              const LinOp* tau, const LinOp* x,
+                              const uint8& stopping_id,
+                              const bool& set_finalized)
 
     /**
      * stop::Criterion's check completed event. Parameters are the Criterion,
@@ -387,11 +387,11 @@ public:                                                              \
      * parameter as below.
      */
     GKO_LOGGER_REGISTER_EVENT(
-        20, criterion_check_completed, const stop::Criterion *criterion,
-        const size_type &it, const LinOp *r, const LinOp *tau, const LinOp *x,
-        const uint8 &stopping_id, const bool &set_finalized,
-        const Array<stopping_status> *status, const bool &one_changed,
-        const bool &all_converged)
+        20, criterion_check_completed, const stop::Criterion* criterion,
+        const size_type& it, const LinOp* r, const LinOp* tau, const LinOp* x,
+        const uint8& stopping_id, const bool& set_finalized,
+        const array<stopping_status>* status, const bool& one_changed,
+        const bool& all_converged)
 protected:
     /**
      * stop::Criterion's check completed event. Parameters are the Criterion,
@@ -408,37 +408,33 @@ protected:
      * @param set_finalized  whether this finalizes the iteration
      * @param status  the stopping status of the right hand sides
      * @param one_changed  whether at least one right hand side converged or not
-     * @param all_converged  whether all right hand sides
+     * @param all_converged  whether all right hand sides are converged
      */
     virtual void on_criterion_check_completed(
-        const stop::Criterion *criterion, const size_type &it, const LinOp *r,
-        const LinOp *tau, const LinOp *implicit_tau_sq, const LinOp *x,
-        const uint8 &stopping_id, const bool &set_finalized,
-        const Array<stopping_status> *status, const bool &one_changed,
-        const bool &all_converged) const
+        const stop::Criterion* criterion, const size_type& it, const LinOp* r,
+        const LinOp* tau, const LinOp* implicit_tau_sq, const LinOp* x,
+        const uint8& stopping_id, const bool& set_finalized,
+        const array<stopping_status>* status, const bool& one_changed,
+        const bool& all_converged) const
     {
         this->on_criterion_check_completed(criterion, it, r, tau, x,
                                            stopping_id, set_finalized, status,
                                            one_changed, all_converged);
     }
 
-    /**
-     * Register the `iteration_complete` event which logs every completed
-     * iterations.
-     *
-     * @param it  the current iteration count
-     * @param r  the residual
-     * @param x  the solution vector (optional)
-     * @param tau  the residual norm (optional)
-     *
-     * @note The on_iteration_complete function that this macro declares is
-     * deprecated. Please use the one with the additional implicit_tau_sq
-     * parameter as below.
-     */
-    GKO_LOGGER_REGISTER_EVENT(21, iteration_complete, const LinOp *solver,
-                              const size_type &it, const LinOp *r,
-                              const LinOp *x = nullptr,
-                              const LinOp *tau = nullptr)
+public:
+    static constexpr size_type iteration_complete{21};
+    static constexpr mask_type iteration_complete_mask{mask_type{1} << 21};
+
+    template <size_type Event, typename... Params>
+    std::enable_if_t<Event == 21 && (21 < event_count_max)> on(
+        Params&&... params) const
+    {
+        if (enabled_events_ & (mask_type{1} << 21)) {
+            this->on_iteration_complete(std::forward<Params>(params)...);
+        }
+    }
+
 protected:
     /**
      * Register the `iteration_complete` event which logs every completed
@@ -448,17 +444,125 @@ protected:
      * @param r  the residual
      * @param x  the solution vector (optional)
      * @param tau  the residual norm (optional)
-     * @param implicit_tau_sq  the implicit residual norm squared (optional)
+     *
+     * @warning This on_iteration_complete function that this macro declares is
+     * deprecated. Please use the version with the stopping information.
      */
-    virtual void on_iteration_complete(const LinOp *solver, const size_type &it,
-                                       const LinOp *r, const LinOp *x,
-                                       const LinOp *tau,
-                                       const LinOp *implicit_tau_sq) const
+    [[deprecated(
+        "Please use the version with the additional stopping "
+        "information.")]] virtual void
+    on_iteration_complete(const LinOp* solver, const size_type& it,
+                          const LinOp* r, const LinOp* x = nullptr,
+                          const LinOp* tau = nullptr) const
+    {}
+
+    /**
+     * Register the `iteration_complete` event which logs every completed
+     * iterations.
+     *
+     * @param it  the current iteration count
+     * @param r  the residual
+     * @param x  the solution vector (optional)
+     * @param tau  the residual norm (optional)
+     * @param implicit_tau_sq  the implicit residual norm squared (optional)
+     *
+     * @warning This on_iteration_complete function that this macro declares is
+     * deprecated. Please use the version with the stopping information.
+     */
+    [[deprecated(
+        "Please use the version with the additional stopping "
+        "information.")]] virtual void
+    on_iteration_complete(const LinOp* solver, const size_type& it,
+                          const LinOp* r, const LinOp* x, const LinOp* tau,
+                          const LinOp* implicit_tau_sq) const
     {
+#if defined(__GNUC__) || defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable : 5211, 4973, 4974)
+#endif
         this->on_iteration_complete(solver, it, r, x, tau);
+#if defined(__GNUC__) || defined(__clang__)
+#pragma GCC diagnostic pop
+#endif
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+    }
+
+    /**
+     * Register the `iteration_complete` event which logs every completed
+     * iterations.
+     *
+     * @param solver  the solver executing the iteration
+     * @param b  the right-hand-side vector
+     * @param x  the solution vector
+     * @param it  the current iteration count
+     * @param r  the residual (optional)
+     * @param tau  the implicit residual norm squared (optional)
+     * @param implicit_tau_sq  the residual norm (optional)
+     * @param status  the stopping status of the right hand sides (optional)
+     * @param stopped  whether all right hand sides have stopped (invalid if
+     *                 status is not provided)
+     */
+    virtual void on_iteration_complete(const LinOp* solver, const LinOp* b,
+                                       const LinOp* x, const size_type& it,
+                                       const LinOp* r, const LinOp* tau,
+                                       const LinOp* implicit_tau_sq,
+                                       const array<stopping_status>* status,
+                                       bool stopped) const
+    {
+#if defined(__GNUC__) || defined(__clang__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif  // defined(__GNUC__) || defined(__clang__)
+#ifdef __NVCOMPILER
+#pragma diag_suppress 1445
+#endif  // __NVCOMPILER
+#ifdef _MSC_VER
+#pragma warning(push)
+#pragma warning(disable : 5211, 4973, 4974)
+#endif  // _MSC_VER
+        this->on_iteration_complete(solver, it, r, x, tau, implicit_tau_sq);
+#if defined(__GNUC__) || defined(__clang__)
+#pragma GCC diagnostic pop
+#endif  // defined(__GNUC__) || defined(__clang__)
+#ifdef __NVCOMPILER
+#pragma diag_warning 1445
+#endif  // __NVCOMPILER
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif  // _MSC_VER
     }
 
 public:
+    /**
+     * PolymorphicObject's move started event.
+     *
+     * @param exec  the executor used
+     * @param input  the PolymorphicObject to be move from
+     * @param output  the PolymorphicObject to be move into
+     */
+    GKO_LOGGER_REGISTER_EVENT(22, polymorphic_object_move_started,
+                              const Executor* exec,
+                              const PolymorphicObject* input,
+                              const PolymorphicObject* output)
+
+    /**
+     * PolymorphicObject's move completed event.
+     *
+     * @param exec  the executor used
+     * @param input  the PolymorphicObject to be move from
+     * @param output  the PolymorphicObject to be move into
+     */
+    GKO_LOGGER_REGISTER_EVENT(23, polymorphic_object_move_completed,
+                              const Executor* exec,
+                              const PolymorphicObject* input,
+                              const PolymorphicObject* output)
+
 #undef GKO_LOGGER_REGISTER_EVENT
 
     /**
@@ -483,6 +587,8 @@ public:
         polymorphic_object_create_completed_mask |
         polymorphic_object_copy_started_mask |
         polymorphic_object_copy_completed_mask |
+        polymorphic_object_move_started_mask |
+        polymorphic_object_move_completed_mask |
         polymorphic_object_deleted_mask;
 
     /**
@@ -505,6 +611,14 @@ public:
     static constexpr mask_type criterion_events_mask =
         criterion_check_started_mask | criterion_check_completed_mask;
 
+    /**
+     * Returns true if this logger, when attached to an Executor, needs to be
+     * forwarded all events from objects on this executor.
+     */
+    virtual bool needs_propagation() const { return false; }
+
+    virtual ~Logger() = default;
+
 protected:
     /**
      * Constructor for a Logger object.
@@ -520,13 +634,31 @@ protected:
      *                           logs every event except linop's apply started
      *                           event.
      */
-    explicit Logger(std::shared_ptr<const gko::Executor> exec,
-                    const mask_type &enabled_events = all_events_mask)
-        : exec_{exec}, enabled_events_{enabled_events}
+    [[deprecated("use single-parameter constructor")]] explicit Logger(
+        std::shared_ptr<const gko::Executor> exec,
+        const mask_type& enabled_events = all_events_mask)
+        : Logger{enabled_events}
+    {}
+
+    /**
+     * Constructor for a Logger object.
+     *
+     * @param enabled_events  the events enabled for this Logger. These can be
+     *                        of the following form:
+     *                        1. `all_event_mask` which logs every event;
+     *                        2. an OR combination of masks, e.g.
+     *                           `iteration_complete_mask|linop_apply_started_mask`
+     *                           which activates both of these events;
+     *                        3. all events with exclusion through XOR, e.g.
+     *                           `all_event_mask^linop_apply_started_mask` which
+     *                           logs every event except linop's apply started
+     *                           event.
+     */
+    explicit Logger(const mask_type& enabled_events = all_events_mask)
+        : enabled_events_{enabled_events}
     {}
 
 private:
-    std::shared_ptr<const Executor> exec_;
     mask_type enabled_events_;
 };
 
@@ -556,14 +688,19 @@ public:
      *       Thus, two loggers constructed in the same way are not considered
      *       equal.
      */
-    virtual void remove_logger(const Logger *logger) = 0;
+    virtual void remove_logger(const Logger* logger) = 0;
+
+    void remove_logger(ptr_param<const Logger> logger)
+    {
+        remove_logger(logger.get());
+    }
 
     /**
      * Returns the vector containing all loggers registered at this object.
      *
      * @return the vector containing all registered loggers.
      */
-    virtual const std::vector<std::shared_ptr<const Logger>> &get_loggers()
+    virtual const std::vector<std::shared_ptr<const Logger>>& get_loggers()
         const = 0;
 
     /** Remove all loggers registered at this object. */
@@ -591,11 +728,11 @@ public:
         loggers_.push_back(logger);
     }
 
-    void remove_logger(const Logger *logger) override
+    void remove_logger(const Logger* logger) override
     {
         auto idx =
             find_if(begin(loggers_), end(loggers_),
-                    [&logger](const auto &l) { return lend(l) == logger; });
+                    [&logger](const auto& l) { return l.get() == logger; });
         if (idx != end(loggers_)) {
             loggers_.erase(idx);
         } else {
@@ -604,7 +741,12 @@ public:
         }
     }
 
-    const std::vector<std::shared_ptr<const Logger>> &get_loggers()
+    void remove_logger(ptr_param<const Logger> logger)
+    {
+        remove_logger(logger.get());
+    }
+
+    const std::vector<std::shared_ptr<const Logger>>& get_loggers()
         const override
     {
         return loggers_;
@@ -612,11 +754,49 @@ public:
 
     void clear_loggers() override { loggers_.clear(); }
 
+private:
+    /**
+     * @internal
+     * This struct is used to differentiate between objects that have an
+     * associated executor (PolymorphicObject) and ones that don't (Executor).
+     * For the ones with executor, it handles the event propagation via template
+     * specialization/SFINAE.
+     */
+    template <size_type Event, typename ConcreteLoggableT, typename = void>
+    struct propagate_log_helper {
+        template <typename... Args>
+        static void propagate_log(const ConcreteLoggableT*, Args&&...)
+        {}
+    };
+
+    template <size_type Event, typename ConcreteLoggableT>
+    struct propagate_log_helper<
+        Event, ConcreteLoggableT,
+        xstd::void_t<
+            decltype(std::declval<ConcreteLoggableT>().get_executor())>> {
+        template <typename... Args>
+        static void propagate_log(const ConcreteLoggableT* loggable,
+                                  Args&&... args)
+        {
+            const auto exec = loggable->get_executor();
+            if (exec->should_propagate_log()) {
+                for (auto& logger : exec->get_loggers()) {
+                    if (logger->needs_propagation()) {
+                        logger->template on<Event>(std::forward<Args>(args)...);
+                    }
+                }
+            }
+        }
+    };
+
 protected:
     template <size_type Event, typename... Params>
-    void log(Params &&... params) const
+    void log(Params&&... params) const
     {
-        for (auto &logger : loggers_) {
+        propagate_log_helper<Event, ConcreteLoggable>::propagate_log(
+            static_cast<const ConcreteLoggable*>(this),
+            std::forward<Params>(params)...);
+        for (auto& logger : loggers_) {
             logger->template on<Event>(std::forward<Params>(params)...);
         }
     }
