@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -58,31 +58,33 @@ namespace log {
  */
 struct iteration_complete_data {
     std::unique_ptr<const LinOp> solver;
+    std::unique_ptr<const LinOp> right_hand_side;
+    std::unique_ptr<const LinOp> solution;
     const size_type num_iterations;
     std::unique_ptr<const LinOp> residual;
-    std::unique_ptr<const LinOp> solution;
     std::unique_ptr<const LinOp> residual_norm;
     std::unique_ptr<const LinOp> implicit_sq_residual_norm;
+    array<stopping_status> status;
+    bool all_stopped;
 
-    iteration_complete_data(const LinOp *solver, const size_type num_iterations,
-                            const LinOp *residual = nullptr,
-                            const LinOp *solution = nullptr,
-                            const LinOp *residual_norm = nullptr,
-                            const LinOp *implicit_sq_residual_norm = nullptr)
-        : solver{nullptr},
-          num_iterations{num_iterations},
-          residual{nullptr},
-          solution{nullptr},
-          residual_norm{nullptr},
-          implicit_sq_residual_norm{nullptr}
+    iteration_complete_data(const LinOp* solver, const LinOp* right_hand_side,
+                            const LinOp* solution,
+                            const size_type num_iterations,
+                            const LinOp* residual = nullptr,
+                            const LinOp* residual_norm = nullptr,
+                            const LinOp* implicit_sq_residual_norm = nullptr,
+                            const gko::array<stopping_status>* status = nullptr,
+                            bool all_stopped = false)
+        : num_iterations{num_iterations}, all_stopped(all_stopped)
     {
         this->solver = solver->clone();
+        this->solution = solution->clone();
+        if (right_hand_side != nullptr) {
+            this->right_hand_side = right_hand_side->clone();
+        }
         if (residual != nullptr) {
             this->residual = residual->clone();
         }
-        if (solution != nullptr) {
-            this->solution = solution->clone();
-        }
         if (residual_norm != nullptr) {
             this->residual_norm = residual_norm->clone();
         }
@@ -90,6 +92,9 @@ struct iteration_complete_data {
             this->implicit_sq_residual_norm =
                 implicit_sq_residual_norm->clone();
         }
+        if (status != nullptr) {
+            this->status = *status;
+        }
     }
 };
 
@@ -98,7 +103,7 @@ struct iteration_complete_data {
  * Struct representing Executor related data
  */
 struct executor_data {
-    const Executor *exec;
+    const Executor* exec;
     const size_type num_bytes;
     const uintptr location;
 };
@@ -108,8 +113,8 @@ struct executor_data {
  * Struct representing Operator related data
  */
 struct operation_data {
-    const Executor *exec;
-    const Operation *operation;
+    const Executor* exec;
+    const Operation* operation;
 };
 
 
@@ -117,13 +122,13 @@ struct operation_data {
  * Struct representing PolymorphicObject related data
  */
 struct polymorphic_object_data {
-    const Executor *exec;
+    const Executor* exec;
     std::unique_ptr<const PolymorphicObject> input;
     std::unique_ptr<const PolymorphicObject> output;  // optional
 
-    polymorphic_object_data(const Executor *exec,
-                            const PolymorphicObject *input,
-                            const PolymorphicObject *output = nullptr)
+    polymorphic_object_data(const Executor* exec,
+                            const PolymorphicObject* input,
+                            const PolymorphicObject* output = nullptr)
         : exec{exec}
     {
         this->input = input->clone();
@@ -144,8 +149,8 @@ struct linop_data {
     std::unique_ptr<const LinOp> beta;
     std::unique_ptr<const LinOp> x;
 
-    linop_data(const LinOp *A, const LinOp *alpha, const LinOp *b,
-               const LinOp *beta, const LinOp *x)
+    linop_data(const LinOp* A, const LinOp* alpha, const LinOp* b,
+               const LinOp* beta, const LinOp* x)
     {
         this->A = A->clone();
         if (alpha != nullptr) {
@@ -164,12 +169,12 @@ struct linop_data {
  * Struct representing LinOp factory related data
  */
 struct linop_factory_data {
-    const LinOpFactory *factory;
+    const LinOpFactory* factory;
     std::unique_ptr<const LinOp> input;
     std::unique_ptr<const LinOp> output;
 
-    linop_factory_data(const LinOpFactory *factory, const LinOp *input,
-                       const LinOp *output)
+    linop_factory_data(const LinOpFactory* factory, const LinOp* input,
+                       const LinOp* output)
         : factory{factory}
     {
         this->input = input->clone();
@@ -184,22 +189,22 @@ struct linop_factory_data {
  * Struct representing Criterion related data
  */
 struct criterion_data {
-    const stop::Criterion *criterion;
+    const stop::Criterion* criterion;
     const size_type num_iterations;
     std::unique_ptr<const LinOp> residual;
     std::unique_ptr<const LinOp> residual_norm;
     std::unique_ptr<const LinOp> solution;
     const uint8 stopping_id;
     const bool set_finalized;
-    const Array<stopping_status> *status;
+    const array<stopping_status>* status;
     const bool oneChanged;
     const bool converged;
 
-    criterion_data(const stop::Criterion *criterion,
-                   const size_type &num_iterations, const LinOp *residual,
-                   const LinOp *residual_norm, const LinOp *solution,
+    criterion_data(const stop::Criterion* criterion,
+                   const size_type& num_iterations, const LinOp* residual,
+                   const LinOp* residual_norm, const LinOp* solution,
                    const uint8 stopping_id, const bool set_finalized,
-                   const Array<stopping_status> *status = nullptr,
+                   const array<stopping_status>* status = nullptr,
                    const bool oneChanged = false, const bool converged = false)
         : criterion{criterion},
           num_iterations{num_iterations},
@@ -262,6 +267,10 @@ public:
             polymorphic_object_copy_started;
         std::deque<std::unique_ptr<polymorphic_object_data>>
             polymorphic_object_copy_completed;
+        std::deque<std::unique_ptr<polymorphic_object_data>>
+            polymorphic_object_move_started;
+        std::deque<std::unique_ptr<polymorphic_object_data>>
+            polymorphic_object_move_completed;
         std::deque<std::unique_ptr<polymorphic_object_data>>
             polymorphic_object_deleted;
 
@@ -282,113 +291,158 @@ public:
     };
 
     /* Executor events */
-    void on_allocation_started(const Executor *exec,
-                               const size_type &num_bytes) const override;
+    void on_allocation_started(const Executor* exec,
+                               const size_type& num_bytes) const override;
 
-    void on_allocation_completed(const Executor *exec,
-                                 const size_type &num_bytes,
-                                 const uintptr &location) const override;
+    void on_allocation_completed(const Executor* exec,
+                                 const size_type& num_bytes,
+                                 const uintptr& location) const override;
 
-    void on_free_started(const Executor *exec,
-                         const uintptr &location) const override;
+    void on_free_started(const Executor* exec,
+                         const uintptr& location) const override;
 
-    void on_free_completed(const Executor *exec,
-                           const uintptr &location) const override;
+    void on_free_completed(const Executor* exec,
+                           const uintptr& location) const override;
 
-    void on_copy_started(const Executor *from, const Executor *to,
-                         const uintptr &location_from,
-                         const uintptr &location_to,
-                         const size_type &num_bytes) const override;
+    void on_copy_started(const Executor* from, const Executor* to,
+                         const uintptr& location_from,
+                         const uintptr& location_to,
+                         const size_type& num_bytes) const override;
 
-    void on_copy_completed(const Executor *from, const Executor *to,
-                           const uintptr &location_from,
-                           const uintptr &location_to,
-                           const size_type &num_bytes) const override;
+    void on_copy_completed(const Executor* from, const Executor* to,
+                           const uintptr& location_from,
+                           const uintptr& location_to,
+                           const size_type& num_bytes) const override;
 
     /* Operation events */
-    void on_operation_launched(const Executor *exec,
-                               const Operation *operation) const override;
+    void on_operation_launched(const Executor* exec,
+                               const Operation* operation) const override;
 
-    void on_operation_completed(const Executor *exec,
-                                const Operation *operation) const override;
+    void on_operation_completed(const Executor* exec,
+                                const Operation* operation) const override;
 
     /* PolymorphicObject events */
     void on_polymorphic_object_create_started(
-        const Executor *exec, const PolymorphicObject *po) const override;
+        const Executor* exec, const PolymorphicObject* po) const override;
 
     void on_polymorphic_object_create_completed(
-        const Executor *exec, const PolymorphicObject *input,
-        const PolymorphicObject *output) const override;
+        const Executor* exec, const PolymorphicObject* input,
+        const PolymorphicObject* output) const override;
 
     void on_polymorphic_object_copy_started(
-        const Executor *exec, const PolymorphicObject *from,
-        const PolymorphicObject *to) const override;
+        const Executor* exec, const PolymorphicObject* from,
+        const PolymorphicObject* to) const override;
 
     void on_polymorphic_object_copy_completed(
-        const Executor *exec, const PolymorphicObject *from,
-        const PolymorphicObject *to) const override;
+        const Executor* exec, const PolymorphicObject* from,
+        const PolymorphicObject* to) const override;
+
+    void on_polymorphic_object_move_started(
+        const Executor* exec, const PolymorphicObject* from,
+        const PolymorphicObject* to) const override;
+
+    void on_polymorphic_object_move_completed(
+        const Executor* exec, const PolymorphicObject* from,
+        const PolymorphicObject* to) const override;
 
     void on_polymorphic_object_deleted(
-        const Executor *exec, const PolymorphicObject *po) const override;
+        const Executor* exec, const PolymorphicObject* po) const override;
 
     /* LinOp events */
-    void on_linop_apply_started(const LinOp *A, const LinOp *b,
-                                const LinOp *x) const override;
+    void on_linop_apply_started(const LinOp* A, const LinOp* b,
+                                const LinOp* x) const override;
 
-    void on_linop_apply_completed(const LinOp *A, const LinOp *b,
-                                  const LinOp *x) const override;
+    void on_linop_apply_completed(const LinOp* A, const LinOp* b,
+                                  const LinOp* x) const override;
 
-    void on_linop_advanced_apply_started(const LinOp *A, const LinOp *alpha,
-                                         const LinOp *b, const LinOp *beta,
-                                         const LinOp *x) const override;
+    void on_linop_advanced_apply_started(const LinOp* A, const LinOp* alpha,
+                                         const LinOp* b, const LinOp* beta,
+                                         const LinOp* x) const override;
 
-    void on_linop_advanced_apply_completed(const LinOp *A, const LinOp *alpha,
-                                           const LinOp *b, const LinOp *beta,
-                                           const LinOp *x) const override;
+    void on_linop_advanced_apply_completed(const LinOp* A, const LinOp* alpha,
+                                           const LinOp* b, const LinOp* beta,
+                                           const LinOp* x) const override;
 
     /* LinOpFactory events */
-    void on_linop_factory_generate_started(const LinOpFactory *factory,
-                                           const LinOp *input) const override;
+    void on_linop_factory_generate_started(const LinOpFactory* factory,
+                                           const LinOp* input) const override;
 
     void on_linop_factory_generate_completed(
-        const LinOpFactory *factory, const LinOp *input,
-        const LinOp *output) const override;
+        const LinOpFactory* factory, const LinOp* input,
+        const LinOp* output) const override;
 
     /* Criterion events */
-    void on_criterion_check_started(const stop::Criterion *criterion,
-                                    const size_type &num_iterations,
-                                    const LinOp *residual,
-                                    const LinOp *residual_norm,
-                                    const LinOp *solution,
-                                    const uint8 &stopping_id,
-                                    const bool &set_finalized) const override;
+    void on_criterion_check_started(const stop::Criterion* criterion,
+                                    const size_type& num_iterations,
+                                    const LinOp* residual,
+                                    const LinOp* residual_norm,
+                                    const LinOp* solution,
+                                    const uint8& stopping_id,
+                                    const bool& set_finalized) const override;
 
     void on_criterion_check_completed(
-        const stop::Criterion *criterion, const size_type &num_iterations,
-        const LinOp *residual, const LinOp *residual_norm,
-        const LinOp *implicit_residual_norm_sq, const LinOp *solution,
-        const uint8 &stopping_id, const bool &set_finalized,
-        const Array<stopping_status> *status, const bool &one_changed,
-        const bool &all_converged) const override;
+        const stop::Criterion* criterion, const size_type& num_iterations,
+        const LinOp* residual, const LinOp* residual_norm,
+        const LinOp* implicit_residual_norm_sq, const LinOp* solution,
+        const uint8& stopping_id, const bool& set_finalized,
+        const array<stopping_status>* status, const bool& one_changed,
+        const bool& all_converged) const override;
 
     void on_criterion_check_completed(
-        const stop::Criterion *criterion, const size_type &num_iterations,
-        const LinOp *residual, const LinOp *residual_norm,
-        const LinOp *solution, const uint8 &stopping_id,
-        const bool &set_finalized, const Array<stopping_status> *status,
-        const bool &one_changed, const bool &all_converged) const override;
+        const stop::Criterion* criterion, const size_type& num_iterations,
+        const LinOp* residual, const LinOp* residual_norm,
+        const LinOp* solution, const uint8& stopping_id,
+        const bool& set_finalized, const array<stopping_status>* status,
+        const bool& one_changed, const bool& all_converged) const override;
 
     /* Internal solver events */
     void on_iteration_complete(
-        const LinOp *solver, const size_type &num_iterations,
-        const LinOp *residual, const LinOp *solution = nullptr,
-        const LinOp *residual_norm = nullptr) const override;
+        const LinOp* solver, const LinOp* right_hand_side, const LinOp* x,
+        const size_type& num_iterations, const LinOp* residual,
+        const LinOp* residual_norm, const LinOp* implicit_resnorm_sq,
+        const array<stopping_status>* status, bool stopped) const override;
 
-    void on_iteration_complete(
-        const LinOp *solver, const size_type &num_iterations,
-        const LinOp *residual, const LinOp *solution,
-        const LinOp *residual_norm,
-        const LinOp *implicit_sq_residual_norm) const override;
+    [[deprecated(
+        "Please use the version with the additional stopping "
+        "information.")]] void
+    on_iteration_complete(const LinOp* solver, const size_type& num_iterations,
+                          const LinOp* residual, const LinOp* solution,
+                          const LinOp* residual_norm) const override;
+
+    [[deprecated(
+        "Please use the version with the additional stopping "
+        "information.")]] void
+    on_iteration_complete(
+        const LinOp* solver, const size_type& num_iterations,
+        const LinOp* residual, const LinOp* solution,
+        const LinOp* residual_norm,
+        const LinOp* implicit_sq_residual_norm) const override;
+
+    /**
+     * Creates a Record logger. This dynamically allocates the memory,
+     * constructs the object and returns an std::unique_ptr to this object.
+     *
+     * @param exec  the executor
+     * @param enabled_events  the events enabled for this logger. By default all
+     *                        events.
+     * @param max_storage  the size of storage (i.e. history) wanted by the
+     *                     user. By default 0 is used, which means unlimited
+     *                     storage. It is advised to control this to reduce
+     *                     memory overhead of this logger.
+     *
+     * @return an std::unique_ptr to the the constructed object
+     *
+     * @internal here I cannot use EnableCreateMethod due to complex circular
+     * dependencies. At the same time, this method is short enough that it
+     * shouldn't be a problem.
+     */
+    [[deprecated("use two-parameter create")]] static std::unique_ptr<Record>
+    create(std::shared_ptr<const Executor> exec,
+           const mask_type& enabled_events = Logger::all_events_mask,
+           size_type max_storage = 1)
+    {
+        return std::unique_ptr<Record>(new Record(enabled_events, max_storage));
+    }
 
     /**
      * Creates a Record logger. This dynamically allocates the memory,
@@ -409,12 +463,10 @@ public:
      * shouldn't be a problem.
      */
     static std::unique_ptr<Record> create(
-        std::shared_ptr<const Executor> exec,
-        const mask_type &enabled_events = Logger::all_events_mask,
+        const mask_type& enabled_events = Logger::all_events_mask,
         size_type max_storage = 1)
     {
-        return std::unique_ptr<Record>(
-            new Record(exec, enabled_events, max_storage));
+        return std::unique_ptr<Record>(new Record(enabled_events, max_storage));
     }
 
     /**
@@ -422,12 +474,12 @@ public:
      *
      * @return the logged data
      */
-    const logged_data &get() const noexcept { return data_; }
+    const logged_data& get() const noexcept { return data_; }
 
     /**
      * @copydoc ::get()
      */
-    logged_data &get() noexcept { return data_; }
+    logged_data& get() noexcept { return data_; }
 
 protected:
     /**
@@ -441,10 +493,26 @@ protected:
      *                     storage. It is advised to control this to reduce
      *                     memory overhead of this logger.
      */
-    explicit Record(std::shared_ptr<const gko::Executor> exec,
-                    const mask_type &enabled_events = Logger::all_events_mask,
+    [[deprecated("use two-parameter constructor")]] explicit Record(
+        std::shared_ptr<const gko::Executor> exec,
+        const mask_type& enabled_events = Logger::all_events_mask,
+        size_type max_storage = 0)
+        : Record(enabled_events, max_storage)
+    {}
+
+    /**
+     * Creates a Record logger.
+     *
+     * @param enabled_events  the events enabled for this logger. By default all
+     *                        events.
+     * @param max_storage  the size of storage (i.e. history) wanted by the
+     *                     user. By default 0 is used, which means unlimited
+     *                     storage. It is advised to control this to reduce
+     *                     memory overhead of this logger.
+     */
+    explicit Record(const mask_type& enabled_events = Logger::all_events_mask,
                     size_type max_storage = 0)
-        : Logger(exec, enabled_events), max_storage_{max_storage}
+        : Logger(enabled_events), max_storage_{max_storage}
     {}
 
     /**
@@ -456,7 +524,7 @@ protected:
      * @param object  the object to append
      */
     template <typename deque_type>
-    void append_deque(std::deque<deque_type> &deque, deque_type object) const
+    void append_deque(std::deque<deque_type>& deque, deque_type object) const
     {
         if (this->max_storage_ && deque.size() == this->max_storage_) {
             deque.pop_front();
