@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -35,6 +35,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 
 #include <ginkgo/core/base/array.hpp>
+#include <ginkgo/core/base/index_set.hpp>
 #include <ginkgo/core/base/lin_op.hpp>
 #include <ginkgo/core/base/math.hpp>
 
@@ -46,6 +47,9 @@ namespace matrix {
 template <typename ValueType>
 class Dense;
 
+template <typename ValueType>
+class Diagonal;
+
 template <typename ValueType, typename IndexType>
 class Coo;
 
@@ -64,6 +68,9 @@ class SparsityCsr;
 template <typename ValueType, typename IndexType>
 class Csr;
 
+template <typename ValueType, typename IndexType>
+class Fbcsr;
+
 template <typename ValueType, typename IndexType>
 class CsrBuilder;
 
@@ -72,7 +79,7 @@ namespace detail {
 
 
 template <typename ValueType = default_precision, typename IndexType = int32>
-void strategy_rebuild_helper(Csr<ValueType, IndexType> *result);
+void strategy_rebuild_helper(Csr<ValueType, IndexType>* result);
 
 
 }  // namespace detail
@@ -123,6 +130,7 @@ class Csr : public EnableLinOp<Csr<ValueType, IndexType>>,
             public ConvertibleTo<Dense<ValueType>>,
             public ConvertibleTo<Coo<ValueType, IndexType>>,
             public ConvertibleTo<Ell<ValueType, IndexType>>,
+            public ConvertibleTo<Fbcsr<ValueType, IndexType>>,
             public ConvertibleTo<Hybrid<ValueType, IndexType>>,
             public ConvertibleTo<Sellp<ValueType, IndexType>>,
             public ConvertibleTo<SparsityCsr<ValueType, IndexType>>,
@@ -132,25 +140,47 @@ class Csr : public EnableLinOp<Csr<ValueType, IndexType>>,
             public Transposable,
             public Permutable<IndexType>,
             public EnableAbsoluteComputation<
-                remove_complex<Csr<ValueType, IndexType>>> {
+                remove_complex<Csr<ValueType, IndexType>>>,
+            public ScaledIdentityAddable {
     friend class EnableCreateMethod<Csr>;
     friend class EnablePolymorphicObject<Csr, LinOp>;
     friend class Coo<ValueType, IndexType>;
     friend class Dense<ValueType>;
+    friend class Diagonal<ValueType>;
     friend class Ell<ValueType, IndexType>;
     friend class Hybrid<ValueType, IndexType>;
     friend class Sellp<ValueType, IndexType>;
     friend class SparsityCsr<ValueType, IndexType>;
+    friend class Fbcsr<ValueType, IndexType>;
     friend class CsrBuilder<ValueType, IndexType>;
     friend class Csr<to_complex<ValueType>, IndexType>;
 
 public:
+    using EnableLinOp<Csr>::convert_to;
+    using EnableLinOp<Csr>::move_to;
+    using ConvertibleTo<Csr<next_precision<ValueType>, IndexType>>::convert_to;
+    using ConvertibleTo<Csr<next_precision<ValueType>, IndexType>>::move_to;
+    using ConvertibleTo<Dense<ValueType>>::convert_to;
+    using ConvertibleTo<Dense<ValueType>>::move_to;
+    using ConvertibleTo<Coo<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<Coo<ValueType, IndexType>>::move_to;
+    using ConvertibleTo<Ell<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<Ell<ValueType, IndexType>>::move_to;
+    using ConvertibleTo<Fbcsr<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<Fbcsr<ValueType, IndexType>>::move_to;
+    using ConvertibleTo<Hybrid<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<Hybrid<ValueType, IndexType>>::move_to;
+    using ConvertibleTo<Sellp<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<Sellp<ValueType, IndexType>>::move_to;
+    using ConvertibleTo<SparsityCsr<ValueType, IndexType>>::convert_to;
+    using ConvertibleTo<SparsityCsr<ValueType, IndexType>>::move_to;
     using ReadableFromMatrixData<ValueType, IndexType>::read;
 
     using value_type = ValueType;
     using index_type = IndexType;
     using transposed_type = Csr<ValueType, IndexType>;
     using mat_data = matrix_data<ValueType, IndexType>;
+    using device_mat_data = device_matrix_data<ValueType, IndexType>;
     using absolute_type = remove_complex<Csr>;
 
     class automatical;
@@ -172,6 +202,8 @@ public:
          */
         strategy_type(std::string name) : name_(name) {}
 
+        virtual ~strategy_type() = default;
+
         /**
          * Returns the name of strategy
          *
@@ -185,8 +217,8 @@ public:
          * @param mtx_row_ptrs  the row pointers of the matrix
          * @param mtx_srow  the srow of the matrix
          */
-        virtual void process(const Array<index_type> &mtx_row_ptrs,
-                             Array<index_type> *mtx_srow) = 0;
+        virtual void process(const array<index_type>& mtx_row_ptrs,
+                             array<index_type>* mtx_srow) = 0;
 
         /**
          * Computes the srow size according to the number of nonzeros.
@@ -223,14 +255,14 @@ public:
          */
         classical() : strategy_type("classical"), max_length_per_row_(0) {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {
             auto host_mtx_exec = mtx_row_ptrs.get_executor()->get_master();
-            Array<index_type> row_ptrs_host(host_mtx_exec);
+            array<index_type> row_ptrs_host(host_mtx_exec);
             const bool is_mtx_on_host{host_mtx_exec ==
                                       mtx_row_ptrs.get_executor()};
-            const index_type *row_ptrs{};
+            const index_type* row_ptrs{};
             if (is_mtx_on_host) {
                 row_ptrs = mtx_row_ptrs.get_const_data();
             } else {
@@ -273,8 +305,8 @@ public:
          */
         merge_path() : strategy_type("merge_path") {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {}
 
         int64_t clac_size(const int64_t nnz) override { return 0; }
@@ -298,8 +330,8 @@ public:
          */
         cusparse() : strategy_type("cusparse") {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {}
 
         int64_t clac_size(const int64_t nnz) override { return 0; }
@@ -322,8 +354,8 @@ public:
          */
         sparselib() : strategy_type("sparselib") {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {}
 
         int64_t clac_size(const int64_t nnz) override { return 0; }
@@ -341,8 +373,11 @@ public:
     public:
         /**
          * Creates a load_balance strategy.
+         *
+         * @warning this is deprecated! Please rely on the new automatic
+         *          strategy instantiation or use one of the other constructors.
          */
-        load_balance()
+        [[deprecated]] load_balance()
             : load_balance(std::move(
                   gko::CudaExecutor::create(0, gko::OmpExecutor::create())))
         {}
@@ -370,12 +405,10 @@ public:
          *
          * @param exec the DPCPP executor
          *
-         * @note TODO: porting - we hardcode the subgroup size is 16 and the
-         *             number of threads in a SIMD unit is 7
+         * @note TODO: porting - we hardcode the subgroup size is 32
          */
         load_balance(std::shared_ptr<const DpcppExecutor> exec)
-            : load_balance(exec->get_num_computing_units() * 7, 16, false,
-                           "intel")
+            : load_balance(exec->get_num_subgroups(), 32, false, "intel")
         {}
 
         /**
@@ -399,8 +432,8 @@ public:
               strategy_name_(strategy_name)
         {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {
             auto nwarps = mtx_srow->get_num_elems();
 
@@ -411,10 +444,10 @@ public:
                                            mtx_srow->get_executor()};
                 const bool is_mtx_on_host{host_mtx_exec ==
                                           mtx_row_ptrs.get_executor()};
-                Array<index_type> row_ptrs_host(host_mtx_exec);
-                Array<index_type> srow_host(host_srow_exec);
-                const index_type *row_ptrs{};
-                index_type *srow{};
+                array<index_type> row_ptrs_host(host_mtx_exec);
+                array<index_type> srow_host(host_srow_exec);
+                const index_type* row_ptrs{};
+                index_type* srow{};
                 if (is_srow_on_host) {
                     srow = mtx_srow->get_data();
                 } else {
@@ -432,10 +465,12 @@ public:
                 }
                 const auto num_rows = mtx_row_ptrs.get_num_elems() - 1;
                 const auto num_elems = row_ptrs[num_rows];
+                const auto bucket_divider =
+                    num_elems > 0 ? ceildiv(num_elems, warp_size_) : 1;
                 for (size_type i = 0; i < num_rows; i++) {
                     auto bucket =
                         ceildiv((ceildiv(row_ptrs[i + 1], warp_size_) * nwarps),
-                                ceildiv(num_elems, warp_size_));
+                                bucket_divider);
                     if (bucket < nwarps) {
                         srow[bucket]++;
                     }
@@ -516,12 +551,21 @@ public:
         /* Use imbalance strategy when the matrix has more more than 1e8 on AMD
          * hardware */
         const index_type amd_nnz_limit{static_cast<index_type>(1e8)};
+        /* Use imbalance strategy when the maximum number of nonzero per row is
+         * more than 25600 on Intel hardware */
+        const index_type intel_row_len_limit = 25600;
+        /* Use imbalance strategy when the matrix has more more than 3e8 on
+         * Intel hardware */
+        const index_type intel_nnz_limit{static_cast<index_type>(3e8)};
 
     public:
         /**
          * Creates an automatical strategy.
+         *
+         * @warning this is deprecated! Please rely on the new automatic
+         *          strategy instantiation or use one of the other constructors.
          */
-        automatical()
+        [[deprecated]] automatical()
             : automatical(std::move(
                   gko::CudaExecutor::create(0, gko::OmpExecutor::create())))
         {}
@@ -549,12 +593,10 @@ public:
          *
          * @param exec the Dpcpp executor
          *
-         * @note TODO: porting - we hardcode the subgroup size is 16 and the
-         *             number of threads in a SIMD unit is 7
+         * @note TODO: porting - we hardcode the subgroup size is 32
          */
         automatical(std::shared_ptr<const DpcppExecutor> exec)
-            : automatical(exec->get_num_computing_units() * 7, 16, false,
-                          "intel")
+            : automatical(exec->get_num_subgroups(), 32, false, "intel")
         {}
 
         /**
@@ -579,8 +621,8 @@ public:
               max_length_per_row_(0)
         {}
 
-        void process(const Array<index_type> &mtx_row_ptrs,
-                     Array<index_type> *mtx_srow) override
+        void process(const array<index_type>& mtx_row_ptrs,
+                     array<index_type>* mtx_srow) override
         {
             // if the number of stored elements is larger than <nnz_limit> or
             // the maximum number of stored elements per row is larger than
@@ -588,12 +630,8 @@ public:
             index_type nnz_limit = nvidia_nnz_limit;
             index_type row_len_limit = nvidia_row_len_limit;
             if (strategy_name_ == "intel") {
-                /* Use imbalance strategy when the maximum number of nonzero per
-                 * row is more than 25600 on Intel hardware. */
-                nnz_limit = 25600;
-                /* Use imbalance strategy when the matrix has more more than 3e8
-                 * on Intel hardware */
-                row_len_limit = 3e8;
+                nnz_limit = intel_nnz_limit;
+                row_len_limit = intel_row_len_limit;
             }
 #if GINKGO_HIP_PLATFORM_HCC
             if (!cuda_strategy_) {
@@ -604,8 +642,8 @@ public:
             auto host_mtx_exec = mtx_row_ptrs.get_executor()->get_master();
             const bool is_mtx_on_host{host_mtx_exec ==
                                       mtx_row_ptrs.get_executor()};
-            Array<index_type> row_ptrs_host(host_mtx_exec);
-            const index_type *row_ptrs{};
+            array<index_type> row_ptrs_host(host_mtx_exec);
+            const index_type* row_ptrs{};
             if (is_mtx_on_host) {
                 row_ptrs = mtx_row_ptrs.get_const_data();
             } else {
@@ -678,88 +716,70 @@ public:
         index_type max_length_per_row_;
     };
 
-    void convert_to(Csr<ValueType, IndexType> *result) const override
-    {
-        bool same_executor = this->get_executor() == result->get_executor();
-        // NOTE: as soon as strategies are improved, this can be reverted
-        result->values_ = this->values_;
-        result->col_idxs_ = this->col_idxs_;
-        result->row_ptrs_ = this->row_ptrs_;
-        result->srow_ = this->srow_;
-        result->set_size(this->get_size());
-        if (!same_executor) {
-            convert_strategy_helper(result);
-        } else {
-            result->set_strategy(std::move(this->get_strategy()->copy()));
-        }
-        // END NOTE
-    }
-
-    void move_to(Csr<ValueType, IndexType> *result) override
-    {
-        bool same_executor = this->get_executor() == result->get_executor();
-        EnableLinOp<Csr>::move_to(result);
-        if (!same_executor) {
-            detail::strategy_rebuild_helper(result);
-        }
-    }
-
     friend class Csr<next_precision<ValueType>, IndexType>;
 
     void convert_to(
-        Csr<next_precision<ValueType>, IndexType> *result) const override;
+        Csr<next_precision<ValueType>, IndexType>* result) const override;
 
-    void move_to(Csr<next_precision<ValueType>, IndexType> *result) override;
+    void move_to(Csr<next_precision<ValueType>, IndexType>* result) override;
 
-    void convert_to(Dense<ValueType> *other) const override;
+    void convert_to(Dense<ValueType>* other) const override;
 
-    void move_to(Dense<ValueType> *other) override;
+    void move_to(Dense<ValueType>* other) override;
 
-    void convert_to(Coo<ValueType, IndexType> *result) const override;
+    void convert_to(Coo<ValueType, IndexType>* result) const override;
 
-    void move_to(Coo<ValueType, IndexType> *result) override;
+    void move_to(Coo<ValueType, IndexType>* result) override;
 
-    void convert_to(Ell<ValueType, IndexType> *result) const override;
+    void convert_to(Ell<ValueType, IndexType>* result) const override;
 
-    void move_to(Ell<ValueType, IndexType> *result) override;
+    void move_to(Ell<ValueType, IndexType>* result) override;
 
-    void convert_to(Hybrid<ValueType, IndexType> *result) const override;
+    void convert_to(Fbcsr<ValueType, IndexType>* result) const override;
 
-    void move_to(Hybrid<ValueType, IndexType> *result) override;
+    void move_to(Fbcsr<ValueType, IndexType>* result) override;
 
-    void convert_to(Sellp<ValueType, IndexType> *result) const override;
+    void convert_to(Hybrid<ValueType, IndexType>* result) const override;
 
-    void move_to(Sellp<ValueType, IndexType> *result) override;
+    void move_to(Hybrid<ValueType, IndexType>* result) override;
 
-    void convert_to(SparsityCsr<ValueType, IndexType> *result) const override;
+    void convert_to(Sellp<ValueType, IndexType>* result) const override;
 
-    void move_to(SparsityCsr<ValueType, IndexType> *result) override;
+    void move_to(Sellp<ValueType, IndexType>* result) override;
 
-    void read(const mat_data &data) override;
+    void convert_to(SparsityCsr<ValueType, IndexType>* result) const override;
 
-    void write(mat_data &data) const override;
+    void move_to(SparsityCsr<ValueType, IndexType>* result) override;
+
+    void read(const mat_data& data) override;
+
+    void read(const device_mat_data& data) override;
+
+    void read(device_mat_data&& data) override;
+
+    void write(mat_data& data) const override;
 
     std::unique_ptr<LinOp> transpose() const override;
 
     std::unique_ptr<LinOp> conj_transpose() const override;
 
     std::unique_ptr<LinOp> permute(
-        const Array<IndexType> *permutation_indices) const override;
+        const array<IndexType>* permutation_indices) const override;
 
     std::unique_ptr<LinOp> inverse_permute(
-        const Array<IndexType> *inverse_permutation_indices) const override;
+        const array<IndexType>* inverse_permutation_indices) const override;
 
     std::unique_ptr<LinOp> row_permute(
-        const Array<IndexType> *permutation_indices) const override;
+        const array<IndexType>* permutation_indices) const override;
 
     std::unique_ptr<LinOp> column_permute(
-        const Array<IndexType> *permutation_indices) const override;
+        const array<IndexType>* permutation_indices) const override;
 
     std::unique_ptr<LinOp> inverse_row_permute(
-        const Array<IndexType> *inverse_permutation_indices) const override;
+        const array<IndexType>* inverse_permutation_indices) const override;
 
     std::unique_ptr<LinOp> inverse_column_permute(
-        const Array<IndexType> *inverse_permutation_indices) const override;
+        const array<IndexType>* inverse_permutation_indices) const override;
 
     std::unique_ptr<Diagonal<ValueType>> extract_diagonal() const override;
 
@@ -785,7 +805,7 @@ public:
      *
      * @return the values of the matrix.
      */
-    value_type *get_values() noexcept { return values_.get_data(); }
+    value_type* get_values() noexcept { return values_.get_data(); }
 
     /**
      * @copydoc Csr::get_values()
@@ -794,7 +814,7 @@ public:
      *       significantly more memory efficient than the non-constant version,
      *       so always prefer this version.
      */
-    const value_type *get_const_values() const noexcept
+    const value_type* get_const_values() const noexcept
     {
         return values_.get_const_data();
     }
@@ -804,7 +824,7 @@ public:
      *
      * @return the column indexes of the matrix.
      */
-    index_type *get_col_idxs() noexcept { return col_idxs_.get_data(); }
+    index_type* get_col_idxs() noexcept { return col_idxs_.get_data(); }
 
     /**
      * @copydoc Csr::get_col_idxs()
@@ -813,7 +833,7 @@ public:
      *       significantly more memory efficient than the non-constant version,
      *       so always prefer this version.
      */
-    const index_type *get_const_col_idxs() const noexcept
+    const index_type* get_const_col_idxs() const noexcept
     {
         return col_idxs_.get_const_data();
     }
@@ -823,7 +843,7 @@ public:
      *
      * @return the row pointers of the matrix.
      */
-    index_type *get_row_ptrs() noexcept { return row_ptrs_.get_data(); }
+    index_type* get_row_ptrs() noexcept { return row_ptrs_.get_data(); }
 
     /**
      * @copydoc Csr::get_row_ptrs()
@@ -832,7 +852,7 @@ public:
      *       significantly more memory efficient than the non-constant version,
      *       so always prefer this version.
      */
-    const index_type *get_const_row_ptrs() const noexcept
+    const index_type* get_const_row_ptrs() const noexcept
     {
         return row_ptrs_.get_const_data();
     }
@@ -842,7 +862,7 @@ public:
      *
      * @return the starting rows.
      */
-    index_type *get_srow() noexcept { return srow_.get_data(); }
+    index_type* get_srow() noexcept { return srow_.get_data(); }
 
     /**
      * @copydoc Csr::get_srow()
@@ -851,7 +871,7 @@ public:
      *       significantly more memory efficient than the non-constant version,
      *       so always prefer this version.
      */
-    const index_type *get_const_srow() const noexcept
+    const index_type* get_const_srow() const noexcept
     {
         return srow_.get_const_data();
     }
@@ -896,6 +916,128 @@ public:
         this->make_srow();
     }
 
+    /**
+     * Scales the matrix with a scalar.
+     *
+     * @param alpha  The entire matrix is scaled by alpha. alpha has to be a 1x1
+     * Dense matrix.
+     */
+    void scale(ptr_param<const LinOp> alpha)
+    {
+        auto exec = this->get_executor();
+        GKO_ASSERT_EQUAL_DIMENSIONS(alpha, dim<2>(1, 1));
+        this->scale_impl(make_temporary_clone(exec, alpha).get());
+    }
+
+    /**
+     * Scales the matrix with the inverse of a scalar.
+     *
+     * @param alpha  The entire matrix is scaled by 1 / alpha. alpha has to be a
+     * 1x1 Dense matrix.
+     */
+    void inv_scale(ptr_param<const LinOp> alpha)
+    {
+        auto exec = this->get_executor();
+        GKO_ASSERT_EQUAL_DIMENSIONS(alpha, dim<2>(1, 1));
+        this->inv_scale_impl(make_temporary_clone(exec, alpha).get());
+    }
+
+    /**
+     * Creates a constant (immutable) Csr matrix from a set of constant arrays.
+     *
+     * @param exec  the executor to create the matrix on
+     * @param size  the dimensions of the matrix
+     * @param values  the value array of the matrix
+     * @param col_idxs  the column index array of the matrix
+     * @param row_ptrs  the row pointer array of the matrix
+     * @param strategy  the strategy the matrix uses for SpMV operations
+     * @returns A smart pointer to the constant matrix wrapping the input arrays
+     *          (if they reside on the same executor as the matrix) or a copy of
+     *          these arrays on the correct executor.
+     */
+    static std::unique_ptr<const Csr> create_const(
+        std::shared_ptr<const Executor> exec, const dim<2>& size,
+        gko::detail::const_array_view<ValueType>&& values,
+        gko::detail::const_array_view<IndexType>&& col_idxs,
+        gko::detail::const_array_view<IndexType>&& row_ptrs,
+        std::shared_ptr<strategy_type> strategy)
+    {
+        // cast const-ness away, but return a const object afterwards,
+        // so we can ensure that no modifications take place.
+        return std::unique_ptr<const Csr>(new Csr{
+            exec, size, gko::detail::array_const_cast(std::move(values)),
+            gko::detail::array_const_cast(std::move(col_idxs)),
+            gko::detail::array_const_cast(std::move(row_ptrs)), strategy});
+    }
+
+    /**
+     * This is version of create_const with a default strategy.
+     */
+    static std::unique_ptr<const Csr> create_const(
+        std::shared_ptr<const Executor> exec, const dim<2>& size,
+        gko::detail::const_array_view<ValueType>&& values,
+        gko::detail::const_array_view<IndexType>&& col_idxs,
+        gko::detail::const_array_view<IndexType>&& row_ptrs)
+    {
+        return Csr::create_const(exec, size, std::move(values),
+                                 std::move(col_idxs), std::move(row_ptrs),
+                                 Csr::make_default_strategy(exec));
+    }
+
+    /**
+     * Creates a submatrix from this Csr matrix given row and column index_set
+     * objects.
+     *
+     * @param row_index_set  the row index set containing the set of rows to be
+     *                       in the submatrix.
+     * @param column_index_set  the col index set containing the set of columns
+     *                          to be in the submatrix.
+     * @return A new CSR matrix with the elements that belong to the row and
+     *          columns of this matrix as specified by the index sets.
+     * @note This is not a view but creates a new, separate CSR matrix.
+     */
+    std::unique_ptr<Csr<ValueType, IndexType>> create_submatrix(
+        const index_set<IndexType>& row_index_set,
+        const index_set<IndexType>& column_index_set) const;
+
+    /**
+     * Creates a submatrix from this Csr matrix given row and column spans
+     *
+     * @param row_span  the row span containing the contiguous set of rows to be
+     *                  in the submatrix.
+     * @param column_span  the column span containing the contiguous set of
+     *                     columns to be in the submatrix.
+     * @return A new CSR matrix with the elements that belong to the row and
+     *          columns of this matrix as specified by the index sets.
+     * @note This is not a view but creates a new, separate CSR matrix.
+     */
+    std::unique_ptr<Csr<ValueType, IndexType>> create_submatrix(
+        const span& row_span, const span& column_span) const;
+
+    /**
+     * Copy-assigns a Csr matrix. Preserves executor, copies everything else.
+     */
+    Csr& operator=(const Csr&);
+
+    /**
+     * Move-assigns a Csr matrix. Preserves executor, moves the data and leaves
+     * the moved-from object in an empty state (0x0 LinOp with unchanged
+     * executor and strategy, no nonzeros and valid row pointers).
+     */
+    Csr& operator=(Csr&&);
+
+    /**
+     * Copy-constructs a Csr matrix. Inherits executor, strategy and data.
+     */
+    Csr(const Csr&);
+
+    /**
+     * Move-constructs a Csr matrix. Inherits executor and strategy, moves the
+     * data and leaves the moved-from object in an empty state (0x0 LinOp with
+     * unchanged executor and strategy, no nonzeros and valid row pointers).
+     */
+    Csr(Csr&&);
+
 protected:
     /**
      * Creates an uninitialized CSR matrix of the specified size.
@@ -909,22 +1051,38 @@ protected:
     {}
 
     /**
-     * Creates an uninitialized CSR matrix of the specified size.
+     * Creates an uninitialized CSR matrix of the specified size with a user
+     * chosen strategy.
      *
      * @param exec  Executor associated to the matrix
      * @param size  size of the matrix
      * @param num_nonzeros  number of nonzeros
      * @param strategy  the strategy of CSR
      */
-    Csr(std::shared_ptr<const Executor> exec, const dim<2> &size = dim<2>{},
-        size_type num_nonzeros = {},
-        std::shared_ptr<strategy_type> strategy = std::make_shared<sparselib>())
+    Csr(std::shared_ptr<const Executor> exec, const dim<2>& size,
+        size_type num_nonzeros, std::shared_ptr<strategy_type> strategy)
         : EnableLinOp<Csr>(exec, size),
           values_(exec, num_nonzeros),
           col_idxs_(exec, num_nonzeros),
           row_ptrs_(exec, size[0] + 1),
           srow_(exec, strategy->clac_size(num_nonzeros)),
           strategy_(strategy->copy())
+    {
+        row_ptrs_.fill(0);
+        this->make_srow();
+    }
+
+    /**
+     * Creates an uninitialized CSR matrix of the specified size with a
+     * default strategy.
+     *
+     * @param exec  Executor associated to the matrix
+     * @param size  size of the matrix
+     * @param num_nonzeros  number of nonzeros
+     */
+    Csr(std::shared_ptr<const Executor> exec, const dim<2>& size = dim<2>{},
+        size_type num_nonzeros = {})
+        : Csr{exec, size, num_nonzeros, Csr::make_default_strategy(exec)}
     {}
 
     /**
@@ -940,6 +1098,7 @@ protected:
      * @param values  array of matrix values
      * @param col_idxs  array of column indexes
      * @param row_ptrs  array of row pointers
+     * @param strategy  the strategy the matrix uses for SpMV operations
      *
      * @note If one of `row_ptrs`, `col_idxs` or `values` is not an rvalue, not
      *       an array of IndexType, IndexType and ValueType, respectively, or
@@ -949,9 +1108,9 @@ protected:
      */
     template <typename ValuesArray, typename ColIdxsArray,
               typename RowPtrsArray>
-    Csr(std::shared_ptr<const Executor> exec, const dim<2> &size,
-        ValuesArray &&values, ColIdxsArray &&col_idxs, RowPtrsArray &&row_ptrs,
-        std::shared_ptr<strategy_type> strategy = std::make_shared<sparselib>())
+    Csr(std::shared_ptr<const Executor> exec, const dim<2>& size,
+        ValuesArray&& values, ColIdxsArray&& col_idxs, RowPtrsArray&& row_ptrs,
+        std::shared_ptr<strategy_type> strategy)
         : EnableLinOp<Csr>(exec, size),
           values_{exec, std::forward<ValuesArray>(values)},
           col_idxs_{exec, std::forward<ColIdxsArray>(col_idxs)},
@@ -964,24 +1123,63 @@ protected:
         this->make_srow();
     }
 
-    void apply_impl(const LinOp *b, LinOp *x) const override;
+    /**
+     * Creates a CSR matrix from already allocated (and initialized) row
+     * pointer, column index and value arrays.
+     *
+     * @note This is the same as the previous constructor but with a default
+     *       strategy.
+     */
+    template <typename ValuesArray, typename ColIdxsArray,
+              typename RowPtrsArray>
+    Csr(std::shared_ptr<const Executor> exec, const dim<2>& size,
+        ValuesArray&& values, ColIdxsArray&& col_idxs, RowPtrsArray&& row_ptrs)
+        : Csr{exec,
+              size,
+              std::forward<ValuesArray>(values),
+              std::forward<ColIdxsArray>(col_idxs),
+              std::forward<RowPtrsArray>(row_ptrs),
+              Csr::make_default_strategy(exec)}
+    {}
 
-    void apply_impl(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                    LinOp *x) const override;
+    void apply_impl(const LinOp* b, LinOp* x) const override;
+
+    void apply_impl(const LinOp* alpha, const LinOp* b, const LinOp* beta,
+                    LinOp* x) const override;
+
+    // TODO: This provides some more sane settings. Please fix this!
+    static std::shared_ptr<strategy_type> make_default_strategy(
+        std::shared_ptr<const Executor> exec)
+    {
+        auto cuda_exec = std::dynamic_pointer_cast<const CudaExecutor>(exec);
+        auto hip_exec = std::dynamic_pointer_cast<const HipExecutor>(exec);
+        auto dpcpp_exec = std::dynamic_pointer_cast<const DpcppExecutor>(exec);
+        std::shared_ptr<strategy_type> new_strategy;
+        if (cuda_exec) {
+            new_strategy = std::make_shared<automatical>(cuda_exec);
+        } else if (hip_exec) {
+            new_strategy = std::make_shared<automatical>(hip_exec);
+        } else if (dpcpp_exec) {
+            new_strategy = std::make_shared<automatical>(dpcpp_exec);
+        } else {
+            new_strategy = std::make_shared<classical>();
+        }
+        return new_strategy;
+    }
 
     // TODO clean this up as soon as we improve strategy_type
     template <typename CsrType>
-    void convert_strategy_helper(CsrType *result) const
+    void convert_strategy_helper(CsrType* result) const
     {
         auto strat = this->get_strategy().get();
         std::shared_ptr<typename CsrType::strategy_type> new_strat;
-        if (dynamic_cast<classical *>(strat)) {
+        if (dynamic_cast<classical*>(strat)) {
             new_strat = std::make_shared<typename CsrType::classical>();
-        } else if (dynamic_cast<merge_path *>(strat)) {
+        } else if (dynamic_cast<merge_path*>(strat)) {
             new_strat = std::make_shared<typename CsrType::merge_path>();
-        } else if (dynamic_cast<cusparse *>(strat)) {
+        } else if (dynamic_cast<cusparse*>(strat)) {
             new_strat = std::make_shared<typename CsrType::cusparse>();
-        } else if (dynamic_cast<sparselib *>(strat)) {
+        } else if (dynamic_cast<sparselib*>(strat)) {
             new_strat = std::make_shared<typename CsrType::sparselib>();
         } else {
             auto rexec = result->get_executor();
@@ -990,7 +1188,7 @@ protected:
             auto hip_exec = std::dynamic_pointer_cast<const HipExecutor>(rexec);
             auto dpcpp_exec =
                 std::dynamic_pointer_cast<const DpcppExecutor>(rexec);
-            auto lb = dynamic_cast<load_balance *>(strat);
+            auto lb = dynamic_cast<load_balance*>(strat);
             if (cuda_exec) {
                 if (lb) {
                     new_strat =
@@ -1060,17 +1258,11 @@ protected:
                                 this_dpcpp_exec);
                     }
                 } else {
+                    // FIXME: this changes strategies.
                     // We had a load balance or automatical strategy from a non
                     // HIP or Cuda executor and are moving to a non HIP or Cuda
                     // executor.
-                    // FIXME this creates a long delay
-                    if (lb) {
-                        new_strat =
-                            std::make_shared<typename CsrType::load_balance>();
-                    } else {
-                        new_strat =
-                            std::make_shared<typename CsrType::automatical>();
-                    }
+                    new_strat = std::make_shared<typename CsrType::classical>();
                 }
             }
         }
@@ -1086,12 +1278,30 @@ protected:
         strategy_->process(row_ptrs_, &srow_);
     }
 
+    /**
+     * @copydoc scale(const LinOp *)
+     *
+     * @note  Other implementations of Csr should override this function
+     *        instead of scale(const LinOp *alpha).
+     */
+    virtual void scale_impl(const LinOp* alpha);
+
+    /**
+     * @copydoc inv_scale(const LinOp *)
+     *
+     * @note  Other implementations of Csr should override this function
+     *        instead of inv_scale(const LinOp *alpha).
+     */
+    virtual void inv_scale_impl(const LinOp* alpha);
+
 private:
-    Array<value_type> values_;
-    Array<index_type> col_idxs_;
-    Array<index_type> row_ptrs_;
-    Array<index_type> srow_;
+    array<value_type> values_;
+    array<index_type> col_idxs_;
+    array<index_type> row_ptrs_;
+    array<index_type> srow_;
     std::shared_ptr<strategy_type> strategy_;
+
+    void add_scaled_identity_impl(const LinOp* a, const LinOp* b) override;
 };
 
 
@@ -1105,7 +1315,7 @@ namespace detail {
  * @param result  the csr matrix.
  */
 template <typename ValueType, typename IndexType>
-void strategy_rebuild_helper(Csr<ValueType, IndexType> *result)
+void strategy_rebuild_helper(Csr<ValueType, IndexType>* result)
 {
     using load_balance = typename Csr<ValueType, IndexType>::load_balance;
     using automatical = typename Csr<ValueType, IndexType>::automatical;
