@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -109,8 +109,8 @@ struct block_interleaved_storage_scheme {
      *        blocks is not known, for a special input `size_type{} - 1`
      *        the method returns `0` to avoid overallocation of memory.
      */
-    GKO_ATTRIBUTES size_type compute_storage_space(size_type num_blocks) const
-        noexcept
+    GKO_ATTRIBUTES size_type
+    compute_storage_space(size_type num_blocks) const noexcept
     {
         return (num_blocks + 1 == size_type{0})
                    ? size_type{0}
@@ -148,8 +148,8 @@ struct block_interleaved_storage_scheme {
      *
      * @return the offset of the block with ID `block_id`
      */
-    GKO_ATTRIBUTES IndexType get_global_block_offset(IndexType block_id) const
-        noexcept
+    GKO_ATTRIBUTES IndexType
+    get_global_block_offset(IndexType block_id) const noexcept
     {
         return this->get_group_offset(block_id) +
                this->get_block_offset(block_id);
@@ -220,6 +220,8 @@ class Jacobi : public EnableLinOp<Jacobi<ValueType, IndexType>>,
 public:
     using EnableLinOp<Jacobi>::convert_to;
     using EnableLinOp<Jacobi>::move_to;
+    using ConvertibleTo<matrix::Dense<ValueType>>::convert_to;
+    using ConvertibleTo<matrix::Dense<ValueType>>::move_to;
     using value_type = ValueType;
     using index_type = IndexType;
     using mat_data = matrix_data<ValueType, IndexType>;
@@ -243,7 +245,7 @@ public:
      * @internal
      * TODO: replace with ranges
      */
-    const block_interleaved_storage_scheme<index_type> &get_storage_scheme()
+    const block_interleaved_storage_scheme<index_type>& get_storage_scheme()
         const noexcept
     {
         return storage_scheme_;
@@ -260,7 +262,7 @@ public:
      * @internal
      * TODO: replace with ranges
      */
-    const value_type *get_blocks() const noexcept
+    const value_type* get_blocks() const noexcept
     {
         return blocks_.get_const_data();
     }
@@ -274,7 +276,7 @@ public:
      *       implementations of the standard non-adaptive variant are allowed to
      *       omit the calculation of condition numbers.
      */
-    const remove_complex<value_type> *get_conditioning() const noexcept
+    const remove_complex<value_type>* get_conditioning() const noexcept
     {
         return conditioning_.get_const_data();
     }
@@ -289,16 +291,42 @@ public:
         return blocks_.get_num_elems();
     }
 
-    void convert_to(matrix::Dense<value_type> *result) const override;
+    void convert_to(matrix::Dense<value_type>* result) const override;
 
-    void move_to(matrix::Dense<value_type> *result) override;
+    void move_to(matrix::Dense<value_type>* result) override;
 
-    void write(mat_data &data) const override;
+    void write(mat_data& data) const override;
 
     std::unique_ptr<LinOp> transpose() const override;
 
     std::unique_ptr<LinOp> conj_transpose() const override;
 
+    /**
+     * Copy-assigns a Jacobi preconditioner. Preserves executor, copies all
+     * data and parameters.
+     */
+    Jacobi& operator=(const Jacobi& other);
+
+    /**
+     * Move-assigns a Jacobi preconditioner. Preserves executor, moves all data
+     * and parameters. The moved-from object will be empty (0x0 and default
+     * parameters).
+     */
+    Jacobi& operator=(Jacobi&& other);
+
+    /**
+     * Copy-constructs a Jacobi preconditioner. Inherits executor, copies all
+     * data and parameters.
+     */
+    Jacobi(const Jacobi& other);
+
+    /**
+     * Move-assigns a Jacobi preconditioner. Inherits executor, moves all data
+     * and parameters. The moved-from object will be empty (0x0 and default
+     * parameters).
+     */
+    Jacobi(Jacobi&& other);
+
     GKO_CREATE_FACTORY_PARAMETERS(parameters, Factory)
     {
         /**
@@ -366,7 +394,7 @@ public:
          *       has to be respected when setting this parameter. Failure to do
          *       so will lead to undefined behavior.
          */
-        gko::Array<index_type> GKO_FACTORY_PARAMETER_VECTOR(block_pointers,
+        gko::array<index_type> GKO_FACTORY_PARAMETER_VECTOR(block_pointers,
                                                             nullptr);
 
     private:
@@ -378,13 +406,13 @@ public:
             {}
 
             storage_optimization_type(
-                const Array<precision_reduction> &block_wise_opt)
+                const array<precision_reduction>& block_wise_opt)
                 : is_block_wise{block_wise_opt.get_num_elems() > 0},
                   block_wise{block_wise_opt}
             {}
 
             storage_optimization_type(
-                Array<precision_reduction> &&block_wise_opt)
+                array<precision_reduction>&& block_wise_opt)
                 : is_block_wise{block_wise_opt.get_num_elems() > 0},
                   block_wise{std::move(block_wise_opt)}
             {}
@@ -393,7 +421,7 @@ public:
 
             bool is_block_wise;
             precision_reduction of_all_blocks;
-            gko::Array<precision_reduction> block_wise;
+            gko::array<precision_reduction> block_wise;
         };
 
     public:
@@ -401,10 +429,10 @@ public:
          * The precisions to use for the blocks of the matrix.
          *
          * This parameter can either be a single instance of precision_reduction
-         * or an Array of precision_reduction values. If set to
+         * or an array of precision_reduction values. If set to
          * `precision_reduction(0, 0)` (this is the default), a regular
          * full-precision block-Jacobi will be used. Any other value (or an
-         * Array of values) will map to the adaptive variant.
+         * array of values) will map to the adaptive variant.
          *
          * The best starting point when evaluating the potential of the adaptive
          * version is to set this parameter to
@@ -440,7 +468,7 @@ public:
          * on the conditioning of the block.
          *
          * If the number of diagonal blocks is larger than the number of
-         * elements in the passed Array, the entire Array will be replicated
+         * elements in the passed array, the entire array will be replicated
          * until enough values are available. For example, if the original array
          * contained two precisions `(x, y)` and the preconditioner contains 5
          * blocks, the array will be transformed into `(x, y, x, y, x)` before
@@ -462,7 +490,7 @@ public:
          * `precision_reduction::autodetect()` will be replaced with the value
          * representing the precision used for the corresponding block.
          * If the non-adaptive version of Jacobi is used, the
-         * `storage_optimization.block_wise` Array will be empty.
+         * `storage_optimization.block_wise` array will be empty.
          */
         storage_optimization_type GKO_FACTORY_PARAMETER_VECTOR(
             storage_optimization, precision_reduction(0, 0));
@@ -522,7 +550,7 @@ protected:
      * @param system_matrix  the matrix this preconditioner should be created
      *                       from
      */
-    explicit Jacobi(const Factory *factory,
+    explicit Jacobi(const Factory* factory,
                     std::shared_ptr<const LinOp> system_matrix)
         : EnableLinOp<Jacobi>(factory->get_executor(),
                               gko::transpose(system_matrix->get_size())),
@@ -538,7 +566,7 @@ protected:
         parameters_.block_pointers.set_executor(this->get_executor());
         parameters_.storage_optimization.block_wise.set_executor(
             this->get_executor());
-        this->generate(lend(system_matrix), parameters_.skip_sorting);
+        this->generate(system_matrix.get(), parameters_.skip_sorting);
     }
 
     /**
@@ -592,7 +620,7 @@ protected:
      *                      skipped (therefore, marking that it is already
      *                      sorted)
      */
-    void generate(const LinOp *system_matrix, bool skip_sorting);
+    void generate(const LinOp* system_matrix, bool skip_sorting);
 
     /**
      * Detects the diagonal blocks and allocates the memory needed to store the
@@ -601,18 +629,18 @@ protected:
      * @param system_matrix  the source matrix whose diagonal block pattern is
      *                       to be detected
      */
-    void detect_blocks(const matrix::Csr<ValueType, IndexType> *system_matrix);
+    void detect_blocks(const matrix::Csr<ValueType, IndexType>* system_matrix);
 
-    void apply_impl(const LinOp *b, LinOp *x) const override;
+    void apply_impl(const LinOp* b, LinOp* x) const override;
 
-    void apply_impl(const LinOp *alpha, const LinOp *b, const LinOp *beta,
-                    LinOp *x) const override;
+    void apply_impl(const LinOp* alpha, const LinOp* b, const LinOp* beta,
+                    LinOp* x) const override;
 
 private:
     block_interleaved_storage_scheme<index_type> storage_scheme_{};
     size_type num_blocks_;
-    Array<value_type> blocks_;
-    Array<remove_complex<value_type>> conditioning_;
+    array<value_type> blocks_;
+    array<remove_complex<value_type>> conditioning_;
 };
 
 
