@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -46,12 +46,15 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/matrix/csr.hpp>
 #include <ginkgo/core/matrix/diagonal.hpp>
 #include <ginkgo/core/matrix/ell.hpp>
+#include <ginkgo/core/matrix/fbcsr.hpp>
 #include <ginkgo/core/matrix/hybrid.hpp>
 #include <ginkgo/core/matrix/sellp.hpp>
 #include <ginkgo/core/matrix/sparsity_csr.hpp>
 
 
-#include "core/components/prefix_sum.hpp"
+#include "accessor/block_col_major.hpp"
+#include "accessor/range.hpp"
+#include "core/components/prefix_sum_kernels.hpp"
 
 
 namespace gko {
@@ -66,10 +69,51 @@ namespace dense {
 
 
 template <typename ValueType>
-void simple_apply(std::shared_ptr<const OmpExecutor> exec,
-                  const matrix::Dense<ValueType> *a,
-                  const matrix::Dense<ValueType> *b,
-                  matrix::Dense<ValueType> *c)
+void compute_dot_dispatch(std::shared_ptr<const DefaultExecutor> exec,
+                          const matrix::Dense<ValueType>* x,
+                          const matrix::Dense<ValueType>* y,
+                          matrix::Dense<ValueType>* result, array<char>& tmp)
+{
+    // OpenMP uses the unified kernel.
+    compute_dot(exec, x, y, result, tmp);
+}
+
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
+    GKO_DECLARE_DENSE_COMPUTE_DOT_DISPATCH_KERNEL);
+
+
+template <typename ValueType>
+void compute_conj_dot_dispatch(std::shared_ptr<const DefaultExecutor> exec,
+                               const matrix::Dense<ValueType>* x,
+                               const matrix::Dense<ValueType>* y,
+                               matrix::Dense<ValueType>* result,
+                               array<char>& tmp)
+{
+    compute_conj_dot(exec, x, y, result, tmp);
+}
+
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
+    GKO_DECLARE_DENSE_COMPUTE_CONJ_DOT_DISPATCH_KERNEL);
+
+
+template <typename ValueType>
+void compute_norm2_dispatch(std::shared_ptr<const DefaultExecutor> exec,
+                            const matrix::Dense<ValueType>* x,
+                            matrix::Dense<remove_complex<ValueType>>* result,
+                            array<char>& tmp)
+{
+    compute_norm2(exec, x, result, tmp);
+}
+
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
+    GKO_DECLARE_DENSE_COMPUTE_NORM2_DISPATCH_KERNEL);
+
+
+template <typename ValueType>
+void simple_apply(std::shared_ptr<const DefaultExecutor> exec,
+                  const matrix::Dense<ValueType>* a,
+                  const matrix::Dense<ValueType>* b,
+                  matrix::Dense<ValueType>* c)
 {
 #pragma omp parallel for
     for (size_type row = 0; row < c->get_size()[0]; ++row) {
@@ -92,12 +136,12 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_SIMPLE_APPLY_KERNEL);
 
 
 template <typename ValueType>
-void apply(std::shared_ptr<const OmpExecutor> exec,
-           const matrix::Dense<ValueType> *alpha,
-           const matrix::Dense<ValueType> *a, const matrix::Dense<ValueType> *b,
-           const matrix::Dense<ValueType> *beta, matrix::Dense<ValueType> *c)
+void apply(std::shared_ptr<const DefaultExecutor> exec,
+           const matrix::Dense<ValueType>* alpha,
+           const matrix::Dense<ValueType>* a, const matrix::Dense<ValueType>* b,
+           const matrix::Dense<ValueType>* beta, matrix::Dense<ValueType>* c)
 {
-    if (beta->at(0, 0) != zero<ValueType>()) {
+    if (is_nonzero(beta->at(0, 0))) {
 #pragma omp parallel for
         for (size_type row = 0; row < c->get_size()[0]; ++row) {
             for (size_type col = 0; col < c->get_size()[1]; ++col) {
@@ -127,77 +171,11 @@ void apply(std::shared_ptr<const OmpExecutor> exec,
 GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_APPLY_KERNEL);
 
 
-template <typename ValueType>
-void compute_dot(std::shared_ptr<const OmpExecutor> exec,
-                 const matrix::Dense<ValueType> *x,
-                 const matrix::Dense<ValueType> *y,
-                 matrix::Dense<ValueType> *result)
-{
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        result->at(0, j) = zero<ValueType>();
-    }
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        for (size_type i = 0; i < x->get_size()[0]; ++i) {
-            result->at(0, j) += x->at(i, j) * y->at(i, j);
-        }
-    }
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_COMPUTE_DOT_KERNEL);
-
-
-template <typename ValueType>
-void compute_conj_dot(std::shared_ptr<const OmpExecutor> exec,
-                      const matrix::Dense<ValueType> *x,
-                      const matrix::Dense<ValueType> *y,
-                      matrix::Dense<ValueType> *result)
-{
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        result->at(0, j) = zero<ValueType>();
-    }
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        for (size_type i = 0; i < x->get_size()[0]; ++i) {
-            result->at(0, j) += conj(x->at(i, j)) * y->at(i, j);
-        }
-    }
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_COMPUTE_CONJ_DOT_KERNEL);
-
-
-template <typename ValueType>
-void compute_norm2(std::shared_ptr<const OmpExecutor> exec,
-                   const matrix::Dense<ValueType> *x,
-                   matrix::Dense<remove_complex<ValueType>> *result)
-{
-    using norm_type = remove_complex<ValueType>;
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        result->at(0, j) = zero<norm_type>();
-    }
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        for (size_type i = 0; i < x->get_size()[0]; ++i) {
-            result->at(0, j) += squared_norm(x->at(i, j));
-        }
-    }
-#pragma omp parallel for
-    for (size_type j = 0; j < x->get_size()[1]; ++j) {
-        result->at(0, j) = sqrt(result->at(0, j));
-    }
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_COMPUTE_NORM2_KERNEL);
-
-
 template <typename ValueType, typename IndexType>
-void convert_to_coo(std::shared_ptr<const OmpExecutor> exec,
-                    const matrix::Dense<ValueType> *source,
-                    matrix::Coo<ValueType, IndexType> *result)
+void convert_to_coo(std::shared_ptr<const DefaultExecutor> exec,
+                    const matrix::Dense<ValueType>* source,
+                    const int64* row_ptrs,
+                    matrix::Coo<ValueType, IndexType>* result)
 {
     auto num_rows = result->get_size()[0];
     auto num_cols = result->get_size()[1];
@@ -206,27 +184,13 @@ void convert_to_coo(std::shared_ptr<const OmpExecutor> exec,
     auto row_idxs = result->get_row_idxs();
     auto col_idxs = result->get_col_idxs();
     auto values = result->get_values();
-    Array<IndexType> row_ptrs_array(exec, num_rows);
-    auto row_ptrs = row_ptrs_array.get_data();
-
-#pragma omp parallel for
-    for (size_type row = 0; row < num_rows; ++row) {
-        IndexType row_count{};
-        for (size_type col = 0; col < num_cols; ++col) {
-            auto val = source->at(row, col);
-            row_count += val != zero<ValueType>();
-        }
-        row_ptrs[row] = row_count;
-    }
-
-    components::prefix_sum(exec, row_ptrs, num_rows);
 
 #pragma omp parallel for
     for (size_type row = 0; row < num_rows; ++row) {
         auto idxs = row_ptrs[row];
         for (size_type col = 0; col < num_cols; ++col) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 row_idxs[idxs] = row;
                 col_idxs[idxs] = col;
                 values[idxs] = val;
@@ -241,9 +205,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-void convert_to_csr(std::shared_ptr<const OmpExecutor> exec,
-                    const matrix::Dense<ValueType> *source,
-                    matrix::Csr<ValueType, IndexType> *result)
+void convert_to_csr(std::shared_ptr<const DefaultExecutor> exec,
+                    const matrix::Dense<ValueType>* source,
+                    matrix::Csr<ValueType, IndexType>* result)
 {
     auto num_rows = result->get_size()[0];
     auto num_cols = result->get_size()[1];
@@ -253,24 +217,12 @@ void convert_to_csr(std::shared_ptr<const OmpExecutor> exec,
     auto col_idxs = result->get_col_idxs();
     auto values = result->get_values();
 
-#pragma omp parallel for
-    for (size_type row = 0; row < num_rows; ++row) {
-        IndexType row_nnz{};
-        for (size_type col = 0; col < num_cols; ++col) {
-            auto val = source->at(row, col);
-            row_nnz += val != zero<ValueType>();
-        }
-        row_ptrs[row] = row_nnz;
-    }
-
-    components::prefix_sum(exec, row_ptrs, num_rows + 1);
-
 #pragma omp parallel for
     for (size_type row = 0; row < num_rows; ++row) {
         auto cur_ptr = row_ptrs[row];
         for (size_type col = 0; col < num_cols; ++col) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 col_idxs[cur_ptr] = col;
                 values[cur_ptr] = val;
                 ++cur_ptr;
@@ -284,9 +236,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-void convert_to_ell(std::shared_ptr<const OmpExecutor> exec,
-                    const matrix::Dense<ValueType> *source,
-                    matrix::Ell<ValueType, IndexType> *result)
+void convert_to_ell(std::shared_ptr<const DefaultExecutor> exec,
+                    const matrix::Dense<ValueType>* source,
+                    matrix::Ell<ValueType, IndexType>* result)
 {
     auto num_rows = result->get_size()[0];
     auto num_cols = result->get_size()[1];
@@ -295,7 +247,7 @@ void convert_to_ell(std::shared_ptr<const OmpExecutor> exec,
     for (size_type i = 0; i < max_nnz_per_row; i++) {
         for (size_type j = 0; j < result->get_stride(); j++) {
             result->val_at(j, i) = zero<ValueType>();
-            result->col_at(j, i) = 0;
+            result->col_at(j, i) = invalid_index<IndexType>();
         }
     }
 #pragma omp parallel for
@@ -303,7 +255,7 @@ void convert_to_ell(std::shared_ptr<const OmpExecutor> exec,
         size_type col_idx = 0;
         for (size_type col = 0; col < num_cols; col++) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 result->val_at(row, col_idx) = val;
                 result->col_at(row, col_idx) = col;
                 col_idx++;
@@ -317,46 +269,65 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-void convert_to_hybrid(std::shared_ptr<const OmpExecutor> exec,
-                       const matrix::Dense<ValueType> *source,
-                       matrix::Hybrid<ValueType, IndexType> *result)
+void convert_to_fbcsr(std::shared_ptr<const DefaultExecutor> exec,
+                      const matrix::Dense<ValueType>* source,
+                      matrix::Fbcsr<ValueType, IndexType>* result)
+{
+    const auto num_rows = source->get_size()[0];
+    const auto num_cols = source->get_size()[1];
+    const auto bs = result->get_block_size();
+    const auto nzbs = result->get_num_stored_blocks();
+    const auto num_block_rows = num_rows / bs;
+    const auto num_block_cols = num_cols / bs;
+    acc::range<acc::block_col_major<ValueType, 3>> blocks(
+        std::array<acc::size_type, 3>{static_cast<acc::size_type>(nzbs),
+                                      static_cast<acc::size_type>(bs),
+                                      static_cast<acc::size_type>(bs)},
+        result->get_values());
+    auto col_idxs = result->get_col_idxs();
+#pragma omp parallel for
+    for (size_type brow = 0; brow < num_block_rows; ++brow) {
+        auto block = result->get_const_row_ptrs()[brow];
+        for (size_type bcol = 0; bcol < num_block_cols; ++bcol) {
+            bool block_nz = false;
+            for (int lrow = 0; lrow < bs; ++lrow) {
+                for (int lcol = 0; lcol < bs; ++lcol) {
+                    const auto row = lrow + bs * brow;
+                    const auto col = lcol + bs * bcol;
+                    block_nz = block_nz || is_nonzero(source->at(row, col));
+                }
+            }
+            if (block_nz) {
+                col_idxs[block] = bcol;
+                for (int lrow = 0; lrow < bs; ++lrow) {
+                    for (int lcol = 0; lcol < bs; ++lcol) {
+                        const auto row = lrow + bs * brow;
+                        const auto col = lcol + bs * bcol;
+                        blocks(block, lrow, lcol) = source->at(row, col);
+                    }
+                }
+                block++;
+            }
+        }
+    }
+}
+
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
+    GKO_DECLARE_DENSE_CONVERT_TO_FBCSR_KERNEL);
+
+
+template <typename ValueType, typename IndexType>
+void convert_to_hybrid(std::shared_ptr<const DefaultExecutor> exec,
+                       const matrix::Dense<ValueType>* source,
+                       const int64* coo_row_ptrs,
+                       matrix::Hybrid<ValueType, IndexType>* result)
 {
     auto num_rows = result->get_size()[0];
     auto num_cols = result->get_size()[1];
-    auto strategy = result->get_strategy();
-    auto ell_lim = strategy->get_ell_num_stored_elements_per_row();
+    auto ell_lim = result->get_ell_num_stored_elements_per_row();
     auto coo_val = result->get_coo_values();
     auto coo_col = result->get_coo_col_idxs();
     auto coo_row = result->get_coo_row_idxs();
-    Array<IndexType> coo_row_ptrs_array(exec, num_rows);
-    auto coo_row_ptrs = coo_row_ptrs_array.get_data();
-
-    auto ell_nnz_row = result->get_ell_num_stored_elements_per_row();
-    auto ell_stride = result->get_ell_stride();
-#pragma omp parallel for collapse(2)
-    for (size_type i = 0; i < ell_nnz_row; i++) {
-        for (size_type j = 0; j < ell_stride; j++) {
-            result->ell_val_at(j, i) = zero<ValueType>();
-            result->ell_col_at(j, i) = 0;
-        }
-    }
-#pragma omp parallel for
-    for (size_type i = 0; i < result->get_coo_num_stored_elements(); i++) {
-        coo_val[i] = zero<ValueType>();
-        coo_col[i] = 0;
-        coo_row[i] = 0;
-    }
-#pragma omp parallel for
-    for (size_type row = 0; row < num_rows; row++) {
-        size_type total_row_nnz{};
-        for (size_type col = 0; col < num_cols; col++) {
-            auto val = source->at(row, col);
-            total_row_nnz += val != zero<ValueType>();
-        }
-        coo_row_ptrs[row] = std::max(ell_lim, total_row_nnz) - ell_lim;
-    }
-
-    components::prefix_sum(exec, coo_row_ptrs, num_rows);
 
 #pragma omp parallel for
     for (size_type row = 0; row < num_rows; row++) {
@@ -364,16 +335,20 @@ void convert_to_hybrid(std::shared_ptr<const OmpExecutor> exec,
         size_type col = 0;
         for (; col < num_cols && ell_count < ell_lim; col++) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 result->ell_val_at(row, ell_count) = val;
                 result->ell_col_at(row, ell_count) = col;
                 ell_count++;
             }
         }
+        for (; ell_count < ell_lim; ell_count++) {
+            result->ell_val_at(row, ell_count) = zero<ValueType>();
+            result->ell_col_at(row, ell_count) = invalid_index<IndexType>();
+        }
         auto coo_idx = coo_row_ptrs[row];
         for (; col < num_cols; col++) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 coo_val[coo_idx] = val;
                 coo_col[coo_idx] = col;
                 coo_row[coo_idx] = row;
@@ -388,74 +363,41 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-void convert_to_sellp(std::shared_ptr<const OmpExecutor> exec,
-                      const matrix::Dense<ValueType> *source,
-                      matrix::Sellp<ValueType, IndexType> *result)
+void convert_to_sellp(std::shared_ptr<const DefaultExecutor> exec,
+                      const matrix::Dense<ValueType>* source,
+                      matrix::Sellp<ValueType, IndexType>* result)
 {
-    auto num_rows = result->get_size()[0];
-    auto num_cols = result->get_size()[1];
-    auto vals = result->get_values();
-    auto col_idxs = result->get_col_idxs();
-    auto slice_lengths = result->get_slice_lengths();
-    auto slice_sets = result->get_slice_sets();
-    auto slice_size = (result->get_slice_size() == 0)
-                          ? matrix::default_slice_size
-                          : result->get_slice_size();
-    auto stride_factor = (result->get_stride_factor() == 0)
-                             ? matrix::default_stride_factor
-                             : result->get_stride_factor();
-    int slice_num = ceildiv(num_rows, slice_size);
-    slice_sets[0] = 0;
-    for (size_type slice = 0; slice < slice_num; slice++) {
-        if (slice > 0) {
-            slice_sets[slice] =
-                slice_sets[slice - 1] + slice_lengths[slice - 1];
-        }
-        size_type current_slice_length = 0;
-#pragma omp parallel for reduction(max : current_slice_length)
-        for (size_type row = 0; row < slice_size; row++) {
-            size_type global_row = slice * slice_size + row;
-            if (global_row < num_rows) {
-                size_type max_col = 0;
-                for (size_type col = 0; col < num_cols; col++) {
-                    if (source->at(global_row, col) != zero<ValueType>()) {
-                        max_col += 1;
-                    }
-                }
-                current_slice_length = std::max(current_slice_length, max_col);
-            }
-        }
-        slice_lengths[slice] =
-            stride_factor * ceildiv(current_slice_length, stride_factor);
+    const auto num_rows = result->get_size()[0];
+    const auto num_cols = result->get_size()[1];
+    const auto vals = result->get_values();
+    const auto col_idxs = result->get_col_idxs();
+    const auto slice_sets = result->get_slice_sets();
+    const auto slice_size = result->get_slice_size();
+    const auto num_slices = ceildiv(num_rows, slice_size);
 #pragma omp parallel for
-        for (size_type row = 0; row < slice_size; row++) {
-            const size_type global_row = slice * slice_size + row;
-            if (global_row < num_rows) {
-                size_type sellp_ind = slice_sets[slice] * slice_size + row;
-                for (size_type col = 0; col < num_cols; col++) {
-                    auto val = source->at(global_row, col);
-                    if (val != zero<ValueType>()) {
-                        col_idxs[sellp_ind] = col;
-                        vals[sellp_ind] = val;
-                        sellp_ind += slice_size;
-                    }
-                }
-                for (size_type i = sellp_ind;
-                     i <
-                     (slice_sets[slice] + slice_lengths[slice]) * slice_size +
-                         row;
-                     i += slice_size) {
-                    col_idxs[i] = 0;
-                    vals[i] = 0;
+    for (size_type slice = 0; slice < num_slices; slice++) {
+        for (size_type local_row = 0; local_row < slice_size; local_row++) {
+            const auto row = slice * slice_size + local_row;
+            if (row >= num_rows) {
+                break;
+            }
+            auto sellp_idx = slice_sets[slice] * slice_size + local_row;
+            const auto sellp_end =
+                slice_sets[slice + 1] * slice_size + local_row;
+            for (size_type col = 0; col < num_cols; col++) {
+                auto val = source->at(row, col);
+                if (is_nonzero(val)) {
+                    col_idxs[sellp_idx] = col;
+                    vals[sellp_idx] = val;
+                    sellp_idx += slice_size;
                 }
             }
+            for (; sellp_idx < sellp_end; sellp_idx += slice_size) {
+                col_idxs[sellp_idx] = invalid_index<IndexType>();
+                vals[sellp_idx] = zero<ValueType>();
+            }
         }
     }
-
-    if (slice_num > 0) {
-        slice_sets[slice_num] =
-            slice_sets[slice_num - 1] + slice_lengths[slice_num - 1];
-    }
 }
 
 GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
@@ -463,9 +405,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-void convert_to_sparsity_csr(std::shared_ptr<const OmpExecutor> exec,
-                             const matrix::Dense<ValueType> *source,
-                             matrix::SparsityCsr<ValueType, IndexType> *result)
+void convert_to_sparsity_csr(std::shared_ptr<const DefaultExecutor> exec,
+                             const matrix::Dense<ValueType>* source,
+                             matrix::SparsityCsr<ValueType, IndexType>* result)
 {
     auto num_rows = result->get_size()[0];
     auto num_cols = result->get_size()[1];
@@ -475,24 +417,12 @@ void convert_to_sparsity_csr(std::shared_ptr<const OmpExecutor> exec,
     auto value = result->get_value();
     value[0] = one<ValueType>();
 
-#pragma omp parallel for
-    for (size_type row = 0; row < num_rows; ++row) {
-        IndexType row_nnz{};
-        for (size_type col = 0; col < num_cols; ++col) {
-            auto val = source->at(row, col);
-            row_nnz += val != zero<ValueType>();
-        }
-        row_ptrs[row] = row_nnz;
-    }
-
-    components::prefix_sum(exec, row_ptrs, num_rows + 1);
-
 #pragma omp parallel for
     for (size_type row = 0; row < num_rows; ++row) {
         auto cur_ptr = row_ptrs[row];
         for (size_type col = 0; col < num_cols; ++col) {
             auto val = source->at(row, col);
-            if (val != zero<ValueType>()) {
+            if (is_nonzero(val)) {
                 col_idxs[cur_ptr] = col;
                 ++cur_ptr;
             }
@@ -505,108 +435,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType>
-void count_nonzeros(std::shared_ptr<const OmpExecutor> exec,
-                    const matrix::Dense<ValueType> *source, size_type *result)
-{
-    auto num_rows = source->get_size()[0];
-    auto num_cols = source->get_size()[1];
-    auto num_nonzeros = 0;
-
-#pragma omp parallel for reduction(+ : num_nonzeros)
-    for (size_type row = 0; row < num_rows; ++row) {
-        for (size_type col = 0; col < num_cols; ++col) {
-            num_nonzeros += (source->at(row, col) != zero<ValueType>());
-        }
-    }
-
-    *result = num_nonzeros;
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_COUNT_NONZEROS_KERNEL);
-
-
-template <typename ValueType>
-void calculate_max_nnz_per_row(std::shared_ptr<const OmpExecutor> exec,
-                               const matrix::Dense<ValueType> *source,
-                               size_type *result)
-{
-    const auto num_rows = source->get_size()[0];
-    const auto num_cols = source->get_size()[1];
-    size_type max_nonzeros_per_row = 0;
-#pragma omp parallel for reduction(max : max_nonzeros_per_row)
-    for (size_type row = 0; row < num_rows; ++row) {
-        size_type num_nonzeros = 0;
-        for (size_type col = 0; col < num_cols; ++col) {
-            num_nonzeros += (source->at(row, col) != zero<ValueType>());
-        }
-        max_nonzeros_per_row = std::max(num_nonzeros, max_nonzeros_per_row);
-    }
-    *result = max_nonzeros_per_row;
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
-    GKO_DECLARE_DENSE_CALCULATE_MAX_NNZ_PER_ROW_KERNEL);
-
-
-template <typename ValueType>
-void calculate_nonzeros_per_row(std::shared_ptr<const OmpExecutor> exec,
-                                const matrix::Dense<ValueType> *source,
-                                Array<size_type> *result)
-{
-    auto num_rows = source->get_size()[0];
-    auto num_cols = source->get_size()[1];
-    auto row_nnz_val = result->get_data();
-#pragma omp parallel for
-    for (size_type row = 0; row < num_rows; ++row) {
-        size_type num_nonzeros = 0;
-        for (size_type col = 0; col < num_cols; ++col) {
-            num_nonzeros += (source->at(row, col) != zero<ValueType>());
-        }
-        row_nnz_val[row] = num_nonzeros;
-    }
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
-    GKO_DECLARE_DENSE_CALCULATE_NONZEROS_PER_ROW_KERNEL);
-
-
-template <typename ValueType>
-void calculate_total_cols(std::shared_ptr<const OmpExecutor> exec,
-                          const matrix::Dense<ValueType> *source,
-                          size_type *result, size_type stride_factor,
-                          size_type slice_size)
-{
-    auto num_rows = source->get_size()[0];
-    auto num_cols = source->get_size()[1];
-    auto slice_num = ceildiv(num_rows, slice_size);
-    size_type total_cols = 0;
-#pragma omp parallel for reduction(+ : total_cols)
-    for (size_type slice = 0; slice < slice_num; slice++) {
-        size_type slice_temp = 0;
-        for (size_type row = 0;
-             row < slice_size && row + slice * slice_size < num_rows; row++) {
-            size_type temp = 0;
-            for (size_type col = 0; col < num_cols; col++) {
-                temp += (source->at(row + slice * slice_size, col) !=
-                         zero<ValueType>());
-            }
-            slice_temp = (slice_temp < temp) ? temp : slice_temp;
-        }
-        slice_temp = ceildiv(slice_temp, stride_factor) * stride_factor;
-        total_cols += slice_temp;
-    }
-
-    *result = total_cols;
-}
-
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
-    GKO_DECLARE_DENSE_CALCULATE_TOTAL_COLS_KERNEL);
-
-
-template <typename ValueType>
-void transpose(std::shared_ptr<const OmpExecutor> exec,
-               const matrix::Dense<ValueType> *orig,
-               matrix::Dense<ValueType> *trans)
+void transpose(std::shared_ptr<const DefaultExecutor> exec,
+               const matrix::Dense<ValueType>* orig,
+               matrix::Dense<ValueType>* trans)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < orig->get_size()[0]; ++i) {
@@ -620,9 +451,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_TRANSPOSE_KERNEL);
 
 
 template <typename ValueType>
-void conj_transpose(std::shared_ptr<const OmpExecutor> exec,
-                    const matrix::Dense<ValueType> *orig,
-                    matrix::Dense<ValueType> *trans)
+void conj_transpose(std::shared_ptr<const DefaultExecutor> exec,
+                    const matrix::Dense<ValueType>* orig,
+                    matrix::Dense<ValueType>* trans)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < orig->get_size()[0]; ++i) {
@@ -635,6 +466,37 @@ void conj_transpose(std::shared_ptr<const OmpExecutor> exec,
 GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_DENSE_CONJ_TRANSPOSE_KERNEL);
 
 
+template <typename ValueType, typename IndexType>
+void count_nonzero_blocks_per_row(std::shared_ptr<const DefaultExecutor> exec,
+                                  const matrix::Dense<ValueType>* source,
+                                  int bs, IndexType* result)
+{
+    const auto num_rows = source->get_size()[0];
+    const auto num_cols = source->get_size()[1];
+    const auto num_block_rows = num_rows / bs;
+    const auto num_block_cols = num_cols / bs;
+#pragma omp parallel for
+    for (size_type brow = 0; brow < num_block_rows; ++brow) {
+        IndexType num_nonzero_blocks{};
+        for (size_type bcol = 0; bcol < num_block_cols; ++bcol) {
+            bool block_nz = false;
+            for (int lrow = 0; lrow < bs; ++lrow) {
+                for (int lcol = 0; lcol < bs; ++lcol) {
+                    const auto row = lrow + bs * brow;
+                    const auto col = lcol + bs * bcol;
+                    block_nz = block_nz || is_nonzero(source->at(row, col));
+                }
+            }
+            num_nonzero_blocks += block_nz ? 1 : 0;
+        }
+        result[brow] = num_nonzero_blocks;
+    }
+}
+
+GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
+    GKO_DECLARE_DENSE_COUNT_NONZERO_BLOCKS_PER_ROW_KERNEL);
+
+
 }  // namespace dense
 }  // namespace omp
 }  // namespace kernels
