@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -42,6 +42,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <ginkgo/core/stop/stopping_status.hpp>
 
 
+#include "common/unified/base/kernel_launch_reduction.hpp"
 #include "core/solver/cb_gmres_accessor.hpp"
 
 
@@ -60,22 +61,18 @@ namespace {
 
 
 template <typename ValueType, typename Accessor3d>
-void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
+void finish_arnoldi_CGS(std::shared_ptr<const OmpExecutor> exec,
+                        matrix::Dense<ValueType>* next_krylov_basis,
                         Accessor3d krylov_bases,
-                        matrix::Dense<ValueType> *hessenberg_iter,
-                        matrix::Dense<ValueType> *buffer_iter,
-                        matrix::Dense<remove_complex<ValueType>> *arnoldi_norm,
-                        size_type iter, const stopping_status *stop_status)
+                        matrix::Dense<ValueType>* hessenberg_iter,
+                        matrix::Dense<ValueType>* buffer_iter,
+                        matrix::Dense<remove_complex<ValueType>>* arnoldi_norm,
+                        size_type iter, const stopping_status* stop_status)
 {
     using rc_vtype = remove_complex<ValueType>;
     constexpr bool has_scalar =
         gko::cb_gmres::detail::has_3d_scaled_accessor<Accessor3d>::value;
     const rc_vtype eta = 1.0 / sqrt(2.0);
-#pragma omp declare reduction(add:ValueType : omp_out = omp_out + omp_in)
-#pragma omp declare reduction(addnc:rc_vtype : omp_out = omp_out + omp_in)
-#pragma omp declare reduction(infnc:rc_vtype \
-                              : omp_out =    \
-                                    (omp_out >= omp_in ? omp_out : omp_in))
 
     for (size_type i = 0; i < next_krylov_basis->get_size()[1]; ++i) {
         if (stop_status[i].has_stopped()) {
@@ -83,18 +80,22 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
         }
 
         auto nrm = zero<rc_vtype>();
-#pragma omp parallel for reduction(addnc : nrm)
-        for (size_type j = 0; j < next_krylov_basis->get_size()[0]; ++j) {
-            nrm += squared_norm(next_krylov_basis->at(j, i));
-        }
+        run_kernel_reduction(
+            exec,
+            [](auto row, auto col, auto next_krylov_basis) {
+                return squared_norm(next_krylov_basis(row, col));
+            },
+            GKO_KERNEL_REDUCE_SUM(rc_vtype), &nrm,
+            next_krylov_basis->get_size()[0], static_cast<int64>(i),
+            next_krylov_basis);
         arnoldi_norm->at(0, i) = eta * sqrt(nrm);
         // nrmP = norm(next_krylov_basis)
 #pragma omp parallel for
         for (size_type k = 0; k < iter + 1; ++k) {
             ValueType hessenberg_iter_entry = zero<ValueType>();
             for (size_type j = 0; j < next_krylov_basis->get_size()[0]; ++j) {
-                hessenberg_iter_entry += next_krylov_basis->at(j, i) *
-                                         conj(ValueType{krylov_bases(k, j, i)});
+                hessenberg_iter_entry +=
+                    next_krylov_basis->at(j, i) * conj(krylov_bases(k, j, i));
             }
             hessenberg_iter->at(k, i) = hessenberg_iter_entry;
         }
@@ -113,13 +114,22 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
         // end
         nrm = zero<rc_vtype>();
         auto inf = zero<rc_vtype>();
-#pragma omp parallel for reduction(addnc : nrm) reduction(infnc : inf)
-        for (size_type j = 0; j < next_krylov_basis->get_size()[0]; ++j) {
-            nrm += squared_norm(next_krylov_basis->at(j, i));
-            inf = (inf >= abs(next_krylov_basis->at(j, i)))
-                      ? inf
-                      : abs(next_krylov_basis->at(j, i));
-        }
+        auto result_pair = std::make_pair(nrm, inf);
+        run_kernel_reduction(
+            exec,
+            [](auto row, auto col, auto next_krylov_basis) {
+                const auto val = next_krylov_basis(row, col);
+                return std::make_pair(squared_norm(val), abs(val));
+            },
+            [](auto a, auto b) {
+                return std::make_pair(a.first + b.first,
+                                      std::max(a.second, b.second));
+            },
+            [](auto a) { return a; }, std::make_pair(rc_vtype{}, rc_vtype{}),
+            &result_pair, next_krylov_basis->get_size()[0],
+            static_cast<int64>(i), next_krylov_basis);
+        nrm = result_pair.first;
+        inf = result_pair.second;
         arnoldi_norm->at(1, i) = sqrt(nrm);
         if (has_scalar) {
             arnoldi_norm->at(2, i) = inf;
@@ -135,9 +145,8 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
                 ValueType hessenberg_iter_entry = zero<ValueType>();
                 for (size_type j = 0; j < next_krylov_basis->get_size()[0];
                      ++j) {
-                    hessenberg_iter_entry +=
-                        next_krylov_basis->at(j, i) *
-                        conj(ValueType{krylov_bases(k, j, i)});
+                    hessenberg_iter_entry += next_krylov_basis->at(j, i) *
+                                             conj(krylov_bases(k, j, i));
                 }
                 buffer_iter->at(k, i) = hessenberg_iter_entry;
             }
@@ -149,8 +158,7 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
                 for (size_type j = 0; j < next_krylov_basis->get_size()[0];
                      ++j) {
                     next_krylov_basis->at(j, i) -=
-                        buffer_iter->at(k, i) *
-                        conj(ValueType{krylov_bases(k, j, i)});
+                        buffer_iter->at(k, i) * conj(krylov_bases(k, j, i));
                 }
             }
             // for i in 1:iter
@@ -158,13 +166,22 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
             // end
             nrm = zero<rc_vtype>();
             inf = zero<rc_vtype>();
-#pragma omp parallel for reduction(addnc : nrm) reduction(infnc : inf)
-            for (size_type j = 0; j < next_krylov_basis->get_size()[0]; ++j) {
-                nrm += squared_norm(next_krylov_basis->at(j, i));
-                inf = (inf >= abs(next_krylov_basis->at(j, i)))
-                          ? inf
-                          : abs(next_krylov_basis->at(j, i));
-            }
+            run_kernel_reduction(
+                exec,
+                [](auto row, auto col, auto next_krylov_basis) {
+                    const auto val = next_krylov_basis(row, col);
+                    return std::make_pair(squared_norm(val), abs(val));
+                },
+                [](auto a, auto b) {
+                    return std::make_pair(a.first + b.first,
+                                          std::max(a.second, b.second));
+                },
+                [](auto a) { return a; },
+                std::make_pair(rc_vtype{}, rc_vtype{}), &result_pair,
+                next_krylov_basis->get_size()[0], static_cast<int64>(i),
+                next_krylov_basis);
+            nrm = result_pair.first;
+            inf = result_pair.second;
             arnoldi_norm->at(1, i) = sqrt(nrm);
             if (has_scalar) {
                 arnoldi_norm->at(2, i) = inf;
@@ -191,12 +208,12 @@ void finish_arnoldi_CGS(matrix::Dense<ValueType> *next_krylov_basis,
 
 
 template <typename ValueType>
-void calculate_sin_and_cos(matrix::Dense<ValueType> *givens_sin,
-                           matrix::Dense<ValueType> *givens_cos,
-                           matrix::Dense<ValueType> *hessenberg_iter,
+void calculate_sin_and_cos(matrix::Dense<ValueType>* givens_sin,
+                           matrix::Dense<ValueType>* givens_cos,
+                           matrix::Dense<ValueType>* hessenberg_iter,
                            size_type iter, const size_type rhs)
 {
-    if (hessenberg_iter->at(iter, rhs) == zero<ValueType>()) {
+    if (is_zero(hessenberg_iter->at(iter, rhs))) {
         givens_cos->at(iter, rhs) = zero<ValueType>();
         givens_sin->at(iter, rhs) = one<ValueType>();
     } else {
@@ -213,10 +230,10 @@ void calculate_sin_and_cos(matrix::Dense<ValueType> *givens_sin,
 
 
 template <typename ValueType>
-void givens_rotation(matrix::Dense<ValueType> *givens_sin,
-                     matrix::Dense<ValueType> *givens_cos,
-                     matrix::Dense<ValueType> *hessenberg_iter, size_type iter,
-                     const stopping_status *stop_status)
+void givens_rotation(matrix::Dense<ValueType>* givens_sin,
+                     matrix::Dense<ValueType>* givens_cos,
+                     matrix::Dense<ValueType>* hessenberg_iter, size_type iter,
+                     const stopping_status* stop_status)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < hessenberg_iter->get_size()[1]; ++i) {
@@ -252,10 +269,10 @@ void givens_rotation(matrix::Dense<ValueType> *givens_sin,
 
 template <typename ValueType>
 void calculate_next_residual_norm(
-    matrix::Dense<ValueType> *givens_sin, matrix::Dense<ValueType> *givens_cos,
-    matrix::Dense<remove_complex<ValueType>> *residual_norm,
-    matrix::Dense<ValueType> *residual_norm_collection, size_type iter,
-    const stopping_status *stop_status)
+    matrix::Dense<ValueType>* givens_sin, matrix::Dense<ValueType>* givens_cos,
+    matrix::Dense<remove_complex<ValueType>>* residual_norm,
+    matrix::Dense<ValueType>* residual_norm_collection, size_type iter,
+    const stopping_status* stop_status)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < residual_norm->get_size()[1]; ++i) {
@@ -275,9 +292,9 @@ void calculate_next_residual_norm(
 
 template <typename ValueType>
 void solve_upper_triangular(
-    const matrix::Dense<ValueType> *residual_norm_collection,
-    const matrix::Dense<ValueType> *hessenberg, matrix::Dense<ValueType> *y,
-    const size_type *final_iter_nums)
+    const matrix::Dense<ValueType>* residual_norm_collection,
+    const matrix::Dense<ValueType>* hessenberg, matrix::Dense<ValueType>* y,
+    const size_type* final_iter_nums)
 {
 #pragma omp parallel for
     for (size_type k = 0; k < residual_norm_collection->get_size()[1]; ++k) {
@@ -299,9 +316,9 @@ void solve_upper_triangular(
 
 template <typename ValueType, typename ConstAccessor3d>
 void calculate_qy(ConstAccessor3d krylov_bases,
-                  const matrix::Dense<ValueType> *y,
-                  matrix::Dense<ValueType> *before_preconditioner,
-                  const size_type *final_iter_nums)
+                  const matrix::Dense<ValueType>* y,
+                  matrix::Dense<ValueType>* before_preconditioner,
+                  const size_type* final_iter_nums)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < before_preconditioner->get_size()[0]; ++i) {
@@ -320,12 +337,12 @@ void calculate_qy(ConstAccessor3d krylov_bases,
 
 
 template <typename ValueType>
-void initialize_1(std::shared_ptr<const OmpExecutor> exec,
-                  const matrix::Dense<ValueType> *b,
-                  matrix::Dense<ValueType> *residual,
-                  matrix::Dense<ValueType> *givens_sin,
-                  matrix::Dense<ValueType> *givens_cos,
-                  Array<stopping_status> *stop_status, size_type krylov_dim)
+void initialize(std::shared_ptr<const OmpExecutor> exec,
+                const matrix::Dense<ValueType>* b,
+                matrix::Dense<ValueType>* residual,
+                matrix::Dense<ValueType>* givens_sin,
+                matrix::Dense<ValueType>* givens_cos,
+                array<stopping_status>* stop_status, size_type krylov_dim)
 {
     using rc_vtype = remove_complex<ValueType>;
 
@@ -344,18 +361,19 @@ void initialize_1(std::shared_ptr<const OmpExecutor> exec,
     }
 }
 
-GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_CB_GMRES_INITIALIZE_1_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_CB_GMRES_INITIALIZE_KERNEL);
 
 
 template <typename ValueType, typename Accessor3d>
-void initialize_2(std::shared_ptr<const OmpExecutor> exec,
-                  const matrix::Dense<ValueType> *residual,
-                  matrix::Dense<remove_complex<ValueType>> *residual_norm,
-                  matrix::Dense<ValueType> *residual_norm_collection,
-                  matrix::Dense<remove_complex<ValueType>> *arnoldi_norm,
-                  Accessor3d krylov_bases,
-                  matrix::Dense<ValueType> *next_krylov_basis,
-                  Array<size_type> *final_iter_nums, size_type krylov_dim)
+void restart(std::shared_ptr<const OmpExecutor> exec,
+             const matrix::Dense<ValueType>* residual,
+             matrix::Dense<remove_complex<ValueType>>* residual_norm,
+             matrix::Dense<ValueType>* residual_norm_collection,
+             matrix::Dense<remove_complex<ValueType>>* arnoldi_norm,
+             Accessor3d krylov_bases,
+             matrix::Dense<ValueType>* next_krylov_basis,
+             array<size_type>* final_iter_nums, array<char>&,
+             size_type krylov_dim)
 {
     using rc_vtype = remove_complex<ValueType>;
     constexpr bool has_scalar =
@@ -366,18 +384,22 @@ void initialize_2(std::shared_ptr<const OmpExecutor> exec,
         auto res_norm = zero<rc_vtype>();
         auto res_inf = zero<rc_vtype>();
 
-#pragma omp declare reduction(addnc:rc_vtype : omp_out = omp_out + omp_in)
-#pragma omp declare reduction(infnc:rc_vtype \
-                              : omp_out =    \
-                                    (omp_out >= omp_in ? omp_out : omp_in))
-
-#pragma omp parallel for reduction(addnc : res_norm) reduction(infnc : res_inf)
-        for (size_type i = 0; i < residual->get_size()[0]; ++i) {
-            res_norm += squared_norm(residual->at(i, j));
-            res_inf = (res_inf >= abs(residual->at(i, j)))
-                          ? res_inf
-                          : abs(residual->at(i, j));
-        }
+        auto result_pair = std::make_pair(res_norm, res_inf);
+        run_kernel_reduction(
+            exec,
+            [](auto row, auto col, auto residual) {
+                const auto val = residual(row, col);
+                return std::make_pair(squared_norm(val), abs(val));
+            },
+            [](auto a, auto b) {
+                return std::make_pair(a.first + b.first,
+                                      std::max(a.second, b.second));
+            },
+            [](auto a) { return a; }, std::make_pair(rc_vtype{}, rc_vtype{}),
+            &result_pair, next_krylov_basis->get_size()[0],
+            static_cast<int64>(j), residual);
+        res_norm = result_pair.first;
+        res_inf = result_pair.second;
         residual_norm->at(0, j) = sqrt(res_norm);
         if (has_scalar) {
             arnoldi_norm->at(2, j) = res_inf;
@@ -418,23 +440,22 @@ void initialize_2(std::shared_ptr<const OmpExecutor> exec,
     }
 }
 
-GKO_INSTANTIATE_FOR_EACH_CB_GMRES_TYPE(
-    GKO_DECLARE_CB_GMRES_INITIALIZE_2_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_CB_GMRES_TYPE(GKO_DECLARE_CB_GMRES_RESTART_KERNEL);
 
 
 template <typename ValueType, typename Accessor3d>
-void step_1(std::shared_ptr<const OmpExecutor> exec,
-            matrix::Dense<ValueType> *next_krylov_basis,
-            matrix::Dense<ValueType> *givens_sin,
-            matrix::Dense<ValueType> *givens_cos,
-            matrix::Dense<remove_complex<ValueType>> *residual_norm,
-            matrix::Dense<ValueType> *residual_norm_collection,
-            Accessor3d krylov_bases, matrix::Dense<ValueType> *hessenberg_iter,
-            matrix::Dense<ValueType> *buffer_iter,
-            matrix::Dense<remove_complex<ValueType>> *arnoldi_norm,
-            size_type iter, Array<size_type> *final_iter_nums,
-            const Array<stopping_status> *stop_status, Array<stopping_status> *,
-            Array<size_type> *)
+void arnoldi(std::shared_ptr<const OmpExecutor> exec,
+             matrix::Dense<ValueType>* next_krylov_basis,
+             matrix::Dense<ValueType>* givens_sin,
+             matrix::Dense<ValueType>* givens_cos,
+             matrix::Dense<remove_complex<ValueType>>* residual_norm,
+             matrix::Dense<ValueType>* residual_norm_collection,
+             Accessor3d krylov_bases, matrix::Dense<ValueType>* hessenberg_iter,
+             matrix::Dense<ValueType>* buffer_iter,
+             matrix::Dense<remove_complex<ValueType>>* arnoldi_norm,
+             size_type iter, array<size_type>* final_iter_nums,
+             const array<stopping_status>* stop_status, array<stopping_status>*,
+             array<size_type>*)
 {
 #pragma omp parallel for
     for (size_type i = 0; i < final_iter_nums->get_num_elems(); ++i) {
@@ -442,7 +463,7 @@ void step_1(std::shared_ptr<const OmpExecutor> exec,
             (1 - static_cast<size_type>(
                      stop_status->get_const_data()[i].has_stopped()));
     }
-    finish_arnoldi_CGS(next_krylov_basis, krylov_bases, hessenberg_iter,
+    finish_arnoldi_CGS(exec, next_krylov_basis, krylov_bases, hessenberg_iter,
                        buffer_iter, arnoldi_norm, iter,
                        stop_status->get_const_data());
     givens_rotation(givens_sin, givens_cos, hessenberg_iter, iter,
@@ -452,17 +473,17 @@ void step_1(std::shared_ptr<const OmpExecutor> exec,
                                  stop_status->get_const_data());
 }
 
-GKO_INSTANTIATE_FOR_EACH_CB_GMRES_TYPE(GKO_DECLARE_CB_GMRES_STEP_1_KERNEL);
+GKO_INSTANTIATE_FOR_EACH_CB_GMRES_TYPE(GKO_DECLARE_CB_GMRES_ARNOLDI_KERNEL);
 
 
 template <typename ValueType, typename ConstAccessor3d>
-void step_2(std::shared_ptr<const OmpExecutor> exec,
-            const matrix::Dense<ValueType> *residual_norm_collection,
-            ConstAccessor3d krylov_bases,
-            const matrix::Dense<ValueType> *hessenberg,
-            matrix::Dense<ValueType> *y,
-            matrix::Dense<ValueType> *before_preconditioner,
-            const Array<size_type> *final_iter_nums)
+void solve_krylov(std::shared_ptr<const OmpExecutor> exec,
+                  const matrix::Dense<ValueType>* residual_norm_collection,
+                  ConstAccessor3d krylov_bases,
+                  const matrix::Dense<ValueType>* hessenberg,
+                  matrix::Dense<ValueType>* y,
+                  matrix::Dense<ValueType>* before_preconditioner,
+                  const array<size_type>* final_iter_nums)
 {
     solve_upper_triangular(residual_norm_collection, hessenberg, y,
                            final_iter_nums->get_const_data());
@@ -471,7 +492,7 @@ void step_2(std::shared_ptr<const OmpExecutor> exec,
 }
 
 GKO_INSTANTIATE_FOR_EACH_CB_GMRES_CONST_TYPE(
-    GKO_DECLARE_CB_GMRES_STEP_2_KERNEL);
+    GKO_DECLARE_CB_GMRES_SOLVE_KRYLOV_KERNEL);
 
 
 }  // namespace cb_gmres
