@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -58,8 +58,8 @@ namespace isai {
 
 
 template <typename IndexType, typename Callback>
-void forall_matching(const IndexType *fst, IndexType fst_size,
-                     const IndexType *snd, IndexType snd_size, Callback cb)
+void forall_matching(const IndexType* fst, IndexType fst_size,
+                     const IndexType* snd, IndexType snd_size, Callback cb)
 {
     IndexType fst_idx{};
     IndexType snd_idx{};
@@ -78,9 +78,9 @@ void forall_matching(const IndexType *fst, IndexType fst_size,
 
 template <typename ValueType, typename IndexType, typename Callable>
 void generic_generate(std::shared_ptr<const DefaultExecutor> exec,
-                      const matrix::Csr<ValueType, IndexType> *mtx,
-                      matrix::Csr<ValueType, IndexType> *inverse_mtx,
-                      IndexType *excess_rhs_ptrs, IndexType *excess_nz_ptrs,
+                      const matrix::Csr<ValueType, IndexType>* mtx,
+                      matrix::Csr<ValueType, IndexType>* inverse_mtx,
+                      IndexType* excess_rhs_ptrs, IndexType* excess_nz_ptrs,
                       Callable direct_solve, bool tri)
 {
     /*
@@ -105,10 +105,10 @@ void generic_generate(std::shared_ptr<const DefaultExecutor> exec,
     const auto i_cols = inverse_mtx->get_const_col_idxs();
     auto i_vals = inverse_mtx->get_values();
     // RHS for local dense system
-    gko::Array<ValueType> rhs_array{exec, row_size_limit};
+    gko::array<ValueType> rhs_array{exec, row_size_limit};
     auto rhs = rhs_array.get_data();
     // memory for dense dense system
-    gko::Array<ValueType> dense_system_array{exec,
+    gko::array<ValueType> dense_system_array{exec,
                                              row_size_limit * row_size_limit};
     auto dense_system_ptr = dense_system_array.get_data();
     // stores the next free index in the excess rhs/solution
@@ -139,18 +139,33 @@ void generic_generate(std::shared_ptr<const DefaultExecutor> exec,
                 const auto col = i_cols[i_begin + i];
                 const auto m_begin = m_row_ptrs[col];
                 const auto m_size = m_row_ptrs[col + 1] - m_begin;
+                // Loop over all matches that are within the sparsity pattern of
+                // the original matrix instead of the larger pattern of the
+                // inverse because any match outside the narrower sparsity
+                // pattern results in a zero entry
                 forall_matching(
                     m_cols + m_begin, m_size, i_cols + i_begin, i_size,
                     [&](IndexType, IndexType m_idx, IndexType i_idx) {
-                        if (m_cols[m_idx + m_begin] < row && col == row) {
-                            rhs_one_idx++;
-                        }
                         if (tri) {
                             dense_system(i, i_idx) = m_vals[m_idx + m_begin];
                         } else {
                             dense_system(i_idx, i) = m_vals[m_idx + m_begin];
                         }
                     });
+                const auto i_transposed_row_begin = i_row_ptrs[col];
+                const auto i_transposed_row_size =
+                    i_row_ptrs[col + 1] - i_transposed_row_begin;
+                // Loop over all matches that are within the sparsity pattern of
+                // the inverse
+                forall_matching(
+                    i_cols + i_transposed_row_begin, i_transposed_row_size,
+                    i_cols + i_begin, i_size,
+                    [&](IndexType, IndexType m_idx, IndexType i_idx) {
+                        if (i_cols[m_idx + i_transposed_row_begin] < row &&
+                            col == row) {
+                            rhs_one_idx++;
+                        }
+                    });
             }
 
             // solve dense triangular system
@@ -190,14 +205,14 @@ void generic_generate(std::shared_ptr<const DefaultExecutor> exec,
 
 template <typename ValueType, typename IndexType>
 void generate_tri_inverse(std::shared_ptr<const DefaultExecutor> exec,
-                          const matrix::Csr<ValueType, IndexType> *mtx,
-                          matrix::Csr<ValueType, IndexType> *inverse_mtx,
-                          IndexType *excess_rhs_ptrs, IndexType *excess_nz_ptrs,
+                          const matrix::Csr<ValueType, IndexType>* mtx,
+                          matrix::Csr<ValueType, IndexType>* inverse_mtx,
+                          IndexType* excess_rhs_ptrs, IndexType* excess_nz_ptrs,
                           bool lower)
 {
     auto trs_solve =
         [lower](const range<accessor::row_major<ValueType, 2>> trisystem,
-                ValueType *rhs, const IndexType) {
+                ValueType* rhs, const IndexType) {
             const IndexType size = trisystem.length(0);
             if (size <= 0) {
                 return;
@@ -240,7 +255,7 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 
 template <typename ValueType, typename IndexType>
-inline IndexType choose_pivot(IndexType block_size, const ValueType *block,
+inline IndexType choose_pivot(IndexType block_size, const ValueType* block,
                               size_type stride)
 {
     IndexType cp = 0;
@@ -255,7 +270,7 @@ inline IndexType choose_pivot(IndexType block_size, const ValueType *block,
 
 template <typename ValueType, typename IndexType>
 inline void swap_rows(IndexType row1, IndexType row2, IndexType block_size,
-                      ValueType *block, size_type stride)
+                      ValueType* block, size_type stride)
 {
     using std::swap;
     for (IndexType i = 0; i < block_size; ++i) {
@@ -266,15 +281,15 @@ inline void swap_rows(IndexType row1, IndexType row2, IndexType block_size,
 
 template <typename ValueType, typename IndexType>
 void generate_general_inverse(std::shared_ptr<const DefaultExecutor> exec,
-                              const matrix::Csr<ValueType, IndexType> *mtx,
-                              matrix::Csr<ValueType, IndexType> *inverse_mtx,
-                              IndexType *excess_rhs_ptrs,
-                              IndexType *excess_nz_ptrs, bool spd)
+                              const matrix::Csr<ValueType, IndexType>* mtx,
+                              matrix::Csr<ValueType, IndexType>* inverse_mtx,
+                              IndexType* excess_rhs_ptrs,
+                              IndexType* excess_nz_ptrs, bool spd)
 {
     using std::swap;
     auto general_solve = [spd](const range<accessor::row_major<ValueType, 2>>
                                    transposed_system_range,
-                               ValueType *rhs, const IndexType rhs_one_idx) {
+                               ValueType* rhs, const IndexType rhs_one_idx) {
         const IndexType size = transposed_system_range.length(0);
         if (size <= 0) {
             return;
@@ -336,12 +351,12 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType, typename IndexType>
 void generate_excess_system(std::shared_ptr<const DefaultExecutor>,
-                            const matrix::Csr<ValueType, IndexType> *input,
-                            const matrix::Csr<ValueType, IndexType> *inverse,
-                            const IndexType *excess_rhs_ptrs,
-                            const IndexType *excess_nz_ptrs,
-                            matrix::Csr<ValueType, IndexType> *excess_system,
-                            matrix::Dense<ValueType> *excess_rhs,
+                            const matrix::Csr<ValueType, IndexType>* input,
+                            const matrix::Csr<ValueType, IndexType>* inverse,
+                            const IndexType* excess_rhs_ptrs,
+                            const IndexType* excess_nz_ptrs,
+                            matrix::Csr<ValueType, IndexType>* excess_system,
+                            matrix::Dense<ValueType>* excess_rhs,
                             size_type e_start, size_type e_end)
 {
     const auto num_rows = input->get_size()[0];
@@ -399,8 +414,8 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType, typename IndexType>
 void scale_excess_solution(std::shared_ptr<const DefaultExecutor>,
-                           const IndexType *excess_block_ptrs,
-                           matrix::Dense<ValueType> *excess_solution,
+                           const IndexType* excess_block_ptrs,
+                           matrix::Dense<ValueType>* excess_solution,
                            size_type e_start, size_type e_end)
 {
     auto excess_values = excess_solution->get_values();
@@ -427,9 +442,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType, typename IndexType>
 void scatter_excess_solution(std::shared_ptr<const DefaultExecutor>,
-                             const IndexType *excess_block_ptrs,
-                             const matrix::Dense<ValueType> *excess_solution,
-                             matrix::Csr<ValueType, IndexType> *inverse,
+                             const IndexType* excess_block_ptrs,
+                             const matrix::Dense<ValueType>* excess_solution,
+                             matrix::Csr<ValueType, IndexType>* inverse,
                              size_type e_start, size_type e_end)
 {
     auto excess_values = excess_solution->get_const_values();
