@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -64,9 +64,9 @@ namespace {
 
 
 template <typename IndexType>
-inline bool has_same_nonzero_pattern(const IndexType *prev_row_ptr,
-                                     const IndexType *curr_row_ptr,
-                                     const IndexType *next_row_ptr)
+inline bool has_same_nonzero_pattern(const IndexType* prev_row_ptr,
+                                     const IndexType* curr_row_ptr,
+                                     const IndexType* next_row_ptr)
 {
     return std::distance(curr_row_ptr, next_row_ptr) ==
                std::distance(prev_row_ptr, curr_row_ptr) &&
@@ -75,8 +75,8 @@ inline bool has_same_nonzero_pattern(const IndexType *prev_row_ptr,
 
 
 template <typename ValueType, typename IndexType>
-size_type find_natural_blocks(const matrix::Csr<ValueType, IndexType> *mtx,
-                              uint32 max_block_size, IndexType *block_ptrs)
+size_type find_natural_blocks(const matrix::Csr<ValueType, IndexType>* mtx,
+                              uint32 max_block_size, IndexType* block_ptrs)
 {
     const auto rows = mtx->get_size()[0];
     const auto row_ptrs = mtx->get_const_row_ptrs();
@@ -110,7 +110,7 @@ size_type find_natural_blocks(const matrix::Csr<ValueType, IndexType> *mtx,
 template <typename IndexType>
 inline size_type agglomerate_supervariables(uint32 max_block_size,
                                             size_type num_natural_blocks,
-                                            IndexType *block_ptrs)
+                                            IndexType* block_ptrs)
 {
     if (num_natural_blocks == 0) {
         return 0;
@@ -137,9 +137,9 @@ inline size_type agglomerate_supervariables(uint32 max_block_size,
 
 template <typename ValueType, typename IndexType>
 void find_blocks(std::shared_ptr<const DefaultExecutor> exec,
-                 const matrix::Csr<ValueType, IndexType> *system_matrix,
-                 uint32 max_block_size, size_type &num_blocks,
-                 Array<IndexType> &block_pointers)
+                 const matrix::Csr<ValueType, IndexType>* system_matrix,
+                 uint32 max_block_size, size_type& num_blocks,
+                 array<IndexType>& block_pointers)
 {
     num_blocks = find_natural_blocks(system_matrix, max_block_size,
                                      block_pointers.get_data());
@@ -155,9 +155,9 @@ namespace {
 
 
 template <typename ValueType, typename IndexType>
-inline void extract_block(const matrix::Csr<ValueType, IndexType> *mtx,
+inline void extract_block(const matrix::Csr<ValueType, IndexType>* mtx,
                           IndexType block_size, IndexType block_start,
-                          ValueType *block, size_type stride)
+                          ValueType* block, size_type stride)
 {
     for (int i = 0; i < block_size; ++i) {
         for (int j = 0; j < block_size; ++j) {
@@ -181,7 +181,7 @@ inline void extract_block(const matrix::Csr<ValueType, IndexType> *mtx,
 
 
 template <typename ValueType, typename IndexType>
-inline IndexType choose_pivot(IndexType block_size, const ValueType *block,
+inline IndexType choose_pivot(IndexType block_size, const ValueType* block,
                               size_type stride)
 {
     IndexType cp = 0;
@@ -196,7 +196,7 @@ inline IndexType choose_pivot(IndexType block_size, const ValueType *block,
 
 template <typename ValueType, typename IndexType>
 inline void swap_rows(IndexType row1, IndexType row2, IndexType block_size,
-                      ValueType *block, size_type stride)
+                      ValueType* block, size_type stride)
 {
     using std::swap;
     for (IndexType i = 0; i < block_size; ++i) {
@@ -207,11 +207,11 @@ inline void swap_rows(IndexType row1, IndexType row2, IndexType block_size,
 
 template <typename ValueType, typename IndexType>
 inline bool apply_gauss_jordan_transform(IndexType row, IndexType col,
-                                         IndexType block_size, ValueType *block,
+                                         IndexType block_size, ValueType* block,
                                          size_type stride)
 {
     const auto d = block[row * stride + col];
-    if (d == zero<ValueType>()) {
+    if (is_zero(d)) {
         return false;
     }
     for (IndexType i = 0; i < block_size; ++i) {
@@ -236,8 +236,8 @@ template <typename SourceValueType, typename ResultValueType,
           typename IndexType,
           typename ValueConverter =
               default_converter<SourceValueType, ResultValueType>>
-inline void transpose_block(IndexType block_size, const SourceValueType *from,
-                            size_type from_stride, ResultValueType *to,
+inline void transpose_block(IndexType block_size, const SourceValueType* from,
+                            size_type from_stride, ResultValueType* to,
                             size_type to_stride,
                             ValueConverter converter = {}) noexcept
 {
@@ -254,8 +254,8 @@ template <typename SourceValueType, typename ResultValueType,
           typename ValueConverter =
               default_converter<SourceValueType, ResultValueType>>
 inline void conj_transpose_block(IndexType block_size,
-                                 const SourceValueType *from,
-                                 size_type from_stride, ResultValueType *to,
+                                 const SourceValueType* from,
+                                 size_type from_stride, ResultValueType* to,
                                  size_type to_stride,
                                  ValueConverter converter = {}) noexcept
 {
@@ -272,10 +272,10 @@ template <typename SourceValueType, typename ResultValueType,
           typename ValueConverter =
               default_converter<SourceValueType, ResultValueType>>
 inline void permute_and_transpose_block(IndexType block_size,
-                                        const IndexType *col_perm,
-                                        const SourceValueType *source,
+                                        const IndexType* col_perm,
+                                        const SourceValueType* source,
                                         size_type source_stride,
-                                        ResultValueType *result,
+                                        ResultValueType* result,
                                         size_type result_stride,
                                         ValueConverter converter = {})
 {
@@ -289,8 +289,8 @@ inline void permute_and_transpose_block(IndexType block_size,
 
 
 template <typename ValueType, typename IndexType>
-inline bool invert_block(IndexType block_size, IndexType *perm,
-                         ValueType *block, size_type stride)
+inline bool invert_block(IndexType block_size, IndexType* perm,
+                         ValueType* block, size_type stride)
 {
     using std::swap;
     for (IndexType k = 0; k < block_size; ++k) {
@@ -311,7 +311,7 @@ inline bool invert_block(IndexType block_size, IndexType *perm,
 template <typename ReducedType, typename ValueType, typename IndexType>
 inline bool validate_precision_reduction_feasibility(
     std::shared_ptr<const DefaultExecutor> exec, IndexType block_size,
-    const ValueType *block, size_type stride)
+    const ValueType* block, size_type stride)
 {
     using gko::detail::float_traits;
     vector<ValueType> tmp(block_size * block_size, {}, exec);
@@ -332,7 +332,9 @@ inline bool validate_precision_reduction_feasibility(
     }
     cond *= compute_inf_norm(block_size, block_size, tmp.data(), block_size);
     return cond >= 1.0 &&
-           cond * float_traits<remove_complex<ValueType>>::eps < 1e-3;
+           cond * static_cast<remove_complex<ValueType>>(
+                      float_traits<remove_complex<ValueType>>::eps) <
+               remove_complex<ValueType>{1e-3};
 }
 
 
@@ -341,22 +343,22 @@ inline bool validate_precision_reduction_feasibility(
 
 template <typename ValueType, typename IndexType>
 void generate(std::shared_ptr<const DefaultExecutor> exec,
-              const matrix::Csr<ValueType, IndexType> *system_matrix,
+              const matrix::Csr<ValueType, IndexType>* system_matrix,
               size_type num_blocks, uint32 max_block_size,
               remove_complex<ValueType> accuracy,
-              const preconditioner::block_interleaved_storage_scheme<IndexType>
-                  &storage_scheme,
-              Array<remove_complex<ValueType>> &conditioning,
-              Array<precision_reduction> &block_precisions,
-              const Array<IndexType> &block_pointers, Array<ValueType> &blocks)
+              const preconditioner::block_interleaved_storage_scheme<IndexType>&
+                  storage_scheme,
+              array<remove_complex<ValueType>>& conditioning,
+              array<precision_reduction>& block_precisions,
+              const array<IndexType>& block_pointers, array<ValueType>& blocks)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_data();
     const auto group_size = storage_scheme.get_group_size();
     const auto cond = conditioning.get_data();
     for (size_type g = 0; g < num_blocks; g += group_size) {
-        vector<Array<ValueType>> block(group_size, {}, exec);
-        vector<Array<IndexType>> perm(group_size, {}, exec);
+        vector<array<ValueType>> block(group_size, {}, exec);
+        vector<array<IndexType>> perm(group_size, {}, exec);
         vector<uint32> pr_descriptors(group_size, uint32{} - 1, exec);
         // extract group of blocks, invert them, figure out storage precision
         for (IndexType b = 0; b < group_size; ++b) {
@@ -364,8 +366,8 @@ void generate(std::shared_ptr<const DefaultExecutor> exec,
                 break;
             }
             const auto block_size = ptrs[g + b + 1] - ptrs[g + b];
-            block[b] = Array<ValueType>(exec, block_size * block_size);
-            perm[b] = Array<IndexType>(exec, block_size);
+            block[b] = array<ValueType>(exec, block_size * block_size);
+            perm[b] = array<IndexType>(exec, block_size);
             std::iota(perm[b].get_data(), perm[b].get_data() + block_size,
                       IndexType(0));
             extract_block(system_matrix, block_size, ptrs[g + b],
@@ -426,7 +428,7 @@ void generate(std::shared_ptr<const DefaultExecutor> exec,
                 permute_and_transpose_block(
                     block_size, perm[b].get_data(), block[b].get_data(),
                     block_size,
-                    reinterpret_cast<resolved_precision *>(
+                    reinterpret_cast<resolved_precision*>(
                         blocks.get_data() +
                         storage_scheme.get_group_offset(g + b)) +
                         storage_scheme.get_block_offset(g + b),
@@ -446,12 +448,12 @@ template <
     typename ValueType, typename BlockValueType,
     typename ValueConverter = default_converter<BlockValueType, ValueType>>
 inline void apply_block(size_type block_size, size_type num_rhs,
-                        const BlockValueType *block, size_type stride,
-                        ValueType alpha, const ValueType *b, size_type stride_b,
-                        ValueType beta, ValueType *x, size_type stride_x,
+                        const BlockValueType* block, size_type stride,
+                        ValueType alpha, const ValueType* b, size_type stride_b,
+                        ValueType beta, ValueType* x, size_type stride_x,
                         ValueConverter converter = {})
 {
-    if (beta != zero<ValueType>()) {
+    if (is_nonzero(beta)) {
         for (size_type row = 0; row < block_size; ++row) {
             for (size_type col = 0; col < num_rhs; ++col) {
                 x[row * stride_x + col] *= beta;
@@ -481,8 +483,8 @@ inline void apply_block(size_type block_size, size_type num_rhs,
 
 
 void initialize_precisions(std::shared_ptr<const DefaultExecutor> exec,
-                           const Array<precision_reduction> &source,
-                           Array<precision_reduction> &precisions)
+                           const array<precision_reduction>& source,
+                           array<precision_reduction>& precisions)
 {
     const auto source_size = source.get_num_elems();
     for (auto i = 0u; i < precisions.get_num_elems(); ++i) {
@@ -494,14 +496,14 @@ void initialize_precisions(std::shared_ptr<const DefaultExecutor> exec,
 template <typename ValueType, typename IndexType>
 void apply(std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
            uint32 max_block_size,
-           const preconditioner::block_interleaved_storage_scheme<IndexType>
-               &storage_scheme,
-           const Array<precision_reduction> &block_precisions,
-           const Array<IndexType> &block_pointers,
-           const Array<ValueType> &blocks,
-           const matrix::Dense<ValueType> *alpha,
-           const matrix::Dense<ValueType> *b,
-           const matrix::Dense<ValueType> *beta, matrix::Dense<ValueType> *x)
+           const preconditioner::block_interleaved_storage_scheme<IndexType>&
+               storage_scheme,
+           const array<precision_reduction>& block_precisions,
+           const array<IndexType>& block_pointers,
+           const array<ValueType>& blocks,
+           const matrix::Dense<ValueType>* alpha,
+           const matrix::Dense<ValueType>* b,
+           const matrix::Dense<ValueType>* beta, matrix::Dense<ValueType>* x)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_const_data();
@@ -515,7 +517,7 @@ void apply(std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
         GKO_PRECONDITIONER_JACOBI_RESOLVE_PRECISION(
             ValueType, p,
             apply_block(block_size, b->get_size()[1],
-                        reinterpret_cast<const resolved_precision *>(group) +
+                        reinterpret_cast<const resolved_precision*>(group) +
                             storage_scheme.get_block_offset(i),
                         storage_scheme.get_stride(), alpha->at(0, 0), block_b,
                         b->get_stride(), beta->at(0, 0), block_x,
@@ -530,11 +532,11 @@ template <typename ValueType, typename IndexType>
 void simple_apply(
     std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
     uint32 max_block_size,
-    const preconditioner::block_interleaved_storage_scheme<IndexType>
-        &storage_scheme,
-    const Array<precision_reduction> &block_precisions,
-    const Array<IndexType> &block_pointers, const Array<ValueType> &blocks,
-    const matrix::Dense<ValueType> *b, matrix::Dense<ValueType> *x)
+    const preconditioner::block_interleaved_storage_scheme<IndexType>&
+        storage_scheme,
+    const array<precision_reduction>& block_precisions,
+    const array<IndexType>& block_pointers, const array<ValueType>& blocks,
+    const matrix::Dense<ValueType>* b, matrix::Dense<ValueType>* x)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_const_data();
@@ -548,7 +550,7 @@ void simple_apply(
         GKO_PRECONDITIONER_JACOBI_RESOLVE_PRECISION(
             ValueType, p,
             apply_block(block_size, b->get_size()[1],
-                        reinterpret_cast<const resolved_precision *>(group) +
+                        reinterpret_cast<const resolved_precision*>(group) +
                             storage_scheme.get_block_offset(i),
                         storage_scheme.get_stride(), one<ValueType>(), block_b,
                         b->get_stride(), zero<ValueType>(), block_x,
@@ -562,11 +564,11 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType>
 void scalar_apply(std::shared_ptr<const DefaultExecutor> exec,
-                  const Array<ValueType> &diag,
-                  const matrix::Dense<ValueType> *alpha,
-                  const matrix::Dense<ValueType> *b,
-                  const matrix::Dense<ValueType> *beta,
-                  matrix::Dense<ValueType> *x)
+                  const array<ValueType>& diag,
+                  const matrix::Dense<ValueType>* alpha,
+                  const matrix::Dense<ValueType>* b,
+                  const matrix::Dense<ValueType>* beta,
+                  matrix::Dense<ValueType>* x)
 {
     for (size_type i = 0; i < x->get_size()[0]; ++i) {
         for (size_type j = 0; j < x->get_size()[1]; ++j) {
@@ -581,9 +583,9 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_JACOBI_SCALAR_APPLY_KERNEL);
 
 template <typename ValueType>
 void simple_scalar_apply(std::shared_ptr<const DefaultExecutor> exec,
-                         const Array<ValueType> &diag,
-                         const matrix::Dense<ValueType> *b,
-                         matrix::Dense<ValueType> *x)
+                         const array<ValueType>& diag,
+                         const matrix::Dense<ValueType>* b,
+                         matrix::Dense<ValueType>* x)
 {
     for (size_type i = 0; i < x->get_size()[0]; ++i) {
         for (size_type j = 0; j < x->get_size()[1]; ++j) {
@@ -598,7 +600,7 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 
 template <typename ValueType>
 void scalar_conj(std::shared_ptr<const DefaultExecutor> exec,
-                 const Array<ValueType> &diag, Array<ValueType> &conj_diag)
+                 const array<ValueType>& diag, array<ValueType>& conj_diag)
 {
     for (size_type i = 0; i < diag.get_num_elems(); ++i) {
         conj_diag.get_data()[i] = conj(diag.get_const_data()[i]);
@@ -610,10 +612,10 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_JACOBI_SCALAR_CONJ_KERNEL);
 
 template <typename ValueType>
 void invert_diagonal(std::shared_ptr<const DefaultExecutor> exec,
-                     const Array<ValueType> &diag, Array<ValueType> &inv_diag)
+                     const array<ValueType>& diag, array<ValueType>& inv_diag)
 {
     for (size_type i = 0; i < diag.get_num_elems(); ++i) {
-        auto diag_val = diag.get_const_data()[i] == zero<ValueType>()
+        auto diag_val = is_zero(diag.get_const_data()[i])
                             ? one<ValueType>()
                             : diag.get_const_data()[i];
         inv_diag.get_data()[i] = one<ValueType>() / diag_val;
@@ -626,11 +628,11 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(GKO_DECLARE_JACOBI_INVERT_DIAGONAL_KERNEL);
 template <typename ValueType, typename IndexType>
 void transpose_jacobi(
     std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
-    uint32 max_block_size, const Array<precision_reduction> &block_precisions,
-    const Array<IndexType> &block_pointers, const Array<ValueType> &blocks,
-    const preconditioner::block_interleaved_storage_scheme<IndexType>
-        &storage_scheme,
-    Array<ValueType> &out_blocks)
+    uint32 max_block_size, const array<precision_reduction>& block_precisions,
+    const array<IndexType>& block_pointers, const array<ValueType>& blocks,
+    const preconditioner::block_interleaved_storage_scheme<IndexType>&
+        storage_scheme,
+    array<ValueType>& out_blocks)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_const_data();
@@ -648,9 +650,9 @@ void transpose_jacobi(
             ValueType, p,
             transpose_block(
                 block_size,
-                reinterpret_cast<const resolved_precision *>(group) + block_ofs,
+                reinterpret_cast<const resolved_precision*>(group) + block_ofs,
                 block_stride,
-                reinterpret_cast<resolved_precision *>(out_group) + block_ofs,
+                reinterpret_cast<resolved_precision*>(out_group) + block_ofs,
                 block_stride));
     }
 }
@@ -662,11 +664,11 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 template <typename ValueType, typename IndexType>
 void conj_transpose_jacobi(
     std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
-    uint32 max_block_size, const Array<precision_reduction> &block_precisions,
-    const Array<IndexType> &block_pointers, const Array<ValueType> &blocks,
-    const preconditioner::block_interleaved_storage_scheme<IndexType>
-        &storage_scheme,
-    Array<ValueType> &out_blocks)
+    uint32 max_block_size, const array<precision_reduction>& block_precisions,
+    const array<IndexType>& block_pointers, const array<ValueType>& blocks,
+    const preconditioner::block_interleaved_storage_scheme<IndexType>&
+        storage_scheme,
+    array<ValueType>& out_blocks)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_const_data();
@@ -684,9 +686,9 @@ void conj_transpose_jacobi(
             ValueType, p,
             conj_transpose_block(
                 block_size,
-                reinterpret_cast<const resolved_precision *>(group) + block_ofs,
+                reinterpret_cast<const resolved_precision*>(group) + block_ofs,
                 block_stride,
-                reinterpret_cast<resolved_precision *>(out_group) + block_ofs,
+                reinterpret_cast<resolved_precision*>(out_group) + block_ofs,
                 block_stride));
     }
 }
@@ -697,8 +699,8 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_AND_INDEX_TYPE(
 
 template <typename ValueType>
 void scalar_convert_to_dense(std::shared_ptr<const DefaultExecutor> exec,
-                             const Array<ValueType> &blocks,
-                             matrix::Dense<ValueType> *result)
+                             const array<ValueType>& blocks,
+                             matrix::Dense<ValueType>* result)
 {
     auto matrix_size = result->get_size();
     for (size_type i = 0; i < matrix_size[0]; ++i) {
@@ -718,11 +720,11 @@ GKO_INSTANTIATE_FOR_EACH_VALUE_TYPE(
 template <typename ValueType, typename IndexType>
 void convert_to_dense(
     std::shared_ptr<const DefaultExecutor> exec, size_type num_blocks,
-    const Array<precision_reduction> &block_precisions,
-    const Array<IndexType> &block_pointers, const Array<ValueType> &blocks,
-    const preconditioner::block_interleaved_storage_scheme<IndexType>
-        &storage_scheme,
-    ValueType *result_values, size_type result_stride)
+    const array<precision_reduction>& block_precisions,
+    const array<IndexType>& block_pointers, const array<ValueType>& blocks,
+    const preconditioner::block_interleaved_storage_scheme<IndexType>&
+        storage_scheme,
+    ValueType* result_values, size_type result_stride)
 {
     const auto ptrs = block_pointers.get_const_data();
     const auto prec = block_precisions.get_const_data();
@@ -740,13 +742,12 @@ void convert_to_dense(
         const auto p = prec ? prec[i] : precision_reduction();
         GKO_PRECONDITIONER_JACOBI_RESOLVE_PRECISION(
             ValueType, p,
-            transpose_block(
-                block_size,
-                reinterpret_cast<const resolved_precision *>(group) +
-                    storage_scheme.get_block_offset(i),
-                storage_scheme.get_stride(),
-                result_values + ptrs[i] * result_stride + ptrs[i],
-                result_stride));
+            transpose_block(block_size,
+                            reinterpret_cast<const resolved_precision*>(group) +
+                                storage_scheme.get_block_offset(i),
+                            storage_scheme.get_stride(),
+                            result_values + ptrs[i] * result_stride + ptrs[i],
+                            result_stride));
     }
 }
 
