@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -60,10 +60,10 @@ public:
     bool apply_uses_initial_guess() const override { return true; }
 
 protected:
-    void apply_impl(const gko::LinOp *b, gko::LinOp *x) const override {}
+    void apply_impl(const gko::LinOp* b, gko::LinOp* x) const override {}
 
-    void apply_impl(const gko::LinOp *alpha, const gko::LinOp *b,
-                    const gko::LinOp *beta, gko::LinOp *x) const override
+    void apply_impl(const gko::LinOp* alpha, const gko::LinOp* b,
+                    const gko::LinOp* beta, gko::LinOp* x) const override
     {}
 
     explicit DummyLinOp(std::shared_ptr<const gko::Executor> exec)
@@ -108,7 +108,48 @@ protected:
     std::shared_ptr<Mtx> product;
 };
 
-TYPED_TEST_SUITE(Composition, gko::test::ValueTypes);
+TYPED_TEST_SUITE(Composition, gko::test::ValueTypes, TypenameNameGenerator);
+
+
+TYPED_TEST(Composition, CopiesOnSameExecutor)
+{
+    using Mtx = typename TestFixture::Mtx;
+    auto cmp = gko::Composition<TypeParam>::create(this->operators[0],
+                                                   this->operators[1]);
+    auto out = cmp->create_default();
+
+    cmp->convert_to(out);
+
+    ASSERT_EQ(out->get_size(), cmp->get_size());
+    ASSERT_EQ(out->get_executor(), cmp->get_executor());
+    ASSERT_EQ(out->get_operators().size(), cmp->get_operators().size());
+    ASSERT_EQ(out->get_operators().size(), 2);
+    ASSERT_EQ(out->get_operators()[0], cmp->get_operators()[0]);
+    ASSERT_EQ(out->get_operators()[1], cmp->get_operators()[1]);
+}
+
+
+TYPED_TEST(Composition, MovesOnSameExecutor)
+{
+    using Mtx = typename TestFixture::Mtx;
+    auto cmp = gko::Composition<TypeParam>::create(this->operators[0],
+                                                   this->operators[1]);
+    auto cmp2 = cmp->clone();
+    auto out = cmp->create_default();
+
+    cmp->move_to(out);
+
+    ASSERT_EQ(out->get_size(), cmp2->get_size());
+    ASSERT_EQ(out->get_executor(), cmp2->get_executor());
+    ASSERT_EQ(out->get_operators().size(), cmp2->get_operators().size());
+    ASSERT_EQ(out->get_operators().size(), 2);
+    ASSERT_EQ(out->get_operators()[0], cmp2->get_operators()[0]);
+    ASSERT_EQ(out->get_operators()[1], cmp2->get_operators()[1]);
+    // empty size, empty operators, same executor
+    ASSERT_EQ(cmp->get_size(), gko::dim<2>{});
+    ASSERT_EQ(cmp->get_executor(), cmp2->get_executor());
+    ASSERT_EQ(cmp->get_operators().size(), 0);
+}
 
 
 TYPED_TEST(Composition, AppliesSingleToVector)
@@ -122,7 +163,7 @@ TYPED_TEST(Composition, AppliesSingleToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({-13.0, 79.0}), r<TypeParam>::value);
 }
@@ -140,7 +181,7 @@ TYPED_TEST(Composition, AppliesSingleToMixedVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({-13.0, 79.0}),
                         (r_mixed<value_type, TypeParam>()));
@@ -160,7 +201,7 @@ TYPED_TEST(Composition, AppliesSingleToComplexVector)
         {value_type{1.0, -2.0}, value_type{2.0, -4.0}}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res,
                         l({value_type{-13.0, 26.0}, value_type{79.0, -158.0}}),
@@ -181,7 +222,7 @@ TYPED_TEST(Composition, AppliesSingleToMixedComplexVector)
         {value_type{1.0, -2.0}, value_type{2.0, -4.0}}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res,
                         l({value_type{-13.0, 26.0}, value_type{79.0, -158.0}}),
@@ -202,7 +243,7 @@ TYPED_TEST(Composition, AppliesSingleLinearCombinationToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({-40.0, 235.0}), r<TypeParam>::value);
 }
@@ -222,7 +263,7 @@ TYPED_TEST(Composition, AppliesSingleLinearCombinationToMixedVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({-40.0, 235.0}),
                         (r_mixed<value_type, TypeParam>()));
@@ -245,7 +286,7 @@ TYPED_TEST(Composition, AppliesSingleLinearCombinationToComplexVector)
         {value_type{1.0, -2.0}, value_type{2.0, -4.0}}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res,
                         l({value_type{-40.0, 80.0}, value_type{235.0, -470.0}}),
@@ -269,7 +310,7 @@ TYPED_TEST(Composition, AppliesSingleLinearCombinationToMixedComplexVector)
         {value_type{1.0, -2.0}, value_type{2.0, -4.0}}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res,
                         l({value_type{-40.0, 80.0}, value_type{235.0, -470.0}}),
@@ -289,7 +330,7 @@ TYPED_TEST(Composition, AppliesToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({14.0, 7.0}), r<TypeParam>::value);
 }
@@ -309,7 +350,7 @@ TYPED_TEST(Composition, AppliesLinearCombinationToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({41.0, 19.0}), r<TypeParam>::value);
 }
@@ -327,7 +368,7 @@ TYPED_TEST(Composition, AppliesLongerToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({238.0, 119.0}), r<TypeParam>::value);
 }
@@ -347,7 +388,7 @@ TYPED_TEST(Composition, AppliesLongerLinearCombinationToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({713.0, 355.0}), r<TypeParam>::value);
 }
@@ -366,7 +407,7 @@ TYPED_TEST(Composition, AppliesLongestToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({238.0, 119.0}), r<TypeParam>::value);
 }
@@ -387,7 +428,7 @@ TYPED_TEST(Composition, AppliesLongestLinearCombinationToVector)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({713.0, 355.0}), r<TypeParam>::value);
 }
@@ -406,7 +447,7 @@ TYPED_TEST(Composition, AppliesLongestToVectorMultipleRhs)
     auto x = clone(this->identity);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({{54.0, 92.0}, {27.0, 46.0}}),
                         r<TypeParam>::value);
@@ -428,7 +469,7 @@ TYPED_TEST(Composition, AppliesLongestLinearCombinationToVectorMultipleRhs)
     auto x = clone(this->identity);
     auto res = clone(x);
 
-    cmp->apply(lend(alpha), lend(x), lend(beta), lend(res));
+    cmp->apply(alpha, x, beta, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({{161.0, 276.0}, {81.0, 137.0}}),
                         r<TypeParam>::value);
@@ -449,7 +490,7 @@ TYPED_TEST(Composition, AppliesToVectorWithInitialGuess)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({1.0, 2.0}), 0);
 }
@@ -470,7 +511,7 @@ TYPED_TEST(Composition, AppliesToVectorWithInitialGuess2)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({0.0, 0.0}), 0);
 }
@@ -489,7 +530,7 @@ TYPED_TEST(Composition, AppliesToVectorWithInitialGuess3)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({1.0, 2.0}), 0);
 }
@@ -510,7 +551,7 @@ TYPED_TEST(Composition, AppliesToVectorWithInitialGuess4)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({0.0, 0.0}), 0);
 }
@@ -531,7 +572,7 @@ TYPED_TEST(Composition, AppliesToVectorWithInitialGuess5)
     auto x = gko::initialize<Mtx>({1.0, 2.0}, this->exec);
     auto res = clone(x);
 
-    cmp->apply(lend(x), lend(res));
+    cmp->apply(x, res);
 
     GKO_ASSERT_MTX_NEAR(res, l({1.0, 2.0}), 0);
 }
