@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -71,26 +71,30 @@ protected:
           dense1(gko::initialize<Dense>(4, {{1.0, 2.0, 3.0}, {1.5, 2.5, 3.5}},
                                         exec)),
           dense2(gko::initialize<Dense>(4, {{1.0, 2.0, 3.0}, {1.5, 2.5, 3.5}},
+                                        exec)),
+          dense3(gko::initialize<Dense>(4, {{2.0, 3.0, 4.0}, {3.0, 4.5, 6.0}},
                                         exec))
     {
         csr1 = Csr::create(exec);
-        csr1->copy_from(dense1.get());
+        csr1->copy_from(dense1);
         csr2 = Csr::create(exec);
-        csr2->copy_from(dense2.get());
+        csr2->copy_from(dense2);
+        csr3 = Csr::create(exec);
+        csr3->copy_from(dense3);
         this->create_diag1(diag1.get());
         this->create_diag2(diag2.get());
     }
 
-    void create_diag1(Diag *d)
+    void create_diag1(Diag* d)
     {
-        auto *v = d->get_values();
+        auto* v = d->get_values();
         v[0] = 2.0;
         v[1] = 3.0;
     }
 
-    void create_diag2(Diag *d)
+    void create_diag2(Diag* d)
     {
-        auto *v = d->get_values();
+        auto* v = d->get_values();
         v[0] = 2.0;
         v[1] = 3.0;
         v[2] = 4.0;
@@ -99,13 +103,15 @@ protected:
     std::shared_ptr<const gko::Executor> exec;
     std::unique_ptr<Csr> csr1;
     std::unique_ptr<Csr> csr2;
+    std::unique_ptr<Csr> csr3;
     std::unique_ptr<Diag> diag1;
     std::unique_ptr<Diag> diag2;
     std::unique_ptr<Dense> dense1;
     std::unique_ptr<Dense> dense2;
+    std::unique_ptr<Dense> dense3;
 };
 
-TYPED_TEST_SUITE(Diagonal, gko::test::ValueTypes);
+TYPED_TEST_SUITE(Diagonal, gko::test::ValueTypes, TypenameNameGenerator);
 
 
 TYPED_TEST(Diagonal, ConvertsToPrecision)
@@ -121,8 +127,8 @@ TYPED_TEST(Diagonal, ConvertsToPrecision)
                         ? gko::remove_complex<ValueType>{0}
                         : gko::remove_complex<ValueType>{r<OtherType>::value};
 
-    this->diag1->convert_to(tmp.get());
-    tmp->convert_to(res.get());
+    this->diag1->convert_to(tmp);
+    tmp->convert_to(res);
 
     GKO_ASSERT_MTX_NEAR(this->diag1, res, residual);
 }
@@ -141,8 +147,8 @@ TYPED_TEST(Diagonal, MovesToPrecision)
                         ? gko::remove_complex<ValueType>{0}
                         : gko::remove_complex<ValueType>{r<OtherType>::value};
 
-    this->diag1->move_to(tmp.get());
-    tmp->move_to(res.get());
+    this->diag1->move_to(tmp);
+    tmp->move_to(res);
 
     GKO_ASSERT_MTX_NEAR(this->diag1, res, residual);
 }
@@ -151,7 +157,7 @@ TYPED_TEST(Diagonal, MovesToPrecision)
 TYPED_TEST(Diagonal, AppliesToDense)
 {
     using value_type = typename TestFixture::value_type;
-    this->diag1->apply(this->dense1.get(), this->dense2.get());
+    this->diag1->apply(this->dense1, this->dense2);
 
     EXPECT_EQ(this->dense2->at(0, 0), value_type{2.0});
     EXPECT_EQ(this->dense2->at(0, 1), value_type{4.0});
@@ -169,10 +175,10 @@ TYPED_TEST(Diagonal, AppliesToMixedDense)
     using mixed_value_type = typename MixedDense::value_type;
     auto mdense1 = MixedDense::create(this->exec);
     auto mdense2 = MixedDense::create(this->exec);
-    this->dense1->convert_to(mdense1.get());
-    this->dense2->convert_to(mdense2.get());
+    this->dense1->convert_to(mdense1);
+    this->dense2->convert_to(mdense2);
 
-    this->diag1->apply(mdense1.get(), mdense2.get());
+    this->diag1->apply(mdense1, mdense2);
 
     EXPECT_EQ(mdense2->at(0, 0), mixed_value_type{2.0});
     EXPECT_EQ(mdense2->at(0, 1), mixed_value_type{4.0});
@@ -186,7 +192,7 @@ TYPED_TEST(Diagonal, AppliesToMixedDense)
 TYPED_TEST(Diagonal, RightAppliesToDense)
 {
     using value_type = typename TestFixture::value_type;
-    this->diag2->rapply(this->dense1.get(), this->dense2.get());
+    this->diag2->rapply(this->dense1, this->dense2);
 
     EXPECT_EQ(this->dense2->at(0, 0), value_type{2.0});
     EXPECT_EQ(this->dense2->at(0, 1), value_type{6.0});
@@ -204,10 +210,10 @@ TYPED_TEST(Diagonal, RightAppliesToMixedDense)
     using mixed_value_type = typename MixedDense::value_type;
     auto mdense1 = MixedDense::create(this->exec);
     auto mdense2 = MixedDense::create(this->exec);
-    this->dense1->convert_to(mdense1.get());
-    this->dense2->convert_to(mdense2.get());
+    this->dense1->convert_to(mdense1);
+    this->dense2->convert_to(mdense2);
 
-    this->diag2->rapply(mdense1.get(), mdense2.get());
+    this->diag2->rapply(mdense1, mdense2);
 
     EXPECT_EQ(mdense2->at(0, 0), mixed_value_type{2.0});
     EXPECT_EQ(mdense2->at(0, 1), mixed_value_type{6.0});
@@ -218,6 +224,41 @@ TYPED_TEST(Diagonal, RightAppliesToMixedDense)
 }
 
 
+TYPED_TEST(Diagonal, InverseAppliesToDense)
+{
+    using value_type = typename TestFixture::value_type;
+    this->diag1->inverse_apply(this->dense3, this->dense2);
+
+    EXPECT_EQ(this->dense2->at(0, 0), value_type{1.0});
+    EXPECT_EQ(this->dense2->at(0, 1), value_type{1.5});
+    EXPECT_EQ(this->dense2->at(0, 2), value_type{2.0});
+    EXPECT_EQ(this->dense2->at(1, 0), value_type{1.0});
+    EXPECT_EQ(this->dense2->at(1, 1), value_type{1.5});
+    EXPECT_EQ(this->dense2->at(1, 2), value_type{2.0});
+}
+
+
+TYPED_TEST(Diagonal, InverseAppliesToMixedDense)
+{
+    using value_type = typename TestFixture::value_type;
+    using MixedDense = typename TestFixture::MixedDense;
+    using mixed_value_type = typename MixedDense::value_type;
+    auto mdense2 = MixedDense::create(this->exec);
+    auto mdense3 = MixedDense::create(this->exec);
+    this->dense2->convert_to(mdense2);
+    this->dense3->convert_to(mdense3);
+
+    this->diag1->inverse_apply(mdense3, mdense2);
+
+    EXPECT_EQ(mdense2->at(0, 0), mixed_value_type{1.0});
+    EXPECT_EQ(mdense2->at(0, 1), mixed_value_type{1.5});
+    EXPECT_EQ(mdense2->at(0, 2), mixed_value_type{2.0});
+    EXPECT_EQ(mdense2->at(1, 0), mixed_value_type{1.0});
+    EXPECT_EQ(mdense2->at(1, 1), mixed_value_type{1.5});
+    EXPECT_EQ(mdense2->at(1, 2), mixed_value_type{2.0});
+}
+
+
 TYPED_TEST(Diagonal, AppliesLinearCombinationToDense)
 {
     using value_type = typename TestFixture::value_type;
@@ -225,8 +266,7 @@ TYPED_TEST(Diagonal, AppliesLinearCombinationToDense)
     auto alpha = gko::initialize<Dense>({-1.0}, this->exec);
     auto beta = gko::initialize<Dense>({2.0}, this->exec);
 
-    this->diag1->apply(alpha.get(), this->dense1.get(), beta.get(),
-                       this->dense2.get());
+    this->diag1->apply(alpha, this->dense1, beta, this->dense2);
 
     EXPECT_EQ(this->dense2->at(0, 0), value_type{0.0});
     EXPECT_EQ(this->dense2->at(0, 1), value_type{0.0});
@@ -246,10 +286,10 @@ TYPED_TEST(Diagonal, AppliesLinearCombinationToMixedDense)
     auto mdense2 = MixedDense::create(this->exec);
     auto alpha = gko::initialize<MixedDense>({-1.0}, this->exec);
     auto beta = gko::initialize<MixedDense>({2.0}, this->exec);
-    this->dense1->convert_to(mdense1.get());
-    this->dense2->convert_to(mdense2.get());
+    this->dense1->convert_to(mdense1);
+    this->dense2->convert_to(mdense2);
 
-    this->diag1->apply(alpha.get(), mdense1.get(), beta.get(), mdense2.get());
+    this->diag1->apply(alpha, mdense1, beta, mdense2);
 
     EXPECT_EQ(mdense2->at(0, 0), mixed_value_type{0.0});
     EXPECT_EQ(mdense2->at(0, 1), mixed_value_type{0.0});
@@ -267,7 +307,7 @@ TYPED_TEST(Diagonal, ApplyToDenseFailsForWrongInnerDimensions)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 3x3 times 2x3 = 3x3 --> mismatch for inner dimensions
-    ASSERT_THROW(this->diag2->apply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag2->apply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -279,7 +319,7 @@ TYPED_TEST(Diagonal, ApplyToDenseFailsForWrongNumberOfRows)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 2x2 times 2x3 = 3x3 --> mismatch for rows of diagonal and result
-    ASSERT_THROW(this->diag1->apply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag1->apply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -291,7 +331,7 @@ TYPED_TEST(Diagonal, ApplyToDenseFailsForWrongNumberOfCols)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x2 times 2x3 = 2x2 --> mismatch for cols of dense1 and result
-    ASSERT_THROW(this->diag1->apply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag1->apply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -303,7 +343,7 @@ TYPED_TEST(Diagonal, RightApplyToDenseFailsForWrongInnerDimensions)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x3 times 2x2 = 2x2 --> mismatch for inner DimensionMismatch
-    ASSERT_THROW(this->diag1->rapply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag1->rapply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -315,7 +355,7 @@ TYPED_TEST(Diagonal, RightApplyToDenseFailsForWrongNumberOfRows)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 2x3 times 3x3 = 3x3 --> mismatch for rows of dense1 and result
-    ASSERT_THROW(this->diag2->rapply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag2->rapply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -327,7 +367,7 @@ TYPED_TEST(Diagonal, RightApplyToDenseFailsForWrongNumberOfCols)
         gko::matrix::Dense<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x3 times 3x3 = 2x2 --> mismatch for cols of diagonal and result
-    ASSERT_THROW(this->diag2->rapply(this->dense1.get(), result.get()),
+    ASSERT_THROW(this->diag2->rapply(this->dense1, result),
                  gko::DimensionMismatch);
 }
 
@@ -335,7 +375,7 @@ TYPED_TEST(Diagonal, RightApplyToDenseFailsForWrongNumberOfCols)
 TYPED_TEST(Diagonal, AppliesToCsr)
 {
     using value_type = typename TestFixture::value_type;
-    this->diag1->apply(this->csr1.get(), this->csr2.get());
+    this->diag1->apply(this->csr1, this->csr2);
 
     const auto values = this->csr2->get_const_values();
     const auto row_ptrs = this->csr2->get_const_row_ptrs();
@@ -363,7 +403,7 @@ TYPED_TEST(Diagonal, AppliesToCsr)
 TYPED_TEST(Diagonal, RightAppliesToCsr)
 {
     using value_type = typename TestFixture::value_type;
-    this->diag2->rapply(this->csr1.get(), this->csr2.get());
+    this->diag2->rapply(this->csr1, this->csr2);
 
     const auto values = this->csr2->get_const_values();
     const auto row_ptrs = this->csr2->get_const_row_ptrs();
@@ -388,6 +428,34 @@ TYPED_TEST(Diagonal, RightAppliesToCsr)
 }
 
 
+TYPED_TEST(Diagonal, InverseAppliesToCsr)
+{
+    using value_type = typename TestFixture::value_type;
+    this->diag1->inverse_apply(this->csr3, this->csr2);
+
+    const auto values = this->csr2->get_const_values();
+    const auto row_ptrs = this->csr2->get_const_row_ptrs();
+    const auto col_idxs = this->csr2->get_const_col_idxs();
+
+    EXPECT_EQ(this->csr2->get_num_stored_elements(), 6);
+    EXPECT_EQ(values[0], value_type{1.0});
+    EXPECT_EQ(values[1], value_type{1.5});
+    EXPECT_EQ(values[2], value_type{2.0});
+    EXPECT_EQ(values[3], value_type{1.0});
+    EXPECT_EQ(values[4], value_type{1.5});
+    EXPECT_EQ(values[5], value_type{2.0});
+    EXPECT_EQ(row_ptrs[0], 0);
+    EXPECT_EQ(row_ptrs[1], 3);
+    EXPECT_EQ(row_ptrs[2], 6);
+    EXPECT_EQ(col_idxs[0], 0);
+    EXPECT_EQ(col_idxs[1], 1);
+    EXPECT_EQ(col_idxs[2], 2);
+    EXPECT_EQ(col_idxs[3], 0);
+    EXPECT_EQ(col_idxs[4], 1);
+    EXPECT_EQ(col_idxs[5], 2);
+}
+
+
 TYPED_TEST(Diagonal, ApplyToCsrFailsForWrongInnerDimensions)
 {
     using value_type = typename TestFixture::value_type;
@@ -395,7 +463,7 @@ TYPED_TEST(Diagonal, ApplyToCsrFailsForWrongInnerDimensions)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 3x3 times 2x3 = 3x3 --> mismatch for inner dimensions
-    ASSERT_THROW(this->diag2->apply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag2->apply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -407,7 +475,7 @@ TYPED_TEST(Diagonal, ApplyToCsrFailsForWrongNumberOfRows)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 2x2 times 2x3 = 3x3 --> mismatch for rows of diagonal and result
-    ASSERT_THROW(this->diag1->apply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag1->apply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -419,7 +487,7 @@ TYPED_TEST(Diagonal, ApplyToCsrFailsForWrongNumberOfCols)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x2 times 2x3 = 2x2 --> mismatch for cols of csr1 and result
-    ASSERT_THROW(this->diag1->apply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag1->apply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -431,7 +499,7 @@ TYPED_TEST(Diagonal, RightApplyToCsrFailsForWrongInnerDimensions)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x3 times 2x2 = 2x2 --> mismatch for inner DimensionMismatch
-    ASSERT_THROW(this->diag1->rapply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag1->rapply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -443,7 +511,7 @@ TYPED_TEST(Diagonal, RightApplyToCsrFailsForWrongNumberOfRows)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{3});
 
     // 2x3 times 3x3 = 3x3 --> mismatch for rows of csr1 and result
-    ASSERT_THROW(this->diag2->rapply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag2->rapply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -455,7 +523,7 @@ TYPED_TEST(Diagonal, RightApplyToCsrFailsForWrongNumberOfCols)
         gko::matrix::Csr<value_type>::create(this->exec, gko::dim<2>{2});
 
     // 2x3 times 3x3 = 2x2 --> mismatch for cols of diagonal and result
-    ASSERT_THROW(this->diag2->rapply(this->csr1.get(), result.get()),
+    ASSERT_THROW(this->diag2->rapply(this->csr1, result),
                  gko::DimensionMismatch);
 }
 
@@ -464,7 +532,7 @@ TYPED_TEST(Diagonal, ConvertsToCsr)
 {
     using value_type = typename TestFixture::value_type;
 
-    this->diag1->convert_to(this->csr1.get());
+    this->diag1->convert_to(this->csr1);
 
     const auto nnz = this->csr1->get_num_stored_elements();
     const auto row_ptrs = this->csr1->get_const_row_ptrs();
@@ -521,7 +589,7 @@ TYPED_TEST(Diagonal, AppliesToComplex)
                              exec);
     auto dense2 = Vec::create(exec, gko::dim<2>{2, 3});
 
-    this->diag1->apply(dense1.get(), dense2.get());
+    this->diag1->apply(dense1, dense2);
 
     GKO_ASSERT_MTX_NEAR(dense2,
                         l({{complex_type{2.0, 4.0}, complex_type{4.0, 8.0},
@@ -547,7 +615,7 @@ TYPED_TEST(Diagonal, AppliesToMixedComplex)
         exec);
     auto mdense2 = Vec::create(exec, gko::dim<2>{2, 3});
 
-    this->diag1->apply(mdense1.get(), mdense2.get());
+    this->diag1->apply(mdense1, mdense2);
 
     GKO_ASSERT_MTX_NEAR(
         mdense2,
@@ -581,7 +649,7 @@ TYPED_TEST(Diagonal, AppliesLinearCombinationToComplex)
     auto alpha = gko::initialize<Scalar>({-1.0}, this->exec);
     auto beta = gko::initialize<Scalar>({2.0}, this->exec);
 
-    this->diag1->apply(alpha.get(), dense1.get(), beta.get(), dense2.get());
+    this->diag1->apply(alpha, dense1, beta, dense2);
 
     GKO_ASSERT_MTX_NEAR(dense2,
                         l({{complex_type{0.0, 0.0}, complex_type{0.0, 0.0},
@@ -615,7 +683,7 @@ TYPED_TEST(Diagonal, AppliesLinearCombinationToMixedComplex)
     auto alpha = gko::initialize<Scalar>({-1.0}, this->exec);
     auto beta = gko::initialize<Scalar>({2.0}, this->exec);
 
-    this->diag1->apply(alpha.get(), dense1.get(), beta.get(), dense2.get());
+    this->diag1->apply(alpha, dense1, beta, dense2);
 
     GKO_ASSERT_MTX_NEAR(
         dense2,
@@ -634,7 +702,8 @@ protected:
     using Diag = gko::matrix::Diagonal<value_type>;
 };
 
-TYPED_TEST_SUITE(DiagonalComplex, gko::test::ComplexValueTypes);
+TYPED_TEST_SUITE(DiagonalComplex, gko::test::ComplexValueTypes,
+                 TypenameNameGenerator);
 
 
 TYPED_TEST(DiagonalComplex, MtxIsConjugateTransposable)
@@ -643,13 +712,13 @@ TYPED_TEST(DiagonalComplex, MtxIsConjugateTransposable)
     using value_type = typename TestFixture::value_type;
     auto exec = gko::ReferenceExecutor::create();
     auto diag = Diag::create(exec, 3);
-    auto diag_values = diag->get_values();
-    diag_values[0] = value_type{1.0, 2.0};
-    diag_values[1] = value_type{3.0, 0.0};
-    diag_values[2] = value_type{0.0, 1.5};
+    auto local_values = diag->get_values();
+    local_values[0] = value_type{1.0, 2.0};
+    local_values[1] = value_type{3.0, 0.0};
+    local_values[2] = value_type{0.0, 1.5};
 
     auto trans = diag->conj_transpose();
-    auto trans_as_diagonal = static_cast<Diag *>(trans.get());
+    auto trans_as_diagonal = static_cast<Diag*>(trans.get());
     auto trans_values = trans_as_diagonal->get_values();
 
     EXPECT_EQ(trans->get_size(), gko::dim<2>(3));
@@ -665,16 +734,16 @@ TYPED_TEST(DiagonalComplex, InplaceAbsolute)
     using value_type = typename TestFixture::value_type;
     auto exec = gko::ReferenceExecutor::create();
     auto diag = Diag::create(exec, 3);
-    auto diag_values = diag->get_values();
-    diag_values[0] = value_type{3.0, -4.0};
-    diag_values[1] = value_type{-3.0, 0.0};
-    diag_values[2] = value_type{0.0, -1.5};
+    auto local_values = diag->get_values();
+    local_values[0] = value_type{3.0, -4.0};
+    local_values[1] = value_type{-3.0, 0.0};
+    local_values[2] = value_type{0.0, -1.5};
 
     diag->compute_absolute_inplace();
 
-    EXPECT_EQ(diag_values[0], (value_type{5.0, 0.0}));
-    EXPECT_EQ(diag_values[1], (value_type{3.0, 0.0}));
-    EXPECT_EQ(diag_values[2], (value_type{1.5, 0.0}));
+    EXPECT_EQ(local_values[0], (value_type{5.0, 0.0}));
+    EXPECT_EQ(local_values[1], (value_type{3.0, 0.0}));
+    EXPECT_EQ(local_values[2], (value_type{1.5, 0.0}));
 }
 
 
@@ -685,10 +754,10 @@ TYPED_TEST(DiagonalComplex, OutplaceAbsolute)
     using abs_type = gko::remove_complex<value_type>;
     auto exec = gko::ReferenceExecutor::create();
     auto diag = Diag::create(exec, 3);
-    auto diag_values = diag->get_values();
-    diag_values[0] = value_type{3.0, -4.0};
-    diag_values[1] = value_type{-3.0, 0.0};
-    diag_values[2] = value_type{0.0, -1.5};
+    auto local_values = diag->get_values();
+    local_values[0] = value_type{3.0, -4.0};
+    local_values[1] = value_type{-3.0, 0.0};
+    local_values[2] = value_type{0.0, -1.5};
 
     auto abs_diag = diag->compute_absolute();
     auto abs_values = abs_diag->get_values();
