@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -105,14 +105,14 @@ protected:
     }
 
     template <typename T>
-    void init_array(T *arr, std::initializer_list<T> vals)
+    void init_array(T* arr, std::initializer_list<T> vals)
     {
         std::copy(std::begin(vals), std::end(vals), arr);
     }
 
     template <typename ValueType>
-    void assert_same_block(gko::size_type block_size, const ValueType *ptr_a,
-                           gko::size_type stride_a, const ValueType *ptr_b,
+    void assert_same_block(gko::size_type block_size, const ValueType* ptr_a,
+                           gko::size_type stride_a, const ValueType* ptr_b,
                            gko::size_type stride_b)
     {
         for (int i = 0; i < block_size; ++i) {
@@ -124,7 +124,8 @@ protected:
         }
     }
 
-    void assert_same_precond(const Bj *a, const Bj *b)
+    void assert_same_precond(gko::ptr_param<const Bj> a,
+                             gko::ptr_param<const Bj> b)
     {
         ASSERT_EQ(a->get_size()[0], b->get_size()[0]);
         ASSERT_EQ(a->get_size()[1], b->get_size()[1]);
@@ -154,11 +155,11 @@ protected:
                 value_type, prec_a,
                 assert_same_block(
                     b_ptr_a[i + 1] - b_ptr_a[i],
-                    reinterpret_cast<const resolved_precision *>(
+                    reinterpret_cast<const resolved_precision*>(
                         a->get_blocks() + scheme.get_group_offset(i)) +
                         scheme.get_block_offset(i),
                     scheme.get_stride(),
-                    reinterpret_cast<const resolved_precision *>(
+                    reinterpret_cast<const resolved_precision*>(
                         a->get_blocks() + scheme.get_group_offset(i)) +
                         scheme.get_block_offset(i),
                     scheme.get_stride()));
@@ -169,14 +170,14 @@ protected:
     std::unique_ptr<typename Bj::Factory> bj_factory;
     std::unique_ptr<typename Bj::Factory> scalar_j_factory;
     std::unique_ptr<typename Bj::Factory> adaptive_bj_factory;
-    gko::Array<index_type> block_pointers;
-    gko::Array<gko::precision_reduction> block_precisions;
+    gko::array<index_type> block_pointers;
+    gko::array<gko::precision_reduction> block_precisions;
     std::shared_ptr<Mtx> mtx;
     std::unique_ptr<Bj> bj;
     std::unique_ptr<Bj> adaptive_bj;
 };
 
-TYPED_TEST_SUITE(Jacobi, gko::test::ValueIndexTypes);
+TYPED_TEST_SUITE(Jacobi, gko::test::ValueIndexTypes, PairTypenameNameGenerator);
 
 
 TYPED_TEST(Jacobi, GeneratesCorrectStorageScheme)
@@ -193,14 +194,14 @@ TYPED_TEST(Jacobi, CanBeCloned)
 {
     auto bj_clone = clone(this->bj);
 
-    this->assert_same_precond(lend(bj_clone), lend(this->bj));
+    this->assert_same_precond(bj_clone, this->bj);
 }
 
 
 TYPED_TEST(Jacobi, CanBeClonedWithAdaptvePrecision)
 {
     auto bj_clone = clone(this->adaptive_bj);
-    this->assert_same_precond(lend(bj_clone), lend(this->adaptive_bj));
+    this->assert_same_precond(bj_clone, this->adaptive_bj);
 }
 
 
@@ -209,16 +210,16 @@ TYPED_TEST(Jacobi, CanBeCopied)
     using Bj = typename TestFixture::Bj;
     using Mtx = typename TestFixture::Mtx;
     using index_type = typename TestFixture::index_type;
-    gko::Array<index_type> empty(this->exec, 1);
+    gko::array<index_type> empty(this->exec, 1);
     empty.get_data()[0] = 0;
     auto copy = Bj::build()
                     .with_block_pointers(empty)
                     .on(this->exec)
                     ->generate(Mtx::create(this->exec));
 
-    copy->copy_from(lend(this->bj));
+    copy->copy_from(this->bj);
 
-    this->assert_same_precond(lend(copy), lend(this->bj));
+    this->assert_same_precond(copy, this->bj);
 }
 
 
@@ -227,16 +228,16 @@ TYPED_TEST(Jacobi, CanBeCopiedWithAdaptivePrecision)
     using Bj = typename TestFixture::Bj;
     using Mtx = typename TestFixture::Mtx;
     using index_type = typename TestFixture::index_type;
-    gko::Array<index_type> empty(this->exec, 1);
+    gko::array<index_type> empty(this->exec, 1);
     empty.get_data()[0] = 0;
     auto copy = Bj::build()
                     .with_block_pointers(empty)
                     .on(this->exec)
                     ->generate(Mtx::create(this->exec));
 
-    copy->copy_from(lend(this->adaptive_bj));
+    copy->copy_from(this->adaptive_bj);
 
-    this->assert_same_precond(lend(copy), lend(this->adaptive_bj));
+    this->assert_same_precond(copy, this->adaptive_bj);
 }
 
 
@@ -246,16 +247,16 @@ TYPED_TEST(Jacobi, CanBeMoved)
     using Mtx = typename TestFixture::Mtx;
     using index_type = typename TestFixture::index_type;
     auto tmp = clone(this->bj);
-    gko::Array<index_type> empty(this->exec, 1);
+    gko::array<index_type> empty(this->exec, 1);
     empty.get_data()[0] = 0;
     auto copy = Bj::build()
                     .with_block_pointers(empty)
                     .on(this->exec)
                     ->generate(Mtx::create(this->exec));
 
-    copy->copy_from(give(this->bj));
+    copy->move_from(this->bj);
 
-    this->assert_same_precond(lend(copy), lend(tmp));
+    this->assert_same_precond(copy.get(), tmp);
 }
 
 
@@ -265,16 +266,16 @@ TYPED_TEST(Jacobi, CanBeMovedWithAdaptivePrecision)
     using Mtx = typename TestFixture::Mtx;
     using index_type = typename TestFixture::index_type;
     auto tmp = clone(this->adaptive_bj);
-    gko::Array<index_type> empty(this->exec, 1);
+    gko::array<index_type> empty(this->exec, 1);
     empty.get_data()[0] = 0;
     auto copy = Bj::build()
                     .with_block_pointers(empty)
                     .on(this->exec)
                     ->generate(Mtx::create(this->exec));
 
-    copy->copy_from(give(this->adaptive_bj));
+    copy->move_from(this->adaptive_bj);
 
-    this->assert_same_precond(lend(copy), lend(tmp));
+    this->assert_same_precond(copy.get(), tmp);
 }
 
 
@@ -316,11 +317,11 @@ TYPED_TEST(Jacobi, ScalarJacobiConvertsToDense)
     gko::matrix_data<value_type, index_type> data;
     auto csr = gko::share(
         gko::matrix::Csr<value_type, index_type>::create(this->exec));
-    csr->copy_from(gko::lend(this->mtx));
+    csr->copy_from(this->mtx);
     auto scalar_j = this->scalar_j_factory->generate(csr);
 
     auto dense_j = gko::matrix::Dense<value_type>::create(this->exec);
-    dense_j->copy_from(scalar_j.get());
+    dense_j->copy_from(scalar_j);
     auto j_val = scalar_j->get_blocks();
 
     for (auto i = 0; i < dense_j->get_size()[0]; ++i) {
@@ -343,7 +344,7 @@ TYPED_TEST(Jacobi, ScalarJacobiCanBeTransposed)
     gko::matrix_data<value_type, index_type> data;
     auto csr = gko::share(
         gko::matrix::Csr<value_type, index_type>::create(this->exec));
-    csr->copy_from(gko::lend(this->mtx));
+    csr->copy_from(this->mtx);
     auto scalar_j = this->scalar_j_factory->generate(csr);
 
     auto dense_j = gko::matrix::Dense<value_type>::create(this->exec);
@@ -366,7 +367,7 @@ TYPED_TEST(Jacobi, ScalarJacobiCanBeTransposed)
 
 
 template <typename T>
-void init_array(T *arr, std::initializer_list<T> vals)
+void init_array(T* arr, std::initializer_list<T> vals)
 {
     std::copy(std::begin(vals), std::end(vals), arr);
 }
@@ -460,7 +461,7 @@ TYPED_TEST(Jacobi, ScalarJacobiGeneratesCorrectMatrixData)
     using tpl = typename decltype(data)::nonzero_type;
     auto csr = gko::share(
         gko::matrix::Csr<value_type, index_type>::create(this->exec));
-    csr->copy_from(gko::lend(this->mtx));
+    csr->copy_from(this->mtx);
     auto scalar_j = this->scalar_j_factory->generate(csr);
 
     scalar_j->write(data);
@@ -513,7 +514,7 @@ TYPED_TEST(Jacobi, ScalarJacobiGeneratesOnDifferentPrecision)
     using Bj = typename TestFixture::Bj;
     auto csr =
         gko::share(gko::matrix::Csr<next_type, index_type>::create(this->exec));
-    csr->copy_from(gko::lend(this->mtx));
+    csr->copy_from(this->mtx);
     std::shared_ptr<Bj> bj{};
 
     ASSERT_NO_THROW(bj = this->scalar_j_factory->generate(csr));
