@@ -1,5 +1,5 @@
 /*******************************<GINKGO LICENSE>******************************
-Copyright (c) 2017-2021, the Ginkgo authors
+Copyright (c) 2017-2023, the Ginkgo authors
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -103,7 +103,7 @@ protected:
     }
 
     template <typename T>
-    void init_array(T *arr, std::initializer_list<T> vals)
+    void init_array(T* arr, std::initializer_list<T> vals)
     {
         for (auto elem : vals) {
             *(arr++) = elem;
@@ -114,12 +114,12 @@ protected:
     std::unique_ptr<typename Bj::Factory> bj_factory;
     std::unique_ptr<typename Bj::Factory> scalar_j_factory;
     std::unique_ptr<typename Bj::Factory> adaptive_bj_factory;
-    gko::Array<index_type> block_pointers;
-    gko::Array<gko::precision_reduction> block_precisions;
+    gko::array<index_type> block_pointers;
+    gko::array<gko::precision_reduction> block_precisions;
     std::shared_ptr<gko::matrix::Csr<value_type, index_type>> mtx;
 };
 
-TYPED_TEST_SUITE(Jacobi, gko::test::ValueIndexTypes);
+TYPED_TEST_SUITE(Jacobi, gko::test::ValueIndexTypes, PairTypenameNameGenerator);
 
 
 TYPED_TEST(Jacobi, CanBeGenerated)
@@ -309,7 +309,7 @@ TYPED_TEST(Jacobi, InvertsDiagonalBlocksWithAdaptivePrecision)
     const auto b_prec_bj =
         bj->get_parameters().storage_optimization.block_wise.get_const_data();
     using reduced = ::gko::reduce_precision<T>;
-    auto b1 = reinterpret_cast<const reduced *>(
+    auto b1 = reinterpret_cast<const reduced*>(
         bj->get_blocks() + scheme.get_global_block_offset(0));
     GKO_EXPECT_NEAR(b1[0 + 0 * p], reduced{4.0 / 14.0}, half_tol);
     GKO_EXPECT_NEAR(b1[0 + 1 * p], reduced{2.0 / 14.0}, half_tol);
@@ -369,7 +369,7 @@ TYPED_TEST(Jacobi, CanTransposeDiagonalBlocksWithAdaptivePrecision)
     auto scheme = bj->get_storage_scheme();
     auto p = scheme.get_stride();
     using reduced = ::gko::reduce_precision<T>;
-    auto b1 = reinterpret_cast<const reduced *>(
+    auto b1 = reinterpret_cast<const reduced*>(
         bj->get_blocks() + scheme.get_global_block_offset(0));
     GKO_EXPECT_NEAR(b1[0 + 0 * p], reduced{4.0 / 14.0}, half_tol);
     GKO_EXPECT_NEAR(b1[1 + 0 * p], reduced{2.0 / 14.0}, half_tol);
@@ -428,7 +428,7 @@ TYPED_TEST(Jacobi, CanConjTransposeDiagonalBlocksWithAdaptivePrecision)
     auto scheme = bj->get_storage_scheme();
     auto p = scheme.get_stride();
     using reduced = ::gko::reduce_precision<T>;
-    auto b1 = reinterpret_cast<const reduced *>(
+    auto b1 = reinterpret_cast<const reduced*>(
         bj->get_blocks() + scheme.get_global_block_offset(0));
     GKO_EXPECT_NEAR(b1[0 + 0 * p], reduced{4.0 / 14.0}, half_tol);
     GKO_EXPECT_NEAR(b1[1 + 0 * p], reduced{2.0 / 14.0}, half_tol);
@@ -486,7 +486,7 @@ TYPED_TEST(Jacobi, PivotsWhenInvertingBlocks)
     using Mtx = typename TestFixture::Mtx;
     using T = typename TestFixture::value_type;
     using index_type = typename TestFixture::index_type;
-    gko::Array<index_type> bp(this->exec, 2);
+    gko::array<index_type> bp(this->exec, 2);
     this->template init_array<index_type>(bp.get_data(), {0, 3});
     auto mtx = Mtx::create(this->exec, gko::dim<2>{3}, 9);
     /* test matrix:
@@ -528,7 +528,7 @@ TYPED_TEST(Jacobi, PivotsWhenInvertingBlocksWithiAdaptivePrecision)
     using index_type = typename TestFixture::index_type;
     using T = typename TestFixture::value_type;
     auto half_tol = std::sqrt(r<T>::value);
-    gko::Array<index_type> bp(this->exec, 2);
+    gko::array<index_type> bp(this->exec, 2);
     this->template init_array<index_type>(bp.get_data(), {0, 3});
     auto mtx = Mtx::create(this->exec, gko::dim<2>{3}, 9);
     /* test matrix:
@@ -552,7 +552,7 @@ TYPED_TEST(Jacobi, PivotsWhenInvertingBlocksWithiAdaptivePrecision)
     auto scheme = bj->get_storage_scheme();
     auto p = scheme.get_stride();
     using reduced = ::gko::reduce_precision<T>;
-    auto b1 = reinterpret_cast<const reduced *>(
+    auto b1 = reinterpret_cast<const reduced*>(
         bj->get_blocks() + scheme.get_global_block_offset(0));
     GKO_EXPECT_NEAR(b1[0 + 0 * p], reduced{0.0 / 4.0}, half_tol);
     GKO_EXPECT_NEAR(b1[0 + 1 * p], reduced{0.0 / 4.0}, half_tol);
@@ -627,7 +627,7 @@ TYPED_TEST(Jacobi, AvoidsPrecisionsThatOverflow)
     auto bj =
         Bj::build()
             .with_max_block_size(13u)
-            .with_block_pointers(gko::Array<index_type>(this->exec, {0, 2, 4}))
+            .with_block_pointers(gko::array<index_type>(this->exec, {0, 2, 4}))
             .with_storage_optimization(gko::precision_reduction::autodetect())
             .with_accuracy(gko::remove_complex<T>{1e-1})
             .on(this->exec)
@@ -652,7 +652,7 @@ TYPED_TEST(Jacobi, AppliesToVector)
     auto b = gko::initialize<Vec>({4.0, -1.0, -2.0, 4.0, -1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 0.0, 0.0, 1.0, 0.0}), r<value_type>::value);
 }
@@ -666,7 +666,7 @@ TYPED_TEST(Jacobi, ScalarJacobiAppliesToVector)
     auto b = gko::initialize<Vec>({4.0, -1.0, -2.0, 4.0, -1.0}, this->exec);
     auto sj = this->scalar_j_factory->generate(this->mtx);
 
-    sj->apply(b.get(), x.get());
+    sj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, -0.25, -0.5, 1.0, -0.25}),
                         r<value_type>::value);
@@ -681,7 +681,7 @@ TYPED_TEST(Jacobi, AppliesToMixedVector)
     auto b = gko::initialize<Vec>({4.0, -1.0, -2.0, 4.0, -1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({1.0, 0.0, 0.0, 1.0, 0.0}),
@@ -703,7 +703,7 @@ TYPED_TEST(Jacobi, AppliesToComplexVector)
         this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x,
@@ -728,7 +728,7 @@ TYPED_TEST(Jacobi, AppliesToMixedComplexVector)
         this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x,
@@ -747,7 +747,7 @@ TYPED_TEST(Jacobi, AppliesToVectorWithAdaptivePrecision)
     auto b = gko::initialize<Vec>({4.0, -1.0, -2.0, 4.0, -1.0}, this->exec);
     auto bj = this->adaptive_bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 0.0, 0.0, 1.0, 0.0}), half_tol);
 }
@@ -769,7 +769,7 @@ TYPED_TEST(Jacobi, AppliesToVectorWithAdaptivePrecisionAndSmallBlocks)
                   .on(this->exec)
                   ->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 0.0, 0.0, 1.0, 0.0}), half_tol);
 }
@@ -792,7 +792,7 @@ TYPED_TEST(Jacobi, AppliesToMultipleVectors)
                              this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, 0.0}, {0.0, 1.0}, {0.0, 0.0}, {1.0, 0.0}, {0.0, 1.0}}),
@@ -817,7 +817,7 @@ TYPED_TEST(Jacobi, ScalarJacobiAppliesToMultipleVectors)
                              this->exec);
     auto sj = this->scalar_j_factory->generate(this->mtx);
 
-    sj->apply(b.get(), x.get());
+    sj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x,
@@ -844,7 +844,7 @@ TYPED_TEST(Jacobi, AppliesToMultipleVectorsWithAdaptivePrecision)
                              this->exec);
     auto bj = this->adaptive_bj_factory->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, 0.0}, {0.0, 1.0}, {0.0, 0.0}, {1.0, 0.0}, {0.0, 1.0}}),
@@ -877,7 +877,7 @@ TYPED_TEST(Jacobi, AppliesToMultipleVectorsWithAdaptivePrecisionAndSmallBlocks)
                   .on(this->exec)
                   ->generate(this->mtx);
 
-    bj->apply(b.get(), x.get());
+    bj->apply(b, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, 0.0}, {0.0, 1.0}, {0.0, 0.0}, {1.0, 0.0}, {0.0, 1.0}}),
@@ -895,7 +895,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToVector)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 1.0, -2.0, 4.0, -3.0}),
                         r<value_type>::value);
@@ -912,7 +912,7 @@ TYPED_TEST(Jacobi, ScalarJacobiAppliesLinearCombinationToVector)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto sj = this->scalar_j_factory->generate(this->mtx);
 
-    sj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    sj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 0.5, -3.0, 4.0, -3.5}),
                         r<value_type>::value);
@@ -929,7 +929,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToMixedVector)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({1.0, 1.0, -2.0, 4.0, -3.0}),
@@ -953,7 +953,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToComplexVector)
     auto beta = gko::initialize<Dense>({-1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(x,
                         l({T{1.0, 2.0}, T{1.0, 2.0}, T{-2.0, -4.0}, T{4.0, 8.0},
@@ -978,7 +978,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToMixedComplexVector)
     auto beta = gko::initialize<MixedDense>({-1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(
         x,
@@ -999,7 +999,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToVectorWithAdaptivePrecision)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto bj = this->adaptive_bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(x, l({1.0, 1.0, -2.0, 4.0, -3.0}), half_tol);
 }
@@ -1025,7 +1025,7 @@ TYPED_TEST(Jacobi, AppliesLinearCombinationToMultipleVectors)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto bj = this->bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, -0.5}, {1.0, 2.5}, {-2.0, -1.0}, {4.0, 1.0}, {-3.0, 0.5}}),
@@ -1053,7 +1053,7 @@ TYPED_TEST(Jacobi, ScalarAppliesLinearCombinationToMultipleVectors)
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto sj = this->scalar_j_factory->generate(this->mtx);
 
-    sj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    sj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, -1.5}, {0.5, 2.5}, {-3.0, -1.0}, {4.0, 0.0}, {-3.5, 0.5}}),
@@ -1082,7 +1082,7 @@ TYPED_TEST(Jacobi,
     auto beta = gko::initialize<Vec>({-1.0}, this->exec);
     auto bj = this->adaptive_bj_factory->generate(this->mtx);
 
-    bj->apply(alpha.get(), b.get(), beta.get(), x.get());
+    bj->apply(alpha, b, beta, x);
 
     GKO_ASSERT_MTX_NEAR(
         x, l({{1.0, -0.5}, {1.0, 2.5}, {-2.0, -1.0}, {4.0, 1.0}, {-3.0, 0.5}}),
@@ -1096,7 +1096,7 @@ TYPED_TEST(Jacobi, ConvertsToDense)
     using value_type = typename TestFixture::value_type;
     auto dense = Vec::create(this->exec);
 
-    dense->copy_from(this->bj_factory->generate(this->mtx));
+    dense->move_from(this->bj_factory->generate(this->mtx));
 
     // clang-format off
     GKO_ASSERT_MTX_NEAR(dense,
@@ -1116,7 +1116,7 @@ TYPED_TEST(Jacobi, ConvertsToDenseWithAdaptivePrecision)
     auto half_tol = std::sqrt(r<value_type>::value);
     auto dense = Vec::create(this->exec);
 
-    dense->copy_from(this->adaptive_bj_factory->generate(this->mtx));
+    dense->move_from(this->adaptive_bj_factory->generate(this->mtx));
 
     // clang-format off
     GKO_ASSERT_MTX_NEAR(dense,
@@ -1132,11 +1132,10 @@ TYPED_TEST(Jacobi, ConvertsToDenseWithAdaptivePrecision)
 TYPED_TEST(Jacobi, ConvertsEmptyToDense)
 {
     using Vec = typename TestFixture::Vec;
-    auto empty = Vec::create(this->exec);
+    auto empty = gko::share(Vec::create(this->exec));
     auto res = Vec::create(this->exec);
 
-    res->copy_from(
-        TestFixture::Bj::build().on(this->exec)->generate(gko::share(empty)));
+    res->move_from(TestFixture::Bj::build().on(this->exec)->generate(empty));
 
     ASSERT_FALSE(res->get_size());
 }
